/*
 * setup.c
 *
 * Gary Jennejohn (C) 2003 <gj@denx.de>
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * INCAIP specific setup.
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/module.h>

#include <linux/interrupt.h>
#include <asm/time.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>

extern void incaip_reboot_setup(void);

void __init bus_error_init(void) { /* nothing */ }

/* modify inca-ip register with a small delay after writing to the register */
/* this delay is located in the same cache line as the write access, to prevent */
/* erroneous updates of the caches (seen with CGU_DIVCR register !) */
/* Input: reg = address of register */
/*        andmask = bits to clear in the register */
/*        ormask  = bits to set in the register */
static inline unsigned long modify_inca_reg(volatile unsigned int *reg, 
                                            unsigned long andmask, 
                                            unsigned long ormask)
{
    int count=15000;
    unsigned long regValue = 0;

    /* equivalent C-Code: (*reg) = ((*reg) & (~andmask)) | (ormask); */
    asm volatile (" .set noreorder\n"
                  "\tlw\t%2,(%3)\n"
                  "\tand\t%2,%2,%5\n"
                  "\tor\t%2,%2,%6\n"
                  "\t.align\t4\n" /* align on cache line boundary */
                  "\tsw\t%2,(%3)\n"
                  "\t0:\n"
                  "\tbnez\t%4,0b\n"
                  "\taddiu\t%4,%4,-1\n" /* branch delay */
                  "\t.set\treorder\n"
                  :
                  "=r" (regValue), /* %0 - output */
                  "=r" (count)  /* %1 - count is modified */
                  :
                  "0" (regValue),    /* %2 - same register as %0 */
                  "r" (reg),    /* %3 - address to modify */
                  "1" (count),  /* %4 - number of delay cycles */
                  "r" (~andmask), /* %5 - value to 'and' with on-chip register */
                  "r" (ormask)  /* %6 - value to 'or' with on-chip register */
/*                    : */
                  /* nothing clobbered */
        );

    return regValue;
}

/* the CPU clock rate - lifted from u-boot */
unsigned int
incaip_get_cpu_hz(void)
{
	/*----------------------------------------------------------------*/
	/* CPU Clock Input Multiplexer (MUX I)                            */
	/* Multiplexer MUX I selects the maximum input clock to the CPU.  */
	/*----------------------------------------------------------------*/
	if (*INCA_IP_CGU_CGU_MUXCR & INCA_IP_CGU_CGU_MUXCR_MUXI)
	{
		/* MUX I set to 150 MHz clock */
		return 150000000;
	}
	else
	{
		/* MUX I set to 100/133 MHz clock */
		if (*INCA_IP_CGU_CGU_DIVCR & 0x40) 
		{
			 /* Division value is 1/3, maximum CPU operating */
			 /* frequency is 133.3 MHz                       */
			 return 133333333;
		}
		else
		{
			 /* Division value is 1/4, maximum CPU operating */
			 /* frequency is 100 MHz                         */
			 return 100000000;
		}
	}
}

/* the FPI clock rate - lifted from u-boot */
unsigned int
incaip_get_fpi_hz(void)
{
	unsigned int  clkCPU;

	clkCPU = incaip_get_cpu_hz();

	switch (*INCA_IP_CGU_CGU_DIVCR & 0xC)
	{
		case 0x4:
			return clkCPU >> 1; /* divided by 2 */
		break;
		case 0x8:
			return clkCPU >> 2; /* divided by 4 */
		break;
		default:
			return clkCPU;
		break;
	}
}

/* get the CPU version number  - based on sysLib.c from VxWorks sources */
/* this doesn't really belong here, but it's a convenient location */
unsigned int
incaip_get_cpu_ver(void)
{
	static unsigned int cpu_ver = 0;

	if (cpu_ver == 0)
		cpu_ver = *INCA_IP_WDT_CHIPID & 0xFFFFF000;
	return cpu_ver;
}
EXPORT_SYMBOL(incaip_get_cpu_hz);
EXPORT_SYMBOL(incaip_get_fpi_hz);
EXPORT_SYMBOL(incaip_get_cpu_ver);

void incaip_time_init(void)
{
	extern int *cpuoptions;

	/* do not want to set mips_counter_frequency! */
	/* I don't know whether this is kosher, but CPU_COUNTER is broken */
	*cpuoptions &= ~MIPS_CPU_COUNTER;
}

static void prom_printf(const char * fmt, ...);
/* this gets called indirectly from incaip_hw2_irqdispatch */
void incaip_timer6_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
#ifndef COUNTER_WORKS
	timer_interrupt(INT_NUM_IM2_IRL29, NULL, regs);
#endif
}

/*
 * THe CPU timer doesn't work (yet) on INCAIP, so we have to
 * use TIMER6 in its place. Takenb from sysLib.c. Timer is set
 * to 100 Hz.
 * Also misuse this routine to print out the CPU type and clock.
 */
void incaip_timer_setup(struct irqaction *irq)
{
#ifndef COUNTER_WORKS
	volatile unsigned int *pPM_CR = INCA_IP_PMU_PM_CR;
	volatile unsigned int *pPM_GEN = INCA_IP_PMU_PM_GEN;
	volatile unsigned int *pCGU_DIVCR = INCA_IP_CGU_CGU_DIVCR;
	int retval;
	unsigned int ver;

	ver = incaip_get_cpu_ver();
	printk("INCA-IP CPU Ver. ");
	if ((INCA_IP_WDT_CHIPID_VERSION_GET(ver) & 0x01) == 0x01)
		printk("1.1 Clock ");
	else if ((INCA_IP_WDT_CHIPID_VERSION_GET(ver) & 0x02) == 0x02)
		printk("1.2 Clock ");

	/* setup the PMU */
        modify_inca_reg(pPM_CR, /* reg */
                        3,      /* and-mask */
                        INCA_IP_PMU_PM_CR_SWCR | INCA_IP_PMU_PM_CR_SWRST); /* or-mask */

	/* setup the CGU */
        modify_inca_reg(pCGU_DIVCR, /* register */
                        INCAIP_CGU_DIVCR_FPI_3, /* and-mask (which bits to clear) */
                        INCA_IP_CGU_CGU_DIVCR_SWRST | INCAIP_CGU_DIVCR_FPI_1); /* or-mask */

	/* enable the timer in the PMU */
        modify_inca_reg(pPM_GEN, 0, INCA_IP_PMU_PM_GEN_EN10);

	/* setup the GPTU for timer tick */
	*(INCA_IP_GPTU_GPT_CLC) = 0x100;

	if (incaip_get_cpu_hz() == 100000000)
	{
		printk("100MHz\n");
		/* for CPU frequency is 100 MHz */
		*(INCA_IP_GPTU_GPT_CAPREL) = 0x3D09;
		*(INCA_IP_GPTU_GPT_T6CON) = 0x80C3;
	}
	else if (incaip_get_cpu_hz() == 150000000)
	{
		printk("150MHz\n");
		/* for CPU frequency is 150 MHz */
		*(INCA_IP_GPTU_GPT_CAPREL) = 0xB71B;
		*(INCA_IP_GPTU_GPT_T6CON) = 0x80C2;
	}
	else
	{
		printk("133MHz\n");
		/* for CPU frequency of 133 MHz */
		*(INCA_IP_GPTU_GPT_CAPREL) = 0xA2C2;
		*(INCA_IP_GPTU_GPT_T6CON) = 0x80C2;
	}

	/* do it by hand */
	irq->handler = incaip_timer6_interrupt;
	retval = setup_irq(INT_NUM_IM2_IRL29, irq);
	if (retval)
		printk("setup_irq failed\n");
#endif
}

#define MIPS_CPU_TIMER_IRQ 7
/* this gets called straight from incaipIRQ.S */
asmlinkage void incaip_timer_interrupt(struct pt_regs *regs)
{
    prom_printf("incaip_timer_interrupt\n");
#ifdef COUNTER_WORKS
	timer_interrupt(MIPS_CPU_TIMER_IRQ, NULL, regs);
#endif
}

static char buf[1024];

static void prom_printf(const char * fmt, ...)
{
	va_list args;
	int l;
	char *p, *buf_end;

	/* Low level, brute force, not SMP safe... */
	va_start(args, fmt);
	l = vsprintf(buf, fmt, args); /* hopefully i < sizeof(buf) */
	va_end(args);
	buf_end = buf + l;
	
	for (p = buf; p < buf_end; p++) {
		/* Wait for FIFO to empty */
		while (((*INCA_IP_ASC_FSTAT) >> 8) != 0x00) ; 
		/* Crude cr/nl handling is better than none */
		if(*p == '\n') *INCA_IP_ASC_ASC_TBUF=('\r');
		*INCA_IP_ASC_ASC_TBUF=(*p);
	}

}

#define INCAIP_COMPACT		0x00C1
#define INCAIP_STANDARD		0x00C0
extern void inca_mux_init(void);

void __init incaip_setup(void)
{
	u32 chipid = 0;
	u32 part_no = 0;

	prom_printf("Checking for Inca-IP type.\n");
	chipid = *(INCA_IP_WDT_CHIPID);
	part_no = INCA_IP_WDT_CHIPID_PART_NUMBER_GET(chipid);
#ifdef CONFIG_INCAIP_COMPACT
	if (part_no != INCAIP_COMPACT) {
		prom_printf("FATAL ERROR.\n");
		prom_printf("Inca-IP Compact kernel running on Inca-IP\n");
		prom_printf("Died.\n");
		panic("Help!");
	}
#else
	if (part_no == INCAIP_COMPACT) {
		prom_printf("FATAL ERROR.\n");
		prom_printf("Inca-IP kernel running on Inca-IP Compact");
		prom_printf("Died.\n");
		panic("Help!");
	}
#endif		

	/* 
	 * disable the ethernet switch to avoid problems 
	 * with ethernet traffic during startup 
	 */
	*INCA_IP_PMU_PM_GEN &= ~( INCA_IP_PMU_PM_GEN_EN5 | INCA_IP_PMU_PM_GEN_EN6 );    

	inca_mux_init();
	
	incaip_reboot_setup();
	board_time_init = incaip_time_init;
	board_timer_setup = incaip_timer_setup;

}
