/*
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * i2c-inca.c
 *
 * Inca-IP I2C adapter driver
 *
 * Andre Messerschmidt 
 * Infineon Technologies AG (C) 2003
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/string.h>  /* for 2.0 kernels to get NULL   */
#include <asm/errno.h>     /* for 2.0 kernels to get ENODEV */
#include <asm/io.h>

#include <linux/i2c.h>
#include "i2c-algo-inca.h"

#define I2C_HW_INCA 0x00

static int inca_writebyte(int addr, char data)
{
	return 0;
}

static char inca_readbyte(int addr)
{
	return 1;
}

static int inca_init(void)
{
	return 0;
}

static void __exit inca_exit(void)
{	
	;
}


static int inca_reg(struct i2c_client *client)
{
	return 0;
}

static int inca_unreg(struct i2c_client *client)
{
	return 0;
}

static void inca_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void inca_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_algo_inca_data inca_data = {
	NULL,
	inca_writebyte,
	inca_readbyte,
};

static struct i2c_adapter inca_ops = {
	"INCA-IP",
	I2C_HW_INCA,
	NULL,
	&inca_data,
	inca_inc_use,
	inca_dec_use,
	inca_reg,
	inca_unreg,
};

int __init  i2c_inca_init(void)
{
	printk("i2c-inca.o: i2c INCA-IP adapter module\n");
	if (inca_init()==0) {
		if(i2c_inca_add_bus(&inca_ops) < 0)
			return -ENODEV;
	} else {
		return -ENODEV;
	}
	return 0;
}

module_init(i2c_inca_init);

#ifdef MODULE
MODULE_AUTHOR("Andre Messerschmidt");
MODULE_DESCRIPTION("I2C-Bus adapter routines for INCA-IP");
MODULE_LICENSE("GPL");

/*
int init_module(void) 
{
	return i2c_inca_init();
}
*/

void cleanup_module(void) 
{
	i2c_inca_del_bus(&inca_ops);
	printk("i2c-inca.o: i2c INCA-IP adapter removed\n");
	//inca_exit();
}

#endif
