/******************************************************************************
       Copyright (c) 2003, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 

******************************************************************************
   Module      : $RCSfile: incaip_iom2.c,v $
   Date        : $ $
   Description : INCA-IP IOM-2 device driver 

   $Log: incaip_iom2.c,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.2  2003/08/15 13:46:41  de6wthne
   * fixed _LOCK macros (disable interrupts)
   * some cosmetic

   Revision 1.1  2003/08/14 11:32:27  de6wthne
   IOM-2 kernel module delivered by EPSCOM - initial version (after integration)

*****************************************************************************/
#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/types.h>
#include <linux/slab.h>
#include <linux/ioctl.h>
#include <linux/poll.h>
#include <linux/proc_fs.h>

#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>
#include <asm/incaip/incaip_iom2_api.h>
#include <asm/incaip/mux.h>
#include <asm/semaphore.h>
#include <asm/irq.h>

#include "incaip_iom2.h"

/****************************************************************************
*                          Module parameters                                *
****************************************************************************/
static int inca_iom2_bitclock = INCA_IOM2_BITCLOCK;/* 0-single, 1-double */
static int inca_iom2_master   = INCA_IOM2_MASTER;  /* 0-slave, 1-master */
static int inca_iom2_major    = INCA_IOM2_MAJOR;   /* Major Number */

/****************************************************************************
*                          Local variables definition                       *
****************************************************************************/
static int                    cic_irq_enabled = 0;
static int                    st_irq_enabled = 0;
static inca_cda_dev_t         cda_device[INCAIOM2_CDA_DEV_NUM];
static inca_ci_dev_t          ci_handler[INCAIOM2_CI_DEV_NUM];
static inca_d_dev_t           d_channel;
static int                    inca_iom2_module_init = 0;

#ifdef INCAIOM2_DEBUG_ENABLE
u32                           G_debug_mask = _FULL_DEBUG_MASK;
#endif

/****************************************************************************
*                          Local functions definition                       *
****************************************************************************/
static void    inca_iom2_workaround(void);

static void    inca_iom2_st_isr (int irq, void *dev_id, struct pt_regs *regs);
static void    inca_iom2_cic_isr(int irq, void *dev_id, struct pt_regs *regs);
static void    inca_iom2_icd_isr(int irq, void *dev_id, struct pt_regs *regs);

static void    inca_iom2_cda_transfer(inca_cda_dev_t *dev);
static int     inca_iom2_cda_set_opts(inca_cda_dev_t *dev, inca_cda_opts_t *options);
static int     inca_iom2_cda_set_mode(inca_cda_dev_t *dev, inca_cda_mode_e mode);
static void    inca_iom2_cda_mask_irq(inca_cda_dev_t *dev);
static void    inca_iom2_cda_unmask_irq(inca_cda_dev_t *dev);

static void    inca_iom2_ci_out(inca_ci_dev_t *dev);
static int     inca_iom2_ci_set_opts(inca_ci_dev_t *dev, inca_ci_opts_t *options);

static void    inca_iom2_d_receive  (inca_d_dev_t *dev, int msg_complete);
static void    inca_iom2_d_transmit (inca_d_dev_t *dev);
static int     inca_iom2_d_set_opts(inca_d_dev_t *dev, inca_d_opts_t *options);
static int     inca_iom2_d_set_sapi_tei(inca_d_dev_t *dev, inca_sapi_tei_t *sapi_tei);

static void    d_hdlc_buffer_init(inca_hdlc_buffer_t *buf);
static int     d_hdlc_buffer_num_bytes(inca_hdlc_buffer_t *buf);
static int     d_hdlc_buffer_read(inca_hdlc_buffer_t *buf, u8 *dest, u32 buf_len);
static int     d_hdlc_buffer_write(inca_hdlc_buffer_t *buf, u32 rx_bytes, int msg_complete);

static int     inca_iom2_version_proc(char *buf);
static int     inca_iom2_status_proc(char *buf);
static int     inca_iom2_read_proc(char *, char **, off_t ,int , int *, void *);

/****************************************************************************
*                          Special macros definition                        *
*                 to avoid multiple access to IOM2 registers                *
****************************************************************************/
static u32   TMP_REG_VAR;

#define IOM2_READ_DCI_CR_REG(reg_value)                \
      IOM2_READ_REG(INCA_IP_IOM_IOM_CR, TMP_REG_VAR); \
      TMP_REG_VAR &= ~INCA_IP_IOM_IOM_CR_CI_CS;       \
      IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,TMP_REG_VAR); \
      IOM2_READ_REG(INCA_IP_IOM_DCI_CR, reg_value);    \
      IOM2_READ_REG(INCA_IP_IOM_DCI_CR, reg_value);    \

#define IOM2_WRITE_DCI_CR_REG(reg_value)               \
      IOM2_READ_REG(INCA_IP_IOM_IOM_CR, TMP_REG_VAR); \
      TMP_REG_VAR &= ~INCA_IP_IOM_IOM_CR_CI_CS;       \
      IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,TMP_REG_VAR); \
      IOM2_WRITE_REG(INCA_IP_IOM_DCI_CR,reg_value);    \

#define IOM2_READ_DCIC_CR_REG(reg_value)               \
      IOM2_READ_REG(INCA_IP_IOM_IOM_CR, TMP_REG_VAR); \
      TMP_REG_VAR |= INCA_IP_IOM_IOM_CR_CI_CS;        \
      IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,TMP_REG_VAR); \
      IOM2_READ_REG(INCA_IP_IOM_DCIC_CR, reg_value);   \
      IOM2_READ_REG(INCA_IP_IOM_DCIC_CR, reg_value);   \

#define IOM2_WRITE_DCIC_CR_REG(reg_value)              \
      IOM2_READ_REG(INCA_IP_IOM_IOM_CR, TMP_REG_VAR); \
      TMP_REG_VAR |= INCA_IP_IOM_IOM_CR_CI_CS;        \
      IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,TMP_REG_VAR); \
      IOM2_WRITE_REG(INCA_IP_IOM_DCIC_CR,reg_value);   \


#define IOM2_READ_DCI_CR_REG_LOCK(reg_value)           \
     {\
        int flags;\
        save_and_cli(flags);\
        IOM2_READ_DCI_CR_REG(reg_value);\
        restore_flags(flags);\
      }

#define IOM2_WRITE_DCI_CR_REG_LOCK(reg_value)          \
     {\
        int flags;\
        save_and_cli(flags);\
        IOM2_WRITE_DCI_CR_REG(reg_value);\
        restore_flags(flags);\
      }

#define IOM2_READ_DCIC_CR_REG_LOCK(reg_value)          \
     {\
       int flags;\
        save_and_cli(flags);\
        IOM2_READ_DCIC_CR_REG(reg_value);\
        restore_flags(flags);\
      }

#define IOM2_WRITE_DCIC_CR_REG_LOCK(reg_value)         \
      {\
        int flags;\
        save_and_cli(flags);\
        IOM2_WRITE_DCIC_CR_REG(reg_value);\
        restore_flags(flags);\
      }

#define IOM2_WRITE_REG_LOCK(reg, value)               \
       {\
        int flags;\
        save_and_cli(flags);\
        IOM2_WRITE_REG(reg,value);\
        restore_flags(flags);\
      }


#define IOM2_READ_REG_LOCK(reg, value)                \
      {\
        int flags;\
        save_and_cli(flags);\
        IOM2_READ_REG(reg,value);\
        restore_flags(flags);\
      }

#define _INCA_IP_IOM_CIR0_CODR0(value)           ( ((value >> 4) & 0x0F) )
#define _INCA_IP_IOM_CIR1_CODR1(value)           ( ((value >> 2) & 0x3F) )
#define _INCA_IP_IOM_CIX0_CODX0(value)           ( ((value) & 0x0F) << 4 )
#define _INCA_IP_IOM_CIX1_CODX1(value)           ( ((value) & 0x3F) << 2 )
#define _INCA_IP_IOM_EXMR_RFBS(value)            (((( 1 << 2) - 1) & (value)) << 5)
#define _INCA_IP_IOM_SAP1_SAPI1(value)           (((( 1 << 6) - 1) & (value)) << 2)
#define _INCA_IP_IOM_SAP2_SAPI2(value)           (((( 1 << 6) - 1) & (value)) << 2)
#define _INCA_IP_IOM_TEI1_TEI1(value)            (((( 1 << 7) - 1) & (value)) << 1)
#define _INCA_IP_IOM_TEI2_TEI2(value)            (((( 1 << 7) - 1) & (value)) << 1)

#define _INCA_IP_IOM_SDS_CR_TSS_GET(value)       (value & ((1 << 4) - 1))
#define _INCA_IP_IOM_CDA_TSDP10_TSS_GET(value)   (value & ((1 << 4) - 1))
#define _INCA_IP_IOM_CO_TSDP10_TSS_GET(value)    (value & ((1 << 4) - 1))




static inline int cda_ring_create(ring_buf_t *buf, int size)
{
   buf->data = kmalloc(size+1,GFP_KERNEL);
   buf->head = 0;
   buf->tail = 0;
   buf->len = size+1;

   if(!buf->data) return -1;

   memset(buf->data,0,size+1);
   return 0;
}

static inline void cda_ring_delete(ring_buf_t *buf)
{
   if(buf->data) 
   {
      kfree(buf->data);
      buf->data = NULL;
   }
}

static inline void cda_ring_flush(ring_buf_t *buf)
{
   buf->head = buf->tail = 0;
}

static inline int cda_ring_free_bytes(ring_buf_t *buf)
{
   return (buf->tail > buf->head)?
            (buf->tail - buf->head - 1):((buf->len - 1) + buf->tail - buf->head);
}

static inline int cda_ring_free_to_end_bytes(ring_buf_t *buf)
{
   return (buf->tail > buf->head)?
            (buf->tail - buf->head - 1):((buf->len - !(buf->tail)) - buf->head);
}

static inline int cda_ring_num_bytes(ring_buf_t *buf)
{
   return (buf->head >= buf->tail)?
            (buf->head - buf->tail):(buf->len + buf->head - buf->tail);
}

static inline int cda_ring_num_to_end_bytes(ring_buf_t *buf)
{
   return (buf->head >= buf->tail)?
            (buf->head - buf->tail):(buf->len - buf->tail);
}

static inline int cda_ring_get(ring_buf_t *buf, u8 *dst, int len)
{
   int count = cda_ring_num_bytes(buf);
   int count_to_end = cda_ring_num_to_end_bytes(buf);

   /*printk(KERN_ALERT "CNT: %d\n",count);
   printk(KERN_ALERT "CNT_TO_END: %d\n",count_to_end);*/
   if(!buf->data || !dst)
   {
      return -1;  
   }

   if(len > count)
   {
      len = count;
   }

   if(len > count_to_end)
   {
      memcpy(dst, &buf->data[buf->tail], count_to_end);
      memcpy(dst + count_to_end, buf->data, len - count_to_end);
      buf->tail = (len - count_to_end);
   }
   else
   {
      memcpy(dst, &buf->data[buf->tail], len);
      buf->tail += len;
   }

   return len;
}

static inline int cda_ring_put(ring_buf_t *buf, const char *src, int len)
{
   int space = cda_ring_free_bytes(buf);
   int space_to_end = cda_ring_free_to_end_bytes(buf);

   /*printk(KERN_ALERT "SPACE: %d\n",space);
   printk(KERN_ALERT "SPACE_TO_END: %d\n",space_to_end);
   printk(KERN_ALERT "LEN: %d\n",len);*/

   if(!buf->data || !src)
   {
      return -1;  
   }

   if(len > space)
   {
      len = space;
   }

   if(len > space_to_end)
   {
      memcpy(&buf->data[buf->head], src, space_to_end);
      memcpy(buf->data, src + space_to_end, len - space_to_end);
      buf->head = (len - space_to_end);
   }
   else
   {
      memcpy(&buf->data[buf->head], src, len);
      buf->head += len;
   }

   return len;
}

#if 0
static void cda_debug_out(ring_buf_t *buf)
{
   int i;

   printk(KERN_ALERT "--------------------------------------------\n");
   printk(KERN_ALERT "| Buffer:\n");
   for(i=0;i<buf->len;i++)
   {
      printk(KERN_ALERT "  %02d",(u32)buf->data[i]);
   }
   printk(KERN_ALERT "\n");
   printk(KERN_ALERT "| Head: %d\n",buf->head);
   printk(KERN_ALERT "| Tail: %d\n",buf->tail);
   printk(KERN_ALERT "| Free: %d\n",cda_ring_free_bytes(buf));
   printk(KERN_ALERT "| Num : %d\n",cda_ring_num_bytes(buf));
   printk(KERN_ALERT "--------------------------------------------\n");
}


static void cda_ring_test(void)
{
   ring_buf_t buf;
   char tmp[5]={1,2,3,4,5};
   int i;

   printk(KERN_ALERT "CREATE:\n");
   cda_ring_create(&buf,10);
   cda_debug_out(&buf);
   printk(KERN_ALERT "INIT WRITE:\n");
   cda_ring_put(&buf,tmp,5);
   cda_debug_out(&buf);

   for(i=0;i<20;i+=2)
   {
      tmp[0] = i+6;
      tmp[1] = i+7;
      printk(KERN_ALERT "WRITE:\n");
      cda_ring_put(&buf,tmp,2);
      cda_debug_out(&buf);
      printk(KERN_ALERT "READ:\n");
      cda_ring_get(&buf,tmp,2);
      cda_debug_out(&buf);
   }
}
#endif



/****************************************************************************
Description:
   This function initiates HDLC RX buffer management.

Arguments:
   buf     - pointer to the HDLC RX buffer descriptor

Return:
   None

Remarks:
   None
****************************************************************************/
static void d_hdlc_buffer_init(inca_hdlc_buffer_t *buf)
{
   /* clear RX buffer */
   memset(buf->buf,0,INCAIOM2_D_MAX_RXBUFSIZE);

   buf->rd_msg_pending   = 0 ;
   buf->rd_byte_count    = 0 ;
   buf->next_rd_ptr      = buf->buf ;
   buf->msg_rd_ptr       = buf->buf ;

   buf->wr_msg_pending   = 0 ;
   buf->wr_byte_count    = 0 ;
   buf->next_wr_ptr      = buf->buf ;   
   buf->msg_wr_ptr       = buf->buf ;
}


static inline int d_hdlc_buffer_data_avail(inca_hdlc_buffer_t *buf)
{
   return (*(buf->next_rd_ptr) & IOM2_HDLC_MSG_COMPLETE);
}



static int d_hdlc_buffer_num_bytes(inca_hdlc_buffer_t *buf)
{
   u8                      *rd_byte_status_0 = buf->next_rd_ptr; /* begin of the current msg to read */
   u8                      *rd_len_byte_0_1;
   u8                      *rd_len_byte_1_2;
   inca_hdlc_msg_header_t  pending_rec_bytes;

   pending_rec_bytes.msg_len = 0 ;

   if(d_hdlc_buffer_data_avail(buf))
   {
      /* set header ptr, check if wrap around within the header */
      rd_len_byte_0_1 = ( (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                        (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                        (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS) ;
      rd_len_byte_1_2 = ( (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                        (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                        (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS) ;

      /* get number of received bytes in the buffer */
      pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE0] = *rd_len_byte_0_1 ;
      pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE1] = *rd_len_byte_1_2 ;
   }
   
   return pending_rec_bytes.msg_len;
}


/****************************************************************************
Description:
   This function copies data from the current HDLC frame into preallocated
   user's buffer.

Arguments:
   buf      - pointer to the HDLC RX buffer descriptor
   dest     - pointer to the user's buffer
   buf_len  - size of the user's buffer (in bytes)

Return:
   (-1)(no data) or size of received data

Remarks:
   This function should be used for all D-Channel modes except Extended 
   transparent mode. For the Extended transparent mode use 
   dHdlcBufferReadExtTransparent() function.
****************************************************************************/
static int d_hdlc_buffer_read(inca_hdlc_buffer_t *buf, u8 *dest, u32 buf_len)
{
   register u32            first_block, second_block = 0;
   register u8             *msg_rd_ptr;
   u8                      *rd_byte_status_0 = buf->next_rd_ptr; /* begin of the current msg to read */
   u8                      *rd_len_byte_0_1;
   u8                      *rd_len_byte_1_2;
   inca_hdlc_msg_header_t  pending_rec_bytes;
   u32                     read_wrap_around = 0;
   u8                      rd_msg_buffer_state;

   /* check if the current read message is complete */
   rd_msg_buffer_state = *rd_byte_status_0 ;

   /* set header ptr, check if wrap around within the header */
   rd_len_byte_0_1 = ( (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                     (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                     (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS) ;
   rd_len_byte_1_2 = ( (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                     (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                     (rd_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS) ;

   pending_rec_bytes.msg_len = 0 ;
   /* get number of received bytes in the buffer */
   pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE0] = *rd_len_byte_0_1 ;
   pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE1] = *rd_len_byte_1_2 ;

   /* message receive complete --> calculate number of bytes for transfere */
   if (pending_rec_bytes.msg_len <= buf_len)
      buf_len = pending_rec_bytes.msg_len ;

   /* check if the message receive is already pending */
   if (buf->rd_msg_pending)
   {
      msg_rd_ptr = buf->msg_rd_ptr ;
   }
   else
   {
      /* new message --> set msg_rd_ptr after the msg header */
      msg_rd_ptr = ( (rd_byte_status_0+IOM2_HDLC_FRAME_MSG_BEGIN_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                    (rd_byte_status_0+IOM2_HDLC_FRAME_MSG_BEGIN_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                    (rd_byte_status_0+IOM2_HDLC_FRAME_MSG_BEGIN_POS) ;

      /* add header length to the read counter */
      buf->rd_byte_count += IOM2_HDLC_FRAME_MSG_BEGIN_POS ;
   }

   /* Check if a wrap around is within the Rx Buffer block */
   first_block = INCAIOM2_D_MAX_RXBUFSIZE - (msg_rd_ptr - buf->buf);
   if( first_block < buf_len )
   {
      /* Yes, data is wrapped */
      /* we have to copy two blocks */
      second_block = buf_len - first_block;
   }
   else
   {
      /* !!! check wrap around !!! 
         --> if the msg match exactly into the buffer than the msg_rd_ptr will be 
             incremented after read the first block. 
             So it points across the buffer bounderies (last buffer entry + 1)
         --> with reading next message a read will be done to this entry
             ACCESS VIOLATION
      */
      if (first_block == buf_len)
      {
         read_wrap_around = 1 ;
      }
      else
      {
         read_wrap_around = 0 ;
      }

      /* we have to copy only one blocks */
      first_block = buf_len;
   }

   /* Copy first block of data */
   memcpy(dest, msg_rd_ptr, first_block);

   if( second_block )
   {
      /* Copy second block of data from the beginning of RX buffer*/
      msg_rd_ptr = buf->buf;
      memcpy(dest + first_block, msg_rd_ptr, second_block);
      /* Update read msg pointer */
      msg_rd_ptr += second_block;
   }
   else
   {
      if (read_wrap_around == 0)
      {
         /* only one block, no wrap around: Update read msg pointer */
         msg_rd_ptr += first_block;
      }
      else
      {
         /* only one block, but wrap around: set the read msg pointer to the begin of the buffer */
         msg_rd_ptr = buf->buf;
      }
   }

   /* update read control parameter */
   buf->msg_rd_ptr = msg_rd_ptr ;
  
   /* check if FIFO read complete --> check and reset message buffer info */
   /* calculate whole message bytes */
   pending_rec_bytes.msg_len -= buf_len ;
   if (pending_rec_bytes.msg_len == 0)
   {
      /* all bytes received, read msg complete --> free frame */
      buf->next_rd_ptr = msg_rd_ptr ;
      buf->rd_msg_pending = 0 ;

      /* lock D Channel interrupts */
      IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ICD);

      /* update next read ptr and frame status --> block again interrupts */
      buf->next_rd_ptr = msg_rd_ptr ;
      *rd_byte_status_0 = 0 ;

      /* Unlock D Channel interrupts */
      IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ICD);
   }
   else
   {
      /* msg still not complete */
      buf->rd_msg_pending = 1 ;
      /* update remaining msg length in the frame header */
      *rd_len_byte_0_1 = pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE0] ;
      *rd_len_byte_1_2 = pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE1] ;
   }

   /* add number of bytes currently read to the read counter */
   buf->rd_byte_count += buf_len ;

   return(buf_len) ;

}        /* static int d_hdlc_buffer_read(..., ..., ...) */


/****************************************************************************
Description:
   This function copies data from the FIFO into HDLC RX buffer. Also
   it initialize frame descriptor for received frame.

Arguments:
   buf        - pointer to the HDLC RX buffer descriptor
   rx_bytes     - value from the RBCL + RBCH registers, current number of bytes in the RX FIFO
   msg_complete - flag that says is HDLC message received completely(1) or not(0)

Return:
   (-1) - data cannot be stored because of lack of memory or frame descriptors
   >0  - size of stored data

Remarks:
   None
****************************************************************************/
static int d_hdlc_buffer_write(inca_hdlc_buffer_t *buf, u32 rx_bytes, int msg_complete)
{
   register u8             *msg_wr_ptr = buf->msg_wr_ptr;        /* current msg write ptr, begin or pending msg */
   register u8             *wr_byte_status_0 = buf->next_wr_ptr; /* begin of the current frame */
   register u8             *wr_len_byte_0_1;
   register u8             *wr_len_byte_1_2;
   register u32            first_block, second_block = 0;
   inca_hdlc_msg_header_t  pending_rec_bytes;
   u32                     free_buffer_space;

   /* set header ptr, check if wrap around within the header */
   wr_len_byte_0_1 = ( (wr_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                     (wr_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                     (wr_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE0_POS) ;
   wr_len_byte_1_2 = ( (wr_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE ) ?
                     (wr_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                     (wr_byte_status_0+IOM2_HDLC_FRAME_LEN_BYTE1_POS) ;

   pending_rec_bytes.msg_len = 0 ;

   /* check if the current receive message is still pending */
   if (buf->wr_msg_pending)
   {
      /* current write message open --> get current received bytes */
      pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE0] = *wr_len_byte_0_1 ;
      pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE1] = *wr_len_byte_1_2 ;
   }

   if(rx_bytes < pending_rec_bytes.msg_len)
   {
      pending_rec_bytes.msg_len = 0;
   }

   rx_bytes -= pending_rec_bytes.msg_len ;

   INCAIOM2_IRQ_TRACE("D RX: %d bytes\n",rx_bytes);

   /* calculate the free buffer space */
   if (msg_wr_ptr > buf->next_rd_ptr)
   {
      free_buffer_space = INCAIOM2_D_MAX_RXBUFSIZE - (msg_wr_ptr - buf->next_rd_ptr) ;
   }
   else     /* if (buf->next_wr_ptr > buf->next_rd_ptr) {...} else ... */
   {
      if (msg_wr_ptr < buf->next_rd_ptr)
      {
         free_buffer_space = buf->next_rd_ptr - msg_wr_ptr ;
      }
      else     /* if (buf->next_wr_ptr < buf->next_rd_ptr) {...} else ... */
      {
         /* !!! next_wr_ptr == next_rd_ptr:                        */
         /*     --> check if the whole buffer is empty or full */
         if(*wr_byte_status_0 == 0)
         {
            /* buffer is empty, free message code */
            free_buffer_space = INCAIOM2_D_MAX_RXBUFSIZE ;
         }
         else
         {
            return (-1);              /* (-1): BUFFER IS FULL */
         }
      }        /* if (buf->next_wr_ptr < buf->next_rd_ptr) {...} else {...} */
   }        /* if (buf->next_wr_ptr > buf->next_rd_ptr) {...} else {...} */

   INCAIOM2_IRQ_TRACE("D RX: current free space = %d bytes\n",free_buffer_space);

   /* check if space is enough */
   if (buf->wr_msg_pending)
   {
      /* message header already exists */
      if (free_buffer_space < rx_bytes)
      {
         /* clear  pending message --> reset current msg write ptr, clear pending flag */
         buf->msg_wr_ptr = buf->next_wr_ptr ;
         buf->wr_msg_pending = 0 ;
         buf->wr_byte_count -= pending_rec_bytes.msg_len ;
         *wr_byte_status_0 = 0 ;
         *wr_len_byte_0_1 = 0 ;
         *wr_len_byte_1_2 = 0 ;
         return (-1) ;          /* (-1): PENDING MSG - NOT ENOUGH FREE SPACE */
      }
   }
   else     /* if (buf->wr_msg_pending) {...} else ... */
   {
      /* new or single message --> reserve 3 bytes for header */
      if ( (free_buffer_space-IOM2_HDLC_MSG_HEADER_LENGHT) < rx_bytes)
      {
         return (-1) ;          /* (-1): NEW MSG - NOT ENOUGH FREE SPACE */
      }
      else
      {
         /* set msg write ptr to the begin of the new msg */
         msg_wr_ptr = 
         ((wr_byte_status_0+IOM2_HDLC_FRAME_MSG_BEGIN_POS-buf->buf) >= INCAIOM2_D_MAX_RXBUFSIZE)? 
                  (wr_byte_status_0+IOM2_HDLC_FRAME_MSG_BEGIN_POS - INCAIOM2_D_MAX_RXBUFSIZE ) :
                  (wr_byte_status_0+IOM2_HDLC_FRAME_MSG_BEGIN_POS) ;

         /* add header length to the counter */
         buf->wr_byte_count += IOM2_HDLC_FRAME_MSG_BEGIN_POS ;
      }
   }        /* if (buf->wr_msg_pending) {...} else {...} */

   /* calculate the blocks to transfere, wrap around */
   first_block = INCAIOM2_D_MAX_RXBUFSIZE - (msg_wr_ptr - buf->buf) ;
   if(first_block < rx_bytes)
   {
      /* Yes, we will wrap the data in the buffer */
      second_block = rx_bytes - first_block;
   }
   else
   {
      /* No, one copying is enough to store data */
      first_block = rx_bytes;
   }

   /* Initiate reading of data from the FIFO. */
   /* NOTE: first reading returns 0x00 value 
      because of a low speed of the FIFO,
      next reading will return a valid data */
   *msg_wr_ptr = *INCA_IP_IOM_RFIFO & 0xFF;
   
   while(first_block--)
   {
      /* Copy data to the RX buffer */
      *msg_wr_ptr++ = *INCA_IP_IOM_RFIFO & 0xFF;
   }        /* while(first_block--) {...} */

   if(second_block)   /* wrap around */
   {
      /* Write data to the beginning of the buffer */
      msg_wr_ptr = buf->buf; 
      while(second_block--)
      {
         *msg_wr_ptr++ = *INCA_IP_IOM_RFIFO & 0xFF;
      }
   }
   else     /* if(second_block) {...} else ... */
   {
      /* !!! check wrap around !!! 
         --> if the msg match exactly into the buffer than the writePtr will be 
             incremented after write the last byte. 
             So it points across the buffer bounderies (last buffer entry + 1)
         --> with receiving next message a write will be done to this entry
             ACCESS VIOLATION
      */
      if (msg_wr_ptr > (buf->buf + INCAIOM2_D_MAX_RXBUFSIZE - 1) )
         msg_wr_ptr = buf->buf; 
   }        /* if(second_block) {...} else {...} */



   /* FIFO read complete --> check and set message info */
   /* calculate whole message bytes */
   pending_rec_bytes.msg_len += rx_bytes ;

   /* add current write bytes to the counter */
   buf->wr_byte_count += rx_bytes ;

   *wr_len_byte_0_1 = pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE0] ;
   *wr_len_byte_1_2 = pending_rec_bytes.msg_len_bytes[IOM2_HDLC_HEADER_LEN_OFF_BYTE1] ;

   /* save current msg_wr_ptr */
   buf->msg_wr_ptr = msg_wr_ptr ;

   /* Did we receive complete frame */
   if(msg_complete)
   {
      buf->next_wr_ptr = msg_wr_ptr ;
      buf->wr_msg_pending = 0 ;

      /* IMPORTANT:
         clear the first byte (status) of the new message for synchronisation of the read ptr
         therefore check if FIFO is full.
         If the FIFO is full, the status will be cleared by the reader
      */
      if (msg_wr_ptr != buf->next_rd_ptr)
         /* FIFO not full */
         *msg_wr_ptr = 0 ;

      /* message complete --> set status and next_wr_ptr */
      *wr_byte_status_0 |= ( (u8)IOM2_HDLC_MSG_COMPLETE ) ;
   }
   else     /* if(msg_complete) {...} else ... */
   {
      /* message still pending */
      buf->wr_msg_pending = 1 ;
   }        /* if(msg_complete) {...} else {...} */

   return rx_bytes;
}        /* static int d_hdlc_buffer_write(..., ..., ...) */


/****************************************************************************
Description:
   This is an interrupt service routine for the ST(synchronous transfer) IRQ

Arguments:
   irq      - unused parameter
   dev_id   - unused parameter
   regs     - unused parameter

Return:
   None

Remarks:
   None
****************************************************************************/
static void inca_iom2_st_isr (int irq, void *dev_id, struct pt_regs *regs)
{
   register u32 sti;

   INCAIOM2_PROFILING_UP(ST_PROFILING)

   /* Check STI register */
   IOM2_READ_REG(INCA_IP_IOM_STI, sti);

   if(sti)
   {
      INCAIOM2_IRQ_TRACE("STI: 0x%08X\n",sti);

      /* CDA0: Synchronous transfer overflow */
      if(sti & INCA_IP_IOM_STI_STOV10)
      {
         INCAIOM2_IRQ_ERROR("STOV10\n");
         cda_device[0].stov_counter++;
      }

      /* CDA1: Synchronous transfer overflow */
      if(sti & INCA_IP_IOM_STI_STOV11)
      {
         INCAIOM2_IRQ_ERROR("STOV11\n");
         cda_device[1].stov_counter++;
      }

      /* CDA2: Synchronous transfer overflow */
      if(sti & INCA_IP_IOM_STI_STOV20)
      {
         INCAIOM2_IRQ_ERROR("STOV20\n");
         cda_device[2].stov_counter++;
      }

      /* CDA3: Synchronous transfer overflow */
      if(sti & INCA_IP_IOM_STI_STOV21)
      {
         INCAIOM2_IRQ_ERROR("STOV21\n");
         cda_device[3].stov_counter++;
      }

      /* CDA0: Synchronous transfer */
      if(sti & INCA_IP_IOM_STI_STI10)
      {
         cda_device[0].sti_counter++;
         inca_iom2_cda_transfer(&cda_device[0]);
         /* Acknowledge interrupt */
         IOM2_WRITE_REG(INCA_IP_IOM_ASTI, INCA_IP_IOM_ASTI_ACK10);
      }

      /* CDA1: Synchronous transfer */
      if(sti & INCA_IP_IOM_STI_STI11)
      {
         cda_device[1].sti_counter++;
         inca_iom2_cda_transfer(&cda_device[1]);
         /* Acknowledge interrupt */
         IOM2_WRITE_REG(INCA_IP_IOM_ASTI, INCA_IP_IOM_ASTI_ACK11);
      }

      /* CDA2: Synchronous transfer */
      if(sti & INCA_IP_IOM_STI_STI20)
      {
         cda_device[2].sti_counter++;
         inca_iom2_cda_transfer(&cda_device[2]);
         /* Acknowledge interrupt */
         IOM2_WRITE_REG(INCA_IP_IOM_ASTI, INCA_IP_IOM_ASTI_ACK20);
      }

      /* CDA3: Synchronous transfer */
      if(sti & INCA_IP_IOM_STI_STI21)
      {
         cda_device[3].sti_counter++;
         inca_iom2_cda_transfer(&cda_device[3]);
         /* Acknowledge interrupt */
         IOM2_WRITE_REG(INCA_IP_IOM_ASTI, INCA_IP_IOM_ASTI_ACK21);
      }
   }
   
   /* Acknowledge interrupt */
   IOM2_INTERRUPTS_CLEAR(INCAIOM2_IRQ_LINE_ST);

   INCAIOM2_PROFILING_DOWN(ST_PROFILING)
}


/****************************************************************************
Description:
   This is an interrupt service routine for the CIC(CI changed) IRQ

Arguments:
   irq      - unused parameter
   dev_id   - unused parameter
   regs     - unused parameter

Return:
   None

Remarks:
   None
****************************************************************************/
static void inca_iom2_cic_isr (int irq, void *dev_id, struct pt_regs *regs)
{
   register u32 cir0;
   register u32 reg_value;

   INCAIOM2_PROFILING_UP(CIC_PROFILING)

   /* Read CIR0 register (and reset this register) */
   IOM2_READ_REG(INCA_IP_IOM_CIR0, cir0);

   INCAIOM2_IRQ_TRACE("CIR0: 0x%08X\n",cir0);

   /* Check CIC0 and CIC1 bits */
   if( cir0 & INCA_IP_IOM_CIR0_CIC0 )
   {
      /* CI0 channel data is available */
      /* get data from the hardware registers */
      CI_DATA_UPDATE(ci_handler[0].rx_data, _INCA_IP_IOM_CIR0_CODR0(cir0));
      ci_handler[0].rx_counter++;

      INCAIOM2_IRQ_TRACE("CI0 RX: %d\n",ci_handler[0].rx_data);

      /* wake up waiting tasks */
      wake_up_interruptible(&ci_handler[0].rx_queue);
   }

   if( cir0 & INCA_IP_IOM_CIR0_CIC1 )
   {
      /* CI1 channel data is available */
      /* get data from the hardware registers */
      IOM2_READ_REG(INCA_IP_IOM_CIR1,reg_value);

      /* put data to the buffer */
      CI_DATA_UPDATE(ci_handler[1].rx_data, _INCA_IP_IOM_CIR1_CODR1(reg_value));
      if(ci_handler[1].hw_opts.CICW == CI1_CHANNEL_WIDTH_4)
      {
         ci_handler[1].rx_data &= 0xF;
      }
      ci_handler[1].rx_counter++;

      INCAIOM2_IRQ_TRACE("CI1 RX: %d\n",ci_handler[1].rx_data);

      /* wake up waiting tasks */
      wake_up_interruptible(&ci_handler[1].rx_queue);
   }

   /* Acknowledge interrupt */
   IOM2_INTERRUPTS_CLEAR(INCAIOM2_IRQ_LINE_CIC);

   INCAIOM2_PROFILING_DOWN(CIC_PROFILING)
}

/****************************************************************************
Description:
   This is an interrupt service routine for the ICD(D-Channel) IRQ

Arguments:
   irq      - unused parameter
   dev_id   - unused parameter
   regs     - unused parameter

Return:
   None

Remarks:
   None
****************************************************************************/
static void inca_iom2_icd_isr (int irq, void *dev_id, struct pt_regs *regs)
{
   register u32   istah;
   register u8    *msg_wr_ptr ; /* current msg write ptr  */
   register u8    *next_wr_ptr ; /* current next write ptr, begin or pending msg */
   u32            pend_msg_len ;
   inca_d_dev_t   *dev = (inca_d_dev_t*)&d_channel;

   INCAIOM2_PROFILING_UP(ICD_PROFILING)

   /* Read ISTAH register (and reset this register) */
   IOM2_READ_REG(INCA_IP_IOM_ISTAH, istah);

   INCAIOM2_IRQ_TRACE("ISTAH: 0x%08X\n",istah);

   if( istah & INCA_IP_IOM_ISTAH_RFO )
   {
      /* Receive Frame Overflow */
      /* reset receiver */
      *INCA_IP_IOM_CMDR = INCA_IP_IOM_CMDR_RMC;
      *INCA_IP_IOM_CMDR = INCA_IP_IOM_CMDR_RRES;

      INCAIOM2_IRQ_ERROR("RFO\n");

      /* check for pending message frame --> clear */
      if (dev->in_buf.wr_msg_pending)
      {
         /* message currently pending --> clear the pending msg */
         next_wr_ptr = dev->in_buf.next_wr_ptr ;
         msg_wr_ptr = dev->in_buf.msg_wr_ptr ;

         /* calculate pending message len */
         if (msg_wr_ptr>=next_wr_ptr)
         {
            /* current received bytes + header */
            pend_msg_len = msg_wr_ptr-next_wr_ptr ;
         }
         else
         {
            pend_msg_len = INCAIOM2_D_MAX_RXBUFSIZE - (next_wr_ptr-msg_wr_ptr) ;
         }

         /* decrement current wr_byte_count */
         dev->in_buf.wr_byte_count -= pend_msg_len ;

         dev->in_buf.msg_wr_ptr = dev->in_buf.next_wr_ptr ;
         dev->in_buf.wr_msg_pending = 0 ;
         *dev->in_buf.next_wr_ptr = 0 ;
      }

      /* clear further RX interrupt processing */
      istah &= ~(INCA_IP_IOM_MASKH_RX) ;

      /* statistic */
      dev->rx_overflow++;
   }

   if( istah & INCA_IP_IOM_ISTAH_XDU )
   {
      /* Transmit Data Underrun */
      INCAIOM2_IRQ_ERROR("XDU\n");
      dev->tx_underrun++;
   }

   if( istah & INCA_IP_IOM_ISTAH_RME )
   {
      /* Receive Message End */
      /* Frame is Complete and last part of data is in RX FIFO */
      inca_iom2_d_receive(dev, 1);
   }

   if( istah & INCA_IP_IOM_ISTAH_RPF )
   {
      /* Receive Pool Full */
      /* Frame is not yet completed but RX FIFO contains data */
      inca_iom2_d_receive(dev, 0);
   }


   if( istah & INCA_IP_IOM_ISTAH_XPR )
   {
      /* Transmit Pool Ready */
      /* we can transmit next portion of data */
      inca_iom2_d_transmit(dev);
   }

   if( istah & INCA_IP_IOM_ISTAH_XMR )
   {
      /* Transmit Message Repeate */
      /* transmit last frame one more time*/
      INCAIOM2_IRQ_ERROR("XMR\n");
      dev->tx_repeat++;
      dev->out_buf_ptr = 0;
      inca_iom2_d_transmit(dev);
   }

   /* Acknowledge interrupt */
   IOM2_INTERRUPTS_CLEAR(INCAIOM2_IRQ_LINE_ICD);

   INCAIOM2_PROFILING_DOWN(ICD_PROFILING)
}

/****************************************************************************
*                          CDA devices functions                            *
****************************************************************************/
/****************************************************************************
Description:
   This function opens specified CDA device on the IOM2 interface.

Arguments:
   dev   - pointer to the pointer to the device structure
   num   - number of the CDA device (0-4)      

Return:
   (-ENODEV) - wrong number of the device or the device is already open
   (-ENOMEM) - not enough memory to create RX and TX ring buffers 
   0         - success 

Remarks:
   In success, the parameter 'dev' contains the address of the valid 
   device structure.
****************************************************************************/
int inca_iom2_cda_open(inca_cda_dev_t **dev, int num)
{
   u32               reg_value;
   inca_cda_mode_e   mode = CDA_DISABLED;
   inca_cda_opts_t   hw_opts;

   /* Check device number */
   if(num >= INCAIOM2_CDA_DEV_NUM)
   {
      INCAIOM2_ERROR("Wrong CDA device number: %d\n",num);
      return -ENODEV;
   }

   *dev = &cda_device[num];

   if((*dev)->open)
   {
      INCAIOM2_ERROR("CDA device is already open\n");
      return -ENODEV;
   }
   
   (*dev)->type = INCA_IOM2_CDA;

   /* Allocate resources */
   /* create device's tx ring buffer */
   if(cda_ring_create(&((*dev)->in_buf),INCAIOM2_CDA_MAXBUFSIZE))
   {
      INCAIOM2_ERROR("no memory for IN ring buffer.\n");
      return -ENOMEM;
   }

   /* create device's rx ring buffer */
   if(cda_ring_create(&((*dev)->out_buf),INCAIOM2_CDA_MAXBUFSIZE))
   {
      INCAIOM2_ERROR("no memory for OUT ring buffer.\n");
      return -ENOMEM;
   }

   /* create device's access semaphore */
   init_MUTEX(&((*dev)->access_sem));

   init_waitqueue_head (&((*dev)->tx_queue));
   init_waitqueue_head (&((*dev)->rx_queue));

   (*dev)->dev_num = num;
   (*dev)->last_value = 0xFF;

   switch(num)
   {
      case 0:
         hw_opts.DPS  = CDA10_DPS_DEFAULT;
         hw_opts.TSS  = CDA10_TSS_DEFAULT;
         break;

      case 1:
         hw_opts.DPS  = CDA11_DPS_DEFAULT;
         hw_opts.TSS  = CDA11_TSS_DEFAULT;
         break;

      case 2:
         hw_opts.DPS  = CDA20_DPS_DEFAULT;
         hw_opts.TSS  = CDA20_TSS_DEFAULT;
         break;

      case 3:
         hw_opts.DPS  = CDA21_DPS_DEFAULT;
         hw_opts.TSS  = CDA21_TSS_DEFAULT;
         break;
   }

   inca_iom2_cda_set_opts(*dev, &hw_opts);

   /* Enable ST interrupt */
   if(!st_irq_enabled)
   {
      IOM2_INTERRUPTS_CLEAR(INCAIOM2_IRQ_LINE_ST);
      IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ST);
   }
   st_irq_enabled++;

   /* Acknowledge interrupts */
   IOM2_READ_REG_LOCK(INCA_IP_IOM_STI, reg_value);
   switch(num)
   {
      case 0:
         reg_value = INCA_IP_IOM_ASTI_ACK10;
         mode     = CDA10_MODE_DEFAULT;
         break;
   
      case 1:
         reg_value = INCA_IP_IOM_ASTI_ACK11;
         mode     = CDA11_MODE_DEFAULT;
         break;

      case 2:
         reg_value = INCA_IP_IOM_ASTI_ACK20;
         mode     = CDA20_MODE_DEFAULT;
         break;

      case 3:
         reg_value = INCA_IP_IOM_ASTI_ACK21;
         mode     = CDA21_MODE_DEFAULT;
         break;
   }
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_ASTI, reg_value);

   (*dev)->open = 1;
   (*dev)->st_unmasked = 0;

   (*dev)->stov_counter = 0;
   (*dev)->sti_counter = 0;

   inca_iom2_cda_set_mode(*dev, mode);   

   return 0;
}

/****************************************************************************
Description:
   This function closes specified CDA device on the IOM2 interface.

Arguments:
   dev     - pointer to the device structure

Return:
   (-ENODEV)      - wrong parameter or the device is not open
   (-ERESTARTSYS) - error occured while taking of the access semaphore 
   0              - success.

Remarks:
   None
****************************************************************************/
int inca_iom2_cda_close(inca_cda_dev_t *dev)
{
   if(!dev || !dev->open || dev->dev_num >= INCAIOM2_CDA_DEV_NUM)
   {
      INCAIOM2_ERROR("Device is not open\n");
      return -ENODEV;
   }

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   /* Mask interrupts */
   inca_iom2_cda_mask_irq(dev);

   st_irq_enabled--;

   /* Disable ST interrupt */
   if(!st_irq_enabled)
   {
      IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ST);
   }

   dev->open = 0;

   /* Set DISABLED mode for the closing device */
   inca_iom2_cda_set_mode(dev, CDA_DISABLED);   

   /* release all allocated resources */
   cda_ring_delete(&dev->in_buf);
   cda_ring_delete(&dev->out_buf);

   up(&dev->access_sem);

   return 0;
}


/****************************************************************************
Description:
   This function reads the data from the specified CDA device and put it into
   destination buffer.

Arguments:
   dev            - pointer to the device structure
   dest           - pointer to the destination buffer, where incoming data 
                     should be copied
   length         - size of the desctination buffer (in bytes)
   non_blocking   - flag that says: do not wait until data is available

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EAGAIN)      - no data is available (only for non-blocking mode)
   >0             - in success, the number of received bytes.

Remarks:
   None
****************************************************************************/
int inca_iom2_cda_read(inca_cda_dev_t *dev, u8 *dest, int length, int non_blocking)
{
   int num_bytes = 0;

   if (  !dev || !dev->open || !dest ||
         (dev->mode != CDA_MONITORING && dev->mode != CDA_MONITORING_TIC))
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   if (!cda_ring_num_bytes(&dev->in_buf)) 
   { 
      INCAIOM2_TRACE("no data, wait ...\n");

      /* release the lock */
      up(&dev->access_sem); 

      /* nothing to read */
      if (non_blocking)
      {
         INCAIOM2_TRACE("non-blocking -> no data.\n");
         return -EAGAIN;
      }

      if (wait_event_interruptible(dev->rx_queue, cda_ring_num_bytes(&dev->in_buf)))
      {
         INCAIOM2_ERROR("wait_event_interruptible error.\n");
         return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
      }

      if(down_interruptible(&dev->access_sem))
      {
         INCAIOM2_ERROR("down_interruptible error.\n");
         return -ERESTARTSYS;
      }
   }

   /* Get data from the ring buffer */
   num_bytes = cda_ring_get(&dev->in_buf, dest, length);

   if(!dev->st_unmasked)
   {
      /* Unmask interrupts */
      inca_iom2_cda_unmask_irq(dev);
   }

   up(&dev->access_sem);

   return num_bytes;
}


/****************************************************************************
Description:
   This function writes the data from the source buffer into the specified
   CDA device. 

Arguments:
   dev            - pointer to the device structure
   src            - pointer to the source buffer, where data 
                     should be read from.
   length         - size of the data in the source buffer (in bytes)
   non_blocking   - flag that says: do not wait until the space is available

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EAGAIN)      - no space is available (only for non-blocking mode)
   >0             - in success, the number of sent bytes.

Remarks:
   None
****************************************************************************/
int inca_iom2_cda_write(inca_cda_dev_t *dev, const char *src, int length, int non_blocking)
{
   int   num_bytes = 0, free_size;
   u32   reg_value;

   if (  !dev || !dev->open || !src ||
         (dev->mode != CDA_WRITING))
   {
      INCAIOM2_ERROR("wrong parameters or mode.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
   }
    
   while(length)
   {
      /* check a buffer free space */
      free_size = cda_ring_free_bytes(&dev->out_buf);
      if ( !free_size )
      {
         INCAIOM2_TRACE("no space, wait ...\n");
   
         up(&dev->access_sem);

         if (non_blocking)
         {
            INCAIOM2_TRACE("non-blocking -> no space.\n");
            return -EAGAIN;
         }
         
         if (wait_event_interruptible(dev->tx_queue, cda_ring_free_bytes(&dev->out_buf)))
         {
            INCAIOM2_ERROR("wait_event_interruptible error.\n");
            return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
         }

         if(down_interruptible(&dev->access_sem))
         {
            INCAIOM2_ERROR("down_interruptible error.\n");
            return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
         }

         free_size = cda_ring_free_bytes(&dev->out_buf);
      }

      if( free_size > length )
      {
         free_size = length;
      }

      /* Lock ST interrupts */
      IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ST);

      if(dev->dev_num < 2)
      {
         IOM2_READ_REG_LOCK(INCA_IP_IOM_CDA1_CR,reg_value);
      }
      else
      {
         IOM2_READ_REG_LOCK(INCA_IP_IOM_CDA2_CR,reg_value);
      }

      if(dev->dev_num == 0 || dev->dev_num == 2)
      {
         /* Input ENABLE*/
         reg_value |= INCA_IP_IOM_CDA1_CR_EN_I0;
      }
      else
      {
         /* Input ENABLE*/
         reg_value |= INCA_IP_IOM_CDA1_CR_EN_I1;
      }
      
      /* Put data into the ring buffer */
      num_bytes += cda_ring_put(&dev->out_buf, src, free_size);

      if(dev->dev_num < 2)
      {
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA1_CR,reg_value);
      }
      else
      {
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA2_CR,reg_value);
      }

      dev->tx_stopped = 0;

      /* Unlock ST interrupts */
      IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ST);
       
      length -= free_size;
      src += free_size;
   }     /* while(length) {...} */

   up(&dev->access_sem);

   return num_bytes;
}


/****************************************************************************
Description:
   This function helps to configure the specified CDA device.

Arguments:
   dev      - pointer to the device stucture
   cmd      - command identifier
   arg      - argument for the issued command

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EFAULT)      - command or command's arguments are invalid
   0              - success

Remarks:
   None
****************************************************************************/
int inca_iom2_cda_ioctl(inca_cda_dev_t *dev, u32 cmd, u32 arg)
{
   int result = (-EFAULT);

   if (!dev || !dev->open)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   } 

   switch(cmd)
   {
      case FIO_IOM2_CDA_GET_HW_OPT:
         if ( arg )
         {
            memcpy((void*)arg, &dev->hw_opts, sizeof(inca_cda_opts_t));
            result = 0;

            INCAIOM2_TRACE("CDA%d GET HW OPT\n", dev->dev_num);
         }
         break;

      case FIO_IOM2_CDA_SET_HW_OPT:
         INCAIOM2_TRACE("CDA%d SET HW OPT\n", dev->dev_num);
         result = inca_iom2_cda_set_opts(dev, (inca_cda_opts_t*)arg);
         break;

      case FIO_IOM2_CDA_GET_MODE:
         if ( arg )
         {
            *(inca_cda_mode_e*)arg = dev->mode;
            result = 0;

            INCAIOM2_TRACE("CDA%d GET MODE\n", dev->dev_num);
         }
         break;

      case FIO_IOM2_CDA_SET_MODE:
         INCAIOM2_TRACE("CDA%d SET MODE\n", dev->dev_num);
         result = inca_iom2_cda_set_mode(dev, (inca_cda_mode_e)arg);
         break;

      case FIO_IOM2_CDA_CLEAR_COUNTERS:
         dev->stov_counter = 0;
         dev->sti_counter  = 0;
         result = 0;
         break;

      default:
         INCAIOM2_ERROR("Unknown command: 0x%X\n",cmd);
   }

   up(&dev->access_sem);

   return result;
}


/****************************************************************************
Description:
   This function checks the state of the specified CDA device.

Arguments:
   dev      - pointer to the device stucture

Return:
   (POLLIN)          - data can be read without blocking
   (POLLOUT)         - data can be write without blocking 
   (POLLIN|POLLOUT)  - data can be read/write without blocking
   0                 - data cannot be read or write without blocking

Remarks:
   None
****************************************************************************/
int inca_iom2_cda_poll(inca_cda_dev_t *dev)
{
   int mask = 0;
    
   if(cda_ring_num_bytes(&dev->in_buf))   mask |= POLLIN;  /* readable */
   if(cda_ring_free_bytes(&dev->out_buf)) mask |= POLLOUT; /* writable */

   /*INCAIOM2_TRACE("%s%s\n",(mask & POLLIN)?"POLLIN ":"",
                           (mask & POLLOUT)?"POLLOUT ":
                           (mask == 0)?"NONE":"");*/

   return mask;
}

/****************************************************************************
Description:
   This function write data to CDA (if device is in CDA_WRITING mode) or
   receive data from CDA (if device is in CDA_MONITORING mode)

Arguments:
   dev - pointer to the device descriptor

Return:
   None
   
Remarks:
   This function is called from the ST ISR for the corresponding CDA device
****************************************************************************/
static void inca_iom2_cda_transfer(inca_cda_dev_t *dev)
{
   register u32   reg_value;
   u8             cda_data;

   if(dev->mode == CDA_MONITORING || dev->mode == CDA_MONITORING_TIC)
   {
      INCAIOM2_PROFILING_UP(ST_MON_PROFILING)
      INCAIOM2_PROFILING_UP(ST_MON_NO_SPACE_PROFILING)

      /* Read data from the CDA register */
      switch(dev->dev_num)
      {
         case 0:
            IOM2_READ_REG(INCA_IP_IOM_CDA10,reg_value);
            break;
         case 1:
            IOM2_READ_REG(INCA_IP_IOM_CDA11,reg_value);
            break;
         case 2:
            IOM2_READ_REG(INCA_IP_IOM_CDA20,reg_value);
            break;
         case 3:
            IOM2_READ_REG(INCA_IP_IOM_CDA21,reg_value);
            break;
         default:
            return;
      }
      
      cda_data = reg_value & 0xFF;

      INCAIOM2_IRQ_TRACE("CDA%d: RX %02X\n",dev->dev_num,cda_data);
      
      /* Do we have free space in the ring buffer */
      if( cda_ring_free_bytes(&dev->in_buf) == 0)
      {
         INCAIOM2_PROFILING_DOWN(ST_MON_PROFILING)

         INCAIOM2_IRQ_ERROR("CDA%d: No space !\n",dev->dev_num);

         /* if not -> mask this interrupt */
         inca_iom2_cda_mask_irq(dev);

         INCAIOM2_PROFILING_DOWN(ST_MON_NO_SPACE_PROFILING)
      }
      else
      {
         INCAIOM2_PROFILING_DOWN(ST_MON_NO_SPACE_PROFILING)

         /* Put it into the ring buffer */
         cda_ring_put(&dev->in_buf, &cda_data, 1);

         wake_up_interruptible(&dev->rx_queue);

         INCAIOM2_PROFILING_DOWN(ST_MON_PROFILING)
      }
   }
   else     /* if(dev->mode == CDA_MONITORING || dev->mode == CDA_MONITORING_TIC) {...} else ... */
   if(dev->mode == CDA_WRITING)
   {  
      INCAIOM2_PROFILING_UP(ST_TX_PROFILING)
      INCAIOM2_PROFILING_UP(ST_TX_NO_DATA_PROFILING)

      /* Do we have anything to transmit ? */
      if( cda_ring_num_bytes(&dev->out_buf) )
      {
         INCAIOM2_PROFILING_DOWN(ST_TX_NO_DATA_PROFILING)

         /* Get data from the ring buffer */
         cda_ring_get(&dev->out_buf, &cda_data, 1);
         reg_value = cda_data;
         dev->last_value = reg_value;

         INCAIOM2_IRQ_TRACE("CDA%d: TX 0x%08X\n",dev->dev_num,reg_value);

         /* Write data to the CDA register */
         /* NOTE: do not insert NOPs !!! 
            IOM2_WRITE_REG() macro does not work properly  */
         switch(dev->dev_num)
         {
            case 0:
               *INCA_IP_IOM_CDA10 = reg_value;
               break;

            case 1:
               *INCA_IP_IOM_CDA11 = reg_value;
               break;

            case 2:
               *INCA_IP_IOM_CDA20 = reg_value;
               break;

            case 3:
               *INCA_IP_IOM_CDA21 = reg_value;
               break;
         }

         wake_up_interruptible(&dev->tx_queue);
         INCAIOM2_PROFILING_DOWN(ST_TX_PROFILING)
      }
      else if(dev->tx_stopped == 0)   /* if(cda_ring_num_bytes(&dev->out_buf)) {...} else ... */
      {
         INCAIOM2_PROFILING_DOWN(ST_TX_PROFILING)
   
         if(dev->dev_num < 2)
         {
            IOM2_READ_REG(INCA_IP_IOM_CDA1_CR,reg_value);
         }
         else
         {
            IOM2_READ_REG(INCA_IP_IOM_CDA2_CR,reg_value);
         }

         if(dev->dev_num == 0 || dev->dev_num == 2)
         {
            /* Input DISABLE*/
            reg_value &= ~INCA_IP_IOM_CDA1_CR_EN_I0;
         }
         else
         {
            /* Input DISABLE*/
            reg_value &= ~INCA_IP_IOM_CDA1_CR_EN_I1;
         }

         if(dev->dev_num < 2)
         {
            IOM2_WRITE_REG(INCA_IP_IOM_CDA1_CR,reg_value);
         }
         else
         {
            IOM2_WRITE_REG(INCA_IP_IOM_CDA2_CR,reg_value);
         }

         switch(dev->dev_num)
         {
            case 0:
               *INCA_IP_IOM_CDA10 = dev->last_value;
               break;

            case 1:
               *INCA_IP_IOM_CDA11 = dev->last_value;
               break;

            case 2:
               *INCA_IP_IOM_CDA20 = dev->last_value;
               break;

            case 3:
               *INCA_IP_IOM_CDA21 = dev->last_value;
               break;
         }
         dev->tx_stopped = 1;
         INCAIOM2_PROFILING_DOWN(ST_TX_NO_DATA_PROFILING)
      }     /* if(cda_ring_num_bytes(&dev->out_buf)) {...} else if(dev->tx_stopped == 0) {...} */
   }     /* if(dev->mode == ... || ...) {...} else if(dev->mode == CDA_WRITING) {...} */
}


/****************************************************************************
Description:
   This function setup hardware configuration of a specified CDA device.

Arguments:
   dev     - pointer to the CDA device descriptor
   option   - pointer to the new HW options of the CDA device

Return:
   (-1) - error occured during change of HW configuration.
   0    - HW config of the device is successfully changed

Remarks:
   None
****************************************************************************/
static int inca_iom2_cda_set_opts(
      inca_cda_dev_t *dev, 
      inca_cda_opts_t *options
      )
{
   u32   cda_tsdp; 

   INCAIOM2_INFO("******************************************************\n");
   INCAIOM2_INFO(" CDA%d config is\n",dev->dev_num);
   INCAIOM2_INFO("   DPS: %s\n",(options->DPS)?
                        "DD -> CDA -> DU":"DU -> CDA -> DD");
   INCAIOM2_INFO("   TSS: %d\n",options->TSS);
   INCAIOM2_INFO("******************************************************\n");

   cda_tsdp =   ((options->DPS) ? INCA_IP_IOM_CDA_TSDP10_DPS : 0) |
               _INCA_IP_IOM_CDA_TSDP10_TSS_GET(options->TSS);

   switch(dev->dev_num)
   {
      case 0: /* CDA10 */
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA_TSDP10,cda_tsdp);
         break;   

      case 1: /* CDA11 */
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA_TSDP11,cda_tsdp);
         break;   

      case 2:  /* CDA20 */
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA_TSDP20,cda_tsdp);
         break;   

      case 3:  /* CDA21 */
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA_TSDP21,cda_tsdp);
         break;   

      default:
         INCAIOM2_ERROR("Wrong CDA device number:%d\n",dev->dev_num);
         return (-ENODEV);
   }

   dev->hw_opts.DPS = options->DPS;
   dev->hw_opts.TSS = options->TSS;

   return 0;
}

/****************************************************************************
Description:
   This function helps to change a operational mode of the CDA device:
      WRITING, MONITORING, SHIFTING, LOOPING and SWITCHING.

Arguments:
   dev     - pointer to the CDA device descriptor
   mode     - new operational mode

Return:
   Always returns 0.

Remarks:
   None
****************************************************************************/
static int inca_iom2_cda_set_mode(inca_cda_dev_t *dev, inca_cda_mode_e mode)
{
   register u32 cda_cr;

   if(dev->dev_num < 2)
   {
      IOM2_READ_REG_LOCK(INCA_IP_IOM_CDA1_CR,cda_cr);
   }
   else
   {
      IOM2_READ_REG_LOCK(INCA_IP_IOM_CDA2_CR,cda_cr);
   }

   INCAIOM2_INFO("******************************************************\n");
   INCAIOM2_INFO(" CDA%d mode is %s \n",dev->dev_num,
                  (mode == CDA_LOOPING)         ? "LOOPING":
                  (mode == CDA_SHIFTING_A2B)    ? "SHIFTING A->B":
                  (mode == CDA_SHIFTING_B2A)    ? "SHIFTING B->A":
                  (mode == CDA_SWITCHING)       ? "SWITCHING":
                  (mode == CDA_MONITORING)      ? "MONITORING":
                  (mode == CDA_WRITING)         ? "WRITING":"DISABLED");
   INCAIOM2_INFO("******************************************************\n");

   cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_TBM;

   switch(mode)
   {
      case CDA_LOOPING:
      case CDA_WRITING:
         if(dev->dev_num == 0 || dev->dev_num == 2)
         {
            /* Input ENABLE*/
            cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I0;
            /* Output ENABLE*/
            cda_cr |= INCA_IP_IOM_CDA1_CR_EN_O0;
         }
         else
         {
            /* Input ENABLE*/
            cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I1;
            /* Output ENABLE*/
            cda_cr |= INCA_IP_IOM_CDA1_CR_EN_O1;
         }
         /* Swap DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_SWAP;
         break;

      case CDA_SHIFTING_A2B:
         /* CDAx0 Input ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I0;
         /* CDAx0 Output DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_O0;
         /* CDAx1 Input DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_I1;
         /* CDAx1 Output ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_O1;
         /* Swap ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_SWAP;
         break;

      case CDA_SHIFTING_B2A:
         /* CDAx0 Input DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_I0;
         /* CDAx0 Output ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_O0;
         /* CDAx1 Input ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I1;
         /* CDAx1 Output DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_O1;
         /* Swap ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_SWAP;
         break;

      case CDA_SWITCHING:
         /* CDAx0 Input ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I0;
         /* CDAx0 Output ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_O0;
         /* CDAx1 Input ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I1;
         /* CDAx1 Output ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_O1;
         /* Swap ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_SWAP;
         break;

      case CDA_MONITORING_TIC:
         if(dev->dev_num == 1 || dev->dev_num == 3)
         {
            /* Wrong device */
            return (-1);
         }

         /* Input ENABLE*/
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I0;
         /* Output DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_O0;

         /* Swap DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_SWAP;
         /* Enable TIC bus monitoring */
         cda_cr |= INCA_IP_IOM_CDA1_CR_EN_TBM;
         break;

      case CDA_MONITORING:
         if(dev->dev_num == 0 || dev->dev_num == 2)
         {
            /* Input ENABLE*/
            cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I0;
            /* Output DISABLE*/
            cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_O0;
         }
         else
         {
            /* Input ENABLE*/
            cda_cr |= INCA_IP_IOM_CDA1_CR_EN_I1;
            /* Output DISABLE*/
            cda_cr &= ~INCA_IP_IOM_CDA1_CR_EN_O1;
         }
         /* Swap DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_SWAP;
         break;

      default:
         /* mode is DISABLED */
         if(dev->dev_num == 0 || dev->dev_num == 2)
         {
            /* Output and Input DISABLE*/
            cda_cr &= ~( INCA_IP_IOM_CDA1_CR_EN_I0 |
                        INCA_IP_IOM_CDA1_CR_EN_O0 );
         }
         else
         {
            /* Output and Input DISABLE*/
            cda_cr &= ~( INCA_IP_IOM_CDA1_CR_EN_O1 |
                        INCA_IP_IOM_CDA1_CR_EN_I1 );
         }
         /* Swap DISABLE*/
         cda_cr &= ~INCA_IP_IOM_CDA1_CR_SWAP;

         /* if previous mode was SWITCHING or SHIFTING 
           then we have to disable a pair of CDA registers */
         if(dev->mode == CDA_SHIFTING_A2B ||
            dev->mode == CDA_SHIFTING_B2A ||
            dev->mode == CDA_SWITCHING    )
         {
            if(dev->dev_num == 1 || dev->dev_num == 3)
            {
               /* Output and Input DISABLE*/
               cda_cr &= ~( INCA_IP_IOM_CDA1_CR_EN_I0 |
                           INCA_IP_IOM_CDA1_CR_EN_O0 );
               /* Change mode of a pair register */
               cda_device[dev->dev_num-1].mode = CDA_DISABLED;
            }
            else
            {
               /* Output and Input DISABLE*/
               cda_cr &= ~( INCA_IP_IOM_CDA1_CR_EN_O1 |
                           INCA_IP_IOM_CDA1_CR_EN_I1 );
               /* Change mode of a pair register */
               cda_device[dev->dev_num+1].mode = CDA_DISABLED;
            }
         }
   }

   if(mode == CDA_SHIFTING_A2B ||
      mode == CDA_SHIFTING_B2A ||
      mode == CDA_SWITCHING    )
   {
      if(dev->dev_num == 1 || dev->dev_num == 3)
      {
         /* Change mode of a pair register */
         cda_device[dev->dev_num-1].mode = mode;
      }
      else
      {
        /* Change mode of a pair register */
        cda_device[dev->dev_num+1].mode = mode;
      }
   }

   dev->mode = mode;

   /* Flush ring buffers */
   cda_ring_flush(&dev->in_buf);
   cda_ring_flush(&dev->out_buf);
   
   if(dev->dev_num < 2)
   {
      IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA1_CR,cda_cr);
   }
   else
   {
      IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CDA2_CR,cda_cr);
   }

   if( mode == CDA_WRITING || mode == CDA_MONITORING || mode == CDA_MONITORING_TIC )
   {
      inca_iom2_cda_unmask_irq(dev);
   }
   else
   {
      inca_iom2_cda_mask_irq(dev);
   }

   return 0;
}

/****************************************************************************
Description:
   This function helps to mask STI/STOV interrupts for the specified CDA 
   device

Arguments:
   dev - pointer to the CDA device descriptor

Return:
   None

Remarks:
   the IOM2 access must be protected outside of this function
****************************************************************************/
static void inca_iom2_cda_mask_irq(inca_cda_dev_t *dev)
{
   register u32 reg_value;
   
   /* Read current mask */
   IOM2_READ_REG_LOCK(INCA_IP_IOM_MSTI, reg_value);

   /* Unmask interrupts */
   switch(dev->dev_num)
   {
      case 0:
         reg_value |= (INCA_IP_IOM_MSTI_STOV10 | INCA_IP_IOM_MSTI_STI10);
         break;
   
      case 1:
         reg_value |= (INCA_IP_IOM_MSTI_STOV11 | INCA_IP_IOM_MSTI_STI11);
         break;

      case 2:
         reg_value |= (INCA_IP_IOM_MSTI_STOV20 | INCA_IP_IOM_MSTI_STI20);
         break;

      case 3:
         reg_value |= (INCA_IP_IOM_MSTI_STOV21 | INCA_IP_IOM_MSTI_STI21);
         break;

      default:
         return;
   }

   INCAIOM2_IRQ_TRACE("ST mask\n");   

   /* Lock ST interrupts */
   IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ST);

   /* Write a new mask */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MSTI, reg_value);
   
   dev->st_unmasked = 0;

   /* Unlock ST interrupts */
   IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ST);
}


/****************************************************************************
Description:
   This function helps to unmask STI/STOV interrupts for the specified CDA 
   device

Arguments:
   dev - pointer to the CDA device descriptor

Return:
   None

Remarks:
   the IOM2 access must be protected outside of this function
****************************************************************************/
static void inca_iom2_cda_unmask_irq(inca_cda_dev_t *dev)
{
   register u32 reg_value;

   /* Read current mask */
   IOM2_READ_REG_LOCK(INCA_IP_IOM_MSTI, reg_value);

   /* Unmask interrupts */
   switch(dev->dev_num)
   {
      case 0:
         reg_value &= ~(INCA_IP_IOM_MSTI_STOV10 | INCA_IP_IOM_MSTI_STI10);
         break;
   
      case 1:
         reg_value &= ~(INCA_IP_IOM_MSTI_STOV11 | INCA_IP_IOM_MSTI_STI11);
         break;

      case 2:
         reg_value &= ~(INCA_IP_IOM_MSTI_STOV20 | INCA_IP_IOM_MSTI_STI20);
         break;

      case 3:
         reg_value &= ~(INCA_IP_IOM_MSTI_STOV21 | INCA_IP_IOM_MSTI_STI21);
         break;

      default:
         return;
   }

   INCAIOM2_IRQ_TRACE("ST unmask\n");   

   /* Lock ST interrupts */
   IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ST);

   /* Write new mask */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MSTI, reg_value);

   dev->st_unmasked = 1;

   /* Unlock ST interrupts */
   IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ST);
}

/****************************************************************************
*                          CI Handler functions                             *
****************************************************************************/

/****************************************************************************
Description:
   This function opens specified CI Handler device on the IOM2 interface.

Arguments:
   dev   - pointer to the pointer to the device structure
   num   - number of the CI handler (0-1)      

Return:
   (-ENODEV) - wrong number of the device or the device is already open
   0         - success 

Remarks:
   In success, the parameter 'dev' contains the address of the valid 
   device structure.
****************************************************************************/
int inca_iom2_ci_open(inca_ci_dev_t **dev, int num)
{
   u32            reg_value;
   inca_ci_opts_t hw_opts;

   /* Check device number */
   if(num >= INCAIOM2_CI_DEV_NUM)
   {
      INCAIOM2_ERROR("Wrong CI device number: %d\n",num);
      return -ENODEV;
   }

   *dev = &ci_handler[num];

   if((*dev)->open)
   {
      INCAIOM2_ERROR("CI device is already open\n");
      return -ENODEV;
   }

   (*dev)->type = INCA_IOM2_CI_HANDLER;

   /* set CI handler number */
   (*dev)->dev_num = num;

   /* create device's access semaphore */
   init_MUTEX(&((*dev)->access_sem));

   init_waitqueue_head (&((*dev)->tx_queue));
   init_waitqueue_head (&((*dev)->rx_queue));

   /* Set initial HW settings  */
   if(num == 0)
   {
      hw_opts.DPS    = CI0_DPS_DEFAULT;
      (*dev)->tx_data   = 0xF;
      (*dev)->rx_data   = 0xF;
   }
   else
   {
      hw_opts.CICW = CI1_CICW_DEFAULT;
      hw_opts.DPS  = CI1_DPS_DEFAULT;
      if(hw_opts.CICW)
      {
         (*dev)->tx_data   = 0x3F;
         (*dev)->rx_data   = 0x3F;
      }
      else
      {
         (*dev)->tx_data   = 0xF;
         (*dev)->rx_data   = 0xF;
      }
   }
   inca_iom2_ci_set_opts((*dev), &hw_opts); 

   /* enable CIC interrupt */
   if(!cic_irq_enabled)
   {
      IOM2_INTERRUPTS_CLEAR(INCAIOM2_IRQ_LINE_CIC);
      IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_CIC);
   }
   cic_irq_enabled ++;

   if((*dev)->dev_num == 1)
   {
      /* Enable CI1 data */
      IOM2_READ_DCI_CR_REG_LOCK(reg_value);
      reg_value |= INCA_IP_IOM_DCI_CR_EN_CI1;
      IOM2_WRITE_DCI_CR_REG_LOCK(reg_value);
   }
   else
   {
      /* Enable CI0 data */
      IOM2_READ_DCIC_CR_REG_LOCK(reg_value);
      reg_value |= INCA_IP_IOM_DCIC_CR_EN_CI0;
      IOM2_WRITE_DCIC_CR_REG_LOCK(reg_value);
   }

   (*dev)->open         = 1;
   (*dev)->tx_counter   = 0;
   (*dev)->rx_counter   = 0;

   /* this value is default, so we have to clear
      the flag 'UPDATED'  */
   CI_DATA_CLEAR((*dev)->rx_data);

   return 0;
}

/****************************************************************************
Description:
   This function closes specified CI Handler device on the IOM2 interface.

Arguments:
   dev     - pointer to the device structure

Return:
   (-ENODEV)      - wrong parameter or the device is not open
   (-ERESTARTSYS) - error occured while taking of the access semaphore 
   0              - success.

Remarks:
   None
****************************************************************************/
int inca_iom2_ci_close(inca_ci_dev_t *dev)
{
   u32 reg_value;

   if(!dev || !dev->open || dev->dev_num >= INCAIOM2_CI_DEV_NUM)
   {
      INCAIOM2_ERROR("Device is not open\n");
      return -ENODEV;
   }

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   cic_irq_enabled--;

   /* disable CIC interrupt for CI 1 handler */
   if(dev->dev_num == 1)
   {
      /* Disable CI1 data */
      IOM2_READ_DCI_CR_REG_LOCK(reg_value);
      reg_value &= ~INCA_IP_IOM_DCI_CR_EN_CI1;
      IOM2_WRITE_DCI_CR_REG_LOCK(reg_value);
   }
   else
   {
      /* Disable CI0 data */
      IOM2_READ_DCIC_CR_REG_LOCK(reg_value);
      reg_value &= ~INCA_IP_IOM_DCIC_CR_EN_CI0;
      IOM2_WRITE_DCIC_CR_REG_LOCK(reg_value);
   }

   /* disable CIC interrupt */
   if(!cic_irq_enabled)
   {
      IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_CIC);
   }

   dev->open = 0;

   up(&dev->access_sem);

   return 0;
}


/****************************************************************************
Description:
   This function reads the data from the specified CI Handler and puts it into
   the destination buffer.

Arguments:
   dev            - pointer to the device structure
   dest           - pointer to the destination buffer, where incoming data 
                     should be copied
   length         - size of the desctination buffer (in bytes)
   non_blocking   - flag that says: do not wait until data is available

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EAGAIN)      - no data is available (only for non-blocking mode)
   >0             - in success, the number of received bytes.

Remarks:
   None
****************************************************************************/
int inca_iom2_ci_read(inca_ci_dev_t *dev, u8 *dest, int non_blocking)
{
   if (!dev || !dev->open || !dest)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   if(!CI_DATA_UPDATED(dev->rx_data))
   {
      INCAIOM2_TRACE("no data, wait ...\n");

      /* release the lock */
      up(&dev->access_sem); 

      /* nothing to read */
      if (non_blocking)
      {
         INCAIOM2_TRACE("non-blocking -> no data.\n");
         return -EAGAIN;
      }

      if (wait_event_interruptible(dev->rx_queue, CI_DATA_UPDATED(dev->rx_data)))
      {
         INCAIOM2_ERROR("wait_event_interruptible error.\n");
         return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
      }

      if(down_interruptible(&dev->access_sem))
      {
         INCAIOM2_ERROR("down_interruptible error.\n");
         return -ERESTARTSYS;
      }
   }

   /* Lock CIC interrupts */
   IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_CIC);

   *((u8*)dest) = dev->rx_data;

   CI_DATA_CLEAR(dev->rx_data);

   /* Unlock CIC interrupts */
   IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_CIC);


   INCAIOM2_TRACE("CI%d READ: %X\n",dev->dev_num,*(u8*)dest );

   up(&dev->access_sem);

   return 1;
}


/****************************************************************************
Description:
   This function copies data from the source buffer into the internal buffer
   of the specified CI Handler.

Arguments:
   dev            - pointer to the device structure
   src            - pointer to the source buffer, where data 
                     should be read from.
   length         - size of the data in the source buffer (in bytes)

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   >0             - in success, the number of sent bytes.

Remarks:
   This function transmit only one byte of data per one function call.
****************************************************************************/
int inca_iom2_ci_write(inca_ci_dev_t *dev, const char *src, int length)
{
   if (!dev || !dev->open || !src || !length)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   dev->tx_data = src[0];
   inca_iom2_ci_out(dev);
   dev->tx_counter++;
      
   INCAIOM2_TRACE("CI%d WRITE: %X.\n", dev->dev_num, dev->tx_data);

   up(&dev->access_sem);

   return 1;
}
   

/****************************************************************************
Description:
   This function writes the data to the CI handler hardware register .

Arguments:
   dev            - pointer to the device structure

Return:
   None

Remarks:
   None
****************************************************************************/
static void inca_iom2_ci_out(inca_ci_dev_t *dev)
{
   u32 reg_value;

   if(dev->dev_num == 0)
   {           
      /* put data into the hardware registers */
      reg_value =  (INCAIOM2_TIC_BUS_ADDR << 1) | 
                  _INCA_IP_IOM_CIX0_CODX0((u32)dev->tx_data);
      IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CIX0, reg_value);
   }
   else
   {
      /* put data into the hardware registers */
      reg_value  = (dev->open) ? INCA_IP_IOM_CIX1_CI1E : 0;
      reg_value |= (dev->hw_opts.CICW == CI1_CHANNEL_WIDTH_6) ? 
                                 INCA_IP_IOM_CIX1_CICW : 
                                 0x000000C0 /* higher 2 bits are ones */;
      reg_value |= _INCA_IP_IOM_CIX1_CODX1((u32)dev->tx_data);
      IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CIX1, reg_value);
   }
}


/****************************************************************************
Description:
   This function helps to configure the specified CI Handler.

Arguments:
   dev      - pointer to the device stucture
   cmd      - command identifier
   arg      - argument for the issued command

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EFAULT)      - command or command's arguments are invalid
   0              - success

Remarks:
   None
****************************************************************************/
int inca_iom2_ci_ioctl(inca_ci_dev_t *dev, u32 cmd, u32 arg )
{
   int   result = (-EFAULT);

   if (!dev || !dev->open)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   switch(cmd)
   {
      case FIO_IOM2_CI_GET_HW_OPT:
         if ( arg )
         {
            memcpy((void*)arg, &dev->hw_opts, sizeof(inca_ci_opts_t));
            result = 0;

            INCAIOM2_TRACE("CI%d GET HW OPT\n",dev->dev_num);
         }
         break;

      case FIO_IOM2_CI_SET_HW_OPT:
         INCAIOM2_TRACE("CI%d SET HW OPT\n",dev->dev_num);
         result = inca_iom2_ci_set_opts(dev, (inca_ci_opts_t*)arg);
         break;

      case FIO_IOM2_CI_CLEAR_COUNTERS:
         dev->tx_counter   = 0;
         dev->rx_counter   = 0;
         result = 0;
         break;

      default:
         INCAIOM2_ERROR("Unknown command: 0x%X\n",cmd);
   }

   up(&dev->access_sem);

   return result;
}


/****************************************************************************
Description:
   This function checks the state of the specified CI Handler.

Arguments:
   dev      - pointer to the device stucture

Return:
   (POLLIN)          - data can be read without blocking
   (POLLOUT)         - data can be write without blocking 
   (POLLIN|POLLOUT)  - data can be read/write without blocking
   0                 - data cannot be read or write without blocking

Remarks:
   The data always can be write without blocking into the CI Handler
****************************************************************************/
int inca_iom2_ci_poll(inca_ci_dev_t *dev)
{
   int mask = 0;
    
   if(CI_DATA_UPDATED(dev->rx_data)) mask |= POLLIN;  /* readable */
   
   mask |= POLLOUT; /* CI is always writable */

   /*INCAIOM2_TRACE("%s POLLOUT\n",(mask & POLLIN)?"POLLIN":"");*/
   return mask;
}

/****************************************************************************
Description:
   This function sets up the hardware configuration of the  specified 
   CI Handler.

Arguments:
   dev      - pointer to the CI Handler device structure
   option   - pointer to the new HW options of the CI Handler device

Return:
   Always returns 0
   
Remarks:
   None
****************************************************************************/
static int inca_iom2_ci_set_opts(inca_ci_dev_t *dev, inca_ci_opts_t *options)
{
   u32   reg_value = 0;

   /* set options */
   INCAIOM2_INFO("******************************************************\n");
   INCAIOM2_INFO(" CI%d config is\n",dev->dev_num);

   dev->hw_opts.CICW    = options->CICW;
   dev->hw_opts.DPS     = options->DPS;

   if(dev->dev_num == 1)   
   {
      /* 1. Channel Width. Only for CI 1  */
      /*    Update CIX1 register          */
      INCAIOM2_INFO("   CI1 Width: %d\n",
                     (options->CICW == CI1_CHANNEL_WIDTH_6) ? 6:4);
      inca_iom2_ci_out(dev);

      /* 2. Data Port Selection */
      IOM2_READ_DCI_CR_REG_LOCK(reg_value);
      if(options->DPS == CI_DU_OUT_DD_IN)
      {
         reg_value |= INCA_IP_IOM_DCI_CR_DPS_CI1;
         INCAIOM2_INFO("   DPS: DD -> CI1 -> DU\n");
      }
      else
      {
         reg_value &= ~INCA_IP_IOM_DCI_CR_DPS_CI1;
         INCAIOM2_INFO("   DPS: DU -> CI1 -> DD\n");
      }
      IOM2_WRITE_DCI_CR_REG_LOCK(reg_value);
   }
   else
   {
      /* 1. Data Port Selection */
      IOM2_READ_DCIC_CR_REG_LOCK(reg_value);
      if(options->DPS == CI_DU_OUT_DD_IN)
      {
         reg_value |= INCA_IP_IOM_DCIC_CR_DPS_CI0;
         INCAIOM2_INFO("   DPS: DD -> CI0 -> DU\n");
      }
      else
      {
         reg_value &= ~INCA_IP_IOM_DCIC_CR_DPS_CI0;
         INCAIOM2_INFO("   DPS: DU -> CI0 -> DD\n");
      }
      IOM2_WRITE_DCIC_CR_REG_LOCK(reg_value);
   }
   INCAIOM2_INFO("******************************************************\n");

   return 0;
}


/****************************************************************************
*                          D-Channel functions                              *
****************************************************************************/
/****************************************************************************
Description:
   This function opens D-Channel device on the IOM2 interface.

Arguments:
   dev   - pointer to the pointer to the device structure

Return:
   (-ENODEV) - the device is already open
   0         - success 

Remarks:
   In success, the parameter 'dev' contains the address of the valid 
   device structure.
****************************************************************************/
int inca_iom2_d_open(inca_d_dev_t **dev)
{
   u32               reg_value;
   inca_d_opts_t     hw_opts;
   inca_sapi_tei_t   sapi_tei;

   *dev = &d_channel;

   if((*dev)->open)
   {
      INCAIOM2_ERROR("D Channel is already open\n");
      return -ENODEV;
   }

   (*dev)->type = INCA_IOM2_D_CHANNEL;  

   /* reset control and statistic data */
   (*dev)->channel_mask       = INCA_IP_IOM_MASKH_UMASK_ALL;
   (*dev)->tx_completed       = 1;
   (*dev)->open               = 1;

   (*dev)->rx_overflow        = 0;
   (*dev)->rx_invalid_frame   = 0;
   (*dev)->tx_repeat          = 0;
   (*dev)->tx_underrun        = 0;
   (*dev)->tx_msg_counter     = 0;
   (*dev)->tx_byte_counter    = 0;
   (*dev)->rx_msg_counter     = 0;
   (*dev)->rx_byte_counter    = 0;
   (*dev)->lack_free_space    = 0;
             
   /* create device's access semaphore */
   init_MUTEX(&((*dev)->access_sem));

   init_waitqueue_head (&((*dev)->tx_queue));
   init_waitqueue_head (&((*dev)->rx_queue));

   hw_opts.MDS     = D_MDS_DEFAULT;
   hw_opts.DIM0    = D_DIM0_DEFAULT;
   hw_opts.DIM1    = D_DIM1_DEFAULT;
   hw_opts.TxFIFO  = D_TX_FIFO_DEFAULT;
   hw_opts.RxFIFO  = D_RX_FIFO_DEFAULT;
   hw_opts.SRA     = D_SRA_DEFAULT;
   hw_opts.TxCRC   = D_TX_CRC_DEFAULT;
   hw_opts.RxCRC   = D_RX_CRC_DEFAULT;
   hw_opts.ITF     = D_ITF_DEFAULT;
   hw_opts.DPS     = D_DPS_DEFAULT;
   inca_iom2_d_set_opts(*dev, &hw_opts);

   sapi_tei.SAPI1     = D_SAPI1_DEFAULT;
   sapi_tei.SAPI2     = D_SAPI2_DEFAULT;
   sapi_tei.TEI1      = D_TEI1_DEFAULT;
   sapi_tei.TEI2      = D_TEI2_DEFAULT;
   inca_iom2_d_set_sapi_tei(*dev, &sapi_tei);

   /* Enable D Channel */
   IOM2_READ_DCI_CR_REG_LOCK(reg_value);
   reg_value |= INCA_IP_IOM_DCI_CR_EN_D;
   IOM2_WRITE_DCI_CR_REG_LOCK(reg_value);

   /*  Unmask all interrupts */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MASKH, INCA_IP_IOM_MASKH_UMASK_ALL);
   (*dev)->int_disabled = 0 ;

   /* Enable interrupts */
   IOM2_INTERRUPTS_CLEAR(INCAIOM2_IRQ_LINE_ICD);
   IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ICD);

   return 0;
}

/****************************************************************************
Description:
   This function closes D-Channel device on the IOM2 interface.

Arguments:
   dev     - pointer to the device structure

Return:
   (-ENODEV)      - wrong parameter or the device is not open
   (-ERESTARTSYS) - error occured while taking of the access semaphore 
   0              - success.

Remarks:
   None
****************************************************************************/
int inca_iom2_d_close(inca_d_dev_t *dev)
{
   u32 reg_value;


   if(!dev || !dev->open)
   {
      INCAIOM2_ERROR("Device is not open\n");
      return -ENODEV;
   }

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   /* Disable interrupt line */
   IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ICD);

   /* Mask all interrupts */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MASKH, INCA_IP_IOM_MASKH_MASK_ALL);
   dev->channel_mask = INCA_IP_IOM_MASKH_MASK_ALL ;

   /* Deactivate Receiver */
   IOM2_READ_REG_LOCK(INCA_IP_IOM_MODEH, reg_value);
   reg_value &= ~INCA_IP_IOM_MODEH_RAC;
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MODEH, reg_value);

   /* Disable D Channel */
   IOM2_READ_DCI_CR_REG_LOCK(reg_value);
   reg_value &= ~INCA_IP_IOM_DCI_CR_EN_D;
   IOM2_WRITE_DCI_CR_REG_LOCK(reg_value);


   dev->open = 0;
   dev->out_buf_ptr = 0;
   dev->out_buf_len = 0;

   up(&dev->access_sem);

   return 0;
}


/****************************************************************************
Description:
   This function reads the data from the D-Channel device and put it into
   the destination buffer.

Arguments:
   dev            - pointer to the device descriptor
   dest           - pointer to a destination buffer, where incoming data 
                     should be copied
   length         - size of the desctination buffer (in bytes)
   non_blocking   - flag that says: do not wait until data is available

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EAGAIN)      - no data is available (only for non-blocking mode)
   >0             - in success, the number of received bytes.

Remarks:
   None
****************************************************************************/
int inca_iom2_d_read(inca_d_dev_t *dev, u8 *dest, int length, int non_blocking)
{
   int    rx_bytes = 0 ;
   u32    istah ;
   u32    currFillBytes ;

   if (!dev || !dev->open || !dest)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   /* check if a new frame is available */
   if ( d_hdlc_buffer_data_avail(&dev->in_buf) )
   {
      /* message complete received --> get the data */
      rx_bytes = d_hdlc_buffer_read(&dev->in_buf, dest, length);
   }
   else
   {
      INCAIOM2_TRACE("no data, wait ...\n");

      up(&dev->access_sem); 

      /* nothing to read */
      if (non_blocking)
      {
         /* release the lock */
         INCAIOM2_TRACE("non-blocking -> no data.\n");
         return -EAGAIN;
      }

      if (wait_event_interruptible( dev->rx_queue, d_hdlc_buffer_data_avail(&dev->in_buf)))
      {
         INCAIOM2_ERROR("wait_event_interruptible error.\n");
         return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
      }

      if(down_interruptible(&dev->access_sem))
      {
         INCAIOM2_ERROR("down_interruptible error.\n");
         return -ERESTARTSYS;
      }

      /* message complete received --> get the data */
      rx_bytes = d_hdlc_buffer_read(&dev->in_buf, dest, length);
   }

   /* frame has been found and read */

   /* REMARK:
      if the RX interrupts are masked --> check the fill level
   */
   /* unmask the RX-interrupts --> therefore block D-Channel interrupts */
   IOM2_INTERRUPTS_DISABLE(INCAIOM2_IRQ_LINE_ICD);

   if (dev->int_disabled == 1)
   {
      /* 
         no receive interrupts will occure 
         --> access to the control data without block interrupts
      */
      /* calculate current fill level (take care for counter wrap around) */
      if (dev->in_buf.msg_wr_ptr > dev->in_buf.next_rd_ptr)
      {
         currFillBytes = dev->in_buf.msg_wr_ptr - dev->in_buf.next_rd_ptr ;
      }
      else
      {
         /* REMARK : in_buf.next_rd_ptr = dev->in_buf.msg_wr_ptr 
            --> FIFO FULL: INCAIOM2_D_MAX_RXBUFSIZE - 0 = INCAIOM2_D_MAX_RXBUFSIZE
         */
         currFillBytes = INCAIOM2_D_MAX_RXBUFSIZE - (dev->in_buf.next_rd_ptr - dev->in_buf.msg_wr_ptr) ;
      }

      /* check for unmask the interrupts */
      if (currFillBytes < INCAIOM2_D_MAX_UMASK_RXBUFSIZE)
      {
         /* reset receiver */
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CMDR, INCA_IP_IOM_CMDR_RRES);

         dev->channel_mask &= ~(INCA_IP_IOM_MASKH_RX) ;
         IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MASKH, dev->channel_mask);
         dev->int_disabled = 0 ;

         /* Read ISTAH register (and reset this register) */
         IOM2_READ_REG_LOCK(INCA_IP_IOM_ISTAH, istah);
         /* IMPORTANT: !!!!
            save the TX D-Channel interrupts for the TX side
         */
      }
   }

   /* enable int here to protect IOM register access too */
   IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ICD);

   up(&dev->access_sem);

   INCAIOM2_TRACE("D RX: %d bytes\n",rx_bytes);

   return rx_bytes;
}


/****************************************************************************
Description:
   This function writes the data from the source buffer into the D-Channel
   device. 

Arguments:
   dev            - pointer to the device descriptor
   dest           - pointer to a source buffer, from which an incoming data should be
                     read out.
   length         - size of the data in the source buffer (in bytes)
   non_blocking   - flag that says: do not wait until the space is available

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EAGAIN)      - no space is available (only for non-blocking mode)
   >0             - in success, the number of sent bytes.

Remarks:
   None
****************************************************************************/
int inca_iom2_d_write(inca_d_dev_t *dev, const char *src, int length, int non_blocking)
{
   u32 tx_bytes;
   u32 tx_fifo_size;
   u32 command;

   if (!dev || !dev->open || !src || !length)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   if ( !dev->tx_completed )
   {
      INCAIOM2_TRACE("no space, wait ...\n");

      up(&dev->access_sem);

      if (non_blocking)
      {
         INCAIOM2_TRACE("non-blocking -> no space.\n");
         return -EAGAIN;
      }
      
      if (wait_event_interruptible(dev->tx_queue, dev->tx_completed))
      {
         INCAIOM2_ERROR("wait_event_interruptible error.\n");
         return -ERESTARTSYS; /* signal: tell the fs layer to handle it */
      }

      if(down_interruptible(&dev->access_sem))
      {
         INCAIOM2_ERROR("down_interruptible error.\n");
         return -ERESTARTSYS;
      }
   }

   if( length > INCAIOM2_D_MAX_TXBUFSIZE )
   {
      length = INCAIOM2_D_MAX_TXBUFSIZE;
   }

   /* Copy data into the TX buffer */
   memcpy(dev->out_buf, src, length);

   /* Get TX FIFO size */
   tx_fifo_size = (dev->hw_opts.TxFIFO) ? 8 : 16;

   /* How many bytes we can transmit at once? */
   tx_bytes = (length > tx_fifo_size) ? tx_fifo_size : length;

   INCAIOM2_TRACE("D TX: %d bytes\n",tx_bytes);
   
   /* Do we have buffer size more than FIFO size */
   if(length-tx_bytes)
   {
      /* Set pointer to next block of data */
      dev->out_buf_ptr = tx_bytes;
      dev->out_buf_len = length;
   
      command = INCA_IP_IOM_CMDR_XTF;
      INCAIOM2_TRACE("D TX: XTF\n");
   }
   else
   {
      command = (INCA_IP_IOM_CMDR_XTF | INCA_IP_IOM_CMDR_XME);
      INCAIOM2_TRACE("D TX: XTF | XME\n");
   }

   dev->tx_byte_counter += tx_bytes;

   /*INCAIOM2_TRACE("to FIFO: <");*/
   /* Write data to the TX FIFO */
   while(tx_bytes--)
   {
      /*INCAIOM2_OUT("%c",*src);*/
      *INCA_IP_IOM_XFIFO = *src;
      src++ ;
   }
   /*INCAIOM2_OUT("\n");*/

   dev->tx_completed = 0;
   dev->tx_msg_counter++;

   /* Gives a command to HDLC Controller*/
   *INCA_IP_IOM_CMDR = command;

   up(&dev->access_sem);

   return length;
}


/****************************************************************************
Description:
   This function helps to configure the D-Channel device.

Arguments:
   dev      - pointer to the device descriptor
   cmd      - command identifier
   arg      - argument for the issued command

Return:
   (-ENODEV)      - wrong parameters or the device is not open
   (-ERESTARTSYS) - error occured while taking of the semaphore 
   (-EFAULT)      - command or command's arguments are invalid
   0              - success

Remarks:
   None
****************************************************************************/
int inca_iom2_d_ioctl(inca_d_dev_t *dev, u32 cmd, u32 arg)
{
   int   result = (-EFAULT);

   if (!dev || !dev->open)
   {
      INCAIOM2_ERROR("wrong parameters.\n");
      return -ENODEV;
   } 

   if(down_interruptible(&dev->access_sem))
   {
      INCAIOM2_ERROR("down_interruptible error.\n");
      return -ERESTARTSYS;
   }

   switch(cmd)
   {
      case FIO_IOM2_D_GET_HW_OPT:
         if ( arg )
         {
            memcpy((void*)arg, &dev->hw_opts, sizeof(inca_d_opts_t));
            result = 0;

            INCAIOM2_TRACE("D GET HW OPT\n");
         }
         break;

      case FIO_IOM2_D_SET_HW_OPT:
         INCAIOM2_TRACE("D SET HW OPT\n");
         result = inca_iom2_d_set_opts(dev, (inca_d_opts_t*)arg);

         break;
      
      case FIO_IOM2_D_GET_SAPI_TEI:
         if ( arg )
         {
            memcpy((void*)arg, &dev->sapi_tei, sizeof(inca_sapi_tei_t));
            result = 0;

            INCAIOM2_TRACE("D GET SAPI TEI\n");
         }
         break;

      case FIO_IOM2_D_SET_SAPI_TEI:
         INCAIOM2_TRACE("D SET SAPI TEI\n");
         result = inca_iom2_d_set_sapi_tei(dev, (inca_sapi_tei_t*)arg);
         break;
      
      case FIO_IOM2_D_GET_NUM_BYTES:
         if ( arg )
         {
            INCAIOM2_TRACE("D GET NUM BYTES\n");
            *((u32*)arg) = d_hdlc_buffer_num_bytes(&dev->in_buf);
            break;
         }

      case FIO_IOM2_D_TEST_MODE:
         if ( arg )
         {
            INCAIOM2_TRACE("D TEST MODE ON\n");
            /* Enable test loop for HDLC controller */
            IOM2_WRITE_REG(INCA_IP_IOM_TMH,0x01);
         }
         else
         {
            INCAIOM2_TRACE("D TEST MODE OFF\n");
            /* Disable test loop for HDLC controller */
            IOM2_WRITE_REG(INCA_IP_IOM_TMH,0x00);
         }
         result = 0;
         
         break;

      case FIO_IOM2_D_CLEAR_COUNTERS:
         dev->rx_overflow        = 0;
         dev->rx_invalid_frame   = 0;
         dev->tx_repeat          = 0;
         dev->tx_underrun        = 0;
         dev->tx_msg_counter     = 0;
         dev->tx_byte_counter    = 0;
         dev->rx_msg_counter     = 0;
         dev->rx_byte_counter    = 0;
         dev->lack_free_space    = 0;
         result = 0;
         break;

      default:
         INCAIOM2_ERROR("Unknown command: 0x%X",cmd);
         break;
   }

   up(&dev->access_sem);

   return result;
}


/****************************************************************************
Description:
   This function checks the state of the D Channel.

Arguments:
   dev      - pointer to the device stucture

Return:
   (POLLIN)          - data can be read without blocking
   (POLLOUT)         - data can be write without blocking 
   (POLLIN|POLLOUT)  - data can be read/write without blocking
   0                 - data cannot be read or write without blocking

Remarks:
   None
****************************************************************************/
int inca_iom2_d_poll(inca_d_dev_t *dev)
{
   int mask = 0;
    
   if(d_hdlc_buffer_data_avail(&dev->in_buf))
      mask |= POLLIN;  /* readable */

   if(dev->tx_completed)
      mask |= POLLOUT; /* writable */

   /*INCAIOM2_TRACE("%s%s\n",(mask & POLLIN)?"POLLIN ":"",
                           (mask & POLLOUT)?"POLLOUT ":
                           (mask == 0)?"NONE":"");*/
   return mask;
}

/****************************************************************************
Description:
   This function copies data from the RX FIFO into D-Channel device's IN
   buffer.

Arguments:
   dev            - pointer to the D Channel device descriptor
   msg_complete   - message is received completely (1) or it is just a part 
                     of the message (0)
Return:
   None  

Remarks:
   None
****************************************************************************/
static void inca_iom2_d_receive(inca_d_dev_t *dev, int msg_complete)
{
   register u32   rx_bytes;
   register u32   reg_value;
   register u8    *msg_wr_ptr; /* current msg write ptr  */
   register u8    *next_wr_ptr; /* current next write ptr, begin or pending msg */
   u32            pend_msg_len;

   /* Is this frame valid ? */
   IOM2_READ_REG(INCA_IP_IOM_RSTA, reg_value);
   if( msg_complete && !(reg_value & INCA_IP_IOM_RSTA_VFR) )
   {
      /* Frame is not valid - reset receiver */
      dev->rx_invalid_frame++;
      INCAIOM2_IRQ_ERROR("VFR err\n");
      /* reset/clear current open frame buffer */
      if (dev->in_buf.wr_msg_pending)
      {
         /* message currently pending --> clear the pending msg */
         next_wr_ptr = dev->in_buf.next_wr_ptr ;
         msg_wr_ptr = dev->in_buf.msg_wr_ptr ;

         /* calculate pending message len */
         if (msg_wr_ptr>=next_wr_ptr)
         {
            /* current received bytes + header */
            pend_msg_len = msg_wr_ptr-next_wr_ptr ;
         }
         else
         {
            pend_msg_len = INCAIOM2_D_MAX_RXBUFSIZE - (next_wr_ptr-msg_wr_ptr) ;
         }

         /* decrement current wr_byte_count */
         dev->in_buf.wr_byte_count -= pend_msg_len ;

         dev->in_buf.msg_wr_ptr = dev->in_buf.next_wr_ptr ;
         dev->in_buf.wr_msg_pending = 0 ;
         *dev->in_buf.next_wr_ptr = 0 ;
      }
      /* reset receiver */
      *INCA_IP_IOM_CMDR = INCA_IP_IOM_CMDR_RRES;
      return;
   }

   INCAIOM2_IRQ_TRACE("RSTA 0x%08X\n",reg_value);

   /* Get number of received bytes */  
   IOM2_READ_REG(INCA_IP_IOM_RBCL, reg_value);
   rx_bytes = reg_value & 0xFF;
   IOM2_READ_REG(INCA_IP_IOM_RBCH, reg_value);
   rx_bytes |= (reg_value & 0xF) << 8;

   if (dev->hw_opts.MDS == D_MODE_TRANSPARENT_EXT)
   {
      /* no framing --> received bytes are always complete */
      msg_complete = 1 ;
      /* recalculate the number of received bytes */
      reg_value = rx_bytes;
      if(rx_bytes >= dev->last_rx_bytes_num)
         rx_bytes -= dev->last_rx_bytes_num;
      else
         rx_bytes = rx_bytes + (0xFFF-dev->last_rx_bytes_num);
      dev->last_rx_bytes_num = reg_value;
   }

   if(d_hdlc_buffer_write(&dev->in_buf, rx_bytes, msg_complete) == (-1))
   {
      /* ERROR: Cannot receive frame due to lack of free space */
      INCAIOM2_IRQ_ERROR("D: No Space\n");

      /* block further interrupts until next read, mask receiver interrupts */
      IOM2_WRITE_REG(INCA_IP_IOM_MASKH, INCA_IP_IOM_MASKH_RX);
      dev->channel_mask |= INCA_IP_IOM_MASKH_RX ;
      dev->int_disabled = 1 ;
      dev->lack_free_space++;

      /* Awake upper layer */
      msg_complete = 1;
   }

   /* Gives "RX Message Complete" command to the HDLC controller */
   *INCA_IP_IOM_CMDR = INCA_IP_IOM_CMDR_RMC;

   if(msg_complete)
   {
      INCAIOM2_IRQ_TRACE("RX COMPLETE\n");
   
      dev->rx_byte_counter += rx_bytes;
      dev->rx_msg_counter++;
      /* Awake upper layer applications */
      wake_up_interruptible(&dev->rx_queue);
   }
}


/****************************************************************************
Description:
   This function copies data from the device's OUT buffer into TX FIFO.

Arguments:
   dev        - pointer to the D Channel device structure

Return:
   None  

Remarks:
   None
****************************************************************************/
static void inca_iom2_d_transmit(inca_d_dev_t *dev)
{
   register u32   tx_bytes;
   register u32   tx_fifo_size;
   register u32   command;
   register u8    *tx_buf_ptr;

   /* Do we have anything to transmit */
   tx_bytes = dev->out_buf_len - dev->out_buf_ptr;

   if(tx_bytes == 0)
   {
      dev->tx_completed = 1;

      INCAIOM2_IRQ_TRACE("TX COMPLETE\n");

      dev->out_buf_ptr = dev->out_buf_len = 0;

      /* awake the task waiting on WRITE */
      wake_up_interruptible(&dev->tx_queue);

      return;
   }
   
   /* Get Tx FIFO size */
   tx_fifo_size = (dev->hw_opts.TxFIFO) ? 8 : 16;
   
   if(tx_bytes > tx_fifo_size)
   {
      tx_bytes = tx_fifo_size;  
      command = INCA_IP_IOM_CMDR_XTF;
      INCAIOM2_IRQ_TRACE("XTF\n");

   }
   else
   {
      command = (INCA_IP_IOM_CMDR_XTF | INCA_IP_IOM_CMDR_XME);
      INCAIOM2_IRQ_TRACE("XTF|XME\n");
   }

   tx_buf_ptr = dev->out_buf + dev->out_buf_ptr; 
   dev->out_buf_ptr += tx_bytes;
   dev->tx_byte_counter += tx_bytes;

   INCAIOM2_IRQ_TRACE("TX: %d bytes\n",tx_bytes);

   while(tx_bytes--)
   {
      *INCA_IP_IOM_XFIFO = *tx_buf_ptr++;
   }
   
   /* Issue HDLC command */   
   *INCA_IP_IOM_CMDR = command;

}


/****************************************************************************
Description:
   This function sets hardware configuration of the D-Channel device.

Arguments:
   dev      - pointer to the D-Channel device descriptor
   option   - pointer to the new HW options of the D-Channel device

Return:
   Always returns 0

Remarks:
   None
****************************************************************************/
static int inca_iom2_d_set_opts(
      inca_d_dev_t *dev, 
      inca_d_opts_t *options
      )
{
   u32 reg_value;

   /* Set corresponding bits in HW registers */
   INCAIOM2_INFO("******************************************************\n");
   INCAIOM2_INFO(" D channel config is\n");
   INCAIOM2_INFO("   MDS %s\n", 
                     (options->MDS == 2)?"NON-AUTO 1":
                     (options->MDS == 3)?"NON-AUTO 2":
                     (options->MDS == 4)?"TRANSPARENT EXT.":
                     (options->MDS == 5)?"TRANSPARENT 2":
                     (options->MDS == 6)?"TRANSPARENT 0":
                     (options->MDS == 7)?"TRANSPARENT 1":"RESERVED");
   INCAIOM2_INFO("   Collision detection: %s\n",
                     (options->DIM0)?"ENABLE":"DISABLE");
   INCAIOM2_INFO("   TIC bus: %s\n",
                     (!options->DIM1)?"ENABLE":"DISABLE");

   /* MODEH */
   reg_value = (options->MDS << 5)                             |
               ((options->DIM0) ? INCA_IP_IOM_MODEH_DIM0 : 0)  |
               ((options->DIM1) ? INCA_IP_IOM_MODEH_DIM1 : 0)  |
               ((dev->open) ? INCA_IP_IOM_MODEH_RAC : 0);

   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MODEH, reg_value);

   INCAIOM2_INFO("   Tx FIFO: %s\n",
                     (options->TxFIFO)?"8":"16");

   INCAIOM2_INFO("   Rx FIFO: %s\n",
                     (options->RxFIFO == 3)?"4":
                     (options->RxFIFO == 2)?"8":
                     (options->RxFIFO == 1)?"16":"32");

   INCAIOM2_INFO("   Store Receive address: %s\n",
                     (options->SRA)?"ENABLE":"DISABLE");

   INCAIOM2_INFO("   Tx CRC: %s\n",
                     (!options->TxCRC)?"ENABLE":"DISABLE");

   INCAIOM2_INFO("   Rx CRC: %s\n",
                     (options->RxCRC)?"ENABLE":"DISABLE");

   INCAIOM2_INFO("   ITF: %s\n",
                     (options->ITF)?"0111 1110":"1111 1111");

   /* EXMR */
   reg_value =  ((options->TxFIFO) ? INCA_IP_IOM_EXMR_XFBS : 0) |
               _INCA_IP_IOM_EXMR_RFBS(options->RxFIFO)         |
               ((options->SRA) ? INCA_IP_IOM_EXMR_SRA : 0)     |
               ((options->TxCRC) ? INCA_IP_IOM_EXMR_XCRC : 0)  |
               ((options->RxCRC) ? INCA_IP_IOM_EXMR_RCRC : 0)  |
               ((options->ITF) ? INCA_IP_IOM_EXMR_ITF : 0);

   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_EXMR, reg_value);

   /* DCIC_CR */
   IOM2_READ_DCIC_CR_REG_LOCK(reg_value);
   if(options->DPS)
   {
      reg_value |= INCA_IP_IOM_DCIC_CR_DPS_D;
      INCAIOM2_INFO("   DPS: DD -> D -> DU\n");
   }
   else
   {
      reg_value &= ~INCA_IP_IOM_DCIC_CR_DPS_D;
      INCAIOM2_INFO("   DPS: DU -> D -> DD\n");
   }
   IOM2_WRITE_DCIC_CR_REG_LOCK(reg_value);

   INCAIOM2_INFO("******************************************************\n");

   /* Reset Receiver and Transmitter */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CMDR,
                        INCA_IP_IOM_CMDR_RRES |
                        INCA_IP_IOM_CMDR_XRES);
   
   /* Reinit HDLC buffer */
   d_hdlc_buffer_init(&dev->in_buf);

   dev->tx_completed = 1;
   dev->last_rx_bytes_num = 0;

   if(dev->int_disabled == 1)
   {
      dev->int_disabled = 0;
      dev->channel_mask = INCA_IP_IOM_MASKH_UMASK_ALL ;
      
      /*  Unmask all interrupts */
      IOM2_WRITE_REG_LOCK(INCA_IP_IOM_MASKH, INCA_IP_IOM_MASKH_UMASK_ALL);

      /* Read ISTAH register (and reset this register) */
      IOM2_READ_REG_LOCK(INCA_IP_IOM_ISTAH, reg_value);

      /* enable int */
      IOM2_INTERRUPTS_ENABLE(INCAIOM2_IRQ_LINE_ICD);
   }

   /* Store new options */
   memcpy(&dev->hw_opts,options,sizeof(inca_d_opts_t));

   return 0;
}


/****************************************************************************
Description:
   This function sets SAPI and TEI values for the D-Channel device.

Arguments:
   dev      - pointer to the D-Channel device structure
   sapi_tei - pointer to the new SAPI and TEI values of the D-Channel device

Return:
   Always returns 0

Remarks:
   None
****************************************************************************/
static int inca_iom2_d_set_sapi_tei(
      inca_d_dev_t *dev, 
      inca_sapi_tei_t *sapi_tei
      )
{
   INCAIOM2_INFO("******************************************************\n");
   INCAIOM2_INFO("   SAPI1 :%d\n",sapi_tei->SAPI1);
   INCAIOM2_INFO("   SAPI2 :%d\n",sapi_tei->SAPI2);
   INCAIOM2_INFO("   TEI1  :%d\n",sapi_tei->TEI1);
   INCAIOM2_INFO("   TEI2  :%d\n",sapi_tei->TEI2);
   INCAIOM2_INFO("******************************************************\n");

   /* SAPI1 */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_SAP1, sapi_tei->SAPI1);

   /* SAPI2 */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_SAP2, sapi_tei->SAPI2);

   /* TEI1 */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_TEI1, sapi_tei->TEI1);

   /* TEI2 */
   IOM2_WRITE_REG_LOCK(INCA_IP_IOM_TEI2, sapi_tei->TEI2);

   memcpy(&dev->sapi_tei,sapi_tei,sizeof(inca_sapi_tei_t));

   return 0;
}


#ifdef INCAIOM2_DEBUG_ENABLE
/****************************************************************************
   This function is using just for testing purpose: print out values of all 
   IOM2 interface's registers
****************************************************************************/
static void inca_iom2_print_regs(void)
{
   u32 reg_value;

   printk(KERN_ALERT "<**************************************************>\n");

   IOM2_READ_REG(
      INCA_IP_IOM_ISTAH,       
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_ISTAH     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_STAR,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_STAR      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_MODEH,       
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_MODEH     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_EXMR,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_EXMR      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_RBCL,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_RBCL      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_RBCH,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_RBCH      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_RSTA,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_RSTA      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_TMH,         
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_TMH       :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CIR0,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CIR0      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CIR1,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CIR1      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA10,       
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA10     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA11,       
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA11     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA20,       
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA20     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA21,       
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA21     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA_TSDP10,  
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA_TSDP10:%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA_TSDP11,  
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA_TSDP11:%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA_TSDP20,  
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA_TSDP20:%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA_TSDP21,  
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA_TSDP21:%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CO_TSDP10,   
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CO_TSDP10 :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CO_TSDP11,   
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CO_TSDP11 :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CO_TSDP20,   
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CO_TSDP20 :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CO_TSDP21,   
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CO_TSDP21 :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA1_CR,     
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA1_CR   :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_CDA2_CR,     
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_CDA2_CR   :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_BCHA_CR,     
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_BCHA_CR   :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_BCHB_CR,     
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_BCHB_CR   :%08X\n",reg_value);

   /* Get Access to DCI_CR register */
   IOM2_READ_REG(INCA_IP_IOM_IOM_CR, reg_value);
   reg_value &= ~INCA_IP_IOM_IOM_CR_CI_CS;
   IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_DCI_CR,      
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_DCI_CR    :%08X\n",reg_value);

   /* Get Access to DCIC_CR register */
   IOM2_READ_REG(INCA_IP_IOM_IOM_CR, reg_value);
   reg_value |= INCA_IP_IOM_IOM_CR_CI_CS;
   IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_DCIC_CR,     
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_DCIC_CR   :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_SDS_CR,      
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_SDS_CR    :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_IOM_CR,      
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_IOM_CR    :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_STI,         
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_STI       :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_MSTI,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_MSTI      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_SDS_CONF,    
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_SDS_CONF  :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_MCDA,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_MCDA      :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_MODE1,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_MODE1     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_MODE2,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_MODE2     :%08X\n",reg_value);

   IOM2_READ_REG(
      INCA_IP_IOM_MODE3,        
      reg_value);
   printk(KERN_ALERT "INCA_IP_IOM_MODE3     :%08X\n",reg_value);

   printk(KERN_ALERT "<**************************************************>\n");
}
#endif /* INCAIOM2_DEBUG_ENABLE */


/****************************************************************************
Description:
   This function initializes the IOM-2 hardware.

Arguments:
   None.

Return:
   (-EFAULT)   - hardware is already initialized or cannot be initialized
   0           - success.

Remarks:
   None
****************************************************************************/
int inca_iom2_hw_init(void)
{
   u32   reg_value;
   u32   chipver;
   int   result;

   if(inca_iom2_module_init)
   {
      INCAIOM2_ERROR("IOM-2 module is already initialized\n");
      return -EFAULT;
   }

   inca_iom2_module_init = 1;
   
   /* check mux capabilities */
   reg_value = inca_mux_iom_pins();
   /* bits 0-3(FSC,DCL,DU,DD pins) have to be available */
   if((reg_value & 0x0F)!=0x0F)
   {
      INCAIOM2_ERROR("Some important IOM-2 pins aren't available\n");
      return -EFAULT;
   }
   
   /* get the actual chip version */
   chipver = (*INCA_IP_WDT_CHIPID) >> 28;

   /* enable PMU */
   *(INCA_IP_PMU_PM_GEN) |= INCA_IP_PMU_PM_GEN_EN8;

   /* Configure fractional divider to get 1.536MHz DCL: CGU_FDCR = 0x80823540 */
   *(INCA_IP_CGU_CGU_FDCR) |= 0x80823540;

   /* Reset all blocks */
   IOM2_WRITE_REG(INCA_IP_IOM_SRES, (INCA_IP_IOM_SRES_RES_CI   |
                                         INCA_IP_IOM_SRES_RES_DCH   |
                                         INCA_IP_IOM_SRES_RES_IOM  ));

   /* IOM_CR: IOM2 Control Register */
   IOM2_READ_REG(INCA_IP_IOM_IOM_CR,reg_value);
   /* Enable IOM2 */
   reg_value &= ~INCA_IP_IOM_IOM_CR_DIS_IOM;
   /* Enable bit clock */
   reg_value |= INCA_IP_IOM_IOM_CR_EN_BCL;

   /* Mode 2 Register */
   /* Setup mode: master/slave */
   if(inca_iom2_master == 0)
   {
      /* Set clock mode */
      if(inca_iom2_bitclock == 0)
      {
         /* single clock per bit */
         reg_value |= INCA_IP_IOM_IOM_CR_CLKM;
      }
      else
      {
         /* double clock per bit */
         reg_value &= ~INCA_IP_IOM_IOM_CR_CLKM;
      }
   }
   IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,reg_value);

   /* SDS_CR: Control for Serial Data Strobes */
   reg_value =  (INCAIOM2_TSS_ENABLE) ? INCA_IP_IOM_SDS_CR_ENS_TSS : 0 |
               (INCAIOM2_TSS_1_ENABLE) ? INCA_IP_IOM_SDS_CR_ENS_TSS_1 : 0 |
               (INCAIOM2_TSS_3_ENABLE) ? INCA_IP_IOM_SDS_CR_ENS_TSS_3 : 0 |
               _INCA_IP_IOM_SDS_CR_TSS_GET(INCAIOM2_TIMESLOT_SEL);
   IOM2_WRITE_REG(INCA_IP_IOM_SDS_CR,reg_value);

   /* SDS_CONF: Configuration for Serial Data Strobes */
   IOM2_WRITE_REG(INCA_IP_IOM_SDS_CONF,INCAIOM2_ENABLE_IOM2_CLK);

   if (chipver == 2)
   {
      /* Mode 3 Register */
      IOM2_WRITE_REG(INCA_IP_IOM_MODE3,0);
   }

   /* Mode 2 Register */
   /* Setup mode: master/slave */
   if(inca_iom2_master)
   {
      /* Switch IOM-2 from Slave to Master mode: MODE1 */
      if (chipver == 2)
      {
         IOM2_WRITE_REG(INCA_IP_IOM_MODE2,INCA_IP_IOM_MODE2_MSM);
      }
      else
      {
         /* For version 1.1: workaround problem with a master mode */
         inca_iom2_workaround();

         IOM2_WRITE_REG(INCA_IP_IOM_MODE1,INCA_IP_IOM_MODE1_MSM);
      }
   }
   else
   {
      if (chipver == 2)
      {
         IOM2_WRITE_REG(INCA_IP_IOM_MODE2,0);
      }
      else
      {
         IOM2_WRITE_REG(INCA_IP_IOM_MODE1,0);
      }
   }

   /* Mode 1 Register */
   /* Choose configuration */
   IOM2_READ_REG(INCA_IP_IOM_MODE1,reg_value);
   if(INCAIOM2_CONFIGURATION)
   {
      reg_value |= INCA_IP_IOM_MODE1_CFS;
   }
   else
   {
      reg_value &= ~INCA_IP_IOM_MODE1_CFS;
   }
   IOM2_WRITE_REG(INCA_IP_IOM_MODE1,reg_value);

   /* Disable D channel and CI0 channel */
   IOM2_READ_DCI_CR_REG(reg_value);
   reg_value &= ~(INCA_IP_IOM_DCI_CR_EN_CI1 | INCA_IP_IOM_DCI_CR_EN_D);
   IOM2_WRITE_DCI_CR_REG(reg_value);

   /* Disable CI1 channel */
   IOM2_READ_DCIC_CR_REG(reg_value);
   reg_value &= ~INCA_IP_IOM_DCIC_CR_EN_CI0;
   IOM2_WRITE_DCIC_CR_REG(reg_value);

   /* Write a TIC bus address */
   reg_value =  0xF0 | (INCAIOM2_TIC_BUS_ADDR << 1);
   IOM2_WRITE_REG(INCA_IP_IOM_CIX0, reg_value);

   /* Allocate the IRQs */
   result = request_irq(INT_NUM_IM2_IRL13, inca_iom2_st_isr, 0, "iom2_st", NULL);
   if (result)
   {
      INCAIOM2_ERROR("Can't allocate ST IRQ\n");
      return result;
   }
      
   result = request_irq(INT_NUM_IM2_IRL14, inca_iom2_cic_isr, 0, "iom2_cic", NULL);
   if (result)
   {
      INCAIOM2_ERROR("Can't allocate CIC IRQ\n");
      free_irq(INT_NUM_IM2_IRL13, NULL);
      return result;
   }
   
   result = request_irq(INT_NUM_IM2_IRL15, inca_iom2_icd_isr, 0, "iom2_icd", NULL);
   if (result)
   {
      INCAIOM2_ERROR("Can't allocate ICD IRQ\n");
      free_irq(INT_NUM_IM2_IRL13, NULL);
      free_irq(INT_NUM_IM2_IRL14, NULL);
      return result;
   }

   return 0;
}

/****************************************************************************
Description:
   This function uninitializes the IOM-2 hardware.

Arguments:
   None.

Return:
   (-EFAULT)   - hardware is not initialized
   0           - success.

Remarks:
   None
****************************************************************************/
int inca_iom2_hw_uninit(void)
{
   u32   reg_value;

   if(!inca_iom2_module_init) 
   {
      return -EFAULT;
   }

   inca_iom2_module_init = 0;

   /* IOM_CR: IOM2 Control Register */
   IOM2_READ_REG(INCA_IP_IOM_IOM_CR,reg_value);
   /* Disable IOM2 */
   reg_value |= INCA_IP_IOM_IOM_CR_DIS_IOM;
   /* Disable bit clock */
   reg_value &= ~INCA_IP_IOM_IOM_CR_EN_BCL;

   IOM2_WRITE_REG(INCA_IP_IOM_IOM_CR,reg_value);

   /* disable PMU */
   *(INCA_IP_PMU_PM_GEN) |= INCA_IP_PMU_PM_GEN_EN8;

   free_irq(INT_NUM_IM2_IRL13, NULL);
   free_irq(INT_NUM_IM2_IRL14, NULL);
   free_irq(INT_NUM_IM2_IRL15, NULL);

   return 0;
}


static void inca_iom2_workaround()
{
   int i;

   /* Uses P2.12 as alternative function, general purpose I/O: P2_ALTSEL */
   *(INCA_IP_Ports_P2_ALTSEL) |= 0x00001000;

   /* Set P2.12 as output: P2_DIR */
   *(INCA_IP_Ports_P2_DIR) |= 0x00001000;

   /* Supply 3 pulses from P2.12 to DCL: */
   for(i=0;i<3;i++)
   {
      /* Drive P2.12 high: P2_OUT */
      *(INCA_IP_Ports_P2_OUT) |= 0x00001000;

      /* Drive P2.12 low: P2_OUT */
      *(INCA_IP_Ports_P2_OUT) &= ~0x00001000;
   }

   /* Set P2.12 as input: P2_DIR */
   *(INCA_IP_Ports_P2_DIR) &= ~0x00001000;
}


int inca_iom2_open(struct inode *inode, struct file *file)
{
   u32   type = IOM2_DEV_TYPE(inode);
   u32   num = IOM2_DEV_NUM(inode);
   int   result = -ENODEV;
   void  *dev = NULL;

   switch(type)
   {
      case INCA_IOM2_CDA:
         result = inca_iom2_cda_open((inca_cda_dev_t**)&dev, num);
         break;

      case INCA_IOM2_CI_HANDLER:
         result = inca_iom2_ci_open((inca_ci_dev_t**)&dev, num);
         break;

      case INCA_IOM2_D_CHANNEL:
         /* Check device number */
         if(num == 0)
         {
            result = inca_iom2_d_open((inca_d_dev_t**)&dev);
         }
         break;

      default:
         /* Unknown device type */
         INCAIOM2_ERROR("Unknown device type\n");
   }
   
   if(!result)
   {
      file->private_data = dev;

      MOD_INC_USE_COUNT;
   }
   
   return result;
}

int inca_iom2_release(struct inode *inode, struct file *file)
{
   int result = -ENODEV;
   u32 type = IOM2_DEV_TYPE(inode);

   INCAIOM2_TRACE("Close device: %s\n",(type == INCA_IOM2_CDA)?"CDA":
                                       (type == INCA_IOM2_CI_HANDLER)?"CI":
                                       (type == INCA_IOM2_D_CHANNEL)?"D":"unknown");

   switch(type)
   {
      case INCA_IOM2_CDA:
         result = inca_iom2_cda_close((inca_cda_dev_t*)file->private_data);
         break;

      case INCA_IOM2_CI_HANDLER:
         result = inca_iom2_ci_close((inca_ci_dev_t*)file->private_data);
         break;

      case INCA_IOM2_D_CHANNEL:
         result = inca_iom2_d_close((inca_d_dev_t*)file->private_data);
         break;

      default:
         /* Unknown device type */
         INCAIOM2_ERROR("Unknown device type\n");
   }
      
   if(!result)
   {
      file->private_data = NULL;
      MOD_DEC_USE_COUNT;
   }
   return result;
}

ssize_t inca_iom2_read(struct file *file, char *dst, size_t size, loff_t *start)
{
   int               result = -ENODEV;
   inca_iom2_dev_t   *dev = file->private_data;

   switch(dev->type)
   {
      case INCA_IOM2_CDA:
         result = inca_iom2_cda_read((inca_cda_dev_t*)dev, dst, size, file->f_flags & O_NONBLOCK);
         break;

      case INCA_IOM2_CI_HANDLER:
         result = inca_iom2_ci_read((inca_ci_dev_t*)dev, dst, file->f_flags & O_NONBLOCK);
         break;

      case INCA_IOM2_D_CHANNEL:
         result = inca_iom2_d_read((inca_d_dev_t*)dev, dst, size, file->f_flags & O_NONBLOCK);
         break;

      default:
         /* Unknown device type */
         INCAIOM2_ERROR("Unknown device type\n");
   }

   return result;
}

ssize_t inca_iom2_write(struct file *file, const char *src, size_t size, loff_t *start)
{
   int result = -ENODEV;
   inca_iom2_dev_t   *dev = file->private_data;

   switch(dev->type)
   {
      case INCA_IOM2_CDA:
         result = inca_iom2_cda_write((inca_cda_dev_t*)dev, src, size, file->f_flags & O_NONBLOCK);
         break;

      case INCA_IOM2_CI_HANDLER:
         result = inca_iom2_ci_write((inca_ci_dev_t*)dev, src, size);
         break;

      case INCA_IOM2_D_CHANNEL:
         result = inca_iom2_d_write((inca_d_dev_t*)dev, src, size, file->f_flags & O_NONBLOCK);
         break;

      default:
         /* Unknown device type */
         INCAIOM2_ERROR("Unknown device type\n");
   }

   return result;
}

int inca_iom2_ioctl(struct inode *inode, struct file *file, u32 command, unsigned long arg)
{
   int result = -ENODEV;
   u32 type = IOM2_DEV_TYPE(inode);

#ifdef INCAIOM2_DEBUG_ENABLE
   if(command == FIO_IOM2_PRINT_REGS)
   {
      inca_iom2_print_regs();
      return 0;
   }
   if(command == FIO_IOM2_DEBUG_LEVEL)
   {
      G_debug_mask = arg;
      return 0;
   }
#endif

   switch(type)
   {
      case INCA_IOM2_CDA:
         result = inca_iom2_cda_ioctl((inca_cda_dev_t*)file->private_data, command, arg);
         break;

      case INCA_IOM2_CI_HANDLER:
         result = inca_iom2_ci_ioctl((inca_ci_dev_t*)file->private_data, command, arg);
         break;

      case INCA_IOM2_D_CHANNEL:
         result = inca_iom2_d_ioctl((inca_d_dev_t*)file->private_data, command, arg);
         break;

      default:
         /* Unknown device type */
         INCAIOM2_ERROR("Unknown device type\n");
   }

   return result;
}

u32 inca_iom2_poll(struct file *file, struct poll_table_struct *table)
{
   inca_iom2_dev_t   *dev = file->private_data;
   int               result = 0;

   if(!dev || !dev->open)
   {
      return POLLERR;
   }

   /* add to poll queue */
   poll_wait(file, &(dev->rx_queue), table);
   poll_wait(file, &(dev->tx_queue), table);

   switch(dev->type)
   {
      case INCA_IOM2_CDA:
         result = inca_iom2_cda_poll((inca_cda_dev_t*)dev);
         break;

      case INCA_IOM2_CI_HANDLER:
         result = inca_iom2_ci_poll((inca_ci_dev_t*)dev);
         break;

      case INCA_IOM2_D_CHANNEL:
         result = inca_iom2_d_poll((inca_d_dev_t*)dev);
         break;

      default:
         /* Unknown device type */
         INCAIOM2_ERROR("Unknown device type\n");
   }

   return result;
}


struct file_operations inca_iom2_fops =
{
   read:       inca_iom2_read, 
   write:      inca_iom2_write,
   open:       inca_iom2_open,
   release:    inca_iom2_release,
   ioctl:      inca_iom2_ioctl,
   poll:       inca_iom2_poll,
};



static int __init inca_iom2_init(void)
{
   int result;
   int i;
   char dev_name[10];
#if CONFIG_PROC_FS 
   struct proc_dir_entry *driver_proc_node;
#endif

   INCAIOM2_INFO("Start IOM-2 module\n");
   INCAIOM2_INFO("Bit Clock %d\n",inca_iom2_bitclock);
   INCAIOM2_INFO("Mode %s\n",(inca_iom2_master)?"MASTER":"SLAVE");

   SET_MODULE_OWNER(&inca_iom2_fops);

   result = devfs_register_chrdev(inca_iom2_major, INCA_IOM2_DEV_NAME, &inca_iom2_fops);
   if(result < 0)
   {
      INCAIOM2_ERROR("Can't get major %d\n",inca_iom2_major);
      return result;
   }

   if(inca_iom2_major == 0) inca_iom2_major = result;

   for(i=0; i<INCAIOM2_CDA_DEV_NUM; i++)
   {
      sprintf(dev_name,"iomcda%d",i);
      cda_device[i].handle = devfs_register(
                     NULL, dev_name, DEVFS_FL_DEFAULT, inca_iom2_major, 
                     ((INCA_IOM2_CDA << 4) | i), S_IFCHR | S_IRUGO | S_IWUGO,
                     &inca_iom2_fops,  NULL);
   }

   for(i=0; i<INCAIOM2_CI_DEV_NUM; i++)
   {
      sprintf(dev_name,"iomci%d",i);
      ci_handler[i].handle = devfs_register(
                     NULL, dev_name, DEVFS_FL_DEFAULT, inca_iom2_major, 
                     ((INCA_IOM2_CI_HANDLER << 4) | i), S_IFCHR | S_IRUGO | S_IWUGO,
                     &inca_iom2_fops,  NULL);
   }

   d_channel.handle = devfs_register(NULL, "iomd", DEVFS_FL_DEFAULT, inca_iom2_major, 
                  ((INCA_IOM2_D_CHANNEL << 4) | 0x00),  S_IFCHR | S_IRUGO | S_IWUGO, 
                  &inca_iom2_fops,  NULL);

   INCAIOM2_INFO("Major: %d\n",inca_iom2_major);

   result = inca_iom2_hw_init();

#if CONFIG_PROC_FS 
   /* install the proc entry */
   INCAIOM2_INFO("module is using proc fs\n");
   driver_proc_node = proc_mkdir( "driver/iom", NULL);
   if (driver_proc_node != NULL)
   {
      create_proc_read_entry( "version" , S_IFREG|S_IRUGO,
         driver_proc_node, inca_iom2_read_proc, (void *)inca_iom2_version_proc );
      create_proc_read_entry( "status" , S_IFREG|S_IRUGO,
         driver_proc_node, inca_iom2_read_proc, (void *)inca_iom2_status_proc );
   }
   else
   {
      INCAIOM2_INFO("cannot create proc entry\n");
   }
#endif

   INCAIOM2_PROFILING_SETUP()

   return result;
}

static void __exit inca_iom2_cleanup(void)
{
   int i;

   INCAIOM2_INFO("Stop IOM-2 module\n");

   for(i=0; i<INCAIOM2_CDA_DEV_NUM; i++)
   {
      devfs_unregister (cda_device[i].handle);
   }

   for(i=0; i<INCAIOM2_CI_DEV_NUM; i++)
   {
      devfs_unregister (ci_handler[i].handle);
   }

   devfs_unregister (d_channel.handle);

   devfs_unregister_chrdev(inca_iom2_major,INCA_IOM2_DEV_NAME);

   inca_iom2_hw_uninit();

#if CONFIG_PROC_FS 
   remove_proc_entry("driver/iom/version",0);
   remove_proc_entry("driver/iom/status",0);
   remove_proc_entry("driver/iom",0);
#endif

}


#if CONFIG_PROC_FS 
/****************************************************************************
Description:
   Read the version information from the driver.
Arguments:
   buf - destination buffer
Return Value:
   length
Remarks:
   None.
****************************************************************************/
static int inca_iom2_version_proc(char *buf)
{
   int len;

   len = sprintf(buf, "**********************************************************\n");
   len += sprintf(buf + len, "*    INCA-IP IOM-2 driver version 2.0\n");
   len += sprintf(buf + len, "*    Compiled on %s, %s for Linux kernel %s\n", 
      __DATE__, __TIME__, UTS_RELEASE);
   len += sprintf(buf + len, "**********************************************************\n");

   return len;
}

/****************************************************************************
Description:
   Read the status information from the driver.
Arguments:
   buf - destination buffer
Return Value:
   length
Remarks:
   None.
****************************************************************************/
static int inca_iom2_status_proc(char *buf)
{
   int len=0;
   int i;

   len = sprintf(buf, "**********************************************************\n");

   /* CDA Registers */
   for(i=0; i<INCAIOM2_CDA_DEV_NUM; i++)
   {
      len += sprintf(buf+len, " CDA%d register: %s\n",(10*(1+((i&2)>>1))+(i&1)),
                        (cda_device[i].open)?"OPEN":"CLOSED");
      len += sprintf(buf+len, "    mode:           %s\n",   
                        (cda_device[i].mode == CDA_LOOPING)         ? "LOOPING":
                        (cda_device[i].mode == CDA_SHIFTING_A2B)    ? "SHIFTING A->B":
                        (cda_device[i].mode == CDA_SHIFTING_B2A)    ? "SHIFTING B->A":
                        (cda_device[i].mode == CDA_SWITCHING)       ? "SWITCHING":
                        (cda_device[i].mode == CDA_MONITORING)      ? "MONITORING":
                        (cda_device[i].mode == CDA_WRITING)         ? "WRITING":"DISABLED");
      len += sprintf(buf+len, "    STI/STOV:       %s\n",   
                        (cda_device[i].st_unmasked) ? "UNMASKED":"MASKED");
      len += sprintf(buf+len, "    DPS:            %s\n",   
                        (cda_device[i].hw_opts.DPS)? "DD -> CDA -> DU":"DU -> CDA -> DD");
      len += sprintf(buf+len, "    TSS:            %d\n",cda_device[i].hw_opts.TSS);   
      len += sprintf(buf+len, "    STI counter:    %d\n",cda_device[i].sti_counter);   
      len += sprintf(buf+len, "    STOV counter:   %d\n",cda_device[i].stov_counter);  
      len += sprintf(buf+len, "**********************************************************\n");
   }

   /* CI Handlers */
   for(i=0; i<INCAIOM2_CI_DEV_NUM; i++)
   {
      len += sprintf(buf+len, " CI%d register: %s\n",i,
                        (ci_handler[i].open)?"OPEN":"CLOSED");
      len += sprintf(buf+len, "    DPS:            %s\n",   
                        (ci_handler[i].hw_opts.DPS)? 
                        "DD -> CI -> DU":"DU -> CI -> DD");
      if(i==1)
         len += sprintf(buf+len, "    Channel width:  %d\n",
                        (ci_handler[i].hw_opts.CICW)?6:4);
      len += sprintf(buf+len, "    TX data:        %d\n",ci_handler[i].tx_data); 
      len += sprintf(buf+len, "    RX data:        %d\n",CI_DATA_GET(ci_handler[i].rx_data));
      len += sprintf(buf+len, "    TX counter:     %d\n",ci_handler[i].tx_counter); 
      len += sprintf(buf+len, "    RX counter:     %d\n",ci_handler[i].rx_counter); 
      len += sprintf(buf+len, "**********************************************************\n");
   }

   /* D Channel */
   len += sprintf(buf+len, " D channel: %s\n",
                     (d_channel.open)?"OPEN":"CLOSED");
   len += sprintf(buf+len, "    ICD:            %s\n",   
                     (d_channel.int_disabled) ? "MASKED":"UNMASKED");
   len += sprintf(buf+len, "    Max.msg size:   %d\n",INCAIOM2_D_MAX_TXBUFSIZE); 

   len += sprintf(buf+len, "    MDS:            %s\n", 
                     (d_channel.hw_opts.MDS == 2)?"NON-AUTO 1":
                     (d_channel.hw_opts.MDS == 3)?"NON-AUTO 2":
                     (d_channel.hw_opts.MDS == 4)?"TRANSPARENT EXT.":
                     (d_channel.hw_opts.MDS == 5)?"TRANSPARENT 2":
                     (d_channel.hw_opts.MDS == 6)?"TRANSPARENT 0":
                     (d_channel.hw_opts.MDS == 7)?"TRANSPARENT 1":"RESERVED");
   len += sprintf(buf+len, "    Collision Det.: %s\n",
                     (d_channel.hw_opts.DIM0)?"ENABLE":"DISABLE");
   len += sprintf(buf+len, "    TIC bus:        %s\n",
                     (!d_channel.hw_opts.DIM1)?"ENABLE":"DISABLE");

   len += sprintf(buf+len, "    Tx FIFO:        %s\n",
                     (d_channel.hw_opts.TxFIFO)?"8":"16");

   len += sprintf(buf+len, "    Rx FIFO:        %s\n",
                     (d_channel.hw_opts.RxFIFO == 3)?"4":
                     (d_channel.hw_opts.RxFIFO == 2)?"8":
                     (d_channel.hw_opts.RxFIFO == 1)?"16":"32");

   len += sprintf(buf+len, "    Store RX addr:  %s\n",
                     (d_channel.hw_opts.SRA)?"ENABLE":"DISABLE");

   len += sprintf(buf+len, "    Tx CRC:         %s\n",
                     (!d_channel.hw_opts.TxCRC)?"ENABLE":"DISABLE");

   len += sprintf(buf+len, "    Rx CRC:         %s\n",
                     (d_channel.hw_opts.RxCRC)?"ENABLE":"DISABLE");

   len += sprintf(buf+len, "    ITF:            %s\n",
                     (d_channel.hw_opts.ITF)?"0111 1110":"1111 1111");
                                               
   len += sprintf(buf+len, "    DPS:            %s\n",
                     (d_channel.hw_opts.DPS)?"DD -> D -> DU":"DU -> D -> DD");


   len += sprintf(buf+len, "    SAPI1:          %d\n",d_channel.sapi_tei.SAPI1);
   len += sprintf(buf+len, "    SAPI2:          %d\n",d_channel.sapi_tei.SAPI2);
   len += sprintf(buf+len, "    TEI1:           %d\n",d_channel.sapi_tei.TEI1);
   len += sprintf(buf+len, "    TEI2:           %d\n",d_channel.sapi_tei.TEI2);

   len += sprintf(buf+len, "    RxMsgCounter:   %d\n",d_channel.rx_msg_counter); 
   len += sprintf(buf+len, "    RxByteCounter:  %d\n",d_channel.rx_byte_counter);   
   len += sprintf(buf+len, "    RxOverflow:     %d\n",d_channel.rx_overflow); 
   len += sprintf(buf+len, "    RxInvalidFrame: %d\n",d_channel.rx_invalid_frame);  
   len += sprintf(buf+len, "    LackFreeSpace:  %d\n",d_channel.lack_free_space);   
   len += sprintf(buf+len, "    TxRepeat:       %d\n",d_channel.tx_repeat);   
   len += sprintf(buf+len, "    TxUnderrun:     %d\n",d_channel.tx_underrun); 
   len += sprintf(buf+len, "    TxMsgCounter:   %d\n",d_channel.tx_msg_counter); 
   len += sprintf(buf+len, "    TxByteCounter:  %d\n",d_channel.tx_byte_counter);   

   len += sprintf(buf+len, "**********************************************************\n");

   return len;
}


/****************************************************************************
Description:
   The proc filesystem: function to read and entry.
Arguments:
   page - 
   start -
   off -
   count -
   eof -
   data -
Return Value:
   length
Remarks:
   None.
*******************************************************************************/
static int inca_iom2_read_proc(char *page, char **start, off_t off,
                        int count, int *eof, void *data)
{
   int len;
   int (*fn)(char *buf);

   if (!data)
   {
      INCAIOM2_ERROR("Wrong parameters\n");
      return 0;
   }

   fn = data;
   len = fn(page);

   if (len <= off+count) *eof = 1;
   *start = page + off;
   len -= off;
   
   if (len > count) len = count;
   if (len < 0) len = 0;

   return len;
}
#endif

/******************************************************************************/
/*                         DSP <---> IOM 2 interface                          */
/******************************************************************************/
/****************************************************************************
Description:
   This function reset the CO_TSDPxy control structure to the HW default values

Arguments:
   NONE

Return:
   NONE

Remarks:
   The CO_TSDP register of the INCA IP chip are used to set up a communication
   between DSP module and the IOM2 module.
   The setup of this register is IOM2 driver independent.
****************************************************************************/
static inca_co_opts_t  co_ctrl_regs[4] ;

void inca_iom2_co_regs_default(void)
{
   memset(&co_ctrl_regs, 0x0, sizeof(co_ctrl_regs));
   /* control register CO_TSDP10: output on B1-DU */
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL1_LOW].TSS = CO_TSDP10_TSS_DEFAULT;
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL1_LOW].DPS = CO_TSDP10_DPS_DEFAULT;
   /* control register CO_TSDP11: output on B2-DU */
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL1_HIGH].TSS = CO_TSDP11_TSS_DEFAULT;
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL1_HIGH].DPS = CO_TSDP11_DPS_DEFAULT;
   /* control register CO_TSDP20: output on B2-DU */
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL2_LOW].TSS = CO_TSDP20_TSS_DEFAULT;
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL2_LOW].DPS = CO_TSDP20_DPS_DEFAULT;
   /* control register CO_TSDP21: output on IC2-DU */
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL2_HIGH].TSS = CO_TSDP21_TSS_DEFAULT;
   co_ctrl_regs[INCA_IOM_DSP_CHANNEL2_HIGH].DPS = CO_TSDP21_DPS_DEFAULT;
}


/****************************************************************************
Description:
   This function set the CO_TSDPxy value for the given register to establish
   a communication path between DSP and IOM.

Arguments:
   reg_num: given register value 
   co_opts: new register values (TSS, DPS) or NULL
               --> NULL: the CO_TSDP register will not be changed 
                   (used for enable / disable the data channel)
   state:   enable / disable / ignore the corresponding data channel setting
               - DATA_CHANNEL_DISABLE: disable the corresponding data channel
               - DATA_CHANNEL_ENABLE:  enable the corresponding data channel
               - DATA_CHANNEL_IGNORE:  ignore the current setting while change the data

Return:
   OK:      set new value successful
   ERROR:   set not successful --> invalid register number
            (INCA_IOM_DSP_CHANNEL1_LOW .. INCA_IOM_DSP_CHANNEL2_HIGH)

Remarks:
   The CO_TSDP register of the INCA IP chip are used to set up a communication
   between DSP module and the IOM2 module.
   The setup of this register is IOM2 driver independent.

   The CO_TSDP register correspond to the following data channels
      CO_TSDP10 --> BCHA_CR.ENBC1
      CO_TSDP11 --> BCHA_CR.ENBC2
      CO_TSDP20 --> BCHB_CR.ENBC1
      CO_TSDP21 --> BCHB_CR.ENBC2

****************************************************************************/
int inca_iom2_co_set_TSDPxy( int reg_num, 
                      inca_co_opts_t *co_opts, 
                      inca_data_channel_state_e state)
{
   u32            tmp_co_reg;
   u32            tmp_mask = 0;
   volatile u32   *tmp_b_chan_reg = NULL;

   if (co_opts != NULL)
   {
      INCAIOM2_INFO("******************************************************\n");
      INCAIOM2_INFO("%s: CO_TSDP %d config is\n",__FUNCTION__, reg_num)
      INCAIOM2_INFO("   DPS: %s\n",(co_opts->DPS)?
                          "DU -> OUT , DD -> IN":"DD -> OUT , DU --> IN");
      INCAIOM2_INFO("   TSS: %d\n",co_opts->TSS);
      INCAIOM2_INFO("******************************************************\n");

      /* calculate CO_TSDPxy value */
      tmp_co_reg =  ((co_opts->DPS) ? INCA_IP_IOM_CO_TSDP10_DPS : 0) |
                         _INCA_IP_IOM_CO_TSDP10_TSS_GET(co_opts->TSS);

      switch(reg_num)
      {
         case INCA_IOM_DSP_CHANNEL1_LOW:  /* CO_TSDP10: DSP Channel 1  LOW  Byte */
            tmp_b_chan_reg = INCA_IP_IOM_BCHA_CR;
            tmp_mask = INCA_IP_IOM_BCHA_CR_EN_BC1;
            IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CO_TSDP10,tmp_co_reg);
            break;   
         case INCA_IOM_DSP_CHANNEL1_HIGH: /* CO_TSDP11: DSP Channel 1  HIGH  Byte */
            tmp_b_chan_reg = INCA_IP_IOM_BCHA_CR;
            tmp_mask = INCA_IP_IOM_BCHA_CR_EN_BC2;
            IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CO_TSDP11,tmp_co_reg);
            break;   
         case INCA_IOM_DSP_CHANNEL2_LOW:  /* CO_TSDP20: DSP Channel 1  LOW  Byte */
            tmp_b_chan_reg = INCA_IP_IOM_BCHB_CR;
            tmp_mask = INCA_IP_IOM_BCHB_CR_EN_BC1;
            IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CO_TSDP20,tmp_co_reg);
            break;   
         case INCA_IOM_DSP_CHANNEL2_HIGH: /* CO_TSDP21: DSP Channel 1  HIGH  Byte */
            tmp_b_chan_reg = INCA_IP_IOM_BCHB_CR;
            tmp_mask = INCA_IP_IOM_BCHB_CR_EN_BC2;
            IOM2_WRITE_REG_LOCK(INCA_IP_IOM_CO_TSDP21,tmp_co_reg);
            break;   
         default:
            INCAIOM2_ERROR("invalid CO register number:%d\n",reg_num);
            return (-1);
      }

      co_ctrl_regs[reg_num].TSS = co_opts->TSS ;
      co_ctrl_regs[reg_num].DPS = co_opts->DPS ;
   }

   /* enable / disable / ignore Data Channel control register */
   IOM2_READ_REG_LOCK(tmp_b_chan_reg, tmp_co_reg) ;

   /* clear all not required flags */
   tmp_co_reg = tmp_co_reg & 
                    (INCA_IP_IOM_BCHA_CR_EN_BC1 | INCA_IP_IOM_BCHA_CR_EN_BC2) ;
   switch(state)
   {
      case DATA_CHANNEL_DISABLE:
         tmp_co_reg |= 0x80 & ~tmp_mask ;
         break ;
      case DATA_CHANNEL_ENABLE:
         tmp_co_reg |= 0x80 | tmp_mask ;
         break ;
      case DATA_CHANNEL_IGNORE:
         /* do nothing */
         return 0;

      default:
         INCAIOM2_ERROR("invalid Data Channel setting for register:%d\n", reg_num);
         return -EFAULT;
   }

   IOM2_WRITE_REG_LOCK(tmp_b_chan_reg, tmp_co_reg) ;

   return 0;
}


/****************************************************************************
Description:
   This function return the CO_TSDPxy value for the given register.
   The values represent the communication path settings between DSP and IOM.

Arguments:
   reg_num: given register value 
   co_opts: OUT: current used register values (TSS, DPS)

Return:
   OK:      current values successful returned
   ERROR:   return not successful 
            --> no valid ptr to return variable
            --> invalid register number
                (INCA_IOM_DSP_CHANNEL1_LOW .. INCA_IOM_DSP_CHANNEL2_HIGH)

Remarks:
   The CO_TSDP register of the INCA IP chip are used to set up a communication
   between DSP module and the IOM2 module.
   The setup of this register is IOM2 driver independent.
****************************************************************************/
int inca_iom2_co_get_TSDPxy(int reg_num, inca_co_opts_t *co_opts)
{
   if (co_opts != NULL)
   {
      /* check register number */
      if ( (reg_num >= INCA_IOM_DSP_CHANNEL1_LOW) && 
           (reg_num <= INCA_IOM_DSP_CHANNEL2_HIGH) )
      {
         co_opts->TSS = co_ctrl_regs[reg_num].TSS ;
         co_opts->DPS = co_ctrl_regs[reg_num].DPS ;
      }
      else
      {
         INCAIOM2_ERROR("invalid CO register number:%d\n", reg_num);
         return -EFAULT;
      }
   }
   else
   {
      INCAIOM2_ERROR("missing argument for return CO register (%d) settings\n", reg_num);
      return -EFAULT;
   }

   return 0;
}


module_init(inca_iom2_init);
module_exit(inca_iom2_cleanup);

MODULE_PARM (inca_iom2_bitclock, "i");
MODULE_PARM_DESC (inca_iom2_bitclock, "Bit Clock: 0-single, 1-double");

MODULE_PARM (inca_iom2_master, "i");
MODULE_PARM_DESC (inca_iom2_master, "Mode: 0-slave, 1-master");

MODULE_PARM (inca_iom2_major, "i");
MODULE_PARM_DESC (inca_iom2_major, "Major Number of Driver");

MODULE_AUTHOR("Alexandre Oplatchkine");
MODULE_DESCRIPTION("INCA-IP IOM-2 device driver v1.0");
MODULE_LICENSE("GPL");

EXPORT_SYMBOL(inca_iom2_hw_init);
EXPORT_SYMBOL(inca_iom2_hw_uninit);

EXPORT_SYMBOL(inca_iom2_cda_open);
EXPORT_SYMBOL(inca_iom2_cda_close);
EXPORT_SYMBOL(inca_iom2_cda_read);
EXPORT_SYMBOL(inca_iom2_cda_write);
EXPORT_SYMBOL(inca_iom2_cda_ioctl);
EXPORT_SYMBOL(inca_iom2_cda_poll);

EXPORT_SYMBOL(inca_iom2_ci_open);
EXPORT_SYMBOL(inca_iom2_ci_close);
EXPORT_SYMBOL(inca_iom2_ci_read);
EXPORT_SYMBOL(inca_iom2_ci_write);
EXPORT_SYMBOL(inca_iom2_ci_ioctl);
EXPORT_SYMBOL(inca_iom2_ci_poll);

EXPORT_SYMBOL(inca_iom2_d_open);
EXPORT_SYMBOL(inca_iom2_d_close);
EXPORT_SYMBOL(inca_iom2_d_read);
EXPORT_SYMBOL(inca_iom2_d_write);
EXPORT_SYMBOL(inca_iom2_d_ioctl);
EXPORT_SYMBOL(inca_iom2_d_poll);

EXPORT_SYMBOL(inca_iom2_co_regs_default);
EXPORT_SYMBOL(inca_iom2_co_set_TSDPxy);
EXPORT_SYMBOL(inca_iom2_co_get_TSDPxy);




















