#ifndef _DRVINCAIOM2_H_
#define _DRVINCAIOM2_H_
/****************************************************************************
       Copyright (c) 2003, Infineon Technologies.  All rights reserved.

                               No Warranty                                     
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ****************************************************************************   
   Module      : $RCSfile: incaip_iom2.h,v $
   Date        : $ $
   Description : This file contains the includes and the defines 
                 specific to the IOM2 interface of the INCA-IP chip

   $Log: incaip_iom2.h,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.3  2003/11/14 14:58:37  de6wanme
   Disbled interrupts for access to IOM2 registers

   Revision 1.2  2003/08/15 13:49:16  de6wthne
   * moved define for debug output to incaip_iom2.h
   * some cosmetic

   Revision 1.1  2003/08/14 11:32:27  de6wthne
   IOM-2 kernel module delivered by EPSCOM - initial version (after integration)

*******************************************************************************/

/******************************************************************************/
/*                      CDA configuration                                     */
/******************************************************************************/
#define INCAIOM2_CDA_DEV_NUM           4 
#define INCAIOM2_CDA_MAXBUFSIZE        512 /* Bytes */

/******************************************************************************/
/*                      CI Handler configuration                              */
/******************************************************************************/
#define INCAIOM2_CI_DEV_NUM            2 


/******************************************************************************/
/*                      D Channel configuration                               */
/******************************************************************************/
#define INCAIOM2_D_MAX_TXBUFSIZE          1024 /* Bytes */
#define INCAIOM2_D_MAX_RXBUFSIZE          1024 /* Bytes */
#define INCAIOM2_D_MAX_UMASK_THRESHOLD    INCAIOM2_D_MAX_RXBUFSIZE/10
#define INCAIOM2_D_MAX_UMASK_RXBUFSIZE    INCAIOM2_D_MAX_RXBUFSIZE - INCAIOM2_D_MAX_UMASK_THRESHOLD

/******************************************************************************/
/*                   Macros to access to IOM2 Device registers                */
/******************************************************************************/
/* Interrupt module access macros */
#define INCAIOM2_IRQ_LINE_ALL    0x0000E000  /* all IRQs */
#define INCAIOM2_IRQ_LINE_ST     0x00002000  /* line 13 */
#define INCAIOM2_IRQ_LINE_CIC    0x00004000  /* line 14 */
#define INCAIOM2_IRQ_LINE_ICD    0x00008000  /* line 15 */

/* interrupt controller access macros */    
#define IOM2_INTERRUPTS_ENABLE(X)  \
	{						\
		unsigned long flags=0;	\
		save_and_cli(flags);	\
		*(INCA_IP_ICU_IM2_IER) |= X; \
		restore_flags(flags);	\
	}

#define IOM2_INTERRUPTS_DISABLE(X) \
	{						\
		unsigned long flags=0;	\
		save_and_cli(flags);	\
		*(INCA_IP_ICU_IM2_IER) &=~X; \
		restore_flags(flags);	\
	}

#define IOM2_INTERRUPTS_CLEAR(X)   \
	{						\
		unsigned long flags=0;	\
		save_and_cli(flags);	\
		*(INCA_IP_ICU_IM2_ISR) =  X; \
		restore_flags(flags);	\
	}

#define IOM2_WRITE_REG(reg, value)   \
         *(reg) = (u32)value;\
         __asm volatile ("nop;nop;nop;nop;nop");\
         *(reg) = (u32)value;

#define IOM2_READ_REG(reg, value)    \
         value = (u32)*((volatile u32*)reg);\
         __asm volatile ("nop;nop;nop;nop;nop");\
         value = (u32)*((volatile u32*)reg);\

#define IOM2_DEV_TYPE(X)   (MINOR(X->i_rdev) >> 4)
#define IOM2_DEV_NUM(X)    (MINOR(X->i_rdev) & 0x0F)

#define INCA_IP_IOM_MASKH_UMASK_ALL              0
#define INCA_IP_IOM_MASKH_MASK_ALL               0xFC

#define INCA_IP_IOM_MASKH_RX                     ( INCA_IP_IOM_MASKH_RME | \
                                                   INCA_IP_IOM_MASKH_RPF | \
                                                   INCA_IP_IOM_MASKH_RFO )
#define INCA_IP_IOM_MASKH_TX                     ( INCA_IP_IOM_MASKH_XPR | \
                                                   INCA_IP_IOM_MASKH_XMR | \
                                                   INCA_IP_IOM_MASKH_XDU )

/******************************************************************************/
/*                   IOM2 Device Structures                                   */
/******************************************************************************/
/* IOM2 device types */
typedef enum
{
   INCA_IOM2_CDA,
   INCA_IOM2_CI_HANDLER,
   INCA_IOM2_D_CHANNEL

}inca_iom2_device_e;


typedef struct ring_buf
{
   char  *data;
   int   head;
   int   tail;
   int   len;
}ring_buf_t;

typedef struct
{
   devfs_handle_t          handle;
   inca_iom2_device_e      type;
   int                     open;
   wait_queue_head_t       tx_queue;
   wait_queue_head_t       rx_queue;
}inca_iom2_dev_t;

/* IOM2 CDA device structure */
typedef struct
{
   devfs_handle_t          handle;
   inca_iom2_device_e      type;
   int                     open;
   wait_queue_head_t       tx_queue;
   wait_queue_head_t       rx_queue;

   u32                     dev_num;
   inca_cda_mode_e         mode;
   inca_cda_opts_t         hw_opts;
   int                     st_unmasked;

   ring_buf_t              in_buf;         /* input ring buffer */
   ring_buf_t              out_buf;        /* output ring buffer */
   struct semaphore        access_sem;
   u32                     last_value;
   int                     tx_stopped;

   u32                     stov_counter;
   u32                     sti_counter;
}inca_cda_dev_t;


/* IOM2 CI handler device structure */
#define CI_DATA_UPDATED(X)    !(X & 0x80)
#define CI_DATA_CLEAR(X)      X |= 0x80
#define CI_DATA_GET(X)        (X & 0x7F)
#define CI_DATA_UPDATE(X,Y)   X = Y
typedef struct
{
   devfs_handle_t          handle;
   inca_iom2_device_e      type;
   int                     open;
   wait_queue_head_t       tx_queue;
   wait_queue_head_t       rx_queue;

   u32                     dev_num;
   u8                      rx_data;        /* last received data */
   u8                      tx_data;        /* currently transmitted data */
   inca_ci_opts_t          hw_opts;
   struct semaphore        access_sem;

   u32                     tx_counter;         
   u32                     rx_counter;         
}inca_ci_dev_t;

#define IOM2_HDLC_MSG_HEADER_LENGHT       3
#define IOM2_HDLC_MSG_COMPLETE            1

#define IOM2_HDLC_FRAME_LEN_BYTE0_POS     1
#define IOM2_HDLC_FRAME_LEN_BYTE1_POS     2
#define IOM2_HDLC_FRAME_MSG_BEGIN_POS     3

#define IOM2_HDLC_HEADER_LEN_OFF_BYTE0    3
#define IOM2_HDLC_HEADER_LEN_OFF_BYTE1    2
#define IOM2_HDLC_HEADER_LEN_OFF_BYTE2    1
#define IOM2_HDLC_HEADER_LEN_OFF_BYTE3    0

typedef union
{
   u32      msg_len ;
   u8       msg_len_bytes[4] ;
}inca_hdlc_msg_header_t ;

/* 
   for handling the HDLC frames 

   REMARK:
   the both byte counter are used separate to avoid interrupt blocking
   while incrementation.

*/
typedef struct
{
   u8    buf[INCAIOM2_D_MAX_RXBUFSIZE];   /* receive buffer */
   u8    *next_wr_ptr;                      /* global ptr for write */
   u8    *next_rd_ptr;                      /* global ptr for read */
   u8    *msg_wr_ptr;                       /* ptr for write within one message */
   u8    *msg_rd_ptr;                       /* ptr for read within one message */
   int   wr_msg_pending;                    /* current message write not complete */
   int   rd_msg_pending;                    /* current message read not complete */
   u32   wr_byte_count;                     /* number of bytes, write: IOM2 --> rxBuffer */
   u32   rd_byte_count;                     /* number of bytes, read:  rxBuffer --> controller */
}inca_hdlc_buffer_t;

typedef struct
{
   devfs_handle_t          handle;
   inca_iom2_device_e      type;
   int                     open;
   wait_queue_head_t       tx_queue;
   wait_queue_head_t       rx_queue;

   inca_hdlc_buffer_t      in_buf;
   int                     int_disabled ; /* if in_buf full --> disable ICD interrupt */

   u8                      out_buf[INCAIOM2_D_MAX_TXBUFSIZE];
   u32                     out_buf_len;
   u32                     out_buf_ptr;
   struct semaphore        access_sem;
   
   inca_d_opts_t           hw_opts;
   inca_sapi_tei_t         sapi_tei;

   u32                     channel_mask;
   int                     tx_completed;
   u32                     last_rx_bytes_num;   

   u32                     rx_overflow;
   u32                     rx_invalid_frame;
   u32                     tx_repeat;
   u32                     tx_underrun;
   u32                     tx_msg_counter;
   u32                     tx_byte_counter;
   u32                     rx_msg_counter;
   u32                     rx_byte_counter;
   u32                     lack_free_space;
}inca_d_dev_t;

/******************************************************************************/
/*                      Debug output                                          */
/******************************************************************************/
//#define INCAIOM2_DEBUG_ENABLE
/******************************************************************************/
#ifdef INCAIOM2_DEBUG_ENABLE
   extern u32 G_debug_mask;

   #define INCAIOM2_PRINTK(fmt, args...)                            \
                   if(G_debug_mask & _FUNC_NAME_MASK){               \
                  printk(KERN_ALERT "IOM:%s:", __FUNCTION__);  \
                  printk(fmt, ## args);                        \
               }else{                                          \
                  printk(KERN_ALERT "IOM:" fmt, ## args);      \
               }                                                

   #define INCAIOM2_ERROR(fmt, args...)     if(G_debug_mask & _ERROR_MASK){INCAIOM2_PRINTK(fmt, ## args);}
   #define INCAIOM2_INFO(fmt, args...)      if(G_debug_mask & _INFO_MASK){INCAIOM2_PRINTK(fmt, ## args);}
   #define INCAIOM2_TRACE(fmt, args...)     if(G_debug_mask & _TRACE_MASK){INCAIOM2_PRINTK(fmt, ## args);}
   #define INCAIOM2_OUT(fmt, args...)       if(G_debug_mask & _DATA_MASK){printk(fmt, ## args);}
   #define INCAIOM2_IRQ_ERROR(fmt, args...) if(G_debug_mask & _IRQ_ERROR_MASK){printk(KERN_ALERT "IOM2_IRQ:" fmt, ## args);}
   #define INCAIOM2_IRQ_TRACE(fmt, args...) if(G_debug_mask & _IRQ_TRACE_MASK){printk(KERN_ALERT "IOM2_IRQ:" fmt, ## args);}
#else
   #define INCAIOM2_ERROR(fmt, args...)
   #define INCAIOM2_INFO(fmt, args...)
   #define INCAIOM2_TRACE(fmt, args...)
   #define INCAIOM2_OUT(fmt, args...)
   #define INCAIOM2_IRQ_ERROR(fmt, args...)
   #define INCAIOM2_IRQ_TRACE(fmt, args...)
#endif


/******************************************************************************/
/*                   IOM 2 interface functions                                */
/******************************************************************************/
int inca_iom2_hw_init(void);
int inca_iom2_hw_uninit(void);

int inca_iom2_cda_open(inca_cda_dev_t **dev, int num);
int inca_iom2_cda_close(inca_cda_dev_t *dev);
int inca_iom2_cda_read(inca_cda_dev_t *dev, u8 *dest, int length, int non_blocking);
int inca_iom2_cda_write(inca_cda_dev_t *dev, const char *src, int length, int non_blocking);
int inca_iom2_cda_ioctl(inca_cda_dev_t *dev, u32 cmd, u32 arg);
int inca_iom2_cda_poll(inca_cda_dev_t *dev);

int inca_iom2_ci_open(inca_ci_dev_t **dev, int num);
int inca_iom2_ci_close(inca_ci_dev_t *dev);
int inca_iom2_ci_read(inca_ci_dev_t *dev, u8 *dest, int non_blocking);
int inca_iom2_ci_write(inca_ci_dev_t *dev, const char *src, int length);
int inca_iom2_ci_ioctl(inca_ci_dev_t *dev, u32 cmd, u32 arg);
int inca_iom2_ci_poll(inca_ci_dev_t *dev);

int inca_iom2_d_open(inca_d_dev_t **dev);
int inca_iom2_d_close(inca_d_dev_t *dev);
int inca_iom2_d_read(inca_d_dev_t *dev, u8 *dest, int length, int non_blocking);
int inca_iom2_d_write(inca_d_dev_t *dev, const char *src, int length, int non_blocking);
int inca_iom2_d_ioctl(inca_d_dev_t *dev,  u32 cmd, u32 arg);
int inca_iom2_d_poll(inca_d_dev_t *dev);

/******************************************************************************/
/*                   Port Pin Function profiling                              */
/******************************************************************************/
//#define INCAIOM2_PROFILING
/******************************************************************************/
enum
{
    ST_PROFILING,
    CIC_PROFILING,
    ICD_PROFILING,
    ST_MON_PROFILING,
    ST_MON_NO_SPACE_PROFILING,
    ST_TX_PROFILING,
    ST_TX_NO_DATA_PROFILING,

} inca_profile_e; 

#ifdef INCAIOM2_PROFILING
    #define INCAIOM_CURRENT_PROFILING   ST_PROFILING 
    
    #define INCAIOM2_PROFILING_PIN      11 
    #define INCAIOM2_PROFILING_SETUP()    \
       *INCA_IP_Ports_P2_ALTSEL |= (1 << INCAIOM2_PROFILING_PIN);\
       *INCA_IP_Ports_P2_DIR |= (1 << INCAIOM2_PROFILING_PIN);
    #define INCAIOM2_PROFILING_UP(X)    \
        if(INCAIOM_CURRENT_PROFILING == X) *INCA_IP_Ports_P2_OUT |= (1 << INCAIOM2_PROFILING_PIN);
    #define INCAIOM2_PROFILING_DOWN(X)  \
       if(INCAIOM_CURRENT_PROFILING == X) *INCA_IP_Ports_P2_OUT &= ~(1 << INCAIOM2_PROFILING_PIN);
#else
    #define INCAIOM2_PROFILING_SETUP()
    #define INCAIOM2_PROFILING_UP(X)
    #define INCAIOM2_PROFILING_DOWN(X)
#endif /* INCAIOM2_PROFILING */

#endif
