#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <linux/spinlock.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <linux/errno.h>
#include <linux/init.h>

#include <asm/incaip/inca-ip.h>

#undef USE_OWN_MDIO_ACC_ACCESS
#ifndef USE_OWN_MDIO_ACC_ACCESS
#include "../switch/vxWorks_type.h"
#include "../switch/inca_switch_access.h"
#endif

#define SW_WRITE_REG(reg, value)   \
         *((volatile u32*)reg) = (u32)value;\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         *((volatile u32*)reg) = (u32)value;

#define SW_READ_REG(reg, value)    \
         value = (u32)*((volatile u32*)reg);\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         value = (u32)*((volatile u32*)reg);



static spinlock_t mdio_lock;

static u32 mdio_access_acc(u32 acc);

u16 mdio_read(u16 phy_id, u16 reg_num)
{
    u16 phy_reg_value;
    u32 acc;
/*      printk("mdio_read phy_id=%d, reg_num=%d\n", phy_id, reg_num); */
    acc = INCA_IP_Switch_MDIO_ACC_RA |
        INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phy_id) |
        INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(reg_num);

    acc = mdio_access_acc(acc);

    phy_reg_value = acc & 0xffff;

/*      printk("mdio_read: phy_reg_value = 0x%04x\n", phy_reg_value); */

    return phy_reg_value;
}

u16 mdio_write(u16 phy_id, u16 reg_num, u16 val)
{
    u16 phy_reg_value;
    u32 acc;

/*      printk("mdio_write phy_id=%d, reg_num=%d, val=0x%04x\n", */
/*             phy_id, reg_num, val); */

    acc = (INCA_IP_Switch_MDIO_ACC_RA | 
           INCA_IP_Switch_MDIO_ACC_RW |
           INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phy_id) |
           INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(reg_num) |
           INCA_IP_Switch_MDIO_ACC_PHY_DATA_SET(val)) ;

    acc = mdio_access_acc(acc) ;

    phy_reg_value = acc & 0xffff;

/*      printk("mdio_write: phy_reg_value = 0x%04x\n", phy_reg_value); */

    return phy_reg_value;
}

#ifdef USE_OWN_MDIO_ACC_ACCESS
static u32 mdio_access_acc(u32 acc)
{

    u32 acc_val;

/*      printk("mdio_access_acc: acc=0x%08x\n", acc); */
    spin_lock(&mdio_lock);
    SW_WRITE_REG(INCA_IP_Switch_MDIO_ACC, acc);
    do {
        SW_READ_REG(INCA_IP_Switch_MDIO_ACC, acc_val);
    } while (acc_val & INCA_IP_Switch_MDIO_ACC_RA);
    spin_unlock(&mdio_lock);
/*      printk("mdio_access_acc: acc_val=0x%08x\n", acc_val); */

    return acc_val;
}
#else
static u32 mdio_access_acc(u32 acc)
{
    return sw21553_Write_MDIO_ACC(acc);
}
#endif /* USE_OWN_MDIO_ACC_ACCESS */

#ifdef MODULE

MODULE_LICENSE("GPL");

int init_module(void)
{
    spin_lock_init(&mdio_lock);
    return 0;
}

void
cleanup_module(void)
{
}

#endif /* MODULE */
