/* IncaSwitchAutoMdixCfg.h - INCA IP auto MDIX configuration 

   Basic settings and definitions for support the Auto MDIX feature

*/

/*
modification history
--------------------
*/


#ifndef __INCboardAutoMDIX
#define __INCboardAutoMDIX

#if 0
/* includes */
#include "vxWorks.h"
#include "RegIncaIP.h"
#include "config.h"

#endif

#include <asm/incaip/inca-ip.h>

/*    CONFIGURATION of the Auto MDIX feature             */
/*-------------------------------------------------------*/

/* 
   only one GPIO is available to change the external relays
   currently there is no possibility to realise all 3 modes
   (MDI, MDIX and DETECT) with 1 GPIO 
*/
#define INCA_IP_EBU_PX ((volatile u32*)0xB1000000)
#define INCA_IP_AUTOMDIX_ONE_GPIO_PIN

/* only for testing */
#if 0
   #define SW_AMDIX_DEBUG_ENABLE
#else
   #undef  SW_AMDIX_DEBUG_ENABLE
#endif


/*---------------------------------------------------------
   PHY1 (PC): configuration and settings for auto MDIX 

   Board support of the Auto MDIX feature for the PC PHY port
   !!! The feature depends on the board implementation !!!

   By default the board does not supported the feature.
   If the board support the feature it can be activated 
   via the project facility. It depends on the driver setting 
      "INCLUDE_END_AUTOMDIX"
---------------------------------------------------------*/

/*---------------------------------------------------------*/
#define SUPPORT_AUTOMDIX_PC_PORT	1 /* current board support auto MDIX */
#define AUTOMDIX_PC_PORT_ON		0 /* switch ON/OFF auto MDIX */
#define SUPPORT_POWER_OVER_LAN_PC_PORT	0 /* currently not supported */
/*---------------------------------------------------------*/

/* board support auto MDIX */
#ifdef SUPPORT_AUTOMDIX_PC_PORT
   /* PHY1 (PC): auto MDIX feature supported */
   #define INCA_IP_BOARD_SUPPORT_AUTOMDIX_PC    1
#else
   /* PHY1 (PC): auto MDIX feature not supported */
   #define INCA_IP_BOARD_SUPPORT_AUTOMDIX_PC    0
#endif

/* support of power over LAN, currently not supported
   --> check/overwrite project-setting */
#ifndef SUPPORT_POWER_OVER_LAN_PC_PORT
   #define SUPPORT_POWER_OVER_LAN_PC_PORT FALSE
#endif

#if (SUPPORT_POWER_OVER_LAN_PC_PORT == TRUE)
   #define INCA_IP_BOARD_SUPPORT_POL_PC  1
#else
   #define INCA_IP_BOARD_SUPPORT_POL_PC  0
#endif

#define INCA_IP_AUTO_MDIX_PC_SUPPORT_POL_NO  0
#define INCA_IP_AUTO_MDIX_PC_SUPPORT_POL_YES 1
#define INCA_IP_AUTO_MDIX_PC_SUPPORT_POL     INCA_IP_AUTO_MDIX_PC_SUPPORT_POL_NO

/* switch ON/OFF auto MDIX */
#define INCA_IP_AUTO_MDIX_PC_AUTOMDIX_OFF       0
#define INCA_IP_AUTO_MDIX_PC_AUTOMDIX_ON        1

#ifdef AUTOMDIX_PC_PORT_ON
   #define INCA_IP_AUTO_MDIX_PC_AUTOMDIX_ON_OFF    INCA_IP_AUTO_MDIX_PC_AUTOMDIX_ON
#else
   #define INCA_IP_AUTO_MDIX_PC_AUTOMDIX_ON_OFF    INCA_IP_AUTO_MDIX_PC_AUTOMDIX_OFF
#endif
/*--- END PHY1 (PC) settings form vxWorks prj ---------------*/


/*---------------------------------------------------------
   PHY1 (PC): configuration and settings for used GPIO's
---------------------------------------------------------*/
/* PHY1 (PC): config the used port register and GPIO pin for RX relay */
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX  17
/* allow only port 1 or 2 */
#define INCA_IP_AUTO_MDIX_PC_PORT_REG_RX  1 

/* PHY1 (PC): config the used port register and GPIO pin for TX relay */
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX  1
/* allow only port 1 or 2 */
#define INCA_IP_AUTO_MDIX_PC_PORT_REG_TX  1 


/* PHY1 (PC): RX port register */
#if (INCA_IP_AUTO_MDIX_PC_PORT_REG_RX == 2)
   /* use port register 2 */
   #define INCA_IP_AUTO_MDIX_PC_PORTS_DIR_RX       INCA_IP_Ports_P2_DIR
   #define INCA_IP_AUTO_MDIX_PC_PORTS_ALTSEL_RX    INCA_IP_Ports_P2_ALTSEL
   #define INCA_IP_AUTO_MDIX_PC_PORTS_OUT_RX       INCA_IP_Ports_P2_OUT
#else
   /* default use port register 1 */
   #define INCA_IP_AUTO_MDIX_PC_PORTS_DIR_RX       INCA_IP_Ports_P1_DIR
   #define INCA_IP_AUTO_MDIX_PC_PORTS_ALTSEL_RX    INCA_IP_Ports_P1_ALTSEL
   #define INCA_IP_AUTO_MDIX_PC_PORTS_OUT_RX       INCA_IP_Ports_P1_OUT
#endif

/* PHY1 (PC): TX port register */
#if (INCA_IP_AUTO_MDIX_PC_PORT_REG_TX == 2)
   /* use port register 2 */
   #define INCA_IP_AUTO_MDIX_PC_PORTS_DIR_TX       INCA_IP_Ports_P2_DIR
   #define INCA_IP_AUTO_MDIX_PC_PORTS_ALTSEL_TX    INCA_IP_Ports_P2_ALTSEL
   #define INCA_IP_AUTO_MDIX_PC_PORTS_OUT_TX       INCA_IP_Ports_P2_OUT
#else
   /* default use port register 1 */
   #define INCA_IP_AUTO_MDIX_PC_PORTS_DIR_TX       INCA_IP_Ports_P1_DIR
   #define INCA_IP_AUTO_MDIX_PC_PORTS_ALTSEL_TX    INCA_IP_Ports_P1_ALTSEL
   #define INCA_IP_AUTO_MDIX_PC_PORTS_OUT_TX       INCA_IP_EBU_PX
#endif

#if INCA_IP_BOARD_SUPPORT_AUTOMDIX_PC
   #define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_RX   (1 << INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX)
   #define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_TX   0x00000001
#else
   #define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_RX   0x00000000
   #define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_TX   0x00000000
#endif

/* mask register for checks */
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX_STATE     0x00000001
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX_STATE     0x00000002
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RXTX_STATE   (INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX_STATE | INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX_STATE)

/* GPIO pin status for MDIX */
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MDIX_MODE    0x00000000
/* GPIO pin status for MDI */
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MDI_MODE     (INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX_STATE | INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX_STATE)
/* GPIO pin status for DETECT (RESET) */
#define INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RESET_MODE   INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX_STATE


/* 
   PHY1 (PC): change the settings of the external relais 
*/
/* connect incoming RX --> TX: switch ON RX relais: GPIO pin = 1 */
#define INCA_IP_AUTO_MDIX_PC_RX_RELAIS_RX_to_TX(PC_PORTS_OUT) \
            (PC_PORTS_OUT) &= ~(INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_RX)

/* connect incoming TX --> RX: switch ON TX relais: GPIO pin = 1 */
#define INCA_IP_AUTO_MDIX_PC_TX_RELAIS_TX_to_RX(PC_PORTS_OUT) \
            (PC_PORTS_OUT) &= ~(INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_TX)


/* connect incoming RX --> RX: switch OFF RX relais: GPIO pin = 0 */
#define INCA_IP_AUTO_MDIX_PC_RX_RELAIS_RX_to_RX(PC_PORTS_OUT) \
            (PC_PORTS_OUT) |= (INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_RX)

/* connect incoming TX --> TX: switch ON TX relais:  GPIO pin = 0 */
#define INCA_IP_AUTO_MDIX_PC_TX_RELAIS_TX_to_TX(PC_PORTS_OUT) \
            (PC_PORTS_OUT) |= (INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_TX)

/* return the current state of the PC port MDIX GPIO-pin */
#define INCA_IP_AUTO_MDIX_PC_GET_MDIX_GPIO_REG(AMDIX_PC_GPIO, PC_PORT_RX, PC_PORT_TX) \
            AMDIX_PC_GPIO = ( (PC_PORT_RX & INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_RX) ?\
                                 (AMDIX_PC_GPIO | INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX_STATE) :\
                                 (AMDIX_PC_GPIO & ~INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX_STATE) )|\
                            ( (PC_PORT_TX & INCA_IP_AUTO_MDIX_PC_GPIO_PIN_MASK_TX) ? \
                                 (AMDIX_PC_GPIO | INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX_STATE) : \
                                 (AMDIX_PC_GPIO & ~INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX_STATE) )






/**********************************************************
   PHY0 (LAN): configuration and settings for auto MDIX 
**********************************************************/

/*---------------------------------------------------------
   Board support of the Auto MDIX feature for the LAN PHY port
   !!! The feature depends on the board implementation !!!

   By default the board does not supported the feature.
   If the board support the feature it can be activated 
   via the project facility. It depends on the driver setting 
      "INCLUDE_END_AUTOMDIX"
---------------------------------------------------------*/
/*---------------------------------------------------------*/
#define SUPPORT_AUTOMDIX_LAN_PORT	1 /* current board support auto MDIX */
#define AUTOMDIX_LAN_PORT_ON		1 /* switch ON/OFF auto MDIX */
#define SUPPORT_POWER_OVER_LAN_LAN_PORT	1 /* current boart support Power over LAN */
/*---------------------------------------------------------*/


/* board support auto MDIX */
#ifdef SUPPORT_AUTOMDIX_LAN_PORT
   /* PHY0 (LAN): auto MDIX feature supported */
   #define INCA_IP_BOARD_SUPPORT_AUTOMDIX_LAN    1
#else
   /* PHY0 (LAN): auto MDIX feature not supported */
   #define INCA_IP_BOARD_SUPPORT_AUTOMDIX_LAN    0
#endif

/* boart support Power over LAN */
#ifndef SUPPORT_POWER_OVER_LAN_LAN_PORT
   #define SUPPORT_POWER_OVER_LAN_LAN_PORT FALSE
#endif
#if (SUPPORT_POWER_OVER_LAN_LAN_PORT == TRUE)
   #define INCA_IP_BOARD_SUPPORT_POL_LAN  1
#else
   #define INCA_IP_BOARD_SUPPORT_POL_LAN  0
#endif

#define INCA_IP_AUTO_MDIX_LAN_SUPPORT_POL_NO  0
#define INCA_IP_AUTO_MDIX_LAN_SUPPORT_POL_YES 1
#define INCA_IP_AUTO_MDIX_LAN_SUPPORT_POL     INCA_IP_AUTO_MDIX_LAN_SUPPORT_POL_YES

/* switch ON/OFF auto MDIX */
#define INCA_IP_AUTO_MDIX_LAN_AUTOMDIX_OFF      0
#define INCA_IP_AUTO_MDIX_LAN_AUTOMDIX_ON       1
#ifdef AUTOMDIX_LAN_PORT_ON
   #define INCA_IP_AUTO_MDIX_LAN_AUTOMDIX_ON_OFF    INCA_IP_AUTO_MDIX_LAN_AUTOMDIX_ON
#else
   #define INCA_IP_AUTO_MDIX_LAN_AUTOMDIX_ON_OFF    INCA_IP_AUTO_MDIX_LAN_AUTOMDIX_OFF
#endif



/* PHY0 (LAN): config the GPIO pins */
#define INCA_IP_AUTO_MDIX_LAN_PORTS_DIR       INCA_IP_Ports_P1_DIR
#define INCA_IP_AUTO_MDIX_LAN_PORTS_ALTSEL    INCA_IP_Ports_P1_ALTSEL
#define INCA_IP_AUTO_MDIX_LAN_PORTS_OUT       INCA_IP_Ports_P1_OUT

#ifdef INCA_IP_AUTOMDIX_ONE_GPIO_PIN
   /* for the Siemens implementation, only one GPIO will be required */
   #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_RXTX  16

   #if INCA_IP_BOARD_SUPPORT_AUTOMDIX_LAN
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX   (1 << INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_RXTX)
   #else
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX 0x00000000
   #endif

   /* 
      PHY0 (LAN): change the settings of the external relais
   */
   
   /*
      set MDI mode, connect:
         incoming LAN line RX  --> INCA-IP TX 
         incoming LAN line TX  --> INCA-IP RX
   */
   
   /* use 1 GPIO-pin */
   #define INCA_IP_AUTO_MDIX_LAN_SET_MDI(LAN_PORTS_OUT) \
               (LAN_PORTS_OUT) &= ~(INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX)
   /*
      set MDIX mode, connect:
         incoming LAN line RX  --> INCA-IP TX 
         incoming LAN line TX  --> INCA-IP RX
   */
   /* use 1 GPIO-pin */
   #define INCA_IP_AUTO_MDIX_LAN_SET_MDIX(LAN_PORTS_OUT) \
               (LAN_PORTS_OUT) |= (INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX)


#else    /* #ifdef INCA_IP_AUTOMDIX_ONE_GPIO_PIN */


   #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_RX  16
   #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_TX  17
   
   #if INCA_IP_BOARD_SUPPORT_AUTOMDIX_PC
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RX   (1 << INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX)
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_TX   (1 << INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX)
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX ( (1 << INCA_IP_AUTO_MDIX_PC_GPIO_PIN_RX) | \
                                                         (1 << INCA_IP_AUTO_MDIX_PC_GPIO_PIN_TX))
   #else
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RX   0x00000000
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_TX   0x00000000
      #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX 0x00000000
   #endif
   
   /* mask register for checks */
   #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_DETECT_MASK  (1 << INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_RX)
   #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MDIX_MASK    0x00000000
   #define INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MDI_MASK     ( (1 << INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_RX) | \
                                                         (1 << INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_TX))

   /* 
      PHY1 (PC): change the settings of the external relais 
   */
   /* connect incoming RX --> TX: switch ON RX relais: GPIO pin = 1 */
   #define INCA_IP_AUTO_MDIX_LAN_RX_RELAIS_RX_to_TX(LAN_PORTS_OUT) \
               (LAN_PORTS_OUT) |= (INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RX)
   
   /* connect incoming TX --> RX: switch ON TX relais: GPIO pin = 1 */
   #define INCA_IP_AUTO_MDIX_LAN_TX_RELAIS_TX_to_RX(LAN_PORTS_OUT) \
               (LAN_PORTS_OUT) |= (INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_TX)
   
   
   /* connect incoming RX --> RX: switch OFF RX relais: GPIO pin = 0 */
   #define INCA_IP_AUTO_MDIX_LAN_RX_RELAIS_RX_to_RX(LAN_PORTS_OUT) \
               (LAN_PORTS_OUT) &= ~(INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RX)
   
   /* connect incoming TX --> TX: switch ON TX relais:  GPIO pin = 0 */
   #define INCA_IP_AUTO_MDIX_LAN_TX_RELAIS_TX_to_TX(LAN_PORTS_OUT) \
               (LAN_PORTS_OUT) &= ~(INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_TX)

#endif    /* #ifdef INCA_IP_AUTOMDIX_ONE_GPIO_PIN */


/* return the current state of the LAN MDIX GPIO-pin */
#define INCA_IP_AUTO_MDIX_LAN_GET_MDIX_GPIO_REG(AMDIX_LAN_GPIO, LAN_PORTS_OUT) \
            AMDIX_LAN_GPIO = ((LAN_PORTS_OUT) & (INCA_IP_AUTO_MDIX_LAN_GPIO_PIN_MASK_RXTX))











/* number of tries for the AutoMDIX functionality */
#define INCA_IP_AUTOMDIX_PC_TRY_NUM    10
#define INCA_IP_AUTOMDIX_LAN_TRY_NUM   0x7fffffff /* virtually try forever */

/* delay time needed by the PHY module to set the state into the status register */
#define INCA_IP_AUTOMDIX_STATUS_DELAY_ms  2000
#define INCA_IP_AUTOMDIX_PHY_RESET_DELAY_ms  200

/* !!! mode definitions: must match with the driver-definitions !!!! */
/* 
   driver mode definitions of the Auto MDIX 
      #define INCA_IP_AUTOMDIX_MODE_RESET       0
      #define INCA_IP_AUTOMDIX_MODE_MDI         1
      #define INCA_IP_AUTOMDIX_MODE_MDIX        2
*/
#define INCA_IP_AUTOMDIX_UMODE_RESET       0
#define INCA_IP_AUTOMDIX_UMODE_MDI         1
#define INCA_IP_AUTOMDIX_UMODE_MDIX        2

#endif      /* #ifndef __INCautoMDIXCfg ... */

