#ifndef __INCA_MDIO_SETTING_H__
#define __INCA_MDIO_SETTING_H__

/* init MDIO configuration: 
      MDS (Poll speed):       0x01 (4ms)
      PHY_LAN_ADDR:           0x06
      PHY_PC_ADDR:            0x05
      UEP (Use External PHY): 0x00 (Internal PHY is used)
      PS (Port Select):       0x00 (PT/UMM for LAN)
      PT (PHY Test):          0x00 (no test mode)
      UMM (Use MDIO Mode):    0x01 (state machine is enabled)
*/

/* BIT[30:16], BIT[1] Reserved  */
#define INCA_IP_Switch_MDIO_CFG_RESERVED_MASK       0x7FFF0002

/* BIT[15:14]: MDIO poll speed */
#define INCA_IP_Switch_MDIO_CFG_MDS_DEF_VALUE  INCA_IP_Switch_MDIO_CFG_MDS_4ms
#define INCA_IP_Switch_MDIO_CFG_MDS_DEFAULT    INCA_IP_Switch_MDIO_CFG_MDS_SET(INCA_IP_Switch_MDIO_CFG_MDS_DEF_VALUE)

/* Bit[13:9]:  LAN Phy Address --> get form the MDIO-CFG itself */
#define INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR    0x06
#define INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_DEFAULT  (INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_SET(INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR) )

/* Bit[8:4]:   PC Phy Address  --> get form the MDIO-CFG itself */
#define INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR     0x05
#define INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_DEFAULT   (INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_SET(INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR))

/* Bit[3]:     Use External Phy 
               0: Use internal phy
               1: Use external phy */
#define INCA_IP_Switch_MDIO_CFG_UEP_INTERNAL   0
#define INCA_IP_Switch_MDIO_CFG_UEP_EXTERNAL  INCA_IP_Switch_MDIO_CFG_UEP
#define INCA_IP_Switch_MDIO_CFG_UEP_DEFAULT   INCA_IP_Switch_MDIO_CFG_UEP_INTERNAL

/* Bit[2]:     Port select 
               0: Use external phy / test mode settings for LAN 
               1: Use external phy / test mode settings for PC  */
#define INCA_IP_Switch_MDIO_CFG_PS_LAN       0
#define INCA_IP_Switch_MDIO_CFG_PS_PC        INCA_IP_Switch_MDIO_CFG_PS
#define INCA_IP_Switch_MDIO_CFG_PS_DEFAULT   INCA_IP_Switch_MDIO_CFG_PS_LAN

/* Bit[0]:     Use MDIO mode
               0: MDIO hardware state machine is disabled
               1: MDIO hardware state machine is enabled  */
#define INCA_IP_Switch_MDIO_CFG_UMM_DISABLED 0
#define INCA_IP_Switch_MDIO_CFG_UMM_ENABLED  INCA_IP_Switch_MDIO_CFG_UMM
#define INCA_IP_Switch_MDIO_CFG_UMM_DEFAULT  INCA_IP_Switch_MDIO_CFG_UMM_ENABLED

/* END default settings: MDIO_CFG - configuration of the MDIO interface */
#endif /* __INCA_MDIO_SETTING_H__ */
