/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : INCAIPSwitchAccess.c
   Date        : 22.05.2003
   Description : provide functional interface to read and configure the INCA-IP 
                 ethernet parts like PHY's and switch module
                
   $Log: inca_switch_access.c,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.11  2004/02/24 12:38:58  de6wanme
   Initial Auto-MDIX support added

   Revision 1.9  2003/11/24 11:43:51  de6wmigr
   Function sw21553_swNonCritPacketDrop() added to configure PM_CTL register.
   Printing of configuration values added to watermark access functions.

   Revision 1.8  2003/10/27 11:52:30  de6wmigr
   Added MIB counter proc-file functionality.

   Revision 1.7  2003/10/21 13:29:34  de6wanme
   Fixed missing EXPORT_SYMBOL

   Revision 1.6  2003/10/20 14:16:02  de6wanme
   - Added function to read rx_dropped MIB counter
   - Disabled interrupts for Read-Modify-Write functions

   Revision 1.5  2003/07/25 10:02:40  rjanse_a
   Bug fix (UnDestPort modification/display)

   Revision 1.4  2003/07/24 11:35:59  rjanse_a
   Bug Fixed: Corruption of vlan table if an entry is overwritten
              If vlan table is full and an entry has to be modified, no error is returned

   Revision 1.3  2003/07/16 11:45:28  rjanse_a
   Cleanup.
   Fixes to suppress warnings.

   Revision 1.2  2003/07/15 11:09:33  rjanse_a
   Bug fixes.
   Improvements in API handling.

   Revision 1.1  2003/07/14 08:57:08  rjanse_a
   Initial revision of ported vxWorks switch driver (inca_switch_hash, inca_switch_access) and a simple control interface (inca_switch_api, api.sh)

   Revision 1.4  2003/07/07 17:41:38  mangele
   - Add Switch LED API

   Revision 1.3  2003/06/04 15:54:59  mangele
   Add further API functions for access switch INCA IP module

   Revision 1.2  2003/06/02 21:52:09  mangele
   Add further functionallity for the switch API




   
******************************************************************************/

/*  #include "vxWorks.h" */
/*  #include "stdio.h" */
/*  #include "string.h" */

/*  #include "RegIncaIP.h" */
/*  #include "IncaSwitchEnd.h" */
/*  #include "IncaSwitchHash.h" */
/*  #include "IncaSwitchAccess.h" */

#define EXPORT_SYMTAB

#include <asm/incaip/inca-ip.h>
#include <asm/io.h>
#include <linux/spinlock.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>

#include "vxWorks_type.h"
#include "inca_switch_access.h"
#include "inca_switch_phy.h"
#include "inca_phy_regs.h"
#include "inca_switch_settings.h"
#include "inca_mdio_settings.h"
#include "inca_switch_actl.h"
#include "inca_switch_hash.h"


#ifdef INCLUDE_LED_MULTIPLEXER
   #include "../LED/incaLEDMultiplexer.h"
#endif

IMPORT unsigned int sysIncaVer(void);

/******************************************************************************
   Common definitions
******************************************************************************/
#if 0
#define SW_WRITE_REG(reg, value)   \
         *((volatile u32*)reg) = (u32)value;\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         *((volatile u32*)reg) = (u32)value;

#define SW_READ_REG(reg, value)    \
         value = (u32)*((volatile u32*)reg);\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         value = (u32)*((volatile u32*)reg);
#endif
#define INCA_IP_DEBUG_VLAN
#undef  INCA_IP_DEBUG_VLAN

/* DEBUG MACROS */
#ifdef SW_DEBUG_ENABLE
   /*#define SW_DEBUG_RX_PRINTOUT*/

   /* debug print out masks */
   #define SW_DEBUG_OFF          0x00000000
   #define SW_DEBUG_INIT         0x00000001
   #define SW_DEBUG_RX           0x00000002
   #define SW_DEBUG_TX           0x00000004
   #define SW_DEBUG_POLL_RX      0x00000008
   #define SW_DEBUG_POLL_TX      0x00000010
   #define SW_DEBUG_POLL         0x00000018 /*(POLL_TX + POLL_RX)*/
   #define SW_DEBUG_IOCTL        0x00000020
   #define SW_DEBUG_INT          0x00000040
   #define SW_DEBUG_QUEUE        0x00000080
   #define SW_DEBUG_EVENT        0x00000100
   #define SW_DEBUG_PHY          0x00000200
   #define SW_DEBUG_FULL         0xFFFFFFFF

/*   #define SW_DEBUG_MASK       (SW_DEBUG_FULL & ~SW_DEBUG_QUEUE & ~SW_DEBUG_INT)*/
/*   #define SW_DEBUG_MASK         (SW_DEBUG_INIT | SW_DEBUG_RX | SW_DEBUG_TX)*/
/*   #define SW_DEBUG_MASK         (SW_DEBUG_PHY | SW_DEBUG_INT) */
/*   #define SW_DEBUG_MASK         (SW_DEBUG_PHY | SW_DEBUG_INT) */
/*   #define SW_DEBUG_MASK         (SW_DEBUG_FULL) */
/*   #define SW_DEBUG_MASK         (SW_DEBUG_INIT) */
   #define SW_DEBUG_MASK         (SW_DEBUG_OFF)

   #define SW_DEBUG(type,X)         if((UINT32)type & SW_DEBUG_MASK) printf X
   #define SW_LOG(type,S)           if((UINT32)type & SW_DEBUG_MASK) logMsg(S,0,0,0,0,0,0)
   #define SW_LOG1(type,S,P1)       if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,0,0,0,0,0)
   #define SW_LOG2(type,S,P1,P2)    if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,0,0,0,0)
   #define SW_LOG3(type,S,P1,P2,P3) if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,P3,0,0,0)
   #define SW_LOG4(type,S,P1,P2,P3,P4)    if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,0,0)
   #define SW_LOG5(type,S,P1,P2,P3,P4,P5) if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,P5,0)

   #undef LOCAL
   #define LOCAL

#else
   #define SW_DEBUG(type,X)          
   #define SW_LOG(type,S)            
   #define SW_LOG1(type,S,P1)        
   #define SW_LOG2(type,S,P1,P2)     
   #define SW_LOG3(type,S,P1,P2,P3)
   #define SW_LOG4(type,S,P1,P2,P3,P4)
   #define SW_LOG5(type,S,P1,P2,P3,P4,P5)

#endif /* SW_DEBUG_ENABLE */
#ifdef INCA_IP_DEBUG_VLAN
   #define SW_DEBUG_VLAN(X)         printk(X)
   #define SW_LOG_0_VLAN(s)         printk(s,0,0,0,0,0,0)
   #define SW_LOG_1_VLAN(s,P1)      printk(s,P1,0,0,0,0,0)
   #define SW_LOG_2_VLAN(s,P1,P2)   printk(s,P1,P1,0,0,0,0)
#else
   #define SW_DEBUG_VLAN(X)
   #define SW_LOG_0_VLAN(s)
   #define SW_LOG_1_VLAN(s,P1)
   #define SW_LOG_2_VLAN(s,P1,P2)
#endif

#if 0
/******************************************************************************
*  sw21553_PhyReadReg()
*     read a MDIO PHY register
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     phyRegIndex: select the MDIO PHY register
*
*  Returns:
*     ERROR: in case of invalid MDIO register index
*     else
*     REGISTER VALUE
*
*  REMARK:
*
*/
//LOCAL UINT32 sw21553_PhyReadReg(int portSelect, UINT32 phyRegIndex)
UINT32 sw21553_PhyReadReg(int portSelect, UINT32 phyRegIndex)
{
   UINT32   regValue ;
   UINT32   phyAddr ;

   /* read PHY address */
   SW_READ_REG(INCA_IP_Switch_MDIO_CFG, regValue) ;
   if (portSelect == INCA_IP_MDIO_LAN_PHY)
   {
      phyAddr = INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_GET(regValue) ;
   }
   else
   {
      if (portSelect == INCA_IP_MDIO_PC_PHY)
         phyAddr = INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_GET(regValue) ;
      else
         return(ERROR) ;
   }
   regValue = 0 ;

   /* read register value */
   switch (phyRegIndex)
   {
      case INCA_IP_MDIO_REG0_PHY_BCR   : /* basic control register PHY_BCR */
      case INCA_IP_MDIO_REG1_PHY_BSR   : /* basic status register  PHY_BSR */
      case INCA_IP_MDIO_REG2_PHY_ID1   : /* PHY identifier 1 PHY_ID1 */
      case INCA_IP_MDIO_REG3_PHY_ID2   : /* PHY identifier 2 PHY_ID2 */
      case INCA_IP_MDIO_REG4_PHY_ANAR  : /* Auto-Negotiation Advertisement Register (PHY_ANAR) */
      case INCA_IP_MDIO_REG5_PHY_ANLPAR: /* Auto-Negotiation Link Partner Ability Register (PHY_ANLPAR) */
      case INCA_IP_MDIO_REG6_PHY_ANER  : /* Auto-Negotiation Expansion Register (PHY_ANER) */
      case INCA_IP_MDIO_REG16_PHY_SRR  : /* Silicon Revision Register (PHY_SRR) */
      case INCA_IP_MDIO_REG17_PHY_MCSR : /* Mode Control/Status Register (PHY_MCSR) */
      case INCA_IP_MDIO_REG18_PHY_SMR  : /* Special Modes Register (PHY_SMR) */
      case INCA_IP_MDIO_REG29_PHY_ISR  : /* Interrupt Source Register (PHY_ISR) */
      case INCA_IP_MDIO_REG30_PHY_IMR  : /* Interrupt Mask Register (PHY_IMR) */
      case INCA_IP_MDIO_REG31_PHY_SCSR : /* PHY Special Control/Status Register (PHY_SCSR) */
         /* set MDIO_ACC read command */
         regValue = INCA_IP_Switch_MDIO_ACC_RA | 
                    INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phyAddr) |
                    INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(phyRegIndex) ;

         regValue = sw21553_Write_MDIO_ACC(regValue) ;
         /* extract PHY register value */
         regValue = INCA_IP_Switch_MDIO_ACC_PHY_DATA_GET(regValue) ;

         return(regValue) ;
         break ;

      default:
         /* ERROR: invalid register index */
         return(ERROR) ;
   }

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_PhyWriteReg()
*     read a MDIO PHY register
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     phyRegIndex: select the MDIO PHY register
*     newRegValue: new value to be set
*
*  Returns:
*     ERROR: in case of invalid MDIO register index
*     else
*     REGISTER VALUE
*
*  REMARK:
*
*/
//LOCAL UINT32 sw21553_PhyWriteReg(int portSelect, UINT32 phyRegIndex, UINT32 newRegValue)
UINT32 sw21553_PhyWriteReg(int portSelect, UINT32 phyRegIndex, UINT32 newRegValue)
{
   UINT32   regValue, reservedMask ;
   UINT32   phyAddr ;

   /* read PHY address */
   SW_READ_REG(INCA_IP_Switch_MDIO_CFG, regValue) ;
   if (portSelect == 0)
   {
      phyAddr = INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_GET(regValue) ;
   }
   else
   {
      phyAddr = INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_GET(regValue) ;
   }

   /* set reseve bit mask or return with error */
   switch (phyRegIndex)
   {
      case INCA_IP_MDIO_REG0_PHY_BCR   : /* basic control register PHY_BCR */
         reservedMask = INCA_IP_PHY_REG0_BCR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG4_PHY_ANAR  : /* Auto-Negotiation Advertisement Register (PHY_ANAR) */
         reservedMask = INCA_IP_PHY_REG4_ANAR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG17_PHY_MCSR : /* Mode Control/Status Register (PHY_MCSR) */
         reservedMask = INCA_IP_PHY_REG17_MCSR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG18_PHY_SMR  : /* Special Modes Register (PHY_SMR) */
         reservedMask = INCA_IP_PHY_REG18_SMR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG30_PHY_IMR  : /* Interrupt Mask Register (PHY_IMR) */
         reservedMask = INCA_IP_PHY_REG_INT_RESERVE_MASK ;
         break ;

      /* invalid register index or register only read able */
      case INCA_IP_MDIO_REG1_PHY_BSR   : /* basic status register  PHY_BSR */
      case INCA_IP_MDIO_REG2_PHY_ID1   : /* PHY identifier 1 PHY_ID1 */
      case INCA_IP_MDIO_REG3_PHY_ID2   : /* PHY identifier 2 PHY_ID2 */
      case INCA_IP_MDIO_REG5_PHY_ANLPAR: /* Auto-Negotiation Link Partner Ability Register (PHY_ANLPAR) */
      case INCA_IP_MDIO_REG6_PHY_ANER  : /* Auto-Negotiation Expansion Register (PHY_ANER) */
      case INCA_IP_MDIO_REG16_PHY_SRR  : /* Silicon Revision Register (PHY_SRR) */
      case INCA_IP_MDIO_REG29_PHY_ISR  : /* Interrupt Source Register (PHY_ISR) */
      case INCA_IP_MDIO_REG31_PHY_SCSR : /* PHY Special Control/Status Register (PHY_SCSR) */
      default:
         /* ERROR: invalid register index or read only register */
         return(ERROR) ;
   }        /* switch (phyRegIndex) {...} */

   /* set MDIO_ACC read command */
   regValue = INCA_IP_Switch_MDIO_ACC_RA | 
              INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phyAddr) |
              INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(phyRegIndex) ;
   regValue = sw21553_Write_MDIO_ACC(regValue) ;

   /* set MDIO_ACC write command, keep reserved bits */
   regValue &= reservedMask ;
   regValue |= (INCA_IP_Switch_MDIO_ACC_RA | 
                INCA_IP_Switch_MDIO_ACC_RW |
                INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phyAddr) |
                INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(phyRegIndex) |
                INCA_IP_Switch_MDIO_ACC_PHY_DATA_SET(newRegValue)) ;

   regValue = sw21553_Write_MDIO_ACC(regValue) ;
   /* extract PHY register value */
   regValue = INCA_IP_Switch_MDIO_ACC_PHY_DATA_GET(regValue) ;

   return(regValue) ;
}


/******************************************************************************
*  sw21553_PhyGetProperties()
*     returns the selected PHY properties
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     phyInfo: specify the properties for return
*             0: current properties
*             1: advertised PHY properties for auto-neg.
*             2: partner PHY properties

*
*  Returns:
*     ERROR: in case of invalid selection
*     else
*     PHY properties
*
*  REMARK:
*
*/
UINT32 sw21553_PhyGetProperties(int portSelect, int phyInfo)
{
   UINT32   reg1Value, reg0Value, regAutoNegValue, reg6Value, reg31Value ;
   UINT32   returnState = 0 ;
   /* read current PHY properties form Basic Status Register */
   reg1Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG1_PHY_BSR) ;

   reg31Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG31_PHY_SCSR) ;
   returnState  = sw21553_PHY_CTRL_CURR_SPEED_SET(
                           INCA_IP_PHY_REG31_SCSR_SPEED_GET(reg31Value)
                                                 ) ;
   returnState |= (reg31Value & INCA_IP_PHY_REG31_SCSR_AUTODONE_MASK) ? 
                                      sw21553_PHY_CTRL_AUTO_NEG_DONE  : 0 ;
   switch(phyInfo)
   {
      case sw21553_PHY_PROPS_CURRENT:
         /* read speed and duplex settings */
         returnState |= INCA_IP_PHY_CTRL_LINE_PROPS_GET_BSR(reg1Value) ;
         /* read link status */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_LS_MASK) ? 
                                        sw21553_PHY_CTRL_LINK_STATE : 0 ) ;
         /* read remote fault */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_RF_MASK) ? 
                                      sw21553_PHY_CTRL_REMOTE_FAULT : 0 ) ;
         /* read Extended Capabilities */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_EXC_MASK) ? 
                                sw21553_PHY_CTRL_EXT_CAP : 0 ) ;
         /* read Auto Neg. Ability */
         if(reg1Value & INCA_IP_PHY_REG1_BSR_ANA_MASK)
         {
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE ;
            /* read Auto Neg. complete */
            returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_ANC_MASK) ? 
                                         sw21553_PHY_CTRL_AUTO_NEG_COMPL : 0 ) ;
         }
         /* check validity of the data */
         if ( (returnState & sw21553_PHY_PROPS_CURRENT_VALID_MASK) != 
                                                sw21553_PHY_PROPS_CURRENT_VALID_STATE )
         {
            /* valid mask does not match --> set invalid bit */
            returnState |= sw21553_PHY_CTRL_DATA_CORRUPTED ;
         }
         return(returnState) ;
         break ;

      case sw21553_PHY_PROPS_ADVERTISE:
         /* read current PHY properties form 
                        Basic Control Register (PHY_BCR) */
         reg0Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR) ;
         /* read current PHY properties form 
                        Auto Negotiation Advertisement Register (PHY_ANAR) */
         regAutoNegValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG4_PHY_ANAR) ;

         /* read speed and duplex settings */
         returnState |= (INCA_IP_PHY_CTRL_LINE_PROPS_GET_ANAR(regAutoNegValue)) ;
         /* read Next Page */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG4_ANAR_NP_MASK) ? 
                                                 sw21553_PHY_CTRL_NEXT_PAGE : 0 ) ;
         /* read Pause Operation */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG4_ANAR_PO_MASK) ? 
                                                     sw21553_PHY_CTRL_PAUSE : 0 ) ;

         /* check if Auto Neg. Enable */
         if(reg0Value & INCA_IP_PHY_REG0_BCR_ANE_MASK)
         {
            /* Auto Neg. is enabled */
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_SUPPORT ;
         }

         /*--- read common link status data from PHYReg 1 ---*/
         /* read link status */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_LS_MASK) ? 
                                        sw21553_PHY_CTRL_LINK_STATE : 0 ) ;
         /* read remote fault */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_RF_MASK) ? 
                                      sw21553_PHY_CTRL_REMOTE_FAULT : 0 ) ;
         /* read Extended Capabilities */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_EXC_MASK) ? 
                                sw21553_PHY_CTRL_EXT_CAP : 0 ) ;
         /* read Auto Neg. Ability */
         if(reg1Value & INCA_IP_PHY_REG1_BSR_ANA_MASK)
         {
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE ;
            /* read Auto Neg. complete */
            returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_ANC_MASK) ? 
                                         sw21553_PHY_CTRL_AUTO_NEG_COMPL : 0 ) ;
         }
         return (returnState) ;
         break ;

      case sw21553_PHY_PROPS_PARTNER:
         /*
            # informations about the link partner are only valid if 
              --> link partner is able to perform auto negotiation
              --> auto neg. complete
         */
         /* read current PHY properties form 
                        Auto Negotiation Expansion Register (PHY_ANER) */
         reg6Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG6_PHY_ANER) ;
         /* read current PHY properties form 
                        Auto Negotiation Link Partner Ability Register (PHY_ANLPAR) */
         regAutoNegValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG5_PHY_ANLPAR) ;

         /* read current PHY properties form 
                     Auto Negotiation Link Partner Ability Register (PHY_ANLPAR) */
         returnState |= (INCA_IP_PHY_CTRL_LINE_PROPS_GET_ANLPAR(regAutoNegValue)) ;

         /* check if the link partner is able to do auto-negotiation */
         returnState |= ( (reg6Value & INCA_IP_PHY_REG6_ANER_LPANA_MASK) ? 
                                       sw21553_PHY_CTRL_AUTO_NEG_SUPPORT : 0 ) ;
         /* read Next Page */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG5_ANLPAR_NP_MASK) ? 
                                                   sw21553_PHY_CTRL_NEXT_PAGE : 0 ) ;
         /* read Pause Operation */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG5_ANLPAR_PO_MASK) ? 
                                                 sw21553_PHY_CTRL_PAUSE : 0 ) ;

         /*--- read common link status data from PHYReg 1 ---*/
         /* read Auto Neg. Ability */
         if(reg1Value & INCA_IP_PHY_REG1_BSR_ANA_MASK)
         {
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE ;
            /* read Auto Neg. complete */
            returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_ANC_MASK) ? 
                                         sw21553_PHY_CTRL_AUTO_NEG_COMPL : 0 ) ;
         }
         /* read link status */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_LS_MASK) ? 
                                        sw21553_PHY_CTRL_LINK_STATE : 0 ) ;
         /* read remote fault */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_RF_MASK) ? 
                                      sw21553_PHY_CTRL_REMOTE_FAULT : 0 ) ;
         /* read Extended Capabilities */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_EXC_MASK) ? 
                                sw21553_PHY_CTRL_EXT_CAP : 0 ) ;

         /* check validity of the data */
         if ( (returnState & sw21553_PHY_PROPS_PARTNER_VALID_MASK) != 
                                                sw21553_PHY_PROPS_PARTNER_VALID_STATE )
         {
            /* valid mask does not match --> set invalid bit */
            returnState |= sw21553_PHY_CTRL_DATA_CORRUPTED ;
         }
         return (returnState) ;
         break ;

      default:
         /* no valid parameter */
         return(ERROR) ;

   }        /* switch(phyInfo) {...} */

   return(ERROR) ;
}




/******************************************************************************
*  sw21553_PhySetAdvertiseProperties()
*     set the advertise properties for auto negotiation (PHY reg 4)
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     newSettings: new properies to set
*                  --> line properties: bit[0..4]
*                       bit 0:  10 MBit/s Half Duplex
*                       bit 1:  10 MBit/s Full Duplex
*                       bit 2: 100 MBit/s Half Duplex
*                       bit 3: 100 MBit/s Full Duplex
*                      (bit 4: 100 MBit/s T4 - can not set )
*                      (bit 5..7: not used)
*                  --> support Pause Operation
*                       bit 8: Support Pause Operation
*
*  Returns:
*     ERROR: in case of invalid selection
*     else
*     PHY advertiese properties (PHY reg 4)
*
*  Example: 
*     sw21553_PhySetAdvertiseProperties( 0, 0x003)
*        --> select LAN PHY port
*        --> new settings: 0x003
*            - set "not support Pause"
*            - advertise: 10MBit/s Full Duplex | 10 MBit/s Half Duplex
*
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*
*/
UINT32 sw21553_PhySetAdvertiseProperties(int portSelect, UINT32 newSettings)
{
   UINT32   regValue ;

   /* clear all not writeable bits */
   newSettings &= sw21553_PHY_PROPS_ADVERTISE_SET_MASK ;

   /* read current settings */
   regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG4_PHY_ANAR) ;

   /* clear line properties and set the new settings */
   regValue &= ~(INCA_IP_MDIO_REG4_PROPERTY_MASK) ;
   regValue |= INCA_IP_PHY_CTRL_LINE_PROPS_SET_ANAR(newSettings) ;
   if(newSettings & sw21553_PHY_CTRL_PAUSE)
      regValue |= INCA_IP_PHY_REG4_ANAR_PO_MASK ;

   /* write new register settings */
   regValue = sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG4_PHY_ANAR, regValue) ;

   return(regValue) ;
}


/******************************************************************************
*  sw21553_PhySetFixSpeed()
*     set the fixed speed and duplex settings used without auto-neg
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     newSpeed: new speed and duplex properies to set
*                  - 001B: 10Base-T half-duplex
*                  - 101B: 10Base-T full-duplex
*                  - 010B: 100Base-TX half-duplex
*                  - 110B: 100Base-TX full-duplex
*
*  Returns:
*     ERROR: in case of invalid selection
*     OK:    
*
*  Example: 
*
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*
*/
UINT32 sw21553_PhySetFixSpeed(int portSelect, UINT32 newSpeed)
{
   UINT32   regValue ;

   if ( (regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR)) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   /* clear reset bit */
   regValue &= ~(INCA_IP_PHY_REG0_BCR_RESET_MASK) ;

   if (newSpeed & sw21553_PHY_SPEED_100_BIT)
      /* set PHY Reg0:Bit13 - set 100 MBit/S */
      regValue |= INCA_IP_PHY_REG0_BCR_SS_MASK ;
   else
      if (newSpeed & sw21553_PHY_SPEED_10_BIT)
         /* clear PHY Reg0:Bit13 - set 10 MBit/s */
         regValue &= ~(INCA_IP_PHY_REG0_BCR_SS_MASK) ;
      /*
      else
         no speed setting given --> keep current 
      */

   if (newSpeed & sw21553_PHY_SPEED_DUPLEX_BIT)
      /* set PHY Reg0:Bit8 - set full duplex */
      regValue |= INCA_IP_PHY_REG0_BCR_DM_MASK ;
   else
      /* clear PHY Reg0:Bit8 - set half duplex */
      regValue &= ~(INCA_IP_PHY_REG0_BCR_DM_MASK) ;

   if ( sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   return(OK) ;
}

/******************************************************************************
*  sw21553_PhyDisableAutoNeg()
*     disable / enable auto negotiation
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     disableAutoNeg: enable / disable auto negotiation (PHY reg 0)
*                  - 1: disable auto neg.
*                  - 0: enable auto neg.
*
*  Returns:
*     ERROR: in case of invalid selection
*     OK:    
*
*
*/
UINT32 sw21553_PhyDisableAutoNeg(int portSelect, int disableAutoNeg)
{
   UINT32   regValue ;

   if ( (regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR)) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   /* clear reset bit */
   regValue &= ~(INCA_IP_PHY_REG0_BCR_RESET_MASK) ;

   if (disableAutoNeg == 1)
      /* clear PHY Reg0:Bit12 - disable auto negotiation */
      regValue &= ~(INCA_IP_PHY_REG0_BCR_ANE_MASK) ;
   else
      /* set PHY Reg0:Bit12 - enable auto negotiation */
      regValue |= INCA_IP_PHY_REG0_BCR_ANE_MASK ;

   if ( sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   return(OK) ;
}

/******************************************************************************
*  sw21553_PhyRestartAutoNeg()
*     restart the auto negotiation for the selected PHY
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*
*  Returns:
*     ERROR: in case of invalid selection or restart not possible
*     OK:
*
*  REMARK:
*
*/
int sw21553_PhyRestartAutoNeg(int portSelect)
{
   UINT32 regValue ;

   /* read current register values */
   regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR) ;

   /* clear reset bit */
   regValue &= ~(INCA_IP_PHY_REG0_BCR_RESET_MASK) ;

   /* check if auto neg. is enabled and update restart auto neg. bit */
   if ( regValue & INCA_IP_PHY_REG0_BCR_ANE_MASK)
   {
      /* auto negotiation is enabled --> do restart */
      regValue |= INCA_IP_PHY_REG0_BCR_RAN_MASK ;
      regValue = sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) ;
      if (regValue != ERROR)
         return(OK) ;
   }
   else
   {
      return(ERROR) ;
   }

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_PhyReset()
*     reset the selected PHY
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*
*  Returns:
*     ERROR: in case of invalid selection
*     OK:
*
*  REMARK:
*
*/
int sw21553_PhyReset(int portSelect)
{
   UINT32 regValue ;

   /* set reset bit */
   regValue = INCA_IP_PHY_REG0_BCR_RESET_MASK ;

   regValue = sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) ;
   if (regValue != ERROR)
      return(OK) ;

   return(ERROR) ;
}
#endif


/******************************************************************************
   Switch ACCESS
******************************************************************************/

/******************************************************************************
*  sw21553_SwIOUnDest()
*     set and change the UN_DEST register
*     The current UN_DEST register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the UN_DEST register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*     setFlag:     indicates get or set
*                  0: read the current register value
*                  1: set the new register value
*
*  Returns:
*     new value of the UN_DEST register
*
*  REMARK:
*     reserved bit 1 must be written with 0
*/
UINT32 sw21553_SwIOUnDest(UINT32 newSettings, UINT32 updMask, int setFlag)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current UN_DEST register value */
   SW_READ_REG(INCA_IP_Switch_UN_DEST, regValue) ;

   if (setFlag == 1)
   {
      /* clear all reserved and update bits */
      regValue &= (~updMask & ~INCA_IP_Switch_UN_DEST_RESERVED_MASK ) ;
      /* set new settings */
      regValue |= ( newSettings & updMask & ~INCA_IP_Switch_UN_DEST_RESERVED_MASK ) ;

      /* write new configuration */
      SW_WRITE_REG(INCA_IP_Switch_UN_DEST,regValue) ;
   }

   restore_flags(flags);
   
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwUpdUnDestPorts()
*     set and change the UN_DEST register
*     The current UN_DEST register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     portBehaviour: 3 bit field
*                 forwarding of unknown packets for broadcast/multicast/unicast packets
*                 0: do not forward a unknown packet 
*                 1: forward a unknown packet 
*
*     3 bit field:
*     ============
*        Bit 0: unicast packets
*        Bit 1: multicast packets
*        Bit 2: broadcast packets
*
*     Example: forwarding unknown packets to <portSelect> port
*        portBehaviour = 0x5 = 100b
*                              |||---> do not forward unknown unicast packets 
*                              ||----> do not forward unknown multicast packets 
*                              |-----> do     forward unknown broadcast packets 
*
*  Returns:
*     new value of the UN_DEST register
*
*  REMARK:
*     reserved bit 1 must be written with 0
*/
UINT32 sw21553_SwUpdUnDestPorts(int portSelect, int portBehaviour)
{
    /* portBehaviour -> UN_DEST-Register Mapping
       Uni    (Bit0) -> Bit 0, 1, 2 (PC, LAN, CPU)
       Multi  (BIT1) -> Bit 3, 4, 5 (PC, LAN, CPU)
       Broad  (Bit2) -> Bit 6, 7, 8 (PC, LAN, CPU) */
    static int portBehaviourMap[8] = { 
        0x00, 0x01, 0x08, 0x09, 0x40, 0x41, 0x48, 0x49 };

   /* check parameter range */
   if ( portBehaviour < INCA_IP_Switch_UN_DEST_FORWARD_NO || 
        portBehaviour > INCA_IP_Switch_UN_DEST_FORWARD_ALL    )
      return(ERROR) ;

   portBehaviour = portBehaviourMap[portBehaviour] << portSelect;

   switch(portSelect)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
         return(sw21553_SwIOUnDest( portBehaviour, INCA_IP_Switch_UN_DEST_PC_PORT_MASK, 1 ) ) ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
         return(sw21553_SwIOUnDest( portBehaviour, INCA_IP_Switch_UN_DEST_LAN_PORT_MASK, 1 ) ) ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU:
         return(sw21553_SwIOUnDest( portBehaviour, INCA_IP_Switch_UN_DEST_CPU_PORT_MASK, 1 ) ) ;

      default:
         return(ERROR) ;
   }        /* switch(portSelect) {...} */

   return(ERROR) ;
}

/******************************************************************************
*  sw21553_SwIOSpPt()
*     set and change the SP_PT register (Spanning Tree Port Status)
*     The current SP_PT register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the SP_PT register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*     setFlag:     indicates get or set
*                  0: read the current register value
*                  1: set the new register value
*
*  Returns:
*     new value of the SP_PT register
*
*  REMARK:
*     reserved bit 1 must be written with 0
*/
UINT32 sw21553_SwIOSpPt(UINT32 newSettings, UINT32 updMask, int setFlag)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current SP_PT register value */
   SW_READ_REG(INCA_IP_Switch_ST_PT, regValue) ;

   if (setFlag == 1)
   {
      /* clear all reserved and update bits */
      regValue &= (~updMask & ~INCA_IP_Switch_SP_PT_RESERVED_MASK ) ;
      /* set new settings */
      regValue |= ( newSettings & updMask & ~INCA_IP_Switch_SP_PT_RESERVED_MASK ) ;

      /* write new configuration */
      SW_WRITE_REG(INCA_IP_Switch_ST_PT,regValue) ;
   }

   restore_flags(flags);
   
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwUpdSpPtPortState()
*     update the port state within the Spanning Tree Port Status 
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     newPortState: possible states are
*                    INCA_IP_Switch_SP_PT_STATE_DISABLED  0
*                    INCA_IP_Switch_SP_PT_STATE_LISTEN    1
*                    INCA_IP_Switch_SP_PT_STATE_LEARN     2
*                    INCA_IP_Switch_SP_PT_STATE_FORWARD   3
*
*  Returns:
*     ERROR: parameter not in range
*     else new value of the SP_PT register
*
*  REMARK:
*     reserved bit 1 must be written with 0
*/
UINT32 sw21553_SwUpdSpPtPortState(int portSelect, int newPortState)
{
   /* check the parameter */
   if ( newPortState < INCA_IP_Switch_SP_PT_STATE_DISABLED || 
        newPortState > INCA_IP_Switch_SP_PT_STATE_FORWARD    )
      return(ERROR) ;

   switch(portSelect)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
         return(sw21553_SwIOSpPt( (newPortState << 0), INCA_IP_Switch_SP_PT_PPS_MASK, 1 ) ) ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
         return(sw21553_SwIOSpPt( (newPortState << 2), INCA_IP_Switch_SP_PT_LPS_MASK, 1 ) ) ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU:
         return(sw21553_SwIOSpPt( (newPortState << 4), INCA_IP_Switch_SP_PT_CPS_MASK, 1 ) ) ;

      default:
         return(ERROR) ;
   }        /* switch(portSelect) {...} */

   return(ERROR) ;
}

/******************************************************************************
*  sw21553_SwIOPacketLen()
*     set the PKT_LEN register (Packet Length Register)
*     The current PKT_LEN register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newPktLen: contains the new maximal packet lenght
*     addTagLen: ADD 4 to MAX_PKT_LEN for Tagged Packets
*                0: MAC_PKT_LEN to be used for tagged and untagged packets
*                1: MAX_PKT_LEN + 4 is used for tagged packets.
*     setFlag:     indicates get or set
*                  0: read the current register value
*                  1: set the new register value
*
*  Returns:
*     new value of the PKT_LEN register
*
*  REMARK:
*     Untagged packets of size greater than MAX_PKT_LEN are
*     classified as Oversized packets. Tagged packets of size
*     greater than MAX_PKT_LEN + 4 are classified as
*     Oversized packets if ADD = 1.
*     Note: The max. programmable packet length is 6A4H
*           (= 1700D bytes).
*/
UINT32 sw21553_SwIOPacketLen(UINT32 newPktLen, int addTagLen, int setFlag)
{
   UINT32 regValue ;

   if (setFlag == 1)
   {
      /* check parameter */
      if (newPktLen < 0 || newPktLen > ~INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN)
         return(ERROR) ;

      /* set new settings */
      regValue = ( (INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_SET(newPktLen)) | 
                   (addTagLen) ? (INCA_IP_Switch_PKT_LEN_ADD): 0 )  ;

      /* write new configuration */
      SW_WRITE_REG(INCA_IP_Switch_PKT_LEN, regValue) ;
   }
   else
   {
      /* read current PKT_LEN register value */
      SW_READ_REG(INCA_IP_Switch_PKT_LEN, regValue) ;
   }

   return(regValue) ;
}



/******************************************************************************
*  sw21553_SwSetArlControl()
*     set and change the address resolution handling (ARL) of the MAC 
*     The current ARL_CTL register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the ARL_CTL register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*
*  Returns:
*     new value of the ARL_CTL register
*
*/
UINT32 sw21553_SwSetArlControl(UINT32 newSettings, UINT32 updMask)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current ARL_CTL register value */
   SW_READ_REG(INCA_IP_Switch_ARL_CTL, regValue) ;

   /* clear all reserved and update bits */
   regValue &= (~updMask & ~INCA_IP_Switch_ARL_CTL_RESERVED_MASK ) ;
   /* set new settings */
   regValue |= ( newSettings & updMask & ~INCA_IP_Switch_ARL_CTL_RESERVED_MASK ) ;

   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_ARL_CTL,regValue) ;

   restore_flags(flags);
   
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwSetCpuPControl()
*     set and change the CPU port control settings
*     The current CPU_PCTL register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the CPU_PCTL register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*
*  Returns:
*     new value of the CPU_PCTL register
*
*/
UINT32 sw21553_SwSetCpuPControl(UINT32 newSettings, UINT32 updMask)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current CPU_PCTL register value */
   SW_READ_REG(INCA_IP_Switch_CPU_PCTL, regValue) ;

   /* clear all reserved and update bits */
   regValue &= (~updMask & ~INCA_IP_Switch_CPU_PCTL_RESERVED_MASK ) ;
   /* set new settings */
   regValue |= ( newSettings & updMask & ~INCA_IP_Switch_CPU_PCTL_RESERVED_MASK ) ;

   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_CPU_PCTL,regValue) ;

   restore_flags(flags);
   
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwSetPMacHdCtl()
*     set and change the PM Header Control Register
*     The current PMAC_HD_CTL register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the PMAC_HD_CTL register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*
*  Returns:
*     new value of the PMAC_HD_CTL register
*
*/
UINT32 sw21553_SwSetPMacHdCtl(UINT32 newSettings, UINT32 updMask)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current PMAC_HD_CTL register value */
   SW_READ_REG(INCA_IP_Switch_PMAC_HD_CTL, regValue) ;

   /* clear all reserved and update bits */
   regValue &= (~updMask & ~INCA_IP_Switch_PMAC_HD_CTL_RESERVED_MASK ) ;
   /* set new settings */
   regValue |= ( newSettings & updMask & ~INCA_IP_Switch_PMAC_HD_CTL_RESERVED_MASK ) ;

   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_PMAC_HD_CTL,regValue) ;

   restore_flags(flags);
   
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwSetMdioConfig()
*     set and change the MDIO_CFG settings
*     The current MDIO_CFG register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the MDIO_CFG register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*
*  Returns:
*     new value of the MDIO_CFG register
*
*  REMARK:
*     reserved bit 1 must be written with 0
*/
UINT32 sw21553_SwSetMdioConfig(UINT32 newSettings, UINT32 updMask)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current MDIO_CFG register value */
   SW_READ_REG(INCA_IP_Switch_MDIO_CFG, regValue) ;

   /* clear all reserved and update bits */
   regValue &= (~updMask & ~INCA_IP_Switch_MDIO_CFG_RESERVED_MASK ) ;
   /* set new settings */
   regValue |= ( newSettings & updMask & ~INCA_IP_Switch_MDIO_CFG_RESERVED_MASK ) ;

   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_MDIO_CFG,regValue) ;

   restore_flags(flags);
   
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwSetEPhy()
*     set and change the EPHY register (Ethernet PHY Register)
*     The current EPHY register will be read and updated only with
*     the values marked in the updMask parameter.
*
*  Parameter:
*     newSettings: contains the new settings for the SP_PT register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*
*  Returns:
*     new value of the EPHY register
*
*  REMARK:
*     reserved bit 1 must be written with 0
*/
UINT32 sw21553_SwSetEPhy(UINT32 newSettings, UINT32 updMask)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);
   
   /* read current EPHY register value */
   SW_READ_REG(INCA_IP_Switch_EPHY, regValue) ;

   /* clear all reserved and update bits */
   regValue &= (~updMask & ~INCA_IP_Switch_EPHY_RESERVED_MASK ) ;
   /* set new settings */
   regValue |= ( newSettings & updMask & ~INCA_IP_Switch_EPHY_RESERVED_MASK ) ;

   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_EPHY,regValue) ;

   restore_flags(flags);
   
   return(regValue) ;
}


/******************************************************************************
*  sw21553_SwResetMDIO()
*     MDIO Configuration : reset MDIO state maschine
*
*  Parameter:
*
*  Returns:
*     MDIO_CFG register settings (reset value)
*
*  REMARK:
*     This reset bit is self clearing, i.e. it resets again to 0 after
*     the reset procedure is completed.
*/
UINT32 sw21553_SwResetMDIO()
{
   UINT32 regValue = 0 ;

   /* reset MDIO */
   sw21553_SwSetCpuPControl( (UINT32)INCA_IP_Switch_MDIO_CFG_RST, 
                             (UINT32)INCA_IP_Switch_MDIO_CFG_RST) ;

   /* read the new register value (reset value) for return */
   SW_READ_REG( INCA_IP_Switch_MDIO_CFG,regValue) ;

   return(regValue) ;
}


/******************************************************************************
*  sw21553_SwCpuPCtlEnableDa()
*     CPU Port Control: enable DA lookup for the the CPU port
*
*  Parameter:
*
*  Returns:
*     CPU_PCTL register settings
*
*  REMARK:
*     DA_PORT and DAC fields are not used if DA lookup is enabled
*/
UINT32 sw21553_SwCpuPCtlEnableDa()
{
   return ( sw21553_SwSetCpuPControl( (UINT32)INCA_IP_Switch_CPU_PCTL_EDL_ENABLED, 
                                      (UINT32)INCA_IP_Switch_CPU_PCTL_EDL_ENABLED) ) ;
}

/******************************************************************************
*  sw21553_SwCpuPCtlDisableDa()
*     CPU Port Control: disable DA lookup for the the CPU port
*     If DA lookup is disabled, the destination will base on the DA_PORT field
*     and the priority will base on the DAC field.
*
*  Parameter:
*     newDaPort: new DA_PORT value to set
*           -1 --> ignore this parameter (keep current CPU_PCTL:DA_PORT)
*           0 (INCA_IP_API_CPU_PCTL_DA_PORTS_DROP)  : packet dropped
*           1 (INCA_IP_API_CPU_PCTL_DA_PORTS_PC)    : packet forward to PC
*           2 (INCA_IP_API_CPU_PCTL_DA_PORTS_LAN)   : packet forward to LAN
*           3 (INCA_IP_API_CPU_PCTL_DA_PORTS_LAN_PC): packet forward to both
*
*     newDAC:    new DAC value to set
*                -1 --> ignore this parameter (keep current CPU_PCTL:DAC)
*                0 (INCA_IP_API_CPU_PCTL_DAC_LOW) : forward with low priority
*                1 (INCA_IP_API_CPU_PCTL_DAC_HIGH): forward with high prioirty
*  Returns:
*     CPU_PCTL register settings
*
*  REMARK:
*     DA_PORT and DAC fields are not used if DA lookup is enabled
*/
UINT32 sw21553_SwCpuPCtlDisableDa(int newDaPort, int newDAC)
{
   UINT32 regValue = 0, updMask = 0 ;

   /* define updMask */
   updMask |= (newDaPort == -1) ? 0 : INCA_IP_Switch_CPU_PCTL_DA_PORTS_MASK ;
   updMask |= (newDAC == -1)    ? 0 : INCA_IP_Switch_CPU_PCTL_DAC_MASK ;

   /* check range */
   if ( newDaPort >= INCA_IP_API_CPU_PCTL_DA_PORTS_DROP && 
        newDaPort <= INCA_IP_API_CPU_PCTL_DA_PORTS_LAN_PC  )
      regValue = (INCA_IP_Switch_CPU_PCTL_DA_PORTS_SET(newDaPort)) ;
   else
      if (newDaPort != -1)
         return(ERROR) ;

   if ( newDAC == INCA_IP_API_CPU_PCTL_DAC_LOW && 
        newDAC == INCA_IP_API_CPU_PCTL_DAC_HIGH  )
      regValue |= INCA_IP_Switch_CPU_PCTL_DAC ;
   else
      if (newDAC != -1)
         return(ERROR) ;

   return( sw21553_SwSetCpuPControl( regValue, updMask) ) ;
}


/******************************************************************************
*  sw21553_SwSetPMacMacAddr()
*     set the given MAC address to the PMAC
*
*/
void sw21553_SwSetPMacMacAddr( UCHAR inAddr[6] )
{
   sw21553_CdaRegister  regValue ;

   regValue.reg32 = 0 ;

   regValue.reg8[2] = inAddr[0] ;
   regValue.reg8[3] = inAddr[1] ;
   SW_WRITE_REG (INCA_IP_Switch_PMAC_SA1,regValue.reg32);

   regValue.reg8[0] = inAddr[2] ;
   regValue.reg8[1] = inAddr[3] ;
   regValue.reg8[2] = inAddr[4] ;
   regValue.reg8[3] = inAddr[5] ;
   SW_WRITE_REG (INCA_IP_Switch_PMAC_SA2,regValue.reg32);

   return ;
}

/******************************************************************************
*  sw21553_SwGetPMacMacAddr()
*     read and return PMAC MAC address
*  Parameter:
*     pMacAddr: ptr to MAC address array
*
*  Returns:
*     0: OK
*    -1: ERROR ptr to MAC address array not valid 
*
*  REMARK:
*     the user must provide a large enough array (6 byte)
*
*/
int sw21553_SwGetPMacMacAddr(UCHAR *pMacAddr)
{
   sw21553_CdaRegister  reg1, reg2 ;

   if (pMacAddr == NULL)
      return(ERROR) ;

   /* read current SP_PT register value */
   SW_READ_REG(INCA_IP_Switch_PMAC_SA1, reg1.reg32) ;

   /* read current SP_PT register value */
   SW_READ_REG(INCA_IP_Switch_PMAC_SA2, reg2.reg32) ;

   pMacAddr[0] = reg1.reg8[2] ;
   pMacAddr[1] = reg1.reg8[3] ;
   pMacAddr[2] = reg2.reg8[0] ;
   pMacAddr[3] = reg2.reg8[1] ;
   pMacAddr[4] = reg2.reg8[2] ;
   pMacAddr[5] = reg2.reg8[3] ;

   return(OK) ;
}

/******************************************************************************
*  sw21553_SwMacReadTableEntry()
*     read the MAC entry from the MAC table
*
*  Parameter:
*     index: table index of the requested entry
*     pOutMacTableEntry: ptr to the return structure
*
*  Returns:
*     0: no entry found at this index
*     1: one entry found and returned
*    -1: ERROR ptr or index not valid 
*
*  REMARK:
*     The control parameter indicates the validation of the entry
*     concerning aging and port assignment
*/
int sw21553_SwMacReadTableEntry(int index, sw21553_MacTableEntry *pOutMacTableEntry )
{
   UINT32 readCmd ;
   sw21553_CdaRegister  CADReg1, CADReg2 ;

   if ( 
        (pOutMacTableEntry == NULL) ||
        (index < 0) || 
        (index >= INCA_IP_MAX_MAC_TABLE_ENTRIES) 
      )
   {
      return(ERROR) ;
   }

   readCmd = INCA_IP_Switch_CPU_ACTL_RD_MA_TABLE | index ;
   /* MAC table: location 0 */
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, readCmd ) ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, readCmd) ;
      if ( (readCmd & 0x80000000) == 0 )
         break ;
   }

   SW_READ_REG(INCA_IP_Switch_DATA1, CADReg1.reg32) ;
   SW_READ_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;

   if ( (CADReg1.reg32 != 0x00FFFFFF) || (CADReg2.reg32 != 0xFFFFFFFF) )
   {
      /* entry found --> transfere data */
      pOutMacTableEntry->Cntrl = CADReg1.reg8[1] ;
      pOutMacTableEntry->MacAddr[0] = CADReg1.reg8[2] ;
      pOutMacTableEntry->MacAddr[1] = CADReg1.reg8[3] ;
      pOutMacTableEntry->MacAddr[2] = CADReg2.reg8[0] ;
      pOutMacTableEntry->MacAddr[3] = CADReg2.reg8[1] ;
      pOutMacTableEntry->MacAddr[4] = CADReg2.reg8[2] ;
      pOutMacTableEntry->MacAddr[5] = CADReg2.reg8[3] ;
      return(1) ;
   }

   return(0) ;
}

/******************************************************************************
*  sw21553_SwMacWriteTableEntry()
*     read the MAC entry from the MAC table
*
*  Parameter:
*     index: table index where the MAC entry will be written
*     inMacTableEntry: MAC table entry
*
*  Returns:
*     0 (OK):    write of the new entry was successful
*    -1 (ERROR): write of the new entry was not successful
*
*  REMARK:
*     Index must be in range of valid indexes
*     The given MAC table entry must contain valid data
*/
LOCAL int sw21553_SwMacWriteTableEntry(int index, sw21553_MacTableEntry inMacTableEntry )
{
   UINT32 writeCmd, count ;
   sw21553_CdaRegister  CADReg1, CADReg2 ;

   CADReg1.reg32 = 0 ;
   CADReg2.reg32 = 0 ;
   CADReg1.reg8[1] = inMacTableEntry.Cntrl ;
   CADReg1.reg8[2] = inMacTableEntry.MacAddr[0] ;
   CADReg1.reg8[3] = inMacTableEntry.MacAddr[1] ;

   CADReg2.reg8[0] = inMacTableEntry.MacAddr[2] ;
   CADReg2.reg8[1] = inMacTableEntry.MacAddr[3] ;
   CADReg2.reg8[2] = inMacTableEntry.MacAddr[4] ;
   CADReg2.reg8[3] = inMacTableEntry.MacAddr[5] ;

   /* write new MAC entry to the CPU access register */
   SW_WRITE_REG(INCA_IP_Switch_DATA1, CADReg1.reg32) ;
   SW_WRITE_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;

   /* set write commend for the MAC table */
   writeCmd = INCA_IP_Switch_CPU_ACTL_WR_MA_TABLE | (UINT32)index ;
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, writeCmd ) ;

   /* check write */
   count = 0 ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, writeCmd) ;
      if ( (writeCmd & 0x80000000) == 0 || count > 10)
         break ;

      count++ ;
   }        /* while(1) {...} */

   if ( (writeCmd & 0x80000000) == 0)
      return(OK) ;
   else
      return(ERROR) ;
}


/*
   convert an given MAC address string into a byte stream
*/
void sw21553_SwMacConvertStr2Hex(char *strMacAddr, UCHAR *pRetHexMacAddr)
{
   int byte;
   char c1, c2;

   if (strMacAddr != NULL && pRetHexMacAddr != NULL)
   {
       for (byte = 0; byte < 6; byte++)
       {
           /* we assume legal hex-values with upper or lower case letters */
           /* (c & ~20) converts a lower case letter to an upper case letter */
           c1 = strMacAddr[3 * byte + 0];
           c2 = strMacAddr[3 * byte + 1];
           pRetHexMacAddr[byte] = 
               (((c1 >= 'A') ? ((c1 & ~0x20) - 'A' + 10) : (c1 - '0')) << 4) |
               (((c2 >= 'A') ? ((c2 & ~0x20) - 'A' + 10) : (c2 - '0')) << 0);
       }
   }
   return ;
}


/******************************************************************************
*  sw21553_SwMacSetTableEntryCntrl()
*     interprete and set the given properties for the control byte within
*     a MAC table entry.
*
*  Parameter:
*     port:       defines the port to which the address is assigned
*                 possible values are:
*                 0: PC port
*                 1: LAN port
*                 2: CPU port VOICE
*                 3: CPU port MANAGEMENT
*     classification: calssify the frames
*                 0: packets will be classified as normal packets
*                 1: packets will be classified as critical packets
*     aging:      defines the aging behaviour of the entry
*                 0: no aging --> static entry into the MAC table
*                 1: normal aging --> entry will normal timed out
*     pCntrl:     ptr to the MAC entry control byte
*
*  Returns:
*     0 (OK):    given values are valid and control byte was set
*    -1 (ERROR): error while interpretation of the control byte
*                - no ptr for return of the value given
*                - port assignment not valid
*
*  REMARK:
*
*/
int sw21553_SwMacSetTableEntryCntrl(int port, int classification, int aging, UCHAR *pCntrl)
{
   UCHAR tempCntrl = 0;

   if (pCntrl == NULL)
      return(ERROR) ;
   /* check and set assigned port */
   switch(port)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
         tempCntrl = (UCHAR)INCA_IP_MAC_TABLE_ENTRY_PORT_PC_MASK ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
         tempCntrl = (UCHAR)INCA_IP_MAC_TABLE_ENTRY_PORT_LAN_MASK ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU_V:
         tempCntrl = (UCHAR)INCA_IP_MAC_TABLE_ENTRY_PORT_CPU_V_MASK ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU_M:
         tempCntrl = (UCHAR)INCA_IP_MAC_TABLE_ENTRY_PORT_CPU_M_MASK ;
         break ;

      default:
         return(ERROR) ;
   }        /* switch(port) {...} */

   tempCntrl |= (UCHAR)( (classification) ? (INCA_IP_MAC_TABLE_ENTRY_CRITICAL_MASK):(INCA_IP_MAC_TABLE_ENTRY_NO_CRITICAL_MASK) ) ;
   tempCntrl |= (UCHAR)( (aging) ? (INCA_IP_MAC_TABLE_ENTRY_MA_ST_NORMAL_AGING_MASK):(INCA_IP_MAC_TABLE_ENTRY_MA_ST_STATIC_MASK) ) ;

   /* set return value */
   *pCntrl = tempCntrl ;

   return(OK) ;
}

/******************************************************************************
*  sw21553_AddMacEntry()
*     Add an given MAC table entry to the MAC table.
*     The MAC table index will be calculated via the hash algorithm
*
*  Parameter:
*     inMacTableEntry: MAC address and appropriate control structures
*     overWrite:       add this entry always (overwrite existing)
*
*  Returns:
*     0 (OK):    add of the new entry was successful
*    -1 (ERROR): add of the new entry was not successful, 
*                calculated entries not free
*
*  REMARK:
*     The given MAC table entry must contain valid and already complete data
*/
int sw21553_SwMacAddEntry(sw21553_MacTableEntry inMacTableEntry, int overWrite)
{
   UCHAR  macTableHash ;
   UINT   modulateNum = 0 ;
   sw21553_MacTableEntry tempMacTableEntry ;

   for (modulateNum = 0 ; modulateNum <= 3 ; modulateNum++)
   {
      /* 
         calculate the MAC table index via hash algorithm 
         REMEBER: init of the polynome has to be done before 
                  --> sw21553_IncaIpInitHashGenPoly() ;
      */
      macTableHash = sw21553_IncaIpSwitchCalcHash(modulateNum, inMacTableEntry.MacAddr) ;

      if (overWrite == 0)
      {
         /* check if this entry is free */
         if ( sw21553_SwMacReadTableEntry(macTableHash, &tempMacTableEntry) == 1 )
         {
            /* table entry bussy --> check if this entry is still valid */
            if ( (tempMacTableEntry.Cntrl & (UCHAR)INCA_IP_MAC_TABLE_ENTRY_MA_ST_AGED_OUT_MASK) == 
                 INCA_IP_MAC_TABLE_ENTRY_MA_ST_AGED_OUT_MASK  )
            {
               /* entry is valid --> write entry */
               return (sw21553_SwMacWriteTableEntry((int)macTableHash, inMacTableEntry) ) ;
            }
         }
         else
         {
            /* table entry free --> write entry */
            return (sw21553_SwMacWriteTableEntry((int)macTableHash, inMacTableEntry) ) ;
         }
      }
      else
      {
         /* write always */
         return (sw21553_SwMacWriteTableEntry((int)macTableHash, inMacTableEntry) ) ;
      }
   }        /* for (modulateNum = 0 ; modulateNum < 3 ; modulateNum++) {...} */

   /* no free entry found */
   return(ERROR) ;
}

/******************************************************************************
*  sw21553_SwMacClearEntryByIndex()
*     Clear an given MAC table entry from the MAC table identified by an index
*     The MAC table index must be known by the caller.
*
*  Parameter:
*     index: index of the MAC table entry
*
*  Returns:
*     
*
*  REMARK:
*     Via modulation the are 4 possible MAC entry indexes for one MAC address
*     The index of the entry must be known by the caller.
*/
int sw21553_SwMacClearEntryByIndex(int index)
{
   sw21553_MacTableEntry tempMacTableEntry ;

   /* read the calculated index and check if the MAC address match */
   if ( sw21553_SwMacReadTableEntry(index, &tempMacTableEntry) == 1 )
   {
      /* entry found --> clear */
      tempMacTableEntry.Cntrl |= INCA_IP_MAC_TABLE_ENTRY_MA_ST_AGED_OUT_MASK ;

      /* write back the previous read entry with cleared control byte */
      sw21553_SwMacWriteTableEntry(index, tempMacTableEntry) ;

      return(OK) ;
   }

   /* no valid entry for this index */
   return(ERROR) ;
}

/******************************************************************************
*  sw21553_SwMacClearEntryByMac()
*     Clear MAC table entries from the MAC table identified by a given
*     MAC address. The MAC table index will be calculated via the hash algorithm
*
*  Parameter:
*     inMacTableEntry: MAC address 
*                      control (0 --> clear all entries, else clear only marked ports)
*
*  Returns:
*     number of cleared entries within the MAC table
*
*  REMARK:
*     Via modulation there are 4 possible MAC entry indexes for one MAC address
*     This functions checks all 4 possibilities for clearance.
*/
int sw21553_SwMacClearEntryByMac(sw21553_MacTableEntry inMacTableEntry)
{
   UCHAR  macTableHash ;
   UINT   modulateNum = 0 ;
   int    count = 0 ;
   sw21553_MacTableEntry tempMacTableEntry ;

   for (modulateNum = 0 ; modulateNum <= 3 ; modulateNum++)
   {
      /* 
         calculate the MAC table index via hash algorithm 
         REMEBER: init of the polynome has to be done before 
                  --> sw21553_IncaIpInitHashGenPoly() ;
      */
      macTableHash = sw21553_IncaIpSwitchCalcHash(modulateNum, inMacTableEntry.MacAddr) ;

      /* read the calculated index and check if the MAC address match */
      if ( sw21553_SwMacReadTableEntry((int)macTableHash, &tempMacTableEntry) == 1 )
      {
         if( ( memcmp(tempMacTableEntry.MacAddr, inMacTableEntry.MacAddr, 6) ) == 0 )
         {
            /* MAC addresses are equal --> clear entry */
            if ( (inMacTableEntry.Cntrl & INCA_IP_MAC_TABLE_ENTRY_ALL_PORT_MASK) == 0)
            {
               /* no port set: clear always */
               tempMacTableEntry.Cntrl |= INCA_IP_MAC_TABLE_ENTRY_MA_ST_AGED_OUT_MASK ;
               /* write back the previous entry with cleared control byte */
               sw21553_SwMacWriteTableEntry((int)macTableHash, tempMacTableEntry) ;
               count++ ;
            }
            else
            {
               /* check for given ports only */
               if ( (inMacTableEntry.Cntrl & 
                     tempMacTableEntry.Cntrl &
                     INCA_IP_MAC_TABLE_ENTRY_ALL_PORT_MASK) != 0 
                  )
               {
                  /* no port set: clear always */
                  tempMacTableEntry.Cntrl |= INCA_IP_MAC_TABLE_ENTRY_MA_ST_AGED_OUT_MASK ;
                  /* write back the previous entry with cleared control byte */
                  sw21553_SwMacWriteTableEntry((int)macTableHash, tempMacTableEntry) ;
                  count++ ;
               }
            }        /* if ( (inMacTableEntry.Cntrl & INCA_IP_MAC_TABLE_ENTRY_ALL_PORT_MASK) == 0) {...} else {...} */
         }        /* if( ( memcmp(tempMacTableEntry.MacAddr, inMacTableEntry.MacAddr, 6) ) == 0 ) {...} */
      }        /* if ( sw21553_SwMacReadTableEntry((int)macTableHash, &tempMacTableEntry) == 1 ) {...} */
   }        /* for (modulateNum = 0 ; modulateNum < 3 ; modulateNum++) {...} */

   /* number of cleared entries */
   return(count) ;
}

/******************************************************************************
*  sw21553_SwMacResetMacTable()
*     reset the complete MAC table (write 0x00FFFFFF 0xFFFFFFFF --> unused)
*
*  Returns:
*     0 (OK):    reset was successful
*     -1 ERROR:  parameter ERROR or ERROR while reset write
*
*/
int sw21553_SwMacResetMacTable()
{
   int      index, count ;
   UINT32   writeCmd ;
/*     UINT32   currentLevel; */
   unsigned long flags;

   /* block all while write */
/*     currentLevel = intLock(); */
   save_and_cli(flags);

   /* write reset MAC entry to the CPU access register */
   SW_WRITE_REG(INCA_IP_Switch_DATA1, 0x00FFFFFF) ;
   SW_WRITE_REG(INCA_IP_Switch_DATA2, 0xFFFFFFFF) ;

   for (index = 0; index < INCA_IP_MAX_MAC_TABLE_ENTRIES; index++)
   {
      /* set write commend for the MAC table */
      writeCmd = INCA_IP_Switch_CPU_ACTL_WR_MA_TABLE | (UINT32)index ;
      SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, writeCmd ) ;

      count = 0;                /* reset timeout counter */
      while(1)
      {
         SW_READ_REG(INCA_IP_Switch_CPU_ACTL, writeCmd) ;
         if ( (writeCmd & 0x80000000) == 0 || count > 10)
            break ;

         count++ ;
      }        /* while(1) {...} */

      if ( (writeCmd & 0x80000000) != 0)
      {
/*           intUnlock(currentLevel) ; */
          restore_flags(flags);
         return(ERROR) ;
      }
   }

/*     intUnlock(currentLevel) ; */
   restore_flags(flags);
   return(OK) ;
}

/* control sturct for INCA-IP VLAN and CoS */
sw21553_IncaIpSwControl sw21553_VlanCoSCtrl ;

/******************************************************************************
*  sw21553_SwVLanInitControl()
*     init the local VLAN control struct with the default values ;
*
*  Parameter:
*     pDefVLanCfg: ptr to the VLAN control structure 
*
*  Returns:
*     NULL: ERROR, invalid parameter
*     else: returns the given parameter ptr "pDefVLanCfg"
*
*  REMARK:
*
*/
LOCAL void sw21553_SwVLanInitControl(void)
{
   /* reset / set local VLAN table */
   sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount = 0 ;
   sw21553_VlanCoSCtrl.vLanCntl.vLanTableCurrentMaxId = 0 ;
   memset( sw21553_VlanCoSCtrl.vLanCntl.vLanTable, 
           0x00, sizeof(sw21553_VlanCoSCtrl.vLanCntl.vLanTable) ) ;

   return ;
}

/******************************************************************************
*  sw21553_SwVLanSetVLanConfig()
*     set new "PRIO" and "VALN_ID" for the selected port
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*
*     newPrio:    new priority value to set (range 0 - 7)
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*
*  Returns:
*     ERROR: invalid parameter (out of range)
*     else : new register settings have been set
*
*  REMARK:
*
*/
UINT32 sw21553_SwVLanSetVLanConfig(int portSelect, UINT32 newPrio, UINT32 newVLanId)
{
   UINT32 regValue ;

   /* check range of parameters */
   if (newPrio > INCA_IP_VLAN_MAX_PRIO || newVLanId > INCA_IP_VLAN_MAX_VLAN_ID)
   {
      /* parameter not in range */
      return(ERROR) ;
   }

   switch(portSelect)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
         /* set new value: clear all reserved and update bits */
         regValue = ( ~INCA_IP_Switch_XXX_VLAN_RESERVED_MASK &
                     (INCA_IP_Switch_PC_VLAN_VLAN_ID_SET(newVLanId)|
                      INCA_IP_Switch_PC_VLAN_PRI_SET(newPrio) ) );
         /* write PC_VLAN register */
         SW_WRITE_REG( INCA_IP_Switch_PC_VLAN, regValue ) ;
         return(regValue) ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
         /* set new value: clear all reserved and update bits */
         regValue = ( ~INCA_IP_Switch_XXX_VLAN_RESERVED_MASK &
                     (INCA_IP_Switch_LAN_VLAN_VLAN_ID_SET(newVLanId)|
                      INCA_IP_Switch_LAN_VLAN_PRI_SET(newPrio) ) );
         /* write LAN_VLAN register */
         SW_WRITE_REG( INCA_IP_Switch_LAN_VLAN, regValue ) ;
         return(regValue) ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU:
         /* set new value: clear all reserved and update bits */
         regValue = ( ~INCA_IP_Switch_XXX_VLAN_RESERVED_MASK &
                      (INCA_IP_Switch_CPU_VLAN_VLAN_ID_SET(newVLanId)|
                       INCA_IP_Switch_CPU_VLAN_PRI_SET(newPrio) ) );
         /* write CPU_VLAN register */
         SW_WRITE_REG( INCA_IP_Switch_CPU_VLAN, regValue ) ;
         return(regValue) ;
         break ;

      default:
         /* invalid port selection */
         return(ERROR) ;
   }

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_SwVLanGetVLanConfig()
*     read the port specific VLAN register settings
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*
*  Returns:
*     ERROR: invalid parameter (out of range)
*     else : current register value
*
*  REMARK:
*
*/
UINT32 sw21553_SwVLanGetVLanConfig(int portSelect)
{
   UINT32 regValue ;

   switch(portSelect)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
         /* read PC_VLAN register */
         SW_READ_REG( INCA_IP_Switch_PC_VLAN, regValue ) ;
         return(regValue) ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
         /* read LAN_VLAN register */
         SW_READ_REG( INCA_IP_Switch_LAN_VLAN, regValue ) ;
         return(regValue) ;
         break ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU:
         /* read CPU_VLAN register */
         SW_READ_REG( INCA_IP_Switch_CPU_VLAN, regValue ) ;
         return(regValue) ;
         break ;

      default:
         /* invalid port selection */
         return(ERROR) ;
   }

   return(ERROR) ;
}

/******************************************************************************
*  sw21553_SwVLanIOVLanCntrl()
*     get or set and change the VLAN_CTRL register (VLAN Control Register)
*
*  Parameter:
*     newSettings: contains the new settings for the VLAN_CTRL register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*     setFlag:     indicates get or set
*                  0: read the current register value
*                  1: set the new register value
*
*  Returns:
*     new value of the VLAN_CTRL register
*
*  REMARK:
*
*/
UINT32 sw21553_SwVLanIOVLanCntrl(UINT32 newSettings, UINT32 updMask, int setFlag)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
   save_and_cli(flags);

   /* read current VLAN_CTRL register value */
   SW_READ_REG(INCA_IP_Switch_VLAN_CTRL, regValue) ;

   if (setFlag == 1)
   {
      /* clear all reserved and update bits */
      regValue &= (~updMask & ~INCA_IP_Switch_VLAN_CTRL_RESERVED_MASK ) ;
      /* set new settings */
      regValue |= ( newSettings & updMask & ~INCA_IP_Switch_VLAN_CTRL_RESERVED_MASK ) ;

      /* write new configuration */
      SW_WRITE_REG(INCA_IP_Switch_VLAN_CTRL,regValue) ;
   }

   restore_flags(flags);
   return(regValue) ;
}

/******************************************************************************
*  sw21553_SwVLanReadTableEntry()
*     read the VLAN entry from the VLAN table
*
*  Parameter:
*     index: table index of the requested entry
*
*  Returns:
*     0: no entry found at this index
*    -1: ERROR index not valid 
*     else: VLAN table entry
*
*/
UINT32 sw21553_SwVLanReadTableEntry(int index)
{
   UINT32 readCmd ;
   sw21553_CdaRegister  CADReg2 ;

   if ( (index < 0) || (index >= INCA_IP_MAX_VLAN_TABLE_ENTRIES) 
      )
   {
      return(ERROR) ;
   }

   /* set read command */
   readCmd = INCA_IP_Switch_CPU_ACTL_RD_VLAN_TABLE | index ;

   /* read from VLAN table via CPU_ACTL */
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, readCmd ) ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, readCmd) ;
      if ( (readCmd & 0x80000000) == 0 )
         break ;
   }

   /* read VLAN table entry */
   SW_READ_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;

   if ( ( /* 0x0007FFFF indicates an unused entry */
          (CADReg2.reg32 & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK) 
                                          != ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK ) &&
        ( /* check valid bit: 0 --> invalid entry */
          (CADReg2.reg32 & INCA_IP_VLAN_TABLE_ENTRY_V_MASK) != 0 )
      )
   {
      /* not a valid entry */
      return( (CADReg2.reg32 & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK) ) ;
   }

   return(0) ;   
}

/******************************************************************************
*  sw21553_SwVLanWriteTableEntry()
*     write a VLAN entry to the VLAN table
*
*  Parameter:
*     index: table index where the VLAN entry will be written
*     inVLanTableEntry: MAC table entry
*
*  Returns:
*     0 (OK):    write of the new entry was successful
*    -1 (ERROR): write of the new entry was not successful
*
*  REMARK:
*     Index must be in range of valid indexes
*     The given VLAN table entry must contain valid data
*/
int sw21553_SwVLanWriteTableEntry(int index, sw21553_VLanTableEntry inVLanTableEntry )
{
   UINT32 writeCmd, count ;
   sw21553_CdaRegister  CADReg2 ;

   if ( (index < 0) || (index >= INCA_IP_MAX_VLAN_TABLE_ENTRIES) )
   {
      return(ERROR) ;
   }

   /* set data */
   CADReg2.reg32 = (INCA_IP_VLAN_TABLE_ENTRY_SET_VLAN_ID(inVLanTableEntry.vLanId)) |
                   (INCA_IP_VLAN_TABLE_ENTRY_SET_CTRL_FLAGS(inVLanTableEntry.Cntrl)) ;

   /* write new VLAN entry via the CPU access register */
   writeCmd = INCA_IP_Switch_CPU_ACTL_WR_VLAN_TABLE | (UINT32)index ;
   SW_WRITE_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, writeCmd ) ;

   /* check write */
   count = 0 ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, writeCmd) ;
      if ( (writeCmd & 0x80000000) == 0 || count > 10)
         break ;

      count++ ;
   }        /* while(1) {...} */

   if ( (writeCmd & 0x80000000) == 0)
      return(OK) ;
   else
      return(ERROR) ;
}

/******************************************************************************
*  sw21553_SwVLanWriteTableEntryReg32()
*     write a VLAN entry (given by his register value) to the VLAN table
*
*  Parameter:
*     index: table index where the VLAN entry will be written
*     newRegEntry: VLAN table entry
*
*  Returns:
*     0 (OK):    write of the new entry was successful
*    -1 (ERROR): write of the new entry was not successful
*
*  REMARK:
*     Index must be in range of valid indexes
*     The given VLAN table entry must contain valid data
*/
int sw21553_SwVLanWriteTableEntryReg32(int index, UINT32 newRegEntry )
{
   UINT32 writeCmd, count ;

   if ( (index < 0) || (index >= INCA_IP_MAX_VLAN_TABLE_ENTRIES) )
   {
      return(ERROR) ;
   }

   /* write new VLAN entry via the CPU access register */
   writeCmd = INCA_IP_Switch_CPU_ACTL_WR_VLAN_TABLE | (UINT32)index ;
   SW_WRITE_REG(INCA_IP_Switch_DATA2, newRegEntry) ;
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, writeCmd ) ;

   /* check write */
   count = 0 ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, writeCmd) ;
      if ( (writeCmd & 0x80000000) == 0 || count > 10)
         break ;

      count++ ;
   }        /* while(1) {...} */

   if ( (writeCmd & 0x80000000) == 0)
      return(OK) ;
   else
      return(ERROR) ;
}


/******************************************************************************
*  sw21553_SwVLanClearTableEntryByIndex()
*     clear a VLAN entry from the VLAN table
*
*  Parameter:
*     index: table index where the VLAN entry will be cleared
*
*  Returns:
*     0 (OK):    index was already cleared
*    -1 (ERROR): clear not possible (index out of range)
*     else:      old VLAN table entry
*
*  REMARK:
*     Index must be in range of valid indexes
*     The given VLAN table entry must contain valid data
*/
int sw21553_SwVLanClearTableEntryByIndex(int index )
{
   UINT32 cdaCmd, count, oldValue ;
   sw21553_CdaRegister  CADReg2 ;

   if ( (index < 0) || (index >= INCA_IP_MAX_VLAN_TABLE_ENTRIES) )
   {
      return(ERROR) ;
   }

   /* read from VLAN table via CPU_ACTL */
   cdaCmd = INCA_IP_Switch_CPU_ACTL_RD_VLAN_TABLE | index ;
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, cdaCmd ) ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, cdaCmd) ;
      if ( (cdaCmd & 0x80000000) == 0 )
         break ;
   }
   /* read VLAN table entry */
   SW_READ_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;
   if ( /* 0x0007FFFF indicates an unused entry */
        (CADReg2.reg32 & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK) 
                                        != INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK )
   {
      return(0) ;
   }

   oldValue = (CADReg2.reg32 & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK) ;

   /* set data */
   CADReg2.reg32 = ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK ;
   /* write new VLAN entry via the CPU access register */
   cdaCmd = INCA_IP_Switch_CPU_ACTL_WR_VLAN_TABLE | (UINT32)index ;
   SW_WRITE_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;
   SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, cdaCmd ) ;
   /* check write */
   count = 0 ;
   while(1)
   {
      SW_READ_REG(INCA_IP_Switch_CPU_ACTL, cdaCmd) ;
      if ( (cdaCmd & 0x80000000) == 0 || count > 10)
         break ;

      count++ ;
   }        /* while(1) {...} */

   if ( (cdaCmd & 0x80000000) == 0)
      return(oldValue) ;
   else
      return(ERROR) ;
}


/******************************************************************************
*  sw21553_SwVLanResetVLanTable()
*     reset the complete VLAN table (write 0x0007FFFF --> unused)
*
*  Parameter:
*
*  Returns:
*     0 (OK):    reset was successful
*     -1 ERROR:  parameter ERROR or ERROR while reset write
*
*/
int sw21553_SwVLanResetVLanTable(void)
{
   int index ;
   UINT32 cdaCmd, count ;
   sw21553_CdaRegister  CADReg2 ;
/*     UINT32 currentLevel; */
   unsigned long flags;
  
   /* block all while reset */
/*     currentLevel = intLock(); */
   save_and_cli(flags);

   /* set data */
   CADReg2.reg32 = ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK ;
   
   /* write reset value only once */   
   SW_WRITE_REG(INCA_IP_Switch_DATA2, CADReg2.reg32) ;

   for(index = (INCA_IP_MAX_VLAN_TABLE_ENTRIES-1) ; index >= 0 ; index--)
   {
      /* reset local VLAN table */
      sw21553_VlanCoSCtrl.vLanCntl.vLanTable[index] = ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK ;

      /* write new VLAN entry via the CPU access register */
      cdaCmd = INCA_IP_Switch_CPU_ACTL_WR_VLAN_TABLE | (UINT32)index ;
      SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, cdaCmd ) ;
      /* check write */
      count = 0 ;
      while(1)
      {
         SW_READ_REG(INCA_IP_Switch_CPU_ACTL, cdaCmd) ;
         if ( (cdaCmd & 0x80000000) == 0 || count > 10)
            break ;
         count++ ;
      }        /* while(1) {...} */

      if ( (cdaCmd & 0x80000000) != 0)
      {
         SW_LOG_0_VLAN("ERROR: sw21553_SwVLanResetVLanTable - no write ack") ;
/*           intUnlock(currentLevel); */
         restore_flags(flags);
         return(ERROR) ;
      }
   }
   sw21553_VlanCoSCtrl.vLanCntl.vLanTableCurrentMaxId = 0 ;
   sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount = 0 ;
/*     intUnlock(currentLevel); */
   restore_flags(flags);

   return(OK) ;
}


/******************************************************************************
*  sw21553_SwVLanAddTableEntry()
*     Calculate the table index and add a given VLAN entry to the VLAN table
*     
*  Parameter:
*     newRegEntry: VLAN table entry
*
*  Returns:
*    -1 (ERROR): write of the new entry was not successful
*     else index within the VLAN table
*
*  REMARK:
*     Because the VLAN table entries must be in a increasing order a local 
*     copy of the current VLAN table will be used for sorting.
*     After insertion of a new entry into the local VLAN table the
*     INCA IP VLAN table will be updated.
*/
int sw21553_SwVLanAddTableEntry(sw21553_VLanTableEntry inVLanTableEntry)
{
/*     UINT32 currentLevel; */
   int overwrite = 0 ;
   int newIndex, tempIndex = 0;
   UINT32 tempEntry ;
   unsigned long flags;

   /* check parameter */
   if ( inVLanTableEntry.vLanId <= 0 || 
        inVLanTableEntry.vLanId >= INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK )
      /* no valid VLAN ID */
      return(ERROR) ;

   /*--------------------
      insert new entry 
   ----------------------*/

   /* block all while update */
/*     currentLevel = intLock(); */
   save_and_cli(flags);

   for ( newIndex = sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount - 1 ;
         newIndex >= 0 ; newIndex-- )
   {
      tempEntry = sw21553_VlanCoSCtrl.vLanCntl.vLanTable[newIndex] ;
      if ( inVLanTableEntry.vLanId > INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(tempEntry) )
      {
         /* entry found --> set new VLAN entry into local array */
         newIndex++ ;
         /* leave loop */
         break ;
      }
      else
      {
         if ( inVLanTableEntry.vLanId == INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(tempEntry) )
         {
            /* entry already exist --> overwrite */
            overwrite = 1 ;
            break ;
         }
      }        /* if ( inVLanTableEntry.vLanId > <current VLAN ID> ) {...} */
   }        /* for ( ... ; newIndex >= 0 ; newIndex-- ) {...} */


   if (newIndex == -1)
   {
      /* VLAN table currently empty or set first entry */
      newIndex = 0 ;
      overwrite = 0;            /* do not overwrite, but insert entry */
   }

   /* update INCA IP VLAN table */
   if (overwrite == 1)
   {
      /* overwrite already existing entry */
       sw21553_VlanCoSCtrl.vLanCntl.vLanTable[newIndex] = 
           INCA_IP_VLAN_TABLE_ENTRY_SET_VLAN_ID(inVLanTableEntry.vLanId) |
           INCA_IP_VLAN_TABLE_ENTRY_SET_CTRL_FLAGS(inVLanTableEntry.Cntrl) ;
       sw21553_SwVLanWriteTableEntryReg32( newIndex, 
                                           sw21553_VlanCoSCtrl.vLanCntl.vLanTable[newIndex] ) ;

   }
   else
   {
       /* insert new entry, if space is available */

       if (sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount >= INCA_IP_MAX_VLAN_TABLE_ENTRIES)
           return(ERROR); /* VLAN table full */

       /* newIndex = index in VLAN table to receive new entry;
          all entries from index newIndex up to index vLanTableEntryCount 
          have to be moved one entry up */

       /* move entries from index (vLanTableEntryCount-1) down to index (newIndex) 
        to the next higher index and write the corresponding entry to the on-chip
        VLAN table */
       for (tempIndex = sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount ;
            tempIndex > newIndex; tempIndex-- )
       {
           /* write and move current VLAN entries */
           tempEntry = sw21553_VlanCoSCtrl.vLanCntl.vLanTable[tempIndex - 1];
           sw21553_VlanCoSCtrl.vLanCntl.vLanTable[tempIndex] = tempEntry ;
           sw21553_SwVLanWriteTableEntryReg32( tempIndex, tempEntry ) ;
       }
       /* now write new entry to cache table and to on-chip table */
       sw21553_VlanCoSCtrl.vLanCntl.vLanTable[newIndex] = 
           INCA_IP_VLAN_TABLE_ENTRY_SET_VLAN_ID(inVLanTableEntry.vLanId) |
           INCA_IP_VLAN_TABLE_ENTRY_SET_CTRL_FLAGS(inVLanTableEntry.Cntrl) ;
       sw21553_SwVLanWriteTableEntryReg32( newIndex, 
                                           sw21553_VlanCoSCtrl.vLanCntl.vLanTable[newIndex]);
      
      /* update control settings */
      sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount++ ;
      if (inVLanTableEntry.vLanId > sw21553_VlanCoSCtrl.vLanCntl.vLanTableCurrentMaxId)
         sw21553_VlanCoSCtrl.vLanCntl.vLanTableCurrentMaxId = inVLanTableEntry.vLanId ;
   }

   /* unlock */
/*     intUnlock(currentLevel); */
   restore_flags(flags);

   return(newIndex) ;
}

/******************************************************************************
*  sw21553_SwVLanReadTableEntry()
*     read and return the current VLAN table (protected)
*
*  Parameter:
*     pVLanTable: ptr to a user given VLAN table (default size 63) 
*     localVlanTable:
*                 0: read and return the INCA IP VLAN table
*                 1: read and return the local VLAN table
*
*  Returns:
*     number of VLAN table entries
*
*/
UINT32 sw21553_SwVLanReadTable(UINT32 *pVLanTable, int localVlanTable)
{
   int index, count ;
   UINT32 readCmd, regValue ;
/*     UINT32 currentLevel; */
   unsigned long flags;

   if (pVLanTable == NULL)
      return(ERROR) ;

   count = 0 ;

   /* block all while read */
/*     currentLevel = intLock(); */
   save_and_cli(flags);

   if (localVlanTable == 0)
   {
      for (index = 0 ; index < INCA_IP_MAX_VLAN_TABLE_ENTRIES; index++)
      {
         /* set read command */
         readCmd = INCA_IP_Switch_CPU_ACTL_RD_VLAN_TABLE | index ;

         /* read from VLAN table via CPU_ACTL */
         SW_WRITE_REG( INCA_IP_Switch_CPU_ACTL, readCmd ) ;
         while(1)
         {
            SW_READ_REG(INCA_IP_Switch_CPU_ACTL, readCmd) ;
            if ( (readCmd & 0x80000000) == 0 )
               break ;
         }

         /* read VLAN table entry */
         SW_READ_REG(INCA_IP_Switch_DATA2, regValue) ;

         if ( /* 0x0007FFFF indicates an unused entry */
              (regValue & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK) 
                                         == ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK 
            )
         {
            /* NO gaps allowded --> end of table */
/*              intUnlock(currentLevel); */
             restore_flags(flags);
            return( count ) ;
         }

         /* save to user array */
         pVLanTable[index] = regValue ;
         count++ ;
      }        /* for (index = 0 ; index <= INCA_IP_MAX_VLAN_TABLE_ENTRIES; index++) {...} */

/*        intUnlock(currentLevel); */
      restore_flags(flags);
      return(count) ;
   }
   else     /* if (localVlanTable = 0) {...} else ... */
   {
      for (index = 0 ; index < sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount ; index++)
      {
         pVLanTable[index] = sw21553_VlanCoSCtrl.vLanCntl.vLanTable[index] ;
      }

/*        intUnlock(currentLevel); */
      restore_flags(flags);
      return(sw21553_VlanCoSCtrl.vLanCntl.vLanTableEntryCount) ;
   }        /* if (localVlanTable = 0) {...} else {...} */
}


/******************************************************************************
*  sw21553_SwCoSInitControl()
*     init the local CoS control struct with the default values ;
*
*  Parameter:
*
*  Returns:
*
*  REMARK:
*
*/
LOCAL void sw21553_SwCoSInitControl(void)
{
   /*---------------------*/
   /* local configuration */
   /*---------------------*/

   /* mapping of DSCP <--> CoS */
   memset( sw21553_VlanCoSCtrl.coSCntl.dscpPrioTable, 
           0x00, sizeof(sw21553_VlanCoSCtrl.coSCntl.dscpPrioTable) ) ;

   return ;
}

/******************************************************************************
*  sw21553_SwCoSSetPrioCos()
*     set and change the PRIO_CoS register (Priority CoS Mapping Register)
*     The current PRIO_CoS is locally stored within the CoS control struct
*     The local structure will be updated and then written to the switch module
*
*  Parameter:
*     newSettings: contains the new settings for the PRIO_CoS register
*
*  Returns:
*     new value of the PRIO_CoS register
*
*  REMARK:
*
*/
UINT32 sw21553_SwCoSSetPrioCos(UINT32 newSettings)
{
   /* check and set new settings */
   newSettings &= INCA_IP_Switch_PRIO_COS_RESERVED_MASK ;

   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_VLAN_CTRL,newSettings) ;

   return(newSettings) ;
}

/******************************************************************************
*  sw21553_SwCoSSetDscpCoS1()
*     set and change the DSCP_COS1/DSCP_COS1 registers (DSCP CoS Mapping Register 1/2)
*
*  Parameter:
*     newDscpCoSx: contains the new settings for the DSCP_COSx register
*
*  Returns:
*     new value of the DSCP_COS1/DSCP_COS2 register
*
*  REMARK:
*     -1 (0xFFFFFFFF) is a valid return value
*
*/
UINT32 sw21553_SwCoSSetDscpCoS1(UINT32 newDscpCoS1 )
{
   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_DSCP_COS1, newDscpCoS1) ;
   
   return(newDscpCoS1) ;
}

UINT32 sw21553_SwCoSSetDscpCoS2(UINT32 newDscpCoS2)
{
   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_DSCP_COS2, newDscpCoS2) ;

   return(newDscpCoS2) ;
}

/******************************************************************************
*  sw21553_SwCoSSetWFQCtl()
*     set new settings for CoS "Weighted Fair Queuing"
*     (PC_WFQ_CTL, LAN_WFQ_CTL, CPU_WFQ_CTL)
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     SchedulingScheme: Select Scheduling Scheme
*                    0 --> 00b: WFQ scheduling
*                    3 --> 11b: Strict priority scheduling
*                    -1 --> keep current settings, no change 
*     WFQWeightQ_0, WFQWeightQ_1: WFQ Weight for Queue 0/1
*                    Range 0-7: weigth factor for WFQ
*                    -1 --> keep current settings, no change 
*                    (Note: This value is used when SchedulingScheme = 00B)
*     PrioQ_0, PrioQ_1: Priority for Queue 0/1
*                    0/1: used priority for queue 0/1
*                    -1 --> keep current settings, no change 
*                    (Note: This value is used when SchedulingScheme = 11B)
*
*  Returns:
*     ERROR: invalid parameter (out of range)
*     else : new register value of the LAN_WFQ_CTL, 
*
*  REMARK:
*
*/
UINT32 sw21553_SwCoSSetWFQCtl( int portSelect,   int SchedulingScheme ,
                               int WFQWeightQ_0, int WFQWeightQ_1,
                               int PrioQ_0,      int PrioQ_1 )
{
   UINT32 newRegValue, currRegValue, updMask = 0 ;
   unsigned int flags=0;
   
   /* build update mask */
   updMask |= ( (SchedulingScheme == -1) ? 0 : INCA_IP_Switch_XXX_WFQ_CTL_SCH_SEL_MASK ) |
              ( (WFQWeightQ_0 == -1) ? 0 : INCA_IP_Switch_XXX_WFQ_CTL_WT0_MASK ) |
              ( (WFQWeightQ_1 == -1) ? 0 : INCA_IP_Switch_XXX_WFQ_CTL_WT1_MASK ) |
              ( (PrioQ_0 == -1) ? 0 : INCA_IP_Switch_XXX_WFQ_CTL_P0_MASK ) |
              ( (PrioQ_1 == -1) ? 0 : INCA_IP_Switch_XXX_WFQ_CTL_P1_MASK ) ;

   /* build register value, check range of parameters */
   newRegValue = 0 ;

   if ( SchedulingScheme == INCA_IP_Switch_XXX_WFQ_CTL_STRICT_SCHED ||
        SchedulingScheme == INCA_IP_Switch_XXX_WFQ_CTL_STRICT_SCHED )
      newRegValue |= (INCA_IP_Switch_PC_WFQ_CTL_SCH_SEL_SET(SchedulingScheme)) ;
   else
      if (SchedulingScheme != -1)
         return(ERROR) ;

   if (WFQWeightQ_0 >= 0 && WFQWeightQ_0 <=7)
      newRegValue |= (INCA_IP_Switch_PC_WFQ_CTL_WT0_SET(WFQWeightQ_0)) ;
   else
      if (WFQWeightQ_0 != -1)
         return(ERROR) ;

   if (WFQWeightQ_1 >= 0 && WFQWeightQ_1 <=7)
      newRegValue |= (INCA_IP_Switch_PC_WFQ_CTL_WT1_SET(WFQWeightQ_1)) ;
   else
      if (WFQWeightQ_1 != -1)
         return(ERROR) ;

   if (PrioQ_0 == 1)
      newRegValue |= INCA_IP_Switch_PC_WFQ_CTL_P0 ;
   else
      if (PrioQ_0 |= 0 || PrioQ_0 == -1)
         return(ERROR) ;

   if (PrioQ_1 == 1)
      newRegValue |= INCA_IP_Switch_PC_WFQ_CTL_P1 ;
   else
      if (PrioQ_1 |= 0 || PrioQ_1 == -1)
         return(ERROR) ;


   switch(portSelect)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
	    save_and_cli(flags);
         /* read current register settings */
         SW_READ_REG(INCA_IP_Switch_PC_WFQ_CTL, currRegValue) ;
         /* clear all reserved and update bits */
         currRegValue &= INCA_IP_Switch_XXX_WFQ_CTL_RESERVED_MASK & ~updMask ;
         /* update current settings */
         currRegValue |= (newRegValue & updMask) ;
         /* write new configuration */
         SW_WRITE_REG(INCA_IP_Switch_PC_WFQ_CTL, currRegValue) ;
	    restore_flags(flags);
         return(currRegValue) ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
	    save_and_cli(flags);
         /* read current register settings */
         SW_READ_REG(INCA_IP_Switch_LAN_WFQ_CTL, currRegValue) ;
         /* clear all reserved and update bits */
         currRegValue &= INCA_IP_Switch_XXX_WFQ_CTL_RESERVED_MASK & ~updMask ;
         /* update current settings */
         currRegValue |= (newRegValue & updMask) ;
         /* write new configuration */
         SW_WRITE_REG(INCA_IP_Switch_LAN_WFQ_CTL, currRegValue) ;
	     restore_flags(flags);
         return(currRegValue) ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU:
	    save_and_cli(flags);
         /* read current register settings */
         SW_READ_REG(INCA_IP_Switch_CPU_WFQ_CTL, currRegValue) ;
         /* clear all reserved and update bits */
         currRegValue &= INCA_IP_Switch_XXX_WFQ_CTL_RESERVED_MASK & ~updMask ;
         /* update current settings */
         currRegValue |= (newRegValue & updMask) ;
         /* write new configuration */
         SW_WRITE_REG(INCA_IP_Switch_CPU_WFQ_CTL, currRegValue) ;
	    restore_flags(flags);
         return(currRegValue) ;

      default:
         /* invalid port selection */
         return(ERROR) ;
   }

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_SwCoSSetRxWatermarks()
*     set new settings for CoS RX watermarks
*     (PC_WM, LAN_WM, CPU_WM)
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     rxWm1: Once this watermark is reached the corresponding MAC
*            sends pause frames (full duplex mode) or performs
*            collisions (half duplex mode).
*     rxWm2: Once this watermark is reached only critical packets are
*            admitted from the xxx port if so enabled by the PDN bit in
*            the PM_CTL. Else non critical packets are admitted until
*            RX_WM3 is reached and PM stops issuing pbnums.
*     rxWm3: Once this watermark is reached PM stops issuing pbnums
*            to the RX module.
*     rxWm4: Once rx_cnt drops below this watermark, MAC stops flow
*            control.
*
*  Returns:
*     ERROR: invalid parameter (out of range)
*     else : new register value of the PC_WM, LAN_WM, CPU_WM* register
*
*  REMARK:

*
*/
UINT32 sw21553_SwCoSSetRxWatermarks( int portSelect, 
                                     UCHAR rxWm1, UCHAR rxWm2, UCHAR rxWm3, UCHAR rxWm4 )
{
   sw21553_CoSRxWatermarks tempWm ;
   register unsigned long regval=0;   

   /* set new settings */
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM1_INDEX] = rxWm1 ;
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM2_INDEX] = rxWm2 ;
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM3_INDEX] = rxWm3 ;
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM4_INDEX] = rxWm4 ;

   switch(portSelect)
   {
      case INCA_IP_SWITCH_PORT_SELECT_PC:
         /* write new configuration */
         SW_WRITE_REG(INCA_IP_Switch_PC_WM, tempWm.rxWaterMarksReg) ;
         SW_READ_REG(INCA_IP_Switch_PC_WM, regval); 
#ifdef SW_DEBUG_ENABLE
         printk("INCA_IP_Switch_PC_WM register set to 0x%08X\n", regval);
#endif
         return(tempWm.rxWaterMarksReg) ;

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
         /* write new configuration */
         SW_WRITE_REG( INCA_IP_Switch_LAN_WM, tempWm.rxWaterMarksReg) ;
         SW_READ_REG(INCA_IP_Switch_LAN_WM, regval); 
#ifdef SW_DEBUG_ENABLE
         printk("INCA_IP_Switch_LAN_WM register set to 0x%08X\n", regval);
#endif
         return(tempWm.rxWaterMarksReg) ;

      case INCA_IP_SWITCH_PORT_SELECT_CPU:
         /* write new configuration */
         SW_WRITE_REG( INCA_IP_Switch_CPU_WM, tempWm.rxWaterMarksReg) ;
         SW_READ_REG(INCA_IP_Switch_CPU_WM, regval); 
#ifdef SW_DEBUG_ENABLE
         printk("INCA_IP_Switch_CPU_WM register set to 0x%08X\n", regval);
#endif
         return(tempWm.rxWaterMarksReg) ;

      default:
         /* invalid port selection */
         return(ERROR) ;
   }

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_SwCoSSetGlobalWatermarks()
*     set new settings for CoS global watermark register
*
*  Parameter:
*     rxWm1: Once this watermark is reached all MACs should send
*            pause frames (full duplex mode) or performs collisions (half
*            duplex mode).
*     rxWm2: Once this watermark is reached only critical packets are
*            admitted from all ports if so enabled by the GDN bit in the
*            PM_CTL. Else non critical packets are admitted until
*            GBL_RX_WM3 is reached and PM stops issuing pbnums.
*     rxWm3: Once this watermark is reached pm_rx_drop_all signal is
*            asserted to all ports - all packets are dropped.
*     rxWm4: Once the freeq counter rises above this value all MACs
*            stop flow control.
*
*  Returns:
*     new register value of the GLB_WM register
*
*  REMARK:
*/
UINT32 sw21553_SwCoSSetGlobalWatermarks( UCHAR rxWm1, UCHAR rxWm2, 
                                         UCHAR rxWm3, UCHAR rxWm4 )
{
   sw21553_CoSRxWatermarks tempWm ;
   register unsigned long regval=0;

   /* set new settings */
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM1_INDEX] = rxWm1 ;
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM2_INDEX] = rxWm2 ;
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM3_INDEX] = rxWm3 ;
   tempWm.rxWaterMark[INCA_IP_COS_RX_WM4_INDEX] = rxWm4 ;

   
   /* write new configuration */
   SW_WRITE_REG(INCA_IP_Switch_GBL_WM, tempWm.rxWaterMarksReg) ;

  
   SW_READ_REG(INCA_IP_Switch_GBL_WM, regval); 
#ifdef SW_DEBUG_ENABLE
   printk("INCA_IP_Switch_GBL_WM register set to 0x%08X\n", regval);
#endif


   
   return(tempWm.rxWaterMarksReg) ;
}




/******************************************************************************
*  sw21553_SwNonCritPacketDrop()
*     configures INCA-IP's PM_CTL register to control the dropping of 
*     non critical packets in case of WM-2 level reached
*
*  Parameter:
*     gdn:     global drop/non drop of non critical packets
*     cdn:     drop/non drop of non critical packets on cpu port
*     ldn:     drop/non drop of non critical packets on lan port
*     pdn:     drop/non drop of non critical packets on pc port.
*
*
*
*  REMARK:
*
*/
UINT32 sw21553_SwNonCritPacketDrop( BOOL gdn, BOOL cdn, BOOL ldn, BOOL pdn )
{
   register unsigned long regval=0;

   /* 
    * set drop control bits for non critical packets 
    * acc. to passed parameters 
	*/
   if( gdn )
      regval |= INCA_IP_Switch_PM_CTL_GDN; 
   if( cdn )
      regval |= INCA_IP_Switch_PM_CTL_CDN; 
   if( ldn )
      regval |= INCA_IP_Switch_PM_CTL_LDN; 
   if( pdn )
      regval |= INCA_IP_Switch_PM_CTL_PDN;    
   SW_WRITE_REG(INCA_IP_Switch_PM_CTL, regval);

   /* read back written value and print it out */
   SW_READ_REG(INCA_IP_Switch_PM_CTL, regval); 
#ifdef SW_DEBUG_ENABLE
   printk("INCA_IP_Switch_PM_CTL register set to 0x%08X\n", regval);
#endif

   return ((UINT32) regval);

}


/******************************************************************************
*  sw21553_SwIoSetControl()
*     write access to the INCA IP switch register for configuration
*
*  Parameter:
*     command: indicate the requested action
*     pdata:   parameter for execute the requestet action
*              (depends on the command)
*
*     currently implemented command:
*        eSwIoSetPortVLanConfig: set the VLAN config (VLAN ID, PRIO) 
*           for PC, LAN or CPU port.
*           (parameter type: pdata --> sw21553_SwIoVLanCfg)
*        eSwIoSetVlanCntrl: set the VLAN control (symetrical, send only tagged packets)
*           for PC, LAN or CPU port.
*           (parameter type: pdata --> sw21553_SwIoVLanCtrl )
*        eSwIoSetVlanCntrl_VA: enable VLAN
*           (parameter type: pdata --> UINT32: 0 clear, 1 set)
*        eSwIoSetArlCtl: set new settings of the ARL_CTL register
*           (parameter type: pdata --> UINT32, new register value)
*        eSwIoUpdArlCtl: update the settings of the ARL_CTL register
*           only the marked settings within the update mask will be changed
*           (parameter type: pdata --> sw21553_SwIoRegUpd)
*        eSwIoUpdArlCtl_ENL: ARL_CTL[0]: Enable Learning from LAN Port
*        eSwIoUpdArlCtl_MAF: ARL_CTL[1]: MA Freeze
*        eSwIoUpdArlCtl_SRC: ARL_CTL[6]: Send Reserved Packets to CPU
*        eSwIoUpdArlCtl_MRO: ARL_CTL[7]: MA Replace Oldest Entry
*        eSwIoUpdArlCtl_CG:  ARL_CTL[9]: CRC GEN for CPU Port
*           (parameter type: pdata --> UINT32: 0 clear, 1 set)
*        eSwIoSetPMacHdCtl: set new settings of the PMAC_HD_CTL register
*           (parameter type: pdata --> UINT32, new register value)
*        eSwIoUpdMacHdCtl: update the settings of the PMAC_HD_CTL register
*           only the marked settings within the update mask will be changed
*           (parameter type: pdata --> sw21553_SwIoRegUpd)
*        eSwIoUpdMacHdCtl_ADD: PMAC_HD_CTL[0]: ADD Header to Packets from DMA to PMAC
*        eSwIoUpdMacHdCtl_TAG: PMAC_HD_CTL[1]: Add TAG to Packets from DMA to PMAC
*           (parameter type: pdata --> UINT32: 0 clear, 1 set)
*        eSwIoSetPrioCoS:  set new settings of the PRI_CoS register
*        eSwIoSetDscpCos1: set new settings of the DSCP_COS1 register
*        eSwIoSetDscpCos2: set new settings of the DSCP_COS2 register
*           (parameter type: pdata --> UINT32, new register value)
*        eSwIoSetPortWfqCtl: set new settings for WFQ Control Register
*           for PC, LAN or CPU port.
*           (parameter type: pdata --> sw21553_SwIoCoSWfqCtl)
*        eSwIoSetPortRxWm:    set new settings for RX watermarks
*           for PC, LAN or CPU port.
*           (parameter type: pdata --> sw21553_SwIoCoSWm)
*        eSwIoSetGlobalRxWm:  set new settings for GLOBAL watermarks
*           (parameter type: pdata --> sw21553_SwIoCoSWm)
*
*  REMARK:
*
*/
UINT32 sw21553_SwIoSetControl(int command, void *pdata)
{
   UINT32 regValue ;
   UINT32 updMask ;

   regValue = 0 ;
   updMask = 0 ;
   
   switch(command)
   {
      case eSwIoSetPortVLanConfig:
         /* pdata --> sw21553_SwIoVLanCfg */
         if (pdata != NULL)
            return( sw21553_SwVLanSetVLanConfig( ((sw21553_SwIoVLanCfg*)pdata)->portSelect, 
                                                 ((sw21553_SwIoVLanCfg*)pdata)->newPrio,
                                                 ((sw21553_SwIoVLanCfg*)pdata)->newVLanId ) ) ;
         else
            return(ERROR) ;

      case eSwIoSetVlanCntrl:
         /* pdata --> sw21553_SwIoVLanCtrl */
         if (pdata == NULL)
            return(ERROR) ;

         switch( ((sw21553_SwIoVLanCtrl*)pdata)->portSelect )
         {
            case INCA_IP_SWITCH_PORT_SELECT_PC:
               if ( ((sw21553_SwIoVLanCtrl*)pdata)->sendOnlyToMember )
               {
                  regValue =  INCA_IP_Switch_VLAN_CTRL_SP;
               }

               if ( ((sw21553_SwIoVLanCtrl*)pdata)->notAcceptUntaggedPkt )
               {
                  regValue |= INCA_IP_Switch_VLAN_CTRL_TP;
               }
               updMask = INCA_IP_Switch_VLAN_CTRL_SP | INCA_IP_Switch_VLAN_CTRL_TP ;
               break ;
            case INCA_IP_SWITCH_PORT_SELECT_LAN:
               if ( ((sw21553_SwIoVLanCtrl*)pdata)->sendOnlyToMember )
               {
                  regValue =  INCA_IP_Switch_VLAN_CTRL_SL;
               }

               if ( ((sw21553_SwIoVLanCtrl*)pdata)->notAcceptUntaggedPkt )
               {
                  regValue |= INCA_IP_Switch_VLAN_CTRL_TL;
               }
               updMask = INCA_IP_Switch_VLAN_CTRL_SL | INCA_IP_Switch_VLAN_CTRL_TL ;
               break ;
            case INCA_IP_SWITCH_PORT_SELECT_CPU:
               if ( ((sw21553_SwIoVLanCtrl*)pdata)->sendOnlyToMember )
               {
                  regValue =  INCA_IP_Switch_VLAN_CTRL_SC;
               }

               if ( ((sw21553_SwIoVLanCtrl*)pdata)->notAcceptUntaggedPkt )
               {
                  regValue |= INCA_IP_Switch_VLAN_CTRL_TC;
               }
               updMask = INCA_IP_Switch_VLAN_CTRL_SC | INCA_IP_Switch_VLAN_CTRL_TC ;
               break ;
            default:
               return(ERROR) ;
         }        /* switch( (sw21553_SwIoVLanCtrl*)pdata)->portSelect ) {...} */

         /* update settings */
         return( sw21553_SwVLanIOVLanCntrl(regValue, updMask, 1) ) ;

      case eSwIoSetVlanCntrl_VA:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_VLAN_CTRL_VA;
         updMask = INCA_IP_Switch_VLAN_CTRL_VA ;

         /* update settings */
         return( sw21553_SwVLanIOVLanCntrl(regValue, updMask, 1) ) ;

      /*--------------------------
            ARL_CTL register
      ---------------------------*/
      case eSwIoSetArlCtl:
         /* pdata --> UINT32, new register value */
         /* update all settings (mask = 0xffffffff)  */
         return( sw21553_SwSetArlControl( (UINT32)pdata, 0xffffffff ) ) ;

      case eSwIoUpdArlCtl:
         /* pdata --> sw21553_SwIoRegUpd */
         /* update marked settings */
         if (pdata != NULL)
            return( sw21553_SwSetArlControl( ((sw21553_SwIoRegUpd*)pdata)->newRegSettings , 
                                             ((sw21553_SwIoRegUpd*)pdata)->updRegMask  ) ) ;
         else
            return(ERROR) ;

      case eSwIoUpdArlCtl_ENL:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_ARL_CTL_ENL ;
         updMask = INCA_IP_Switch_ARL_CTL_ENL ;
         /* update settings */
         return( sw21553_SwSetArlControl(regValue, updMask) ) ;
      case eSwIoUpdArlCtl_MAF:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_ARL_CTL_MAF ;
         updMask = INCA_IP_Switch_ARL_CTL_MAF ;
         /* update settings */
         return( sw21553_SwSetArlControl(regValue, updMask) ) ;
      case eSwIoUpdArlCtl_SRC:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_ARL_CTL_SRC ;
         updMask = INCA_IP_Switch_ARL_CTL_SRC ;
         /* update settings */
         return( sw21553_SwSetArlControl(regValue, updMask) ) ;
      case eSwIoUpdArlCtl_MRO:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_ARL_CTL_MRO ;
         updMask = INCA_IP_Switch_ARL_CTL_MRO ;
         /* update settings */
         return( sw21553_SwSetArlControl(regValue, updMask) ) ;
      case eSwIoUpdArlCtl_CG:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_ARL_CTL_CG ;
         updMask = INCA_IP_Switch_ARL_CTL_CG ;
         /* update settings */
         return( sw21553_SwSetArlControl(regValue, updMask) ) ;

      /*--------------------------
            PMAC_HD_CTL register
      ---------------------------*/
      case eSwIoSetPMacHdCtl:
         /* pdata --> UINT32, new register value */
         /* update all settings (mask = 0xffffffff)  */
         return( sw21553_SwSetPMacHdCtl( (UINT32)pdata, 0xffffffff ) ) ;
      case eSwIoUpdMacHdCtl:
         /* pdata --> sw21553_SwIoRegUpd */
         /* update marked settings */
         if (pdata != NULL)
            return( sw21553_SwSetPMacHdCtl( ((sw21553_SwIoRegUpd*)pdata)->newRegSettings , 
                                            ((sw21553_SwIoRegUpd*)pdata)->updRegMask  )  ) ;
         else
            return(ERROR) ;
      case eSwIoUpdMacHdCtl_ADD:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_PMAC_HD_CTL_ADD ;
         updMask = INCA_IP_Switch_PMAC_HD_CTL_ADD ;
         /* update settings */
         return( sw21553_SwSetPMacHdCtl(regValue, updMask) ) ;
      case eSwIoUpdMacHdCtl_TAG:
         /* pdata --> UINT32: 0 clear, 1 set */
         if ( (UINT32)pdata )
            regValue =  INCA_IP_Switch_PMAC_HD_CTL_TAG ;
         updMask = INCA_IP_Switch_PMAC_HD_CTL_TAG ;
         /* update settings */
         return( sw21553_SwSetPMacHdCtl(regValue, updMask) ) ;

      case eSwIoSetPrioCoS:
         /* pdata --> UINT32, new register value */
         return( sw21553_SwCoSSetPrioCos( (UINT32)pdata) ) ;
      case eSwIoSetDscpCos1:
         /* pdata --> UINT32, new register value */
         return( sw21553_SwCoSSetDscpCoS1( (UINT32)pdata) ) ;
      case eSwIoSetDscpCos2:
         /* pdata --> UINT32, new register value */
         return( sw21553_SwCoSSetDscpCoS2( (UINT32)pdata) ) ;

      case eSwIoSetPortWfqCtl:
         /* pdata --> sw21553_SwIoCoSWfqCtl */
         if (pdata != NULL)
            return( sw21553_SwCoSSetWFQCtl( ((sw21553_SwIoCoSWfqCtl*)pdata)->portSelect, 
                                            ((sw21553_SwIoCoSWfqCtl*)pdata)->scheme,
                                            ((sw21553_SwIoCoSWfqCtl*)pdata)->weightQueue_0,
                                            ((sw21553_SwIoCoSWfqCtl*)pdata)->weightQueue_1,
                                            ((sw21553_SwIoCoSWfqCtl*)pdata)->prioQueue_0,
                                            ((sw21553_SwIoCoSWfqCtl*)pdata)->prioQueue_1  ) ) ;
         else
            return(ERROR) ;
      case eSwIoSetPortRxWm:
         /* pdata --> sw21553_SwIoCoSWm */
         if (pdata != NULL)
            return( sw21553_SwCoSSetRxWatermarks( ((sw21553_SwIoCoSWm*)pdata)->portSelect, 
                                                  ((sw21553_SwIoCoSWm*)pdata)->rxWm1 ,
                                                  ((sw21553_SwIoCoSWm*)pdata)->rxWm2 ,
                                                  ((sw21553_SwIoCoSWm*)pdata)->rxWm3 ,
                                                  ((sw21553_SwIoCoSWm*)pdata)->rxWm4  ) ) ;
         else
            return(ERROR) ;

      case eSwIoSetGlobalRxWm:
         /* pdata --> sw21553_SwIoCoSWm */
         if (pdata != NULL)
            return( sw21553_SwCoSSetGlobalWatermarks( ((sw21553_SwIoCoSWm*)pdata)->rxWm1 ,
                                                      ((sw21553_SwIoCoSWm*)pdata)->rxWm2 ,
                                                      ((sw21553_SwIoCoSWm*)pdata)->rxWm3 ,
                                                      ((sw21553_SwIoCoSWm*)pdata)->rxWm4  ) ) ;
         else
            return(ERROR) ;

      default:
         return(ERROR) ;
   }        /* switch(command) {...} */

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_SwVLanCoSInitControl()
*     init the local VLAN and CoS control struct with the default values ;
*
*  REMARK:
*
*/
int sw21553_SwVLanCoSInitControl(void)
{
   sw21553_VlanCoSCtrl.bSoftWareProcessVLanTag = 0 ;
   sw21553_VlanCoSCtrl.bL2PrioOnly = 0 ;
   sw21553_VlanCoSCtrl.bAddLocalVLanTag = 0 ;

   /* create Semaphore for Address Form Function */
/*     if ( ( sw21553_VlanCoSCtrl.endAdrFormSem = semBCreate(SEM_Q_PRIORITY, SEM_FULL) ) == NULL ) */
/*     { */
/*        SW_LOG_0_VLAN("ERROR: sw21553_SwVLanCoSInitControl - create semaphore") ; */
/*        return(ERROR) ; */
/*     } */
   spin_lock_init(&sw21553_VlanCoSCtrl.endAdrFormSem);

   /* init the control structures */
   sw21553_SwVLanInitControl() ;
   sw21553_SwCoSInitControl() ;

   return(OK) ;
}

/******************************************************************************
*  sw21553_SwVLanCoSClearControl()
*     clear the allocated parts of the local "sw21553_VlanCoSCtrl" struct
*
*     - delete semaphore
*
*  REMARK:
*
*/
void sw21553_SwVLanCoSClearControl(void)
{
/*     if (sw21553_VlanCoSCtrl.endAdrFormSem != NULL) */
/*     { */
/*        semDelete(sw21553_VlanCoSCtrl.endAdrFormSem) ; */
/*        sw21553_VlanCoSCtrl.endAdrFormSem = NULL ; */
/*     } */

   return ;
}


#if (INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING == TRUE)
/*--------------------------------------------------------
   allow VLAN processing by software
   !!! it is recommended to use this only for testing !!!

   for use this feature, default VxWorks END driver 
   callback functions have to be replaced.
----------------------------------------------------------*/


/******************************************************************************
*  sw21553_SwVLanSetLocalVLanTag()
*     set new "PRIO" and "VALN_ID" for software VLAN
*
*  Parameter:
*     newPrio:    new priority value to set (range 0 - 7)
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*
*  Returns:
*     ERROR: invalid parameter (out of range)
*     OK:    set
*
*/
int sw21553_SwVLanSetLocalVLanTag(UINT32 newPrio, UINT32 newVLanId)
{

   /* check range of parameters */
   if (newPrio > INCA_IP_VLAN_MAX_PRIO || newVLanId > INCA_IP_VLAN_MAX_VLAN_ID)
   {
      /* parameter not in range */
      return(ERROR) ;
   }

   sw21553_VlanCoSCtrl.vLanCntl.vLanId   = newVLanId ;
   sw21553_VlanCoSCtrl.vLanCntl.vLanPrio = newPrio ;
   return(OK) ;
}

/******************************************************************************
*  sw21553_SwVLanSetLocalVLanTag()
*     set new "PRIO" and "VALN_ID" for software VLAN
*
*  Parameter:
*     pOutPrio:    ptr to return value VLAN tag prio
*     pOutVLanId:  ptr to return value VLAN tag VLAN ID
*
*  Returns: bAddLocalVLanTag --> indicate if fixed local tag will be added
*     bAddLocalVLanTag = 0: added VLAN tag prio from L3
*     bAddLocalVLanTag = 1: add fixed local VLAN tag prio
*
*/
int sw21553_SwVLanGetLocalVLanTag(UINT32 *pOutPrio, UINT32 *pOutVLanId)
{

   /* check range of parameters */
   if (pOutPrio == NULL || pOutVLanId == NULL)
   {
      /* parameter error */
      return(ERROR) ;
   }

   *pOutVLanId = sw21553_VlanCoSCtrl.vLanCntl.vLanId ;
   *pOutPrio   = sw21553_VlanCoSCtrl.vLanCntl.vLanPrio ;

   return(sw21553_VlanCoSCtrl.bAddLocalVLanTag) ;
}

/******************************************************************************
*  sw21553_SwVLanSetLocalVLanCtrlTag()
*     set the control settings for software VLAN handling
*
*  Parameter:
*     bSoftWareProcessVLanTag: VLAN processing can be done by software
*     bL2PrioOnly:      the MAC frame is extended only if l3 priority is set > 0
*     bAddLocalVLanTag: Software adds always fixed VLAN tag
*
*
*/
void sw21553_SwVLanSetLocalVLanCtrlTag( int bSoftWareProcessVLanTag,
                                        int bL2PrioOnly,
                                        int bAddLocalVLanTag)
{
   if (bAddLocalVLanTag)
      sw21553_VlanCoSCtrl.bAddLocalVLanTag = 1 ;
   else
      sw21553_VlanCoSCtrl.bAddLocalVLanTag = 0 ;

   if (bL2PrioOnly)
      sw21553_VlanCoSCtrl.bL2PrioOnly = 1 ;
   else
      sw21553_VlanCoSCtrl.bL2PrioOnly = 0 ;

   if (bSoftWareProcessVLanTag)
      sw21553_VlanCoSCtrl.bSoftWareProcessVLanTag = 1 ;
   else
      sw21553_VlanCoSCtrl.bSoftWareProcessVLanTag = 0 ;

   return ;
}


/* local VLAN buffer for create VLAN tag */
LOCAL sw21553_VLanTag loaclVLanTag ;

/******************************************************************************
*  sw21553_EndEtherAddressForm() - routine that forms the Ethernet and VLAN Tag Header
*
*     These is a wrapper function to the vxWorks default function endEtherAddressForm
*
*  REMARK:
*     If VLAN support is enabled (via compiler switch) these function will be used 
*     by the VxWorks IP stack for ethernet handling
*
*     The behaviour depends on local settings from the "sw21553_VlanCoSCtrl" struct
*
*     bSoftWareProcessVLanTag: VLAN processing is done by software
*        0: no VLAN processing --> the default VxWorks function will be called
*        1: do VLAN processing --> depending on further settings
*           (requires correct switch configuration)
*
*     if (bSoftWareProcessVLanTag == 1) --> process VLAN tags by software
*        bAddLocalVLanTag: add a fixed VLAN tag defined within the "sw21553_VlanCoSCtrl" struct
*           0: do not add a fixed VLAN tag
*           1: add a fixed VLAN tag
*
*        if (bAddLocalVLanTag == 0) --> create VLAN tag depending on L3 CoS
*           bL2PrioOnly: add priority via VLAN tag only if the L3 prio is not 0
*           0: add allways VLAN tag with L3 prio
*           1: if L3 prio = 0 --> do not add VLAN tag
*
*/
M_BLK_ID sw21553_EndEtherAddressForm( M_BLK_ID pNBuff,   /* mBlk chain containing outgoing data */
                                      M_BLK_ID pSrcAddr, /* source address, in an mBlk */
                                      M_BLK_ID pDstAddr, /* destination address, in an mBlk */
                                      BOOL bcastFlag)    /* use link-level broadcast ? */
{
   /* local variables */
   UINT   lHeaderLength;
   UINT   lDscp;

   /* check VLAN enabled */
   if (sw21553_VlanCoSCtrl.bSoftWareProcessVLanTag == 0)
   {
      /*-------------------------------------------------------------------
          no VLAN processing --> use the original "endEtherAddressForm()"
      -------------------------------------------------------------------*/
      /* SW_DEBUG_VLAN( "call org. endEtherAddressForm()\n" ) ; */
      SW_LOG_0_VLAN( "call org. endEtherAddressForm()\n" ) ;
      return( endEtherAddressForm(pNBuff, pSrcAddr, pDstAddr)  ) ;
   }

   /*------------------------------
        software VLAN processing
   ------------------------------*/
   SW_LOG_0_VLAN( "VLAN SW (ADDR form) - sw21553_EndEtherAddressForm\n" ) ;

   /* check for valid pointer */
   if ((pSrcAddr == NULL) || (pDstAddr == NULL))
   {
      SW_LOG_0_VLAN( "VLAN SW (ADDR form) - ERROR: addr ptr = NULL\n" ) ;
      return (NULL);
   }

   /* ensure that we are not disturbed by other addressform calls */
   /* ??? create the VLAN tag local --> no semaphore necessary ??? */
   semTake (sw21553_VlanCoSCtrl.endAdrFormSem, WAIT_FOREVER);

   /* fill local Vlan Tag buffer */
   loaclVLanTag.wTpid = INCA_IP_VLAN_TPID ;

   if (sw21553_VlanCoSCtrl.bAddLocalVLanTag == 1)
   {
      SW_LOG_2_VLAN( "VLAN SW (ADDR form) - add local tag\n", 
                     sw21553_VlanCoSCtrl.vLanCntl.vLanId,
                     sw21553_VlanCoSCtrl.vLanCntl.vLanPrio ) ;
      /* Src. + Dest. Addrs + Tag + Length/Ether-Type */
      lHeaderLength = INCA_IP_VLAN_ENET_HEAD_SIZE ; 
      /* create VLAN tag from local settings */
      loaclVLanTag.wTci = sw21553_VlanCoSCtrl.vLanCntl.vLanId | 
                          (sw21553_VlanCoSCtrl.vLanCntl.vLanPrio << 13) ;
   }
   else     /* if (sw21553_VlanCoSCtrl.bAddLocalVLanTag == 1) {...} else ... */
   {
      /* reserved field contains "network protocol type" */
      /* check for all IP packet types not just ETHERNET_IP = 0x0800 */
      if ( (pSrcAddr->mBlkHdr.reserved & 0xff00) == ETHERTYPE_IP)
      {
         /* IP packet, evaluate layer 3 priority */
         /* IP Header:    byte 0        byte 1         byte 2 / 3
                        version/       Type Of        total lenght 
                        header Len     Service
         */
         /* this must be right shifted 2 bits as it's a 6 bit field stored in bits 2-7 of the TOS field */
         lDscp = *(unsigned char*) ((pNBuff->mBlkHdr.mData)+1);
         lDscp = lDscp >> 2;
      }
      else     /* if ( (pSrcAddr->mBlkHdr.reserved & 0xff00) == ETHERTYPE_IP) {...} else ... */
      {
         lDscp = 0;
      }        /* if ( (pSrcAddr->mBlkHdr.reserved & 0xff00) == ETHERTYPE_IP) {...} else {...} */


      if (sw21553_VlanCoSCtrl.bL2PrioOnly == 1 && lDscp == 0 )
      {
         /* If L2_PRIO_ONLY is defined, the MAC frame is extended only if l3 priority is set > 0 */

         /* Src. + Dest. Addrs + Length/Ether-Type */
         lHeaderLength = INCA_IP_ENET_HEAD_SIZE ; 
      }
      else     /* if (sw21553_VlanCoSCtrl.bL2PrioOnly == 1 && lDscp == 0 ) {...} else ... */
      {
         /* Src. + Dest. Addrs + Tag + Length/Ether-Type */
         lHeaderLength = INCA_IP_VLAN_ENET_HEAD_SIZE ; 
         /* Change MAC frame, insert priority bytes */

         /* create VLAN tag CoS settings */
         loaclVLanTag.wTci = sw21553_VlanCoSCtrl.vLanCntl.vLanId | 
                            (sw21553_VlanCoSCtrl.coSCntl.dscpPrioTable[lDscp] << 13) ;
      }        /* if (sw21553_VlanCoSCtrl.bL2PrioOnly == 1 && lDscp == 0 ) {...} else {...} */
   }        /* if (sw21553_VlanCoSCtrl.bAddLocalVLanTag == 1) {...} else {...} */


   /* copy header into data element if possible or */
   /* create a new data element including ethernet header */
   /* check for enough space in pNBuff to copy header into pNBuff */
   M_PREPEND (pNBuff, lHeaderLength, M_DONTWAIT);
   if (pNBuff == NULL)
   {
      semGive (sw21553_VlanCoSCtrl.endAdrFormSem);
      SW_LOG_0_VLAN( "VLAN SW (ADDR form) - ERROR: not enough space\n" ) ;
      return (NULL);
   }


   /* 
      copy Ethernet Header:
       6 bytes      6 bytes      4 bytes    2 bytes
      dest. addr | source addr | VLAN tag | Length/type | ....
   */

   #if 0
      /* Causes alignment exception on MIPs!! */
      /* copy 6 bytes destination and 6 bytes src ethernet addresses into pNBuff */
      *(UINT *) (pNBuff->mBlkHdr.mData)     = *(UINT *) (pDstAddr->mBlkHdr.mData);
      *(USHORT *) (pNBuff->mBlkHdr.mData + 4) = *(USHORT *) (pDstAddr->mBlkHdr.mData + 4);
      *(UINT *) (pNBuff->mBlkHdr.mData + INCA_IP_ENET_ADDR_SIZE) = *(UINT *) (pSrcAddr->mBlkHdr.mData);
      *(USHORT *) (pNBuff->mBlkHdr.mData + INCA_IP_ENET_ADDR_SIZE + 4) = *(USHORT *) (pSrcAddr->mBlkHdr.mData + 4);
   #else
      bcopyBytes( pDstAddr->mBlkHdr.mData,      /* source */
                  pNBuff->mBlkHdr.mData,        /* destination */
                  INCA_IP_ENET_ADDR_SIZE );     /* size */
      bcopyBytes( pSrcAddr->mBlkHdr.mData, 
                  pNBuff->mBlkHdr.mData + INCA_IP_ENET_ADDR_SIZE, 
                  INCA_IP_ENET_ADDR_SIZE );
   #endif

   if (lHeaderLength == INCA_IP_ENET_HEAD_SIZE)
   {
      /* standard header, no VLAN tag to add */
      /* no layer 2 qos: copy Ether-Type only */
      bcopyBytes( (UCHAR *)(&pSrcAddr->mBlkHdr.reserved), 
                  (pNBuff->mBlkHdr.mData + (2 * INCA_IP_ENET_ADDR_SIZE)), 
                  sizeof( USHORT ) );

      SW_LOG_0_VLAN( "VLAN SW (ADDR form) - no VLAN tag\n" ) ;
   }
   else
   {
      /* CoS enabled: add VLAN tag and Ether-Type */
      /* copy VLAN tag (4 bytes) */
      bcopyBytes( (UCHAR *)(&loaclVLanTag), 
                  (pNBuff->mBlkHdr.mData + (2 * INCA_IP_ENET_ADDR_SIZE)), 
                  sizeof (sw21553_VLanTag) );
      /* copy Ether-Type (2 bytes) */
      bcopyBytes( (UCHAR *)(&pSrcAddr->mBlkHdr.reserved), 
                  (pNBuff->mBlkHdr.mData + (2 * INCA_IP_ENET_ADDR_SIZE) + INCA_IP_VLAN_TAG_SIZE), 
                  sizeof( USHORT ));
      SW_LOG_0_VLAN( "VLAN SW (ADDR form) - VLAN tag added\n" ) ;
   }

   /* now everything should work again */
   semGive (sw21553_VlanCoSCtrl.endAdrFormSem);

   return (pNBuff);

}

/******************************************************************************
*  sw21553_EndEtherPacketDataGet()  - get a pointer to packet data
*     These is a wrapper function to the vxWorks default function endEtherAddressForm
*
*  REMARK:
*     If VLAN support is enabled (via compiler switch) these function will be used 
*     by the VxWorks IP stack for ethernet handling
*
*     The behaviour depends on local settings from the "sw21553_VlanCoSCtrl" struct
*     bSoftWareProcessVLanTag: VLAN processing by software 
*                              (requires correct switch configuration)
*        0: no VLAN software processing --> the default VxWorks function will be called
*        1: do VLAN software processing --> depending on further settings
*
*/
STATUS sw21553_EndEtherPacketDataGet(M_BLK_ID pMblk, LL_HDR_INFO * pLinkHdrInfo)
{
   /* check VLAN enabled */
   if (sw21553_VlanCoSCtrl.bSoftWareProcessVLanTag == 0)
   {
      /*-------------------------------------------------------------------
          no VLAN processing --> use the original "endEtherAddressForm()"
      -------------------------------------------------------------------*/
      /* SW_DEBUG_VLAN( "call org. endEtherPacketDataGet()\n" ) ; */
      SW_LOG_0_VLAN( "call org. endEtherPacketDataGet()\n" ) ; 
      return( endEtherPacketDataGet(pMblk, pLinkHdrInfo) ) ;
   }

   /*------------------------------
        software VLAN processing
   ------------------------------*/
   SW_LOG_0_VLAN( "VLAN SW (Get data) - sw21553_EndEtherPacketDataGet\n" ) ;
   
   /* set LL_HDR_INFO packets */
   pLinkHdrInfo->destAddrOffset = 0;
   pLinkHdrInfo->destSize      = INCA_IP_ENET_ADDR_SIZE ;
   pLinkHdrInfo->srcAddrOffset = INCA_IP_ENET_ADDR_SIZE ;
   pLinkHdrInfo->srcSize       = INCA_IP_ENET_ADDR_SIZE ;

   if ( ( *(USHORT *) (pMblk->mBlkHdr.mData + (2 * INCA_IP_ENET_ADDR_SIZE))) == INCA_IP_VLAN_TPID)
   {   
      /* 
         VLAN tag protocol identifier found, remove:
          6 bytes      6 bytes      4 bytes    2 bytes
         dest. addr | source addr | VLAN tag | Length/type | ....
      */
      pLinkHdrInfo->dataOffset = INCA_IP_VLAN_ENET_HEAD_SIZE ;
      /* set the ptr to the type field */
      pLinkHdrInfo->pktType = *(USHORT *) (pMblk->mBlkHdr.mData + (INCA_IP_ENET_ADDR_SIZE + 
                                                                   INCA_IP_ENET_ADDR_SIZE + 
                                                                   INCA_IP_VLAN_TAG_SIZE));
      SW_LOG_1_VLAN( "VLAN SW (Get data) - VLAN tag = %d found\n", 
                     ( *(USHORT *)( pMblk->mBlkHdr.mData + (2 * INCA_IP_ENET_ADDR_SIZE) ) ) 
                   ) ;
   }
   else
   {
      /* 
         no VLAN tag protocol identifier found, remove:
          6 bytes      6 bytes        2 bytes
         dest. addr | source addr | Length/type | ....
      */
      pLinkHdrInfo->dataOffset = INCA_IP_ENET_HEAD_SIZE ;
      pLinkHdrInfo->pktType = *(USHORT *) (pMblk->mBlkHdr.mData + (INCA_IP_ENET_ADDR_SIZE +
                                                                   INCA_IP_ENET_ADDR_SIZE));
      SW_LOG_0_VLAN( "VLAN SW (Get data) - NO VLAN tag found\n" ) ;
   }
   return OK;

}   /* sw21553_EndEtherPacketDataGet */


/*

   +------+                 +------------------+      VLAN ID = 5, PRIO = 1
   |  PC  |<--------------->|(LAN)      Board 1|      =====================
   +------+                 |                  |      EGRESS ports with VLAN:
     ping   -------------+  | (SW processing   |      --> PC  port: sends VLAN tagged frames
   (Board2)     untagged |  | without VLAN tag)|      --> LAN port: sends no VLAN tagged frames
            <-----+      |  |       |          |      --> CPU port: sends no VLAN tagged frames
                  |      |  | remove VLAN tag  |
                  -      -  |       PC         |      MEMBERSHIP of this VLAN: PC, LAN, CPU
                  _      _  +------------------+
                t |    t |          |
                a |    a |          |  VLAN tagged
                g |    g |          |  comminication
                g |    g |          |
                e |    e |  +------------------+      VLAN ID = 5, PRIO = 1
                d |    d |  |      PC          |      =====================
                            |  keep VLAN tag   |      EGRESS ports with VLAN:
                  |      |  |       |          |      --> PC  port: sends VLAN tagged frames
                  |      |  | (SW processing   |      --> LAN port: sends no VLAN tagged frames
                  <------+  | with VLAN tag)   |      --> CPU port: sends VLAN tagged frames
                  replay    |                  |
                            |           Board 2|      MEMBERSHIP of this VLAN: PC, LAN, CPU
                            +------------------+

*/
int sw21553_ActivateVLanFixed(void)
{
   int index ;
   UINT32 retValue ;
   sw21553_SwIoVLanCfg vLanConfig ;
   sw21553_VLanTableEntry vLanTableEntry ;

   /* setup CPU port */
   vLanConfig.portSelect = INCA_IP_SWITCH_PORT_SELECT_CPU ;
   vLanConfig.newPrio = 1 ;
   vLanConfig.newVLanId = 5 ;
   retValue = sw21553_SwIoSetControl(eSwIoSetPortVLanConfig, (void *)&vLanConfig) ;

   /* setup PC port */
   vLanConfig.portSelect = INCA_IP_SWITCH_PORT_SELECT_PC ;
   retValue = sw21553_SwIoSetControl(eSwIoSetPortVLanConfig, (void *)&vLanConfig) ;

   /* add VLAN Tag into VLAN table 
      VLAN_ID = 5 ;
      EC = 1
      EP = 1
      V  = 1
   */
   index = 0 ;
   vLanTableEntry.vLanId = 5 ;
   vLanTableEntry.Cntrl = ( INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC_MASK |  /* CPU port is member */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP_MASK |  /* PC  port is member */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_ML_MASK |  /* LAN port is member */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP_MASK |  /* tag egress on PC */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK     /* entry is valid ??? */
                          ) ;
   retValue = sw21553_SwVLanWriteTableEntry(index, vLanTableEntry ) ;

   /* VLAN aware */
   retValue = sw21553_SwIoSetControl(eSwIoSetVlanCntrl_VA, (void *)1) ;

   return(0) ;
}

int sw21553_ActivateVLanSoft(void)
{

   int index ;
   UINT32 retValue ;
   sw21553_SwIoVLanCfg vLanConfig ;
   sw21553_VLanTableEntry vLanTableEntry ;

   /* basic initialization has already been done while start system */

   /* setup VLAN software processing */
   sw21553_VlanCoSCtrl.vLanCntl.vLanId = 5 ;
   sw21553_VlanCoSCtrl.vLanCntl.vLanPrio = 1 ;
   sw21553_VlanCoSCtrl.bAddLocalVLanTag = 1 ;

   /* setup CPU port */
   vLanConfig.portSelect = INCA_IP_SWITCH_PORT_SELECT_CPU ;
   vLanConfig.newPrio = 1 ;
   vLanConfig.newVLanId = 5 ;
   retValue = sw21553_SwIoSetControl(eSwIoSetPortVLanConfig, (void *)&vLanConfig) ;

   /* setup PC port */
   vLanConfig.portSelect = INCA_IP_SWITCH_PORT_SELECT_PC ;
   retValue = sw21553_SwIoSetControl(eSwIoSetPortVLanConfig, (void *)&vLanConfig) ;

   /* add VLAN Tag into VLAN table 
      VLAN_ID = 5 ;
      EC = 1
      EP = 1
      V  = 1
   */
   index = 0 ;
   vLanTableEntry.vLanId = 5 ;
   vLanTableEntry.Cntrl = ( INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC_MASK |  /* CPU port is member */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP_MASK |  /* PC  port is member */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_EC_MASK |  /* tag egress on CPU */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP_MASK |  /* tag egress on PC */
                            INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK     /* entry is valid ??? */
                          ) ;
   retValue = sw21553_SwVLanWriteTableEntry(index, vLanTableEntry ) ;

   /* setup VLAN aware: SW and HW VLAN tagging */
   sw21553_VlanCoSCtrl.bSoftWareProcessVLanTag = 1 ;
   retValue = sw21553_SwIoSetControl(eSwIoSetVlanCntrl_VA, (void *)1) ;

   return(0) ;
}


#endif /* #if (INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING == TRUE) */



#ifdef INCLUDE_LED_MULTIPLEXER

/******************************************************************************
   Switch/END LED's ACCESS: functions
******************************************************************************/


/******************************************************************************
*  sw21553_SwLedSetLedMuxConfig()
*     Set the new Switch/END configuration of the LED multiplexer. 
*
*  Parameter:
*     newSettings: contains the register mask of the new settings
*                  see "LEDMUX_CONF" register
*     updMask:     only the marked settings will be changed
*                  (for change all set updMask = 0xFFFF FFFF)
*     Remark: there are several properties for display. By setting the appropriate
*             bit per port the corresponding property will be signaled via a LED
*        status:  display the status of the ethernet port
*        speed:   display the speed of the ethernet port
*        Duplex:  display the duplex mode of the ethernet port
*        Activity:    signal any receive activities on the ethernet port
*        TX Activity: signal also any transmit activities on the appropriate port
*
*  Returns:
*     new value of the LEDMUX_CONF register
*
*  REMARK:
*
*/
UINT32 sw21553_SwLedSetLedMuxConfig(UINT32 newSettings, UINT32 updMask)
{
   UINT32 regValue ;
   unsigned int flags=0;
   
#if defined(INCA_COMPACT) && defined(INCA_STANDARD)

   /* 'Standard' and 'Compact' INCA-IP are supported */

   /* check if the device is a 'Compact' */
   if ((sysIncaVer() & 0x0000F000) == 0x00001000)
   {
      /* Compact INCA-IP: only LAN port available */
      newSettings &= (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN & updMask) ;
   }
   else
   {
      /* Standard INCA-IP: LAN and PC port available */
      newSettings &= (INCA_IP_TSF_LEDMUX_CONF_SW_MASK & updMask) ;
   }

#elif defined(INCA_STANDARD)

   /* Standard INCA-IP: LAN and PC port supported */
   newSettings &= (INCA_IP_TSF_LEDMUX_CONF_SW_MASK & updMask) ;

#elif defined(INCA_COMPACT)

   /* Compact INCA-IP: only LAN port supported */
   newSettings &= (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN & updMask) ;

#else

   /* Missing parameter for 'Standard' and 'Compact' INCA-IP */
   #error Standard or Compact INCA-IP has to be supported!

#endif /* #if defined(INCA_COMPACT) && defined(INCA_STANDARD) */

   save_and_cli(flags);
   /* read current LEDMUX_CONF register */
   regValue = *((volatile UINT32*)INCA_IP_TSF_LEDMUX_CONF) ;

   /* clear old settings */
   regValue &= ~(INCA_IP_TSF_LEDMUX_CONF_SW_MASK) ;

   /* set new settings */
   regValue |= newSettings ;

   /* write back */
   *((volatile UINT32*)INCA_IP_TSF_LEDMUX_CONF) = regValue ;

   restore_flags(flags);
   /* return new value */
   return(regValue) ;
}


/******************************************************************************
*  sw21553_SwLedAssignLeds()
*     Assign the Switch/END signals to the LEDs
*
*  Parameter:
*     portSelect: select a port (PC or LAN)
*        PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*        LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*
*     status: display the status of the ethernet port
*             LED off: normal
*             LED on:  ehternet port is disabled
*             LED flashing: ERROR condition
*     speed:  displays the speed of the ethernet port
*             LED off: 10 Mbit/s
*             LED on: 100 Mbit/s
*     Duplex: displays the duplex mode of the ethernet port
*             LED off: Half Duplex
*             LED on: Full Duplex
*     Activity: signals any receive activities on the ethernet port
*             LED on (non-flashing): Link without activity
*             LED on (flashing):     Port receiving (or transmitting, see txAct)
*
*  Returns:
*     ERROR: in case of invalid port selection
*     ELSE:  new value of the LED_REG register
*
*  REMARK:
*
*/
UINT32 sw21553_SwLedAssignLeds( UINT32 portSelect, 
                                int status, int speed, int duplex, int Activity )
{
   switch (portSelect)
   {
      #if defined(INCA_STANDARD)
      case INCA_IP_SWITCH_PORT_SELECT_PC:
      {
         /* Standard INCA-IP: LAN and PC port supported */
         /* check if the device is a 'Standard' */
         if ((sysIncaVer() & 0x0FFFF000) == 0x000C0000)
         {
            /* Standard INCA-IP: LAN and PC port available */

            /* set PC settings */
            if (status)
               sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_PC_STATUS) ;
            else
               sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_PC_STATUS) ;

            if (speed)
               sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_PC_SPEED) ;
            else
               sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_PC_SPEED) ;

            if (duplex)
               sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_PC_DUPLEX) ;
            else
               sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_PC_DUPLEX) ;

            if (Activity)
               sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_PC_ACT) ;
            else
               sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_PC_ACT) ;
         }
         else
         {
            /* Compact INCA-IP: no PC port available */
            return (ERROR) ;
         }
      }
      break ;
      #endif

      case INCA_IP_SWITCH_PORT_SELECT_LAN:
      {
         /* set LAN settings */
         if (status)
            sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_LAN_STATUS) ;
         else
            sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_LAN_STATUS) ;

         if (speed)
            sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_LAN_SPEED) ;
         else
            sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_LAN_SPEED) ;

         if (duplex)
            sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_LAN_DUPLEX) ;
         else
            sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_LAN_DUPLEX) ;

         if (Activity)
            sysIncaLEDMuxOn(INCA_IP_SWITCH_LEDREG_LAN_ACT) ;
         else
            sysIncaLEDMuxOff(INCA_IP_SWITCH_LEDREG_LAN_ACT) ;
      }
      break ;

      default:
         /* parameter error */
         return(ERROR) ;
   }

   return(OK) ;
}

#endif /* #ifdef INCLUDE_LED_MULTIPLEXER */


/******************************************************************************
*  sw21553_SwGetMIBTable()
*     read and return MIB counter table for the specified port
*  Parameter:
*     portSelect:  select a port 
*                               PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                               LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                               CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     pMIBTable:   ptr to MIB table to fill out
*
*  Returns:
*     0: OK
*    -1: ERROR 
*
*  REMARK:
*     the user must allocate and free memory space for a variable of type 
*     sw21553_MIBCounterTable
*
*/
int sw21553_SwGetMIBTable( unsigned int portSelect, sw21553_MIBCounterTable *pMIBTable )
{
   UINT32 MIBbase;
   int index = 0;
   int value;
   int ret_value = -1;

   if ((portSelect < 3) && (pMIBTable != NULL ))
   {
     /* get semaphore */
/*     semTake(sw21553_SwitchAccessSem, WAIT_FOREVER) ;  */

     MIBbase = (UINT32)INCA_IP_Switch_PORT_MIB_BASE_ADDRESS(portSelect);

     for ( index = 0; index < sizeof(sw21553_MIBCounterTable); index +=4 )
     {
       /* read MIB counter values and copy them to MIB counter structure */
       SW_READ_REG( (MIBbase + index), value );
       *((UINT32*)((UINT32)pMIBTable+index)) = (UINT32)value;
     }

     /* free semaphore before return */
/*     semGive(sw21553_SwitchAccessSem) ;  */
     ret_value = 0;


   }
   return(ret_value);
}


    

/****************************************************************************
Description:
   This function prints the MIB counters of LAN-, PC- and CPU-port 
   into the passed buffer in order to display them by means of 
   proc-file access.
Arguments:
   *buf 
   **start
   offset 
   count 
   *eof     
   *data
Return Value:
   len : number of characters added by this function
Remarks:
   None
****************************************************************************/
int inca_switch_mib_read_procmem( char *buf, char **start, off_t offset, 
                                  int count, int *eof,     void *data)
{
	long	len = 0;
	unsigned int 	t;

	sw21553_MIBCounterTable pMIBTable[3];   /* MIB counter array */
	
    /* 
     * initialize counter array and
     * get MIB counter values for all ports 
     */
	memset( pMIBTable, 0, sizeof(pMIBTable));
	for( t=0; t<3; t++ )
	{
        sw21553_SwGetMIBTable( t, &pMIBTable[t] );
	}
	
	/* 
	 * print counter value table to passed buffer  
	 */
	len = sprintf( buf,      "\nInca-IP MIB Counters\n");
	len=len+sprintf( buf+len,"                       PC Port      LAN Port     CPU Port\n");
	len=len+sprintf( buf+len,"------------------------------------------------------------\n");
	len=len+sprintf( buf+len,"rxBytes                0x%08X   0x%08X   0x%08X\n",
	                 pMIBTable[0].rxBytes,
	                 pMIBTable[1].rxBytes,        
	                 pMIBTable[2].rxBytes );
	len=len+sprintf( buf+len,"rxUnicastPkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxUnicastPkts,   
	                 pMIBTable[1].rxUnicastPkts,  
	                 pMIBTable[2].rxUnicastPkts );                                   
	len=len+sprintf( buf+len,"rxBroadcastPkts        0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxBroadcastPkts, 
	                 pMIBTable[1].rxBroadcastPkts, 
	                 pMIBTable[2].rxBroadcastPkts ); 
	len=len+sprintf( buf+len,"rxMulticastPkts        0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxMulticastPkts, 
	                 pMIBTable[1].rxMulticastPkts, 
	                 pMIBTable[2].rxMulticastPkts ); 
	len=len+sprintf( buf+len,"rxCRCErrorPkts         0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxCRCErrorPkts,  
	                 pMIBTable[1].rxCRCErrorPkts,  
	                 pMIBTable[2].rxCRCErrorPkts ); 
	len=len+sprintf( buf+len,"rxDroppedPkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxDroppedPkts,   
	                 pMIBTable[1].rxDroppedPkts,   
	                 pMIBTable[2].rxDroppedPkts ); 
	len=len+sprintf( buf+len,"rxUnderSizeGoodPkts    0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxUnderSizeGoodPkts, 
	                 pMIBTable[1].rxUnderSizeGoodPkts, 
	                 pMIBTable[2].rxUnderSizeGoodPkts ); 
	len=len+sprintf( buf+len,"rxFragments            0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxFragments,     
	                 pMIBTable[1].rxFragments,     
	                 pMIBTable[2].rxFragments ); 
	len=len+sprintf( buf+len,"rxOversizeGoodPkts     0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxOversizeGoodPkts,   
	                 pMIBTable[1].rxOversizeGoodPkts,   
	                 pMIBTable[2].rxOversizeGoodPkts );
	len=len+sprintf( buf+len,"rxOversizeBadPkts      0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxOversizeBadPkts,   
	                 pMIBTable[1].rxOversizeBadPkts,   
	                 pMIBTable[2].rxOversizeBadPkts );
	len=len+sprintf( buf+len,"rxGoodBytes            0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxGoodBytes,   
	                 pMIBTable[1].rxGoodBytes,   
	                 pMIBTable[2].rxGoodBytes );
	len=len+sprintf( buf+len,"rxGoodPausePkts        0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxGoodPausePkts,   
	                 pMIBTable[1].rxGoodPausePkts,   
	                 pMIBTable[2].rxGoodPausePkts );
	len=len+sprintf( buf+len,"rxAlignmentErrors      0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxAlignmentErrors,   
	                 pMIBTable[1].rxAlignmentErrors,   
	                 pMIBTable[2].rxAlignmentErrors );
	len=len+sprintf( buf+len,"rxErrors               0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxErrors,   
	                 pMIBTable[1].rxErrors,   
	                 pMIBTable[2].rxErrors );
	len=len+sprintf( buf+len,"rx64BytePkts           0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rx64BytePkts,   
	                 pMIBTable[1].rx64BytePkts,   
	                 pMIBTable[2].rx64BytePkts );
	len=len+sprintf( buf+len,"rx127BytePkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rx127BytePkts,   
	                 pMIBTable[1].rx127BytePkts,   
	                 pMIBTable[2].rx127BytePkts );
	len=len+sprintf( buf+len,"rx255BytePkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rx255BytePkts,   
	                 pMIBTable[1].rx255BytePkts,   
	                 pMIBTable[2].rx255BytePkts );
	len=len+sprintf( buf+len,"rx511BytePkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rx511BytePkts,   
	                 pMIBTable[1].rx511BytePkts,   
	                 pMIBTable[2].rx511BytePkts );
	len=len+sprintf( buf+len,"rx1023BytePkts         0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rx1023BytePkts,   
	                 pMIBTable[1].rx1023BytePkts,   
	                 pMIBTable[2].rx1023BytePkts );
	len=len+sprintf( buf+len,"rxMaxBytePkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].rxMaxBytePkts,   
	                 pMIBTable[1].rxMaxBytePkts,       
	                 pMIBTable[2].rxMaxBytePkts );
	len=len+sprintf( buf+len,"txLateCollisions       0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txLateCollisions,   
	                 pMIBTable[1].txLateCollisions,   
	                 pMIBTable[2].txLateCollisions );
	len=len+sprintf( buf+len,"txGoodBytes            0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txGoodBytes,   
	                 pMIBTable[1].txGoodBytes,   
	                 pMIBTable[2].txGoodBytes );
	len=len+sprintf( buf+len,"txUnicastPkts          0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txUnicastPkts,   
	                 pMIBTable[1].txUnicastPkts,   
	                 pMIBTable[2].txUnicastPkts );
	len=len+sprintf( buf+len,"txBroadcastPkts        0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txBroadcastPkts,   
	                 pMIBTable[1].txBroadcastPkts,   
	                 pMIBTable[2].txBroadcastPkts );
	len=len+sprintf( buf+len,"txMulticastPkts        0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txMulticastPkts,   
	                 pMIBTable[1].txMulticastPkts,   
	                 pMIBTable[2].txMulticastPkts );
	len=len+sprintf( buf+len,"txCollisions           0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txCollisions,   
	                 pMIBTable[1].txCollisions,   
	                 pMIBTable[2].txCollisions );
	len=len+sprintf( buf+len,"txSingleCollisions     0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txSingleCollisions,   
	                 pMIBTable[1].txSingleCollisions,   
	                 pMIBTable[2].txSingleCollisions );	
	len=len+sprintf( buf+len,"txMultipleCollisions   0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txMultipleCollisions,   
	                 pMIBTable[1].txMultipleCollisions,   
	                 pMIBTable[2].txMultipleCollisions );
	len=len+sprintf( buf+len,"txExcessCollisions     0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txExcessCollisions,   
	                 pMIBTable[1].txExcessCollisions,   
	                 pMIBTable[2].txExcessCollisions );
	len=len+sprintf( buf+len,"txDeferredPkts         0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txDeferredPkts,   
	                 pMIBTable[1].txDeferredPkts,   
	                 pMIBTable[2].txDeferredPkts );
	len=len+sprintf( buf+len,"txGoodPausePkts        0x%08lX   0x%08lX   0x%08lX\n",
	                 pMIBTable[0].txGoodPausePkts,   
	                 pMIBTable[1].txGoodPausePkts,   
	                 pMIBTable[2].txGoodPausePkts );

	len=len+sprintf(buf+len,"\n\n");

	*eof = 1;	// No more data available
	return len;


}




/*
   only for testing
   ================
   sw21553_PrintSwReg - read and show a switch register identified by his offset

   PARAMETER:
      addrOffset: address offset of the requested register within the 
                  switch register space (base address = INCA_IP_Switch)

*/
void sw21553_PrintSwReg(UINT32 addrOffset)
{
   UINT32 regValue ;

   regValue = 0 ;
   SW_READ_REG( (INCA_IP_Switch | addrOffset),regValue);
   printk("Register (0x%08x + 0x%x) = 0x%08x \n", INCA_IP_Switch, addrOffset, regValue) ;

   return ;
}

/*
   only for testing
   ================
   sw21553_PrintReg - read and show a register

   PARAMETER:
      addrOffset: address offset of the requested register within the 
                  switch register space (base address = INCA_IP_Switch)

*/
void sw21553_PrintReg(UINT32 addr)
{
   UINT32 regValue ;

   regValue = (UINT32)*((volatile UINT32*)addr);
   printk("Register (0x%08x) = 0x%08x \n", addr, regValue) ;

   return ;
}

/*
   only for testing
   ================
   sw21553_PrintSwReg - read and show a switch register identified by his offset

   PARAMETER:
      addrOffset: address offset of the requested register within the 
                  switch register space (base address = INCA_IP_Switch)

*/
void sw21553_WrReg(UINT32 addr, UINT32 value)
{
   UINT32 regValue ;

   (UINT32)*((volatile UINT32*)addr) = value ;
   regValue = (UINT32)*((volatile UINT32*)addr);
   printk("Register (0x%08x) = 0x%08x \n", addr, regValue) ;

   return ;
}


/*******************************************************************************
*
* sw21553_Write_MDIO_ACC - Access the 'MDIO ACC' register of the PHY
*
* Writes 'value' to the 'MDIO ACC' - register and waits for the hardware request
* acknowledgment (INCA_IP_Switch_MDIO_ACC_RA). The function returns the value 
* of the 'MDIO ACC' - register, which is reads from the register.
*
* Remark:
*    The value of the value-parameter indicates read or write request
*    Bit 30: 0 --> Read Access
*            1 --> Write Access
*    Bit 31: this is a Request / Acknowledge flag
*            1 --> indicates a new request
*            0 --> acknowledege the previous request
*
* RETURNS: UINT32 value of the 'MDIO ACC' - register
*/
UINT32 sw21553_Write_MDIO_ACC(
   UINT32   value)
{
/*
   printf("MDIO_ACC (0x%08x): command = 0x%08x \n", 
          INCA_IP_Switch_MDIO_ACC, value );
*/
   SW_WRITE_REG(INCA_IP_Switch_MDIO_ACC, value);
   do{
      SW_READ_REG(INCA_IP_Switch_MDIO_ACC, value);
   } while (value & INCA_IP_Switch_MDIO_ACC_RA);

   return value;
}

UINT32 sw21553_Read_CPU_MIB_RxDropped(void)
{
	UINT32   value=0;
	SW_READ_REG(INCA_IP_Switch+0x0700+0x14, value);
	
	return(value);
}

/* Linux specific enhancements */

MODULE_LICENSE("Infineon");

int inca_switch_access_init(void)
{
	
	printk("Inca-IP Switch Access Initialization\n");
		
	/* Create proc file */ 
	create_proc_read_entry("driver/inca_mib", 0, NULL, inca_switch_mib_read_procmem, NULL);    
    
    return 0;
}

void inca_switch_access_exit(void)
{
	/* Remove proc file */ 
	remove_proc_entry( "driver/inca_mib", 0 );    
}

module_init(inca_switch_access_init);
module_exit(inca_switch_access_exit);
/*
EXPORT_SYMBOL(sw21553_PhyReadReg);
EXPORT_SYMBOL(sw21553_PhyWriteReg);
*/
EXPORT_SYMBOL(sw21553_Write_MDIO_ACC);
/*
EXPORT_SYMBOL(sw21553_PhyGetProperties);
EXPORT_SYMBOL(sw21553_PhySetAdvertiseProperties);

EXPORT_SYMBOL(sw21553_PhySetFixSpeed);
EXPORT_SYMBOL(sw21553_PhyDisableAutoNeg);
EXPORT_SYMBOL(sw21553_PhyRestartAutoNeg);
EXPORT_SYMBOL(sw21553_PhyReset);
*/
EXPORT_SYMBOL(sw21553_SwIOUnDest);
EXPORT_SYMBOL(sw21553_SwUpdUnDestPorts);
EXPORT_SYMBOL(sw21553_SwIOSpPt);
EXPORT_SYMBOL(sw21553_SwUpdSpPtPortState);
EXPORT_SYMBOL(sw21553_SwIOPacketLen);
EXPORT_SYMBOL(sw21553_SwSetArlControl);
EXPORT_SYMBOL(sw21553_SwSetCpuPControl);
EXPORT_SYMBOL(sw21553_SwSetPMacHdCtl);
EXPORT_SYMBOL(sw21553_SwSetMdioConfig);
EXPORT_SYMBOL(sw21553_SwSetEPhy);
EXPORT_SYMBOL(sw21553_SwResetMDIO);
EXPORT_SYMBOL(sw21553_SwCpuPCtlEnableDa);
EXPORT_SYMBOL(sw21553_SwCpuPCtlDisableDa);
EXPORT_SYMBOL(sw21553_SwSetPMacMacAddr);
EXPORT_SYMBOL(sw21553_SwGetPMacMacAddr);
EXPORT_SYMBOL(sw21553_SwMacReadTableEntry);
EXPORT_SYMBOL(sw21553_SwMacConvertStr2Hex);
EXPORT_SYMBOL(sw21553_SwMacSetTableEntryCntrl);
EXPORT_SYMBOL(sw21553_SwMacAddEntry);
EXPORT_SYMBOL(sw21553_SwMacClearEntryByIndex);
EXPORT_SYMBOL(sw21553_SwMacClearEntryByMac);
EXPORT_SYMBOL(sw21553_SwMacResetMacTable);
EXPORT_SYMBOL(sw21553_SwIoSetControl);
EXPORT_SYMBOL(sw21553_SwVLanCoSInitControl);
EXPORT_SYMBOL(sw21553_SwVLanCoSClearControl);
EXPORT_SYMBOL(sw21553_SwVLanSetVLanConfig);
EXPORT_SYMBOL(sw21553_SwVLanIOVLanCntrl);
EXPORT_SYMBOL(sw21553_SwCoSSetPrioCos);
EXPORT_SYMBOL(sw21553_SwCoSSetDscpCoS1);
EXPORT_SYMBOL(sw21553_SwCoSSetDscpCoS2);
EXPORT_SYMBOL(sw21553_SwCoSSetWFQCtl);

EXPORT_SYMBOL(sw21553_SwCoSSetRxWatermarks);
EXPORT_SYMBOL(sw21553_SwCoSSetGlobalWatermarks);

EXPORT_SYMBOL(sw21553_SwVLanGetVLanConfig);
EXPORT_SYMBOL(sw21553_SwVLanReadTableEntry);
EXPORT_SYMBOL(sw21553_SwVLanWriteTableEntry);
EXPORT_SYMBOL(sw21553_SwVLanWriteTableEntryReg32);
EXPORT_SYMBOL(sw21553_SwVLanClearTableEntryByIndex);
EXPORT_SYMBOL(sw21553_SwVLanResetVLanTable);
EXPORT_SYMBOL(sw21553_SwVLanAddTableEntry);
EXPORT_SYMBOL(sw21553_SwVLanReadTable);

#if (INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING == TRUE)
EXPORT_SYMBOL(sw21553_SwVLanSetLocalVLanTag);
EXPORT_SYMBOL(sw21553_SwVLanGetLocalVLanTag);
EXPORT_SYMBOL(sw21553_SwVLanSetLocalVLanCtrlTag);
EXPORT_SYMBOL(sw21553_EndEtherAddressForm);
EXPORT_SYMBOL(sw21553_EndEtherPacketDataGet);
#endif


#ifdef INCLUDE_LED_MULTIPLEXER
EXPORT_SYMBOL(sw21553_SwLedSetLedMuxConfig);
EXPORT_SYMBOL(sw21553_SwLedAssignLeds);
#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */

EXPORT_SYMBOL(sw21553_Read_CPU_MIB_RxDropped);

