/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : INCAIPSwitchAccess.h
   Date        : 22.05.2003
   Description : provide functional interface to read and configure the INCA-IP 
                 ethernet parts like PHY's and switch module


   $Log: inca_switch_access.h,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.5  2004/01/12 09:19:28  de6wmigr
   Several definitions and phy driver function prototypes shifted to inca_switch_phy.h.

   Revision 1.4  2003/11/24 11:44:35  de6wmigr
   Function prototype sw21553_swNonCritPacketDrop() added to configure PM_CTL register.

   Revision 1.3  2003/10/27 11:51:05  de6wmigr
   Added MIB counter table and counter addresses.

   Revision 1.2  2003/10/20 14:16:28  de6wanme
   - Added function to read rx_dropped MIB counter

   Revision 1.1  2003/07/14 08:57:09  rjanse_a
   Initial revision of ported vxWorks switch driver (inca_switch_hash, inca_switch_access) and a simple control interface (inca_switch_api, api.sh)

   Revision 1.4  2003/07/07 17:41:38  mangele
   - Add Switch LED API

   Revision 1.3  2003/06/04 15:55:20  mangele
   Add further API functions for access switch INCA IP module

   Revision 1.2  2003/06/02 21:52:17  mangele
   Add further functionallity for the switch API


   
******************************************************************************/
#ifndef _INCASWITCHACCESS_H
#define _INCASWITCHACCESS_H




/*--------------------------------------------------------
   allow VLAN processing by software
   !!! it is recommended to use this only for testing !!!

   for use this feature, default VxWorks END driver 
   callback functions have to be replaced.
----------------------------------------------------------*/
/* for testing */
#define INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING   FALSE

#ifndef INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING
   #define INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING   FALSE
#endif /* #ifndef INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING */
/*-------------------------------------------------------*/



/******************************************************************************
   PHY ACCESS: definitions
******************************************************************************/

/* 
   get/set line properties from the 
      Basic Status Register (reg 1) BIT[11..15] 
*/
#define INCA_IP_PHY_CTRL_LINE_PROPS_GET_BSR(value)  ( ((value)>> 11) & ((1 << 5) - 1)  )
#define INCA_IP_PHY_CTRL_LINE_PROPS_SET_BSR(value)  ( (((1 << 5) - 1) &(value)) << 11 )

/* 
   get/set line properties from the 
      Auto Negotiation Advertisement Register (reg 4) BIT[5..9] 
*/
#define INCA_IP_PHY_CTRL_LINE_PROPS_GET_ANAR(value)  ( ((value)>> 5) & ((1 << 5) - 1)  )
#define INCA_IP_PHY_CTRL_LINE_PROPS_SET_ANAR(value)  ( (((1 << 5) - 1) &(value)) << 5 )

/* 
   get/set line properties from the 
      Auto Negotiation Link Partner Ability Register (reg 5) BIT[5..9] 
*/
#define INCA_IP_PHY_CTRL_LINE_PROPS_GET_ANLPAR(value)  ( ((value)>> 5) & ((1 << 5) - 1)  )
#define INCA_IP_PHY_CTRL_LINE_PROPS_SET_ANLPAR(value)  ( (((1 << 5) - 1) &(value)) << 5 )


/* select one of the both PHY (LAN/PC) accessable via MDIO_ACC register */
#define INCA_IP_MDIO_LAN_PHY     0
#define INCA_IP_MDIO_PC_PHY      1

/* definition of existing MDIO register */
#define INCA_IP_MDIO_REG0_PHY_BCR      0
#define INCA_IP_MDIO_REG1_PHY_BSR      1
#define INCA_IP_MDIO_REG2_PHY_ID1      2
#define INCA_IP_MDIO_REG3_PHY_ID2      3
#define INCA_IP_MDIO_REG4_PHY_ANAR     4
#define INCA_IP_MDIO_REG5_PHY_ANLPAR   5
#define INCA_IP_MDIO_REG6_PHY_ANER     6
#define INCA_IP_MDIO_REG16_PHY_SRR     16
#define INCA_IP_MDIO_REG17_PHY_MCSR    17
#define INCA_IP_MDIO_REG18_PHY_SMR     18
#define INCA_IP_MDIO_REG29_PHY_ISR     29
#define INCA_IP_MDIO_REG30_PHY_IMR     30
#define INCA_IP_MDIO_REG31_PHY_SCSR    31

/*** PHY register definitions ***/
#define INCA_IP_MDIO_REG4_PROPERTY_MASK   0x000007E0


/* Ethernet line 100 MBit/s T4 */
#define sw21553_ETHERNET_LINE_100_T4      (1<<4)
/* Ethernet line 100 MBit/s Full Duplex */
#define sw21553_ETHERNET_LINE_100_FULL    (1<<3)
/* Ethernet line 100 MBit/s Half Duplex */
#define sw21553_ETHERNET_LINE_100_HALF    (1<<2)
/* Ethernet line 10 MBit/s Full Duplex */
#define sw21553_ETHERNET_LINE_10_FULL     (1<<1)
/* Ethernet line 10 MBit/s Half Duplex */
#define sw21553_ETHERNET_LINE_10_HALF     1

/* support pause */
#define sw21553_ENET_EXT_SUPPORT_PAUSE       0x01
/* support next page */
#define sw21553_ENET_EXT_SUPPORT_NEXT_PAGE   0x02
/* support auto negotiation */
#define sw21553_ENET_EXT_SUPPORT_AUTO_NEG    0x04


typedef struct sw21553_ephy_properties
{
   /* 100 MBit/s T4, 100 MBit/s Full D., 100 MBit/s Half D., 10 MBit/s FD, 10 MBit/s Hd */
   UINT  LineProperties ;
   UINT  ExtProperties ;
} sw21553_EPhyProperties ;



/*********************************************************************
      port link status control word
      =============================

         Control part     |    Property part
      --------------------|--------------------
      3322 2222 2222 1111   1111 11
      1098 7654 3210 9876   5432 1098 7654 3210
      |    |___ |||| ||||         |||    |    |
      |    | |  |||| ||||         |||    |-------> line properties (speed/duplex)
      |    | |  |||| ||||         |||
      |    | |  |||| ||||         |||--> Pause 
      |    | |  |||| ||||         ||---> Next Page
      |    | |  |||| ||||         |----> Auto Nego.
      |    | |  |||| ||||
      |    | |  |||| ||||--> extended Capabilities
      |    | |  |||| |||---> Auto Neg possible (Reg 1), maybe Auto-Neg. disabled
      |    | |  ||||
      |    | |  ||||--> Link Status
      |    | |  |||---> Remote Fault
      |    | |  ||----> Auto Neg. Complete
      |    | |  |----->
      |    | |
      |    | |--> current Speed
      |    | 
      |    |----> Auto Neg Done
      |
      |----------> data may corrupted
*/

#define sw21553_PHY_CTRL_LINE_PROPS_GET(value)  ( (value) & ((1 << 5) - 1)  )
#define sw21553_PHY_CTRL_LINE_PROPS_SET(value)  ( ((1 << 5) - 1) &(value) )
#define sw21553_PHY_CTRL_PAUSE                  (1<<8)
#define sw21553_PHY_CTRL_NEXT_PAGE              (1<<9)
#define sw21553_PHY_CTRL_AUTO_NEG_SUPPORT       (1<<10)

#define sw21553_PHY_CTRL_EXT_CAP                (1<<16)
#define sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE      (1<<17)

#define sw21553_PHY_CTRL_LINK_STATE             (1<<20)
#define sw21553_PHY_CTRL_REMOTE_FAULT           (1<<21)
#define sw21553_PHY_CTRL_AUTO_NEG_COMPL         (1<<22)


#if 0
/* see PHY register 31: speed indicaton */
#define sw21553_PHY_CTRL_CURR_SPEED_GET(value)  ( ((value) >> 24) & ((1 << 3) - 1) )
#define sw21553_PHY_CTRL_CURR_SPEED_SET(value)  ( ( ((1 << 3) - 1) &(value) ) << 24 )

/* 001B 10Base-T half-duplex */
#define sw21553_PHY_SPEED_IND_10_HALF     1
/* 101B 10Base-T full-duplex */
#define sw21553_PHY_SPEED_IND_10_FULL     5
/* 010B 100Base-TX half-duplex */
#define sw21553_PHY_SPEED_IND_100_HALF    2
/* 110B 100Base-TX full-duplex */
#define sw21553_PHY_SPEED_IND_100_FULL    6

/* interprete speed bits */
#define sw21553_PHY_SPEED_10_BIT        (1)
#define sw21553_PHY_SPEED_100_BIT       (1<<1)
#define sw21553_PHY_SPEED_DUPLEX_BIT    (1<<2)


/* interprete current speed indication */
#define sw21553_PHY_CTRL_SPEED_10_IND        (1<<24)
#define sw21553_PHY_CTRL_SPEED_100_IND       (1<<25)
#define sw21553_PHY_CTRL_SPEED_DUPLEX_IND    (1<<26)

#define sw21553_PHY_CTRL_AUTO_NEG_DONE       (1<<27)

#define sw21553_PHY_CTRL_DATA_CORRUPTED         (1<<31)


/* current used properties from "Basic Status Register" (PHY_BSR) */
#define sw21553_PHY_PROPS_CURRENT     0
#define sw21553_PHY_PROPS_CURRENT_STR "STATUS"
/* take care about: link state | Remote Fault */
#define sw21553_PHY_PROPS_CURRENT_VALID_MASK   (sw21553_PHY_CTRL_LINK_STATE | \
                                                sw21553_PHY_CTRL_REMOTE_FAULT )
/* data are valid if the state mask matches */
#define sw21553_PHY_PROPS_CURRENT_VALID_STATE  (sw21553_PHY_CTRL_LINK_STATE)


/* advertised properties from "Auto-Negotiation Advertisement Register" (PHY_ANAR) */
#define sw21553_PHY_PROPS_ADVERTISE   1
#define sw21553_PHY_PROPS_ADVERTISE_STR "ADVERTISEMENT"

/* only this settings are writeable within the advertise register */
#define sw21553_PHY_PROPS_ADVERTISE_SET_MASK    sw21553_PHY_CTRL_PAUSE | \
                                                sw21553_ETHERNET_LINE_100_FULL | \
                                                sw21553_ETHERNET_LINE_100_HALF | \
                                                sw21553_ETHERNET_LINE_10_FULL | \
                                                sw21553_ETHERNET_LINE_10_HALF

/* partner link properties from "Auto-Negotiation Link Partner Ability Register" (PHY_ANLPAR) */
#define sw21553_PHY_PROPS_PARTNER     2
#define sw21553_PHY_PROPS_PARTNER_STR "LINK-PARTNER"

/* data are valid only if: 
      partner supports Auto Neg | 
      Auto Neg. must be complete    */
#define sw21553_PHY_PROPS_PARTNER_VALID_MASK   (sw21553_PHY_CTRL_AUTO_NEG_SUPPORT | \
                                                sw21553_PHY_CTRL_AUTO_NEG_COMPL )
/* data are valid if the state mask matches */
#define sw21553_PHY_PROPS_PARTNER_VALID_STATE  (sw21553_PHY_CTRL_AUTO_NEG_SUPPORT | \
                                                sw21553_PHY_CTRL_AUTO_NEG_COMPL )


/*
      END: port link status control word
******************************************************************************/
#endif

/******************************************************************************
   SWITCH ACCESS: definitions
******************************************************************************/

/* include from if_ether.h: 0xe = 14 */
#define INCA_IP_ENET_HEAD_SIZE  SIZEOF_ETHERHEADER
/* include from if_ether.h: 0x12 = 18 */
#define INCA_IP_VLAN_ENET_HEAD_SIZE    SIZEOF_TAG_ETHERHEADER
/* size of Ethernet src/dest addresses */
#define INCA_IP_ENET_ADDR_SIZE   0x06
/* size of ethernet type field */
#define INCA_IP_ENET_TYPE_SIZE   0x02
/* 2 byte "VLAN protocol ID" + 2 byte "Tag control Info" --> 4 byte */
#define INCA_IP_VLAN_TAG_SIZE    4

/* values and ranges defined by the standard */

/* VLAN Tag Protocol identifier field */
#define INCA_IP_VLAN_TPID           0x8100

/* VLAN TCI-field (3 bits) */
#define INCA_IP_VLAN_MAX_PRIO       0x7
/* VLAN_ID field (12 bits) */
#define INCA_IP_VLAN_MAX_VLAN_ID    0x0FFF




/* identify the available ports: PC, LAN, CPU */
#define INCA_IP_SWITCH_PORT_SELECT_PC      0
#define INCA_IP_SWITCH_PORT_SELECT_LAN     1
#define INCA_IP_SWITCH_PORT_SELECT_CPU     2

/* for MAC table: decide between voice and management */
#define INCA_IP_SWITCH_PORT_SELECT_CPU_V   2
#define INCA_IP_SWITCH_PORT_SELECT_CPU_M   3


/*
   CPU_PCTL settings/values 
*/
/* BIT[11:13]: DA destination
   000:  Packet is dropped
   001:  Packet is forward to the PC port
   010:  Packet is forward to the LAN port
   011:  Packet is forward to both LAN and PC port
*/
#define INCA_IP_API_CPU_PCTL_DA_PORTS_DROP   0x00
#define INCA_IP_API_CPU_PCTL_DA_PORTS_PC     0x01
#define INCA_IP_API_CPU_PCTL_DA_PORTS_LAN    0x02
#define INCA_IP_API_CPU_PCTL_DA_PORTS_LAN_PC 0x03

#define INCA_IP_API_CPU_PCTL_DAC_LOW         0
#define INCA_IP_API_CPU_PCTL_DAC_HIGH        1

/* END: CPU_PCTL settings/values */


/* max. number of INCA-IP MAC entries within the switch MAC table */
#define INCA_IP_MAX_MAC_TABLE_ENTRIES 256

/* bit defs for access a mac table entry */
#define INCA_IP_MAC_TABLE_ENTRY_PORT_PC_MASK       0x01
#define INCA_IP_MAC_TABLE_ENTRY_PORT_LAN_MASK      0x02
#define INCA_IP_MAC_TABLE_ENTRY_PORT_CPU_V_MASK    0x04
#define INCA_IP_MAC_TABLE_ENTRY_PORT_CPU_M_MASK    0x08
#define INCA_IP_MAC_TABLE_ENTRY_ALL_PORT_MASK      0x0F

#define INCA_IP_MAC_TABLE_ENTRY_CRITICAL_MASK      0x10
#define INCA_IP_MAC_TABLE_ENTRY_NO_CRITICAL_MASK   0x00

/* MAC table entry - control byte: 111x xxxx --> 0xE0 */
#define INCA_IP_MAC_TABLE_ENTRY_MA_ST_AGED_OUT_MASK      0xE0
#define INCA_IP_MAC_TABLE_ENTRY_MA_ST_STATIC_MASK        0x00
#define INCA_IP_MAC_TABLE_ENTRY_MA_ST_NORMAL_AGING_MASK  0x40

/* 
   union definition for the Switch CPU Access Data (CAD) register 
   --> access to the MAC table via CPU_ACTL register
*/
typedef union sw21553_cad_register
{
   UINT32   reg32 ;
   UCHAR    reg8[8] ;
} sw21553_CdaRegister ;

/* definition of an MAC entry used within the INCA-IP switch module */
typedef struct sw21553_mac_table_entry
{
   UCHAR MacAddr[6] ;
   UCHAR Cntrl ;
} sw21553_MacTableEntry ;



/******************************************************************************
   SWITCH ACCESS: VLAN access / config
******************************************************************************/


/* definition of an VLAN entry used within the INCA-IP switch module */
typedef struct sw21553_vlan_table_entry
{
   USHORT vLanId ;      /* VLAN ID [0..11] of an VLAN table entry */
   USHORT Cntrl ;       /* control data of an VLAN table entry */
} sw21553_VLanTableEntry ;



#define INCA_IP_MAX_VLAN_TABLE_ENTRIES  63


/* 
   definitions for access a VLAN entry   
*/

/* BIT[19:31] Reserved: Returns 0 upon read; must be written with 0.  */
#define INCA_IP_VLAN_TABLE_ENTRY_VLAN_RESERVED_MASK   0xFFF80000

/* BIT[0..11]: VLAN ID */
#define INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_POS      0
#define INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_WIDTH    12
#define INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK     0x00000FFF
#define INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(VALUE) (\
                                   ( (VALUE) >> INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_POS  ) & \
                                   ( (1 << INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_WIDTH) -1 ) )

#define INCA_IP_VLAN_TABLE_ENTRY_SET_VLAN_ID(VALUE) (\
                        ( ( (1 << INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_WIDTH) - 1 ) & (VALUE) ) \
                                                    << INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_POS )

/* BIT[12]: Membership information for PC Port 
      0 Packets from PC port is not a member of VLAN
      1 Packets from PC port is member of this VLAN   */
#define INCA_IP_VLAN_TABLE_ENTRY_MP       12
#define INCA_IP_VLAN_TABLE_ENTRY_MP_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_MP)

/* BIT[13]: Membership information for LAN Port 
      0 Packets from LAN port is not a member of VLAN
      1 Packets from LAN port is member of this VLAN  */
#define INCA_IP_VLAN_TABLE_ENTRY_ML       13
#define INCA_IP_VLAN_TABLE_ENTRY_ML_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_ML)

/* BIT[14]: Membership information for CPU Port 
      0 Packets from CPU port is not a member of VLAN
      1 Packets from CPU port is member of this VLAN  */
#define INCA_IP_VLAN_TABLE_ENTRY_MC       14
#define INCA_IP_VLAN_TABLE_ENTRY_MC_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_MC)

/* BIT[15]: Egress Tagging information for PC Port
      0 Packets to PC egress port is not tagged for this VLAN
      1 Packets to PC egress port is tagged for this VLAN      */
#define INCA_IP_VLAN_TABLE_ENTRY_EP       15
#define INCA_IP_VLAN_TABLE_ENTRY_EP_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_EP)

/* BIT[16]: Egress Tagging information for LAN Port
      0 Packets to LAN egress port is not tagged for this VLAN
      1 Packets to LAN egress port is tagged for this VLAN     */
#define INCA_IP_VLAN_TABLE_ENTRY_EL       16
#define INCA_IP_VLAN_TABLE_ENTRY_EL_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_EL)

/* BIT[17]: Egress Tagging information for CPU Port
      0 Packets to CPU egress port is not tagged for this VLAN
      1 Packets to CPU egress port is tagged for this VLAN        */
#define INCA_IP_VLAN_TABLE_ENTRY_EC       17
#define INCA_IP_VLAN_TABLE_ENTRY_EC_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_EC)

/* BIT[18]: Valid
      0 Current entry is invalid
      1 Current entry is valid   */
#define INCA_IP_VLAN_TABLE_ENTRY_V        18
#define INCA_IP_VLAN_TABLE_ENTRY_V_MASK   (1<<INCA_IP_VLAN_TABLE_ENTRY_V)


/* access the raw control field */
/* BIT[12..18]: control flags */
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_BEGIN_POS    12
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_BEGIN_WIDTH  7
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_MASK         0x0007F000
#define INCA_IP_VLAN_TABLE_ENTRY_GET_CTRL_FLAGS(VALUE) (\
                                   ( (VALUE) >> INCA_IP_VLAN_TABLE_ENTRY_CTRL_BEGIN_POS  ) & \
                                   ( (1 << INCA_IP_VLAN_TABLE_ENTRY_CTRL_BEGIN_WIDTH) -1 ) )

#define INCA_IP_VLAN_TABLE_ENTRY_SET_CTRL_FLAGS(VALUE) (\
                        ( ( (1 << INCA_IP_VLAN_TABLE_ENTRY_CTRL_BEGIN_WIDTH) - 1 ) & (VALUE) ) \
                                                    << INCA_IP_VLAN_TABLE_ENTRY_CTRL_BEGIN_POS )

#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP  0
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP)
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_ML  1
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_ML_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_ML)
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC  2
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC)
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP  3
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP)
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_EL  4
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_EL_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_EL)
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_EC  5
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_EC_MASK  (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_EC)
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_V   6
#define INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK   (1<<INCA_IP_VLAN_TABLE_ENTRY_CTRL_V)

/* END: definitions for access a VLAN entry */


/*
   structure to store and handle a VLAN tag 
*/
typedef struct sw21553_vlan_tag
{
    USHORT wTpid;   /* Tag Protocol Identifier (VLAN_TPID) */
    USHORT wTci;    /* Tag Control Information (VLAN_TCI)  */
} sw21553_VLanTag ;



/************************************************
*  sw21553_VLanControl
*     control informations for VLAN processing
*
*     If software supported VLAN is enabled the VLAN tag will be created
*     by software with 
*        vLanId
*        vLanPrio
*
*     vLanTableEntryCount:    current count of VLAN table entries
*     vLanTableCurrentMaxId:  current max VLAN ID in the VLAN table
*     vLanTable:              local VLAN table
*
*  REMARK:
*     The VLAN table must be set in descending order. Adding and remove of 
*     settings requires an reordering of the table.
*     This will be done on the local VLAN table and after that the INCA IP 
*     VLAN table will be updated.
*/
typedef struct sw21553_vlan_control 
{
   /*---------------------*/
   /* local configuration */
   /*---------------------*/
   /* VLAN Tag Control Info: own VLAN_ID / own Prio */
   USHORT vLanId ;
   USHORT vLanPrio ;

   UINT32 vLanTableEntryCount ;
   UINT32 vLanTableCurrentMaxId ;
   UINT32 vLanTable[INCA_IP_MAX_VLAN_TABLE_ENTRIES] ;

} sw21553_VLanControl ;



/* number of Diff Serv Code Points */
#define INCA_IP_COS_MAX_DSCP      64

#define INCA_IP_COS_RX_WM1_INDEX    0
#define INCA_IP_COS_RX_WM2_INDEX    1
#define INCA_IP_COS_RX_WM3_INDEX    2
#define INCA_IP_COS_RX_WM4_INDEX    3

typedef union sw21553_cos_rx_watermarks
{
   UCHAR    rxWaterMark[4] ;
   UINT32   rxWaterMarksReg ;
}sw21553_CoSRxWatermarks ;


/* definition of an MAC entry used within the INCA-IP switch module */
typedef struct sw21553_cos_control 
{
   /*---------------------*/
   /* local configuration */
   /*---------------------*/

   /* mapping of DSCP <--> CoS */
   UCHAR  dscpPrioTable[INCA_IP_COS_MAX_DSCP] ;

} sw21553_CoSControl ;


/* 
   struct to handle the INCA-IP switch module 
      bSoftWareProcessVLanTag: VLAN processing can be done by software
      bL2PrioOnly:      the MAC frame is extended only if l3 priority is set > 0
      bAddLocalVLanTag: Software adds always fixed VLAN tag

*/
typedef struct sw21553_incaip_sw_control 
{
   int  bSoftWareProcessVLanTag ;
   int  bL2PrioOnly ;
   int  bAddLocalVLanTag ;

   /* semaphore for protect the MAC processing while normal operations */
   SEM_ID   endAdrFormSem ;

   /* control structure for VLAN */
   sw21553_VLanControl  vLanCntl ;

   /* control structure for CoS */
   sw21553_CoSControl   coSCntl ;

} sw21553_IncaIpSwControl ;

/******************************************************************************
   SWITCH ACCESS: IO definitions
******************************************************************************/

/* IO cntrl struct for update switch register 
   the current register value is read and only the marked settings within the 
   "updRegMask" will be changed to the newRegSettings 
*/
typedef struct sw21553_sw_io_reg_upd
{
   UINT32 newRegSettings ;
   UINT32 updRegMask ;
} sw21553_SwIoRegUpd ;

/* IO cntrl struct for set port selected VLAN config --> sw21553_SwVLanSetVLanConfig() */
typedef struct sw21553_sw_io_vlan_cfg
{
   UINT32 portSelect ;     /* select port: PC = 0, LAN = 1, CPU = 2 */
   UINT32 newPrio ;        /* new Default Priority for selected Port */
   UINT32 newVLanId ;      /* new Default VLAN ID for selected Port */
} sw21553_SwIoVLanCfg ;

/* IO cntrl struct for set port selected VLAN_CTRL --> sw21553_SwVLanIOVLanCntrl() */
typedef struct sw21553_sw_io_vlan_ctrl
{
   UINT32 portSelect ;           /* select port: PC = 0, LAN = 1, CPU = 2 */
   UINT32 sendOnlyToMember ;     /* 1: only send packets to VLANs of which it is a member */
   UINT32 notAcceptUntaggedPkt ; /* 1: Untagged packets will not be accepted on this port */
} sw21553_SwIoVLanCtrl ;

/* IO cntrl struct for set port selected WFQ Ctl --> sw21553_SwCoSSetWFQCtl() */
typedef struct sw21553_sw_io_cos_wfq_ctl
{
   UINT32 portSelect ;        /* select port: PC = 0, LAN = 1, CPU = 2 */
   int    scheme ;            /* Scheduling scheme: 
                                       00b = WFQ scheduling
                                       11b = Strict priority scheduling */
   int    prioQueue_0 ;       /* 0/1:  Priority for Queue 1   (scheme = 11b) */
   int    prioQueue_1 ;       /* 0/1:  Priority for Queue 2   (scheme = 11b) */
   int    weightQueue_0 ;     /* 0..7: WFQ weight for Queue 0 (scheme = 00b) */
   int    weightQueue_1 ;     /* 0..7: WFQ weight for Queue 1 (scheme = 00b) */
} sw21553_SwIoCoSWfqCtl ;

/* IO cntrl struct for set port selected and global RX watermarks 
   --> sw21553_SwCoSSetRxWatermarks() 
   --> sw21553_SwCoSSetGlobalWatermarks()
*/
typedef struct sw21553_sw_io_cos_wm
{
   UINT32 portSelect ;     /* select port: PC = 0, LAN = 1, CPU = 2 */
   UCHAR  rxWm1 ;          /* watermark reached: sends pause frames (full duplex mode) 
                                 or performs collisions (half duplex mode). */
   UCHAR  rxWm2 ;          /* watermark reached: only critical packets are admitted 
                                 if so enabled by the PDN bit in the PM_CTL. 
                                 Else non critical packets are admitted until
                                 RX_WM3 is reached and PM stops issuing pbnums. */
   UCHAR  rxWm3 ;          /* watermark reached: PM stops issuing pbnums to the RX module. */
   UCHAR  rxWm4 ;          /* Once rx_cnt drops below this watermark, MAC stops flow control. */
} sw21553_SwIoCoSWm ;

/* switch IO commands */
typedef enum {
   eSwIoSetPortVLanConfig ,
   eSwIoSetVlanCntrl ,
   eSwIoSetVlanCntrl_VA ,
   eSwIoSetArlCtl,
   eSwIoUpdArlCtl,
   eSwIoUpdArlCtl_ENL,
   eSwIoUpdArlCtl_MAF,
   eSwIoUpdArlCtl_SRC,
   eSwIoUpdArlCtl_MRO,
   eSwIoUpdArlCtl_CG,      /* ARL_CTL[9]: CRC GEN for CPU Port */
   eSwIoSetPMacHdCtl,
   eSwIoUpdMacHdCtl,
   eSwIoUpdMacHdCtl_ADD,   /* PMAC_HD_CTL[0]: ADD Header to Packets from DMA to PMAC */
   eSwIoUpdMacHdCtl_TAG,   /* PMAC_HD_CTL[1]: Add TAG to Packets from DMA to PMAC */
   eSwIoSetPrioCoS ,
   eSwIoSetDscpCos1 ,
   eSwIoSetDscpCos2 ,
   eSwIoSetPortWfqCtl ,
   eSwIoSetPortRxWm ,
   eSwIoSetGlobalRxWm
} sw21553_SwIoCntlCmd ;


/* INCA-IP switch MIB counters. 
 * REMARKS: 
 * - MAX_PKT_LEN is defined in register PKT_LEN.
 * - The following counters are not required for the CPU port and will 
 *   therefore not be supported: rxCRCErrorPkts, rxUnderSizeGoodPkts,
 *   rxFragments, rxOversizeBadPkts, rxAlignmentErrors,rxErrors, 
 *   txLateCollisions, txCollisions, txSingleCollisions, txMultipleCollisions,
 *   txExcessCollisions, txDeferredPkts, txGoodPausePkts
 *   The content of these fields is /undefined/ after calling function 
 *   sw21553_swGetMIBTable
 * - For the receive unicast, multicast and broadcast counters, pause packets 
 *   are separately counted in the pause counters. Counters do not increment
 *   for disabled ports (separate in the drop counters).
 */
typedef struct _sw21553_MIBCounterTable {
    UINT32  rxBytes;
    UINT32  rxUnicastPkts;
    UINT32  rxBroadcastPkts;
    UINT32  rxMulticastPkts;
    UINT32  rxCRCErrorPkts;
    UINT32  rxDroppedPkts;
    UINT32  rxUnderSizeGoodPkts;  /* Packet length < 64 */
    UINT32  rxFragments;          /* Packet length < 64 bytes with CRC error */
    UINT32  rxOversizeGoodPkts;   /* Packet length > MAX_PKT_LEN bytes */
    UINT32  rxOversizeBadPkts;    /* Packet length > MAX_PKT_LEN bytes */
    UINT32  rxGoodBytes;
    UINT32  rxGoodPausePkts;
    UINT32  rxAlignmentErrors;
    UINT32  rxErrors;
    UINT32  rx64BytePkts;         /* Packet length == 64 bytes */
    UINT32  rx127BytePkts;        /* 64 < Packet length <= 127 */
    UINT32  rx255BytePkts;        /* 127 < Packet length <= 255 */
    UINT32  rx511BytePkts;        /* 255 < Packet length <= 511 */
    UINT32  rx1023BytePkts;       /* 511 < Packet length <= 1023 */
    UINT32  rxMaxBytePkts;        /* 1023 < Packet length <= MAX_PKT_LEN */
    UINT32  txLateCollisions;
    UINT32  txGoodBytes;
    UINT32  txUnicastPkts;
    UINT32  txBroadcastPkts;
    UINT32  txMulticastPkts;
    UINT32  txCollisions;
    UINT32  txSingleCollisions;
    UINT32  txMultipleCollisions; /* Collision count from 1-16 */
    UINT32  txExcessCollisions;   /* Collision Count > 16 */
    UINT32  txDeferredPkts;
    UINT32  txGoodPausePkts;
} sw21553_MIBCounterTable;

#define INCA_IP_Switch_MIB_BASE_ADDRESS			        (INCA_IP_Switch + 0x0600 )
#define INCA_IP_Switch_PORT_MIB_BASE_ADDRESS( port )	(INCA_IP_Switch_MIB_BASE_ADDRESS + (port * 0x80))
#define INCA_IP_Switch_PORT_MIB_ADDRESS( port, offset )	(VREG32((INCA_IP_Switch_MIB_BASE_ADDRESS + (port * 0x80))+offset))
#define INCA_IP_Switch_MIB_rxBytes              0x00
#define INCA_IP_Switch_MIB_rxUnicastPkts        0x04
#define INCA_IP_Switch_MIB_rxBroadcastPkts      0x08
#define INCA_IP_Switch_MIB_rxMulticastPkts      0x0C
#define INCA_IP_Switch_MIB_rxCRCErrorPkts       0x10
#define INCA_IP_Switch_MIB_rxDroppedPkts        0x14
#define INCA_IP_Switch_MIB_rxUnderSizeGoodPkts  0x18
#define INCA_IP_Switch_MIB_rxFragments          0x1C
#define INCA_IP_Switch_MIB_rxOversizeGoodPkts   0x20
#define INCA_IP_Switch_MIB_rxOversizeBadPkts    0x24
#define INCA_IP_Switch_MIB_rxGoodBytes          0x28
#define INCA_IP_Switch_MIB_rxGoodPausePkts      0x2C
#define INCA_IP_Switch_MIB_rxAlignmentErrors    0x30
#define INCA_IP_Switch_MIB_rxErrors             0x34
#define INCA_IP_Switch_MIB_rx64BytePkts         0x38
#define INCA_IP_Switch_MIB_rx127BytePkts        0x3C
#define INCA_IP_Switch_MIB_rx255BytePkts        0x40
#define INCA_IP_Switch_MIB_rx511BytePkts        0x44
#define INCA_IP_Switch_MIB_rx1023BytePkts       0x48
#define INCA_IP_Switch_MIB_rxMaxBytePkts        0x4C
#define INCA_IP_Switch_MIB_txLateCollisions     0x50
#define INCA_IP_Switch_MIB_txGoodBytes          0x54
#define INCA_IP_Switch_MIB_txUnicastPkts        0x58
#define INCA_IP_Switch_MIB_txBroadcastPkts      0x5C
#define INCA_IP_Switch_MIB_txMulticastPkts      0x60
#define INCA_IP_Switch_MIB_txCollisions         0x64
#define INCA_IP_Switch_MIB_txSingleCollisions   0x68
#define INCA_IP_Switch_MIB_txMultipleCollisions 0x6C
#define INCA_IP_Switch_MIB_txExcessCollisions   0x70
#define INCA_IP_Switch_MIB_txDeferredPkts       0x74
#define INCA_IP_Switch_MIB_txGoodPausePkts      0x78

/* -------------------------------------------------------------------------- */
/* Basic macro definitions                                                    */
/* -------------------------------------------------------------------------- */


#define SW_WRITE_REG(reg, value)   \
         *((volatile u32*)reg) = (u32)value;\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         *((volatile u32*)reg) = (u32)value;

#define SW_READ_REG(reg, value)    \
         value = (u32)*((volatile u32*)reg);\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         value = (u32)*((volatile u32*)reg);




static void nopRepeatTen (void)
{
         asm("nop;nop;nop;nop;nop"); \
         asm("nop;nop;nop;nop;nop"); \
};

#if 0
/* write and read to the Switch module register */
#define SW_WRITE_REG_LOCKED(reg, value, semaphoreId)   \
         semTake(semaphoreId, WAIT_FOREVER); \
         *((volatile UINT32*)reg) = (UINT32)value;\
         nopRepeatTen(); \
         semGive(semaphoreId) ;

#define SW_READ_REG_LOCKED(reg, value, semaphoreId)    \
         semTake(semaphoreId, WAIT_FOREVER); \
         value = (UINT32)*((volatile UINT32*)reg);\
         nopRepeatTen();\
         value = (UINT32)*((volatile UINT32*)reg); \
         semGive(semaphoreId) ;

/* write and read to the Switch module register */
#define SW_WRITE_REG_UNLOCKED(reg, value)   \
         *((volatile UINT32*)reg) = (UINT32)value;\
         nopRepeatTen();

#define SW_READ_REG_UNLOCKED(reg, value)    \
         value = (UINT32)*((volatile UINT32*)reg);\
         nopRepeatTen();\
         value = (UINT32)*((volatile UINT32*)reg);

#else
/* write and read to the Switch module register */
#define SW_WRITE_REG_LOCKED(reg, value, dummy) {  \
         unsigned int flags; \
         save_and_cli(flags); \
         *((volatile UINT32*)reg) = (UINT32)value; \
         nopRepeatTen(); \
         restore_flags(flags); }

#define SW_READ_REG_LOCKED(reg, value, dummy) {   \
         unsigned int flags; \
         save_and_cli(flags); \
         value = (UINT32)*((volatile UINT32*)reg); \
         nopRepeatTen(); \
         value = (UINT32)*((volatile UINT32*)reg); \
         restore_flags(flags); }

#define SW_WRITE_REG_UNLOCKED(reg, value)  SW_WRITE_REG_LOCKED(reg, value, NULL)
#define SW_READ_REG_UNLOCKED(reg, value)   SW_READ_REG_LOCKED(reg, value, NULL)
#endif



/******************************************************************************
   Switch/END LED's ACCESS: definitions
******************************************************************************/

#define INCA_IP_SWITCH_LEDREG_LAN_STATUS     INCA_IP_TSF_LED_REG_L35
#define INCA_IP_SWITCH_LEDREG_LAN_DUPLEX     INCA_IP_TSF_LED_REG_L25
#define INCA_IP_SWITCH_LEDREG_LAN_ACT        INCA_IP_TSF_LED_REG_L15
#define INCA_IP_SWITCH_LEDREG_LAN_SPEED      INCA_IP_TSF_LED_REG_L05

#define INCA_IP_SWITCH_LEDREG_PC_STATUS      INCA_IP_TSF_LED_REG_L34
#define INCA_IP_SWITCH_LEDREG_PC_DUPLEX      INCA_IP_TSF_LED_REG_L24
#define INCA_IP_SWITCH_LEDREG_PC_ACT         INCA_IP_TSF_LED_REG_L14
#define INCA_IP_SWITCH_LEDREG_PC_SPEED       INCA_IP_TSF_LED_REG_L04




/******************************************************************************
   PHY ACCESS: functions
******************************************************************************/

UINT32 sw21553_Write_MDIO_ACC( UINT32   value);

/* returns the selected PHY properties
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      phyInfo: specify properties - 0: current , 1: advertised 2: partner 
*/

#if 0
UINT32 sw21553_PhyGetProperties(int portSelect, int phyInfo) ;
/* set the advertise properties
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      newSettings: bit[0..4] - 10MBit/s HD | 10MBit/s FD | 100MBit/s HD | 100MBit/s FD 
                   bit 4-7 - no used
                   bit 8: pause operation
*/


UINT32 sw21553_PhySetAdvertiseProperties(int portSelect, UINT32 newSettings) ;
/* set the fixed properties (without auto neg)
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      newSpeed:   001B:  10MBit/s HD  
                  101B:  10MBit/s FD 
                  010B: 100MBit/s HD  
                  110B: 100MBit/s FD 
*/
UINT32 sw21553_PhySetFixSpeed(int portSelect, UINT32 newSpeed) ;
/* disable / enable auto negotiation 
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      disableAutoNeg: 0 -> enable, 1 -> disable
*/
UINT32 sw21553_PhyDisableAutoNeg(int portSelect, int disableAutoNeg) ;
/* restart the auto negotiation for the selected PHY
      portSelect: selects PHY (LAN = 0 / PC = 1)
*/
int sw21553_PhyRestartAutoNeg(int portSelect) ;
/* reset the selected PHY
      portSelect: selects PHY (LAN = 0 / PC = 1)
*/
int sw21553_PhyReset(int portSelect) ;
#endif
/******************************************************************************
   SWITCH ACCESS: functions
******************************************************************************/

/* set and change the UN_DEST register (unknown destination)
   (only the settings marked in the "updMask" will be changed)
*/ 
UINT32 sw21553_SwIOUnDest(UINT32 newSettings, UINT32 updMask, int setFlag) ;

/* set and change the UN_DEST register */
UINT32 sw21553_SwUpdUnDestPorts(int portSelect, int portBehaviour) ;

/* set and change the SP_PT register (Spanning Tree Port Status) 
   (only the settings marked in the "updMask" will be changed)
*/
UINT32 sw21553_SwIOSpPt(UINT32 newSettings, UINT32 updMask, int setFlag) ;


/* update the port state within the Spanning Tree Port Status */
UINT32 sw21553_SwUpdSpPtPortState(int portSelect, int newPortState) ;

/* set and change the PKT_LEN register (Packet Length Register) */
UINT32 sw21553_SwIOPacketLen(UINT32 newPktLen, int addTagLen, int setFlag) ;


/* set and change the address resolution handling (ARL) of the MAC 
   (only the settings marked in the "updMask" will be changed)
*/
UINT32 sw21553_SwSetArlControl(UINT32 newSettings, UINT32 updMask) ;

/* set and change the CPU port control settings 
   (only the settings marked in the "updMask" will be changed)
*/
UINT32 sw21553_SwSetCpuPControl(UINT32 newSettings, UINT32 updMask) ;

/* set and change the PM Header Control Register 
   (only the settings marked in the "updMask" will be changed)
*/
UINT32 sw21553_SwSetPMacHdCtl(UINT32 newSettings, UINT32 updMask) ;

/* set and change the MDIO_CFG settings 
   (only the settings marked in the "updMask" will be changed)
*/
UINT32 sw21553_SwSetMdioConfig(UINT32 newSettings, UINT32 updMask) ;

/* set and change the EPHY register (Ethernet PHY Register) 
   (only the settings marked in the "updMask" will be changed)
*/
UINT32 sw21553_SwSetEPhy(UINT32 newSettings, UINT32 updMask) ;

/* MDIO Configuration : reset MDIO state maschine */
UINT32 sw21553_SwResetMDIO(void) ;

/* CPU Port Control: enable DA lookup for the the CPU port */
UINT32 sw21553_SwCpuPCtlEnableDa(void) ;

/* CPU Port Control: disable DA lookup for the the CPU port */
UINT32 sw21553_SwCpuPCtlDisableDa(int newDaPort, int newDAC) ;

/* set the given MAC address to the PMAC */
void sw21553_SwSetPMacMacAddr(UCHAR inAddr[6] ) ;

/* set the given MAC address to the PMAC */
int sw21553_SwGetPMacMacAddr(UCHAR *pMacAddr) ;

/* read the MAC entry from the MAC table
      index: table index of the requested entry
      pOutMacTableEntry: ptr to the return structure
   Returns:
      0: no entry found, 1: one entry found, -1: ERROR ptr or index not valid 
*/
int sw21553_SwMacReadTableEntry(int index, sw21553_MacTableEntry *pOutMacTableEntry ) ;

/* convert an given MAC address string into a byte stream
   strMacAddr: ptr to string of the form "00:11:22:aa:77:cc:99"
   pRetHexMacAddr: return value ptr to UCHAR[6] array
*/
void sw21553_SwMacConvertStr2Hex(char *strMacAddr, UCHAR *pRetHexMacAddr) ;

/* set the control properties of an existing MAC table entry
      port:       0: PC port, 1: LAN port, 2: CPU port VOICE, 3: CPU port MANAGEMENT
      classification: 0: normal packets, 1: critical packets
      aging:      0: no aging, 1: normal aging
      pCntrl:     ptr to the MAC entry control byte
   Returns: 0 (OK), -1 (ERROR)
*/
int sw21553_SwMacSetTableEntryCntrl(int port, int classification, int aging, UCHAR *pCntrl) ;

/* Add an given MAC table entry to the MAC table.
      inMacTableEntry: MAC address and appropriate control structures
      overWrite:       add this entry always (overwrite existing)
   Returns: 0 (OK), -1 (ERROR)
*/
int sw21553_SwMacAddEntry(sw21553_MacTableEntry inMacTableEntry, int overWrite) ;

/* Clear an given MAC table entry identified by its index
      index: index of the MAC table entry
   Returns: 0 (OK), -1 (ERROR)
*/
int sw21553_SwMacClearEntryByIndex(int index) ;

/* Clear MAC table entries identified by a given MAC address
      inMacTableEntry: MAC address 
                       control (0 --> clear all entries with this MAC
                                else clear only MAC's for the marked ports)
   Returns: 0 (OK), -1 (ERROR)
*/
int sw21553_SwMacClearEntryByMac(sw21553_MacTableEntry inMacTableEntry) ;

/* reset the complete MAC table (write 0x00FFFFFF 0xFFFFFFFF --> unused) */
int sw21553_SwMacResetMacTable(void) ;

/* write access to the INCA IP switch register for configuration 
   currently implemented command:
      eSwIoSetPortVLanConfig: set the VLAN config (VLAN ID, PRIO)
      eSwIoSetVlanCntrl:   set the VLAN control (symetrical, send only tagged packets)
      eSwIoSetVlanCntrl_VA: enable VLAN
      eSwIoSetArlCtl:      set new settings of the ARL_CTL register
      eSwIoUpdArlCtl:      update the settings of the ARL_CTL register
      eSwIoUpdArlCtl_ENL:  ARL_CTL[0]: Enable Learning from LAN Port
      eSwIoUpdArlCtl_MAF:  ARL_CTL[1]: MA Freeze
      eSwIoUpdArlCtl_SRC:  ARL_CTL[6]: Send Reserved Packets to CPU
      eSwIoUpdArlCtl_MRO:  ARL_CTL[7]: MA Replace Oldest Entry
      eSwIoUpdArlCtl_CG:   ARL_CTL[9]: CRC GEN for CPU Port
      eSwIoSetPMacHdCtl:   set new settings of the PMAC_HD_CTL register
      eSwIoUpdMacHdCtl:    update the settings of the PMAC_HD_CTL register
      eSwIoUpdMacHdCtl_ADD: PMAC_HD_CTL[0]: ADD Header to Packets from DMA to PMAC
      eSwIoUpdMacHdCtl_TAG: PMAC_HD_CTL[1]: Add TAG to Packets from DMA to PMAC
      eSwIoSetPrioCoS:     set new settings of the PRI_CoS register
      eSwIoSetDscpCos1:    set new settings of the DSCP_COS1 register
      eSwIoSetDscpCos2:    set new settings of the DSCP_COS2 register
      eSwIoSetPortWfqCtl:  set new settings for WFQ Control Register
      eSwIoSetPortRxWm:    set new settings for RX watermarks
      eSwIoSetGlobalRxWm:  set new settings for GLOBAL watermarks
*/
UINT32 sw21553_SwIoSetControl(int command, void *pdata) ;

/* init the local VLAN and CoS control struct with the default values */
int sw21553_SwVLanCoSInitControl(void) ;
/* clear the allocated parts of the local "sw21553_VlanCoSCtrl" struct */
void sw21553_SwVLanCoSClearControl(void) ;



/* set new "PRIO" and "VALN_ID" for the selected port
      port:  0 = PC port, 1 = LAN port, 2 = CPU port
      newPrio:    new priority value to set (range 0 - 7)
      newVLanId:  new VLAN ID to set (range 0 - 4096)
                  consider: 4096 (0xFFF): reserved, 0 (0x000): priority tag 
*/
UINT32 sw21553_SwVLanSetVLanConfig(int portSelect, UINT32 newPrio, UINT32 newVLanId) ;
/* get or set and change the VLAN_CTRL register (VLAN Control Register) */
UINT32 sw21553_SwVLanIOVLanCntrl(UINT32 newSettings, UINT32 updMask, int getFlag) ;

/* set and change the PRIO_CoS register (Priority CoS Mapping Register) */
UINT32 sw21553_SwCoSSetPrioCos(UINT32 newSettings) ;
/* set and change the DSCP_COS1/DSCP_COS1 registers (DSCP CoS Mapping Register 1/2) */
UINT32 sw21553_SwCoSSetDscpCoS1(UINT32 newDscpCoS1 ) ;
UINT32 sw21553_SwCoSSetDscpCoS2(UINT32 newDscpCoS2 ) ;
/* set new settings for CoS "Weighted Fair Queuing" */
UINT32 sw21553_SwCoSSetWFQCtl( int portSelect,   int SchedulingScheme ,
                               int WFQWeightQ_0, int WFQWeightQ_1,
                               int PrioQ_0,      int PrioQ_1 ) ;
/* set new settings for CoS RX watermarks */
UINT32 sw21553_SwCoSSetRxWatermarks( int portSelect, 
                                     UCHAR rxWm1, UCHAR rxWm2, UCHAR rxWm3, UCHAR rxWm4 ) ;
/* set new settings for CoS global watermark register */
UINT32 sw21553_SwCoSSetGlobalWatermarks( UCHAR rxWm1, UCHAR rxWm2, 
                                         UCHAR rxWm3, UCHAR rxWm4 ) ;

/* set new settings for PM control register (drop non critical packets) */										 
UINT32 sw21553_SwNonCritPacketDrop( BOOL gdn, BOOL cdn, BOOL ldn, BOOL pdn );										 
										 
/* read the port specific VLAN register settings */
UINT32 sw21553_SwVLanGetVLanConfig(int portSelect) ;

/* read the VLAN entry from the VLAN table */
UINT32 sw21553_SwVLanReadTableEntry(int index) ;

/* write a VLAN entry to the VLAN table */
int sw21553_SwVLanWriteTableEntry(int index, sw21553_VLanTableEntry inVLanTableEntry ) ;

/* write a VLAN entry (given by his register value) to the VLAN table */
int sw21553_SwVLanWriteTableEntryReg32(int index, UINT32 newRegEntry ) ;

/* clear a VLAN entry from the VLAN table */
int sw21553_SwVLanClearTableEntryByIndex(int index ) ;

/* reset the complete VLAN table (write 0x0007FFFF --> unused) */
int sw21553_SwVLanResetVLanTable(void) ;

/* Calculate the table index and add a given VLAN entry to the VLAN table
   Remark: 
      Because the VLAN table entries must be in a increasing order a local 
      copy of the current VLAN table will be used for sorting.
      After insertion of a new entry into the local VLAN table the
      INCA IP VLAN table will be updated.
*/
int sw21553_SwVLanAddTableEntry(sw21553_VLanTableEntry inVLanTableEntry) ;

/* read and return the current VLAN table */
UINT32 sw21553_SwVLanReadTable(UINT32 *pVLanTable, int localVlanTable) ;



#if (INCA_IP_SUPPORT_VLAN_SOFTWARE_PROCESSING == TRUE)

/* set new "PRIO" and "VALN_ID" for software VLAN */
int sw21553_SwVLanSetLocalVLanTag(UINT32 newPrio, UINT32 newVLanId) ;

/* read and return "PRIO" and "VALN_ID" for software VLAN */
int sw21553_SwVLanGetLocalVLanTag(UINT32 *pOutPrio, UINT32 *pOutVLanId) ;

/* set the control settings for software VLAN handling */
void sw21553_SwVLanSetLocalVLanCtrlTag( int bSoftWareProcessVLanTag,
                                        int bL2PrioOnly,
                                        int bAddLocalVLanTag) ;

M_BLK_ID sw21553_EndEtherAddressForm( M_BLK_ID pNBuff,
                                      M_BLK_ID pSrcAddr, 
                                      M_BLK_ID pDstAddr, 
                                      BOOL bcastFlag) ;
STATUS sw21553_EndEtherPacketDataGet(M_BLK_ID pMblk, LL_HDR_INFO * pLinkHdrInfo) ;
#endif


#ifdef INCLUDE_LED_MULTIPLEXER
/******************************************************************************
   Switch/END LED's ACCESS: functions
******************************************************************************/

/* Set the new Switch/END configuration of the LED multiplexer. */
UINT32 sw21553_SwLedSetLedMuxConfig(UINT32 newSettings, UINT32 updMask) ;

/* Assign the Switch/END signals to the LEDs */
UINT32 sw21553_SwLedAssignLeds( UINT32 portSelect, 
                                int status, int speed, int Duplex, int Activity ) ;

#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */

/******************************************************************************
   Switch MIB functions
******************************************************************************/
/* read and return the MIB counter values */
int sw21553_SwGetMIBTable( unsigned int portSelect, sw21553_MIBCounterTable *pMIBTable );

UINT32 sw21553_Read_CPU_MIB_RxDropped(void);


#endif      /* #ifndef _INCASWITCHACCESS_H */
