/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               

   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : IncaSwitchAmdix.c
   Date        : 26.09.2003
   Description :
   
   $Log: inca_switch_amdix.c,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.1  2004/02/24 12:38:22  de6wanme
   Initial Auto-MDIX support added


******************************************************************************/

/* includes */
#if 0
#include "vxWorks.h"
#include "stdlib.h"
#include "cacheLib.h"
#include "intLib.h"
#include "lstLib.h"        /* Needed to maintain protocol list. */
#include "wdLib.h"
#include "iv.h"
#include "semLib.h"
#include "logLib.h"
#include "netLib.h"
#include "stdio.h"
#include "sysLib.h"
#include "errno.h"
#include "errnoLib.h"
#include "memLib.h"
#include "iosLib.h"

#include "tickLib.h"
#include "timers.h"

#include "sys/socket.h"
#include "sys/ioctl.h"
#include "sys/times.h"
#include "RegIncaIP.h"


#endif

#include <linux/spinlock.h>
#include "vxWorks_type.h"

#include "inca_switch_phy.h"
#include "inca_switch_amdix.h"

/* DEBUG MACROS */
#ifdef AMDIX_DEBUG_ENABLE
   /* debug print out masks */
   #define AMDIX_DEBUG_OFF          0x00000000
   #define AMDIX_DEBUG_INIT         0x00000001
   #define AMDIX_DEBUG_INT          0x00000040
   #define AMDIX_DEBUG_FSM          0x00000100
   #define AMDIX_DEBUG_FULL         0xFFFFFFFF

   #define AMDIX_DEBUG_MASK         (AMDIX_DEBUG_FSM | AMDIX_DEBUG_INT | AMDIX_DEBUG_INIT) 

   #define AMDIX_DEBUG(type,X)         if((UINT32)type & AMDIX_DEBUG_MASK) printf X
   #define AMDIX_LOG(type,S)           if((UINT32)type & AMDIX_DEBUG_MASK) logMsg(S,0,0,0,0,0,0)
   #define AMDIX_LOG1(type,S,P1)       if((UINT32)type & AMDIX_DEBUG_MASK) logMsg(S,P1,0,0,0,0,0)
   #define AMDIX_LOG2(type,S,P1,P2)    if((UINT32)type & AMDIX_DEBUG_MASK) logMsg(S,P1,P2,0,0,0,0)
   #define AMDIX_LOG3(type,S,P1,P2,P3) if((UINT32)type & AMDIX_DEBUG_MASK) logMsg(S,P1,P2,P3,0,0,0)
   #define AMDIX_LOG4(type,S,P1,P2,P3,P4)    if((UINT32)type & AMDIX_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,0,0)
   #define AMDIX_LOG5(type,S,P1,P2,P3,P4,P5) if((UINT32)type & AMDIX_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,P5,0)

   #undef LOCAL
   #define LOCAL

#else
   #define AMDIX_DEBUG(type,X)          
   #define AMDIX_LOG(type,S)            
   #define AMDIX_LOG1(type,S,P1)        
   #define AMDIX_LOG2(type,S,P1,P2)     
   #define AMDIX_LOG3(type,S,P1,P2,P3)
   #define AMDIX_LOG4(type,S,P1,P2,P3,P4)
   #define AMDIX_LOG5(type,S,P1,P2,P3,P4,P5)

#endif /* AMDIX_DEBUG_ENABLE */


swAutoMDIXstruct AutoMDIX[2]; 

/*-------------------------------------------------------*/
void sw21553_AmdixPcCallback(int event);
void sw21553_AmdixLanCallback(int event);


/*******************************************************************************
*
* sw21553_AmdixInit - initialize data structures for the AutoMDIX
*
* This function is called by the switch END Load function
*
* RETURNS: OK or ERROR.
*
*/
int sw21553_AmdixInit()
{
   AMDIX_DEBUG (AMDIX_DEBUG_INIT, ("AmdixInit() called!\n"));

   AutoMDIX[INCA_IP_PHY0_LAN].pGetAutoMDIXSupportPOL   = sysGetAutoMDIX_LAN_Support_POL ;
   AutoMDIX[INCA_IP_PHY0_LAN].pSetAutoMDIXModeStartup  = sysSetAutoMDIX_LAN_StartupMode ;
   AutoMDIX[INCA_IP_PHY0_LAN].pSetupAutoMDIXModeGPIO   = sysSetupAutoMDIX_LAN_GPIO ;
   AutoMDIX[INCA_IP_PHY0_LAN].pSetAutoMDIXModeMDI      = sysSetAutoMDIX_LAN_MDIMode ;
   AutoMDIX[INCA_IP_PHY0_LAN].pSetAutoMDIXModeMDIX     = sysSetAutoMDIX_LAN_MDIXMode ;
   AutoMDIX[INCA_IP_PHY0_LAN].pSetAutoMDIXModeRESET    = sysSetAutoMDIX_LAN_RESETMode ;
   AutoMDIX[INCA_IP_PHY0_LAN].pGetAutoMDIXMode         = sysGetAutoMDIX_LAN_Mode ;
   AutoMDIX[INCA_IP_PHY0_LAN].pCheckAutoMDIXPortSetup  = sysCheckAutoMDIX_LAN_PortSetup ;

   AutoMDIX[INCA_IP_PHY1_PC ].pGetAutoMDIXSupportPOL   = sysGetAutoMDIX_PC_Support_POL ;
   AutoMDIX[INCA_IP_PHY1_PC ].pSetAutoMDIXModeStartup  = sysSetAutoMDIX_PC_StartupMode ;
   AutoMDIX[INCA_IP_PHY1_PC ].pSetupAutoMDIXModeGPIO   = sysSetupAutoMDIX_PC_GPIO ;
   AutoMDIX[INCA_IP_PHY1_PC ].pSetAutoMDIXModeMDI      = sysSetAutoMDIX_PC_MDIMode ;
   AutoMDIX[INCA_IP_PHY1_PC ].pSetAutoMDIXModeMDIX     = sysSetAutoMDIX_PC_MDIXMode ;
   AutoMDIX[INCA_IP_PHY1_PC ].pSetAutoMDIXModeRESET    = sysSetAutoMDIX_PC_RESETMode ;
   AutoMDIX[INCA_IP_PHY1_PC ].pGetAutoMDIXMode         = sysGetAutoMDIX_PC_Mode ;
   AutoMDIX[INCA_IP_PHY1_PC ].pCheckAutoMDIXPortSetup  = sysCheckAutoMDIX_PC_PortSetup ;


	AutoMDIX[INCA_IP_PHY0_LAN].oldCallback = 
			sw21553_PhySetCallback(INCA_IP_PHY0_LAN, sw21553_AmdixLanCallback);
	sw21553_PhySetLinkRetry(INCA_IP_PHY0_LAN, sysGetAutoMDIX_LAN_LinkTries(), 
															sysGetAutoMDIX_PHY_ResetDelay());

	AutoMDIX[INCA_IP_PHY1_PC].oldCallback = 
			sw21553_PhySetCallback(INCA_IP_PHY1_PC, sw21553_AmdixPcCallback);
	sw21553_PhySetLinkRetry(INCA_IP_PHY1_PC, sysGetAutoMDIX_PC_LinkTries(), 
														  sysGetAutoMDIX_PHY_ResetDelay());


   return OK ;

}

/*******************************************************************************
*
* sw21553_AmdixStart - 
*
*/
void sw21553_AmdixStart(int port)
{
   swAutoMDIXstruct *pAutoMDIX; 

   pAutoMDIX = &(AutoMDIX[port]);
	
   if ( pAutoMDIX->pCheckAutoMDIXPortSetup() != 0 ) 
   {
      pAutoMDIX->pSetupAutoMDIXModeGPIO();
      pAutoMDIX->currMode  = pAutoMDIX->pSetAutoMDIXModeStartup() ;
      AMDIX_LOG2(AMDIX_DEBUG_INIT, 
      	"AmdixStart: Setting Port%d to Startup-Mode = %d!\n", port, pAutoMDIX->currMode);
   } 
   else
   {
      pAutoMDIX->currMode  = pAutoMDIX->pGetAutoMDIXMode() ;
      AMDIX_LOG2(AMDIX_DEBUG_INIT, 
      	"AmdixStart: Getting Port%d Mode = %d!\n", port, pAutoMDIX->currMode);
   }
}

/*******************************************************************************
*
* sw21553_AmdixEnergy - 
*
*/
void sw21553_AmdixEnergy(int port)
{
   swAutoMDIXstruct *pAutoMDIX; 

   pAutoMDIX = &(AutoMDIX[port]);
	
   /* change to Startup mode */
   pAutoMDIX->currMode = pAutoMDIX->pSetAutoMDIXModeStartup() ;
   AMDIX_LOG2(AMDIX_DEBUG_INIT, 
      	"AmdixEnergy: Setting Port%d to Startup-Mode = %d!\n", port, pAutoMDIX->currMode);
}

/*******************************************************************************
*
* sw21553_AmdixRetry - 
*
*/
void sw21553_AmdixRetry(int port)
{
   swAutoMDIXstruct *pAutoMDIX; 

   pAutoMDIX = &(AutoMDIX[port]);

   /* LFSR bit is 1 --> change auto MDIX settings */
   if (pAutoMDIX->currMode == INCA_IP_AUTOMDIX_MODE_MDI)
   {
      AMDIX_LOG1(AMDIX_DEBUG_FSM, "AmdixRetry: Port%d, TOGGLE to MDIX-mode!\n", port);
      /* current mode is MDI: change to MDIX mode */
      pAutoMDIX->pSetAutoMDIXModeMDIX();
      pAutoMDIX->currMode = INCA_IP_AUTOMDIX_MODE_MDIX;
   }
   else
   {
      AMDIX_LOG1(AMDIX_DEBUG_FSM, "AmdixRetry: Port%d, TOGGLE to MDI-mode!\n", port);
      /* current mode is MDIX: change to MDI mode */
      pAutoMDIX->pSetAutoMDIXModeMDI();
      pAutoMDIX->currMode = INCA_IP_AUTOMDIX_MODE_MDI;
   }
}

/*******************************************************************************
*
* sw21553_AmdixTriesTout - 
*
*
* RETURNS: OK or ERROR.
*
*/
void sw21553_AmdixTriesTout(int port)
{
   swAutoMDIXstruct *pAutoMDIX; 

   pAutoMDIX = &(AutoMDIX[port]);
	
   /* change to RESET mode */
   pAutoMDIX->pSetAutoMDIXModeRESET() ;
   pAutoMDIX->currMode = INCA_IP_AUTOMDIX_MODE_RESET ;
   AMDIX_LOG2(AMDIX_DEBUG_INIT, 
      	"AmdixTriesTout: Setting Port%d to Reset-Mode = %d!\n", port, pAutoMDIX->currMode);
}


/******************************************************************************
*  sw21553_AmdixPcCallback
*     Callback receiveing PHY events.
*
*  Parameter: 
*     event: PHY_INIT, ENERGY_DETECT. LINK_TRY_FAIL or LINK_TRIES_TOUT
*
*  Returns:
*
*  REMARK:
*
*/
void sw21553_AmdixPcCallback(int event)
{
	/* If there is another function in the callback chain 
		then call that one first */
	if (AutoMDIX[INCA_IP_PHY1_PC].oldCallback) 
		AutoMDIX[INCA_IP_PHY1_PC].oldCallback(event);

	/* process the Phy event */
	switch (event) {
	case PHY_INIT       : sw21553_AmdixStart(INCA_IP_PHY1_PC);     return;
	case ENERGY_DETECT  : sw21553_AmdixEnergy(INCA_IP_PHY1_PC);    break;
	case LINK_TRY_FAIL  : sw21553_AmdixRetry(INCA_IP_PHY1_PC);	   break;
	case LINK_TRIES_TOUT: sw21553_AmdixTriesTout(INCA_IP_PHY1_PC);	break;
	default:
		AMDIX_LOG1(AMDIX_DEBUG_FSM, 
				"AmdixPcCallback: unsupported event (%d)!\n",event);
		return;
	}

	sw21553_PhySetLinkRetry(INCA_IP_PHY1_PC, sysGetAutoMDIX_PC_LinkTries(), 
														  sysGetAutoMDIX_PC_StatusDelay());
}

/******************************************************************************
*  sw21553_AmdixLanCallback
*     Callback receiveing PHY events.
*
*  Parameter: 
*     event: PHY_INIT, ENERGY_DETECT. LINK_TRY_FAIL or LINK_TRIES_TOUT
*
*  Returns:
*
*  REMARK:
*
*/
void sw21553_AmdixLanCallback(int event)
{
	/* If there is another function in the callback chain 
		then call that one first */
	if (AutoMDIX[INCA_IP_PHY0_LAN].oldCallback) 
		AutoMDIX[INCA_IP_PHY0_LAN].oldCallback(event);

	/* process the Phy event */
	switch (event) {
	case PHY_INIT       : sw21553_AmdixStart(INCA_IP_PHY0_LAN);    return;
	case ENERGY_DETECT  : sw21553_AmdixEnergy(INCA_IP_PHY0_LAN);   break;
	case LINK_TRY_FAIL  : sw21553_AmdixRetry(INCA_IP_PHY0_LAN);	   break;
	case LINK_TRIES_TOUT: sw21553_AmdixTriesTout(INCA_IP_PHY0_LAN);break;
	default:
		AMDIX_LOG1(AMDIX_DEBUG_FSM, 
				"AmdixLanCallback: unsupported event (%d)!\n",event);
		return;
	}

	sw21553_PhySetLinkRetry(INCA_IP_PHY0_LAN, sysGetAutoMDIX_LAN_LinkTries(), 
														   sysGetAutoMDIX_LAN_StatusDelay());
}
