/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : IncaSwitchAmdix.h
   Date        : 01.12.2003
   Description :
   
   $Log: inca_switch_amdix.h,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.1  2004/02/24 12:38:23  de6wanme
   Initial Auto-MDIX support added



   
******************************************************************************/
#ifndef _INCASWITCHAMDIX_H
#define _INCASWITCHAMDIX_H

/* switch ON/OFF debugging, only for testing */
//#define AMDIX_DEBUG_ENABLE
#undef  AMDIX_DEBUG_ENABLE

/* -------------------------------------------------------------------------- */
/* Basic macro definitions                                                    */
/* -------------------------------------------------------------------------- */
/* tbd */

/* mode definitions of the Auto MDIX */
#define INCA_IP_AUTOMDIX_MODE_RESET       0
#define INCA_IP_AUTOMDIX_MODE_MDI         1
#define INCA_IP_AUTOMDIX_MODE_MDIX        2


/* control struct for handle the Ethernet PHY port (PC/LAN) */
typedef struct
{
   UINT32            AutoMDIX_ON ;  /* switch ON / OFF the autoMDIX feature */
   UINT32            currMode ;     /* RESET-mode: PC: DETECT RX and TX; LAN: MDI
                                       MDI-mode:   TX --> INCA_IP RX
                                                   incoming RX --> INCA_IP TX
                                       MDIX-mode:  incoming TX --> INCA_IP TX
                                                   incoming RX --> INCA_IP RX 
                                    */
   UINT32            currState ;    /* reset:             after power on, 
                                       wait for "linkup": do auto MDIX, 
                                       "linkup":          it works 
                                    */
   sw21553_PhyCallbackFunction oldCallback;
   
   /* external functions used for external board configuration */
   UINT32 (*pGetAutoMDIXSupportPOL)(void);
   UINT32 (*pSetAutoMDIXModeStartup)(void);
   void   (*pSetupAutoMDIXModeGPIO)(void);
   UINT32 (*pSetAutoMDIXModeMDI)(void);
   UINT32 (*pSetAutoMDIXModeMDIX)(void);
   UINT32 (*pSetAutoMDIXModeRESET)(void);
   int    (*pGetAutoMDIXMode)(void);
   int    (*pCheckAutoMDIXPortSetup)(void);

} swAutoMDIXstruct;


int sw21553_AmdixInit(void);
void sw21553_AmdixStart(int port);
void sw21553_AmdixEnergy(int port);
void sw21553_AmdixRetry(int port);
void sw21553_AmdixTriesTout(int port);

/*******************************************************************************
*   forward declaration for the external user-defined init functions 
*   to set the auto MDIX featuere. This functions must be provided by the BSP
*/
/* PC-Port */
extern UINT32 sysGetAutoMDIX_PC_ON_OFF(void);
extern UINT32 sysGetAutoMDIX_PC_StatusDelay(void);

extern UINT32 sysGetAutoMDIX_PC_Support_POL(void);
extern UINT32 sysGetAutoMDIX_PC_LinkTries(void);
extern UINT32 sysSetAutoMDIX_PC_StartupMode(void);
extern void   sysSetupAutoMDIX_PC_GPIO(void);
extern UINT32 sysSetAutoMDIX_PC_MDIMode(void);
extern UINT32 sysSetAutoMDIX_PC_MDIXMode(void);
extern UINT32 sysSetAutoMDIX_PC_RESETMode(void);
extern int    sysGetAutoMDIX_PC_Mode(void);
extern int    sysCheckAutoMDIX_PC_PortSetup(void);

/* LAN-Port */
extern UINT32 sysGetAutoMDIX_LAN_ON_OFF(void);
extern UINT32 sysGetAutoMDIX_LAN_StatusDelay(void);

extern UINT32 sysGetAutoMDIX_LAN_Support_POL(void);
extern UINT32 sysGetAutoMDIX_LAN_LinkTries(void);
extern UINT32 sysSetAutoMDIX_LAN_StartupMode(void);
extern void   sysSetupAutoMDIX_LAN_GPIO(void);
extern UINT32 sysSetAutoMDIX_LAN_MDIMode(void);
extern UINT32 sysSetAutoMDIX_LAN_MDIXMode(void);
extern UINT32 sysSetAutoMDIX_LAN_RESETMode(void);
extern int    sysGetAutoMDIX_LAN_Mode(void);
extern int    sysCheckAutoMDIX_LAN_PortSetup(void);

/* configure and setup the used AutoMDIX Ports */
extern UINT32 sysGetAutoMDIX_PHY_ResetDelay(void);
extern void   sysSetupAutoMDIX_Ports(void);

#endif /* _INCASWITCHAMDIX_H */
