/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : IncaSwitchApi.c
   Date        : 13.05.2003
   Description : provide an API to read and configure the INCA-IP ethernet parts
                 like PHY's and switch module



   $Log: inca_switch_api.c,v $
   Revision 1.3  2005-08-29 15:08:09  fischesv
   new ioctls INCA_SWITCH_IOC_GET_PROPERTIES and INCA_SWITCH_IOC_SET_PROPERTIES

   Revision 1.2  2004/11/18 16:26:11  fischesv
   added ethernet lan link detection as ioctl (INCA_SWITCH_IOC_LAN_LINK, /dev/inca-switch-lan) and proc filesystem entry (/proc/driver/inca_switch_lan)

   Revision 1.1.1.1  2004/06/03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.5  2004/02/24 12:38:58  de6wanme
   Initial Auto-MDIX support added

   Revision 1.3  2003/07/25 10:02:40  rjanse_a
   Bug fix (UnDestPort modification/display)

   Revision 1.2  2003/07/15 11:09:33  rjanse_a
   Bug fixes.
   Improvements in API handling.

   Revision 1.1  2003/07/14 08:57:09  rjanse_a
   Initial revision of ported vxWorks switch driver (inca_switch_hash, inca_switch_access) and a simple control interface (inca_switch_api, api.sh)

   Revision 1.5  2003/07/07 17:41:38  mangele
   - Add Switch LED API

   Revision 1.4  2003/06/04 15:55:20  mangele
   Add further API functions for access switch INCA IP module

   Revision 1.3  2003/06/02 21:52:33  mangele
   Add further functionallity for the switch API

   Revision 1.2  2003/05/23 13:35:42  mangele
   Splitt of vxWorks user functions and switch/PHY API functions (see IncaSwitchAccess.c)
   Rename vxWorks user functions (sysPhy..., sysSw..)

   Revision 1.1  2003/05/20 13:34:08  mangele
   add PHY API: access LAN and PC PHY
   add SWITCH API: switch MAC table



   
******************************************************************************/
/*  #include "vxWorks.h" */
/*  #include "stdio.h" */
/*  #include "string.h" */

/*  #include "RegIncaIP.h" */
/*  #include "IncaSwitchEnd.h" */
/*  #include "IncaSwitchHash.h" */
/*  #include "IncaSwitchAccess.h" */
/*  #include "IncaSwitchApi.h" */

#define EXPORT_SYMTAB

#include <asm/uaccess.h>

#include <asm/incaip/inca-ip.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>

#include "vxWorks_type.h"
#include "inca_switch_access.h"
#include "inca_switch_settings.h"
#include "inca_switch_hash.h"
#include "inca_switch_phy.h"

#include <asm/incaip/incaip_switch_api.h>

/*  #include "vxWorks_type.h" */
/*  #include "inca_switch_access.h" */
/*  #include "inca_phy_regs.h" */

/*  #include "inca_mdio_settings.h" */
/*  #include "inca_switch_actl.h" */
/*  #include "inca_switch_hash.h" */

#undef DEBUG

static int switch_major = 0;

/******************************************************************************
   Common definitions
******************************************************************************/



/******************************************************************************
   PHY API
******************************************************************************/



/******************************************************************************
*  sw21553_DisplayPhyProperties()
*     display PHY properties previous read
*
*  Parameter:
*     properties: encoded properties of a PHY
*           bit  0.. 4: line properties (current state, advertise, partner)
*           bit  5.. 7: currently not used
*           bit  8:     support pause operation
*           bit  9:     support next page
*           bit 10:     support auto negotiation
*           bit 11..15: currently not used
*           bit 16:     extended capabilities
*           bit 17:     auto negotiation able (PHY Reg 1)
*           bit 18..19: currently not used       
*           bit 20:     link status
*           bit 21:     remote fault (advertise: signal partner a fault)
*           bit 22:     auto negotiation complete
*           bit 23:     currently not used       
*           bit 24..26: current speed (PHY reg 31)
*           bit 27:     auto negotiation done (PHY reg 31)
*           bit 28..30: currently not used       
*           bit 21:     data may be corrupted
*
*  REMARK:
*
*/
void sw21553_DisplayPhyProperties(UINT32 properties)
{
   printk(" data  |link  |remote|A.Neg |         line properties \n") ;
   printk(" valid |state |fault |compl.| 100T4| 100FD| 100HD| 10FD | 10HD\n") ;
   printk("----------------------------------------------------------------\n") ;
   printk("   %c      %c      %c      %c      %c      %c      %c      %c      %c\n",
          ((properties & sw21553_PHY_CTRL_DATA_CORRUPTED) ? '0' : '1'), 
          ((properties & sw21553_PHY_CTRL_LINK_STATE) ? '1' : '0'), 
          ((properties & sw21553_PHY_CTRL_REMOTE_FAULT) ? '1' : '0'), 
          ((properties & sw21553_PHY_CTRL_AUTO_NEG_COMPL) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_100_T4) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_100_FULL) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_100_HALF) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_10_FULL) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_10_HALF) ? '1' : '0') ) ;
   /* display current speed and duplex mode */
   printk("\n Current Line: %s MBit/s %s DUPLEX (Auto-Neg. %s)\n\n",
          ( (properties & sw21553_PHY_CTRL_SPEED_100_IND) ? "100" : "10" ),
          ( (properties & sw21553_PHY_CTRL_SPEED_DUPLEX_IND) ? "FULL" : "HALF" ),
          ( (properties & sw21553_PHY_CTRL_AUTO_NEG_DONE) ? "DONE" : "NOT DONE" ) ) ;

   return ;
}


/******************************************************************************
*  sysPhyShowLan()
*     collect and display the appropriate LAN PHY properties
*
*  Parameter:
*     phyInfo: specify the properties for display
*             0: properties of the given PHY
*             1: advertised PHY properties used for auto-neg.
*             2: partner PHY properties
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyShowLan(int phyInfo)
{
   UINT32  properties ;

   if ( (properties = sw21553_PhyGetProperties(INCA_IP_MDIO_LAN_PHY, phyInfo)) != ERROR)
   {
      /* display and interprete informations */
      printk("Status LAN Port PHY (%s): 0x%08x\n",
              (phyInfo==sw21553_PHY_PROPS_CURRENT) ? 
                    (sw21553_PHY_PROPS_CURRENT_STR):( 
                        (phyInfo==sw21553_PHY_PROPS_ADVERTISE)?
                             (sw21553_PHY_PROPS_ADVERTISE_STR):(sw21553_PHY_PROPS_PARTNER_STR)
                                                    ),

               properties ) ;
      sw21553_DisplayPhyProperties(properties) ;
   }
   else
   {
      printk("ERROR - while request LAN PHY informations\n") ;
   }
   return 0 ;
}

/******************************************************************************
*  sysPhyShowPc()
*     collect and display the appropriate PC PHY properties
*
*  Parameter:
*     phyInfo: specify the properties for display
*             0: properties of the given PHY
*             1: advertised PHY properties used for auto-neg.
*             2: partner PHY properties
*
*  Returns:
*     0 - no dummy*
*  REMARK:
*
*/
int sysPhyShowPc(int phyInfo)
{
   UINT32  properties ;

   if ( (properties = sw21553_PhyGetProperties(INCA_IP_MDIO_PC_PHY, phyInfo)) != ERROR)
   {
      /* display and interprete informations */
      printk("Status PC Port PHY (%s): 0x%08x\n",
              (phyInfo==sw21553_PHY_PROPS_CURRENT) ? 
                    (sw21553_PHY_PROPS_CURRENT_STR):( 
                        (phyInfo==sw21553_PHY_PROPS_ADVERTISE)?
                             (sw21553_PHY_PROPS_ADVERTISE_STR):(sw21553_PHY_PROPS_PARTNER_STR)
                                                    ),

               properties ) ;
      sw21553_DisplayPhyProperties(properties) ;
   }
   else
   {
      printk("ERROR - while request PC PHY informations\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhySetLanAdvertise()
*     restart the auto negotiation for the LAN PHY
*
*  Parameter:
*     newSettings: new properies to set
*                  --> line properties: bit[0..4]
*                       bit 0:  10 MBit/s Half Duplex
*                       bit 1:  10 MBit/s Full Duplex
*                       bit 2: 100 MBit/s Half Duplex
*                       bit 3: 100 MBit/s Full Duplex
*                      (bit 4: 100 MBit/s T4 - can not set )
*                      (bit 5..7: not used)
*                  --> support Pause Operation
*                       bit 8: Support Pause Operation
*     doRestartAutoNeg: if set do auto negotiation of the LAN PHY
*
*  Returns:
*     0 - no dummy
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*/
int sysPhySetLanAdvertise(UINT32 newSettings, int doRestartAutoNeg)
{
   UINT32 tempReg ;
   tempReg = sw21553_PhySetAdvertiseProperties(INCA_IP_MDIO_LAN_PHY, newSettings) ;
   
   if (doRestartAutoNeg)
      if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_LAN_PHY) == ERROR)
         printk("LAN PHY: advertise auto-neg. properties - ERROR while restart auto-neg.\n") ;
      else
         printk("LAN PHY: advertise auto-neg. properties - restart auto neg. done\n") ;
   else
      printk("LAN PHY: advertise auto-neg. properties set - NO restart auto neg.\n") ;

   return(0) ;
}


/******************************************************************************
*  sysPhySetPcAdvertise()
*     restart the auto negotiation for the PC PHY
*
*  Parameter:
*     newSettings: new properies to set
*                  --> line properties: bit[0..4]
*                       bit 0:  10 MBit/s Half Duplex
*                       bit 1:  10 MBit/s Full Duplex
*                       bit 2: 100 MBit/s Half Duplex
*                       bit 3: 100 MBit/s Full Duplex
*                      (bit 4: 100 MBit/s T4 - can not set )
*                      (bit 5..7: not used)
*                  --> support Pause Operation
*                       bit 8: Support Pause Operation
*     doRestartAutoNeg: if set do auto negotiation of the PC PHY
*
*  Returns:
*     0 - no dummy
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*/
int sysPhySetPcAdvertise(UINT32 newSettings, int doRestartAutoNeg)
{
   UINT32 tempReg ;
   tempReg = sw21553_PhySetAdvertiseProperties(INCA_IP_MDIO_PC_PHY, newSettings) ;
   
   if (doRestartAutoNeg)
      if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_PC_PHY) == ERROR)
         printk("PC PHY: advertise auto-neg. properties - ERROR while restart auto-neg.\n") ;
      else
         printk("PC PHY: advertise auto-neg. properties - restart auto neg. done\n") ;
   else
      printk("PC PHY: advertise auto-neg. properties set - NO restart auto neg.\n") ;

   return(0) ;
}

/******************************************************************************
*  sysPhySetLanFixed()
*     set the LAN PHY speed settings used for not auto-neg connections
*
*  Parameter:
*     newSpeed: new speed and duplex properies to set
*                  - 001B: 10Base-T half-duplex
*                  - 101B: 10Base-T full-duplex
*                  - 010B: 100Base-TX half-duplex
*                  - 110B: 100Base-TX full-duplex
*     doDisableAutoNeg: if set disable auto negotiation of the LAN PHY
*                        else keep current value
*
*  Returns:
*     0 - Success
*     -1  ERROR
*  REMARK:
*/
int sysPhySetLanFixed(UINT32 newSpeed, int doDisableAutoNeg)
{
   if (sw21553_PhySetFixSpeed(INCA_IP_MDIO_LAN_PHY, newSpeed) == ERROR)
   {
      printk("ERROR while change fixed LAN PHY speed settings\n") ;
      return(-1) ;
   }

   if (doDisableAutoNeg==1)
   {
      if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, doDisableAutoNeg) == ERROR)
      {
         printk("LAN PHY: fixed speed changed - ERROR while disable auto-neg.\n") ;
      }
      else
      {
         printk("LAN PHY: fixed speed changed, auto-negotiation disabled\n") ;
      }
   }
   else
   {
      printk("LAN PHY: fixed speed changed\n") ;
   }

   return(OK) ;
}

/******************************************************************************
*  sysPhySetPcFixed()
*     set the PC PHY speed settings used for not auto-neg connections
*
*  Parameter:
*     newSpeed: new speed and duplex properies to set
*                  - 001B: 10Base-T half-duplex
*                  - 101B: 10Base-T full-duplex
*                  - 010B: 100Base-TX half-duplex
*                  - 110B: 100Base-TX full-duplex
*     doDisableAutoNeg: if set disable auto negotiation of the PC PHY
*                        else keep current value
*
*  Returns:
*     0 - Success
*     -1  ERROR
*  REMARK:
*/
int sysPhySetPcFixed(UINT32 newSpeed, int doDisableAutoNeg)
{
   if (sw21553_PhySetFixSpeed(INCA_IP_MDIO_PC_PHY, newSpeed) == ERROR)
   {
      printk("ERROR while change fixed PC PHY speed settings\n") ;
      return(ERROR) ;
   }

   if (doDisableAutoNeg==1)
   {
      if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, doDisableAutoNeg) == ERROR)
      {
         printk("PC PHY: fixed speed changed - ERROR while disable auto-neg.\n") ;
      }
      else
      {
         printk("PC PHY: fixed speed changed, auto-negotiation disabled\n") ;
      }
   }
   else
   {
      printk("PC PHY: fixed speed changed\n") ;
   }

   return(OK) ;
}


/******************************************************************************
*  sysPhyLanReAutoNeg()
*     restart the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanReAutoNeg(void)
{
   if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_LAN_PHY) == ERROR)
   {
      printk("ERROR - LAN PHY: Cannot RESTART auto negotiation - check if auto neg is enabled\n") ;
   }
   else
   {
      printk("LAN PHY: RESTART auto negotiation done\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyPcReAutoNeg()
*     restart the auto negotiation for the PC PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcReAutoNeg(void)
{
   if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_PC_PHY) == ERROR)
   {
      printk("ERROR - PC PHY: Cannot RESTART auto negotiation - check if auto neg is enabled\n") ;
   }
   else
   {
      printk("PC PHY: RESTART auto negotiation done\n") ;
   }
   return(0) ;
}


/******************************************************************************
*  sysPhyLanDisableAutoNeg()
*     disable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanDisableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, 1) == ERROR)
   {
      printk("ERROR - LAN PHY: Cannot DISABLE auto negotiation\n") ;
   }
   else
   {
      printk("LAN PHY: auto negotiation DISABLED\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyLanEnableAutoNeg()
*     disable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanEnableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, 0) == ERROR)
   {
      printk("ERROR - LAN PHY: Cannot ENABLE auto negotiation\n") ;
   }
   else
   {
      printk("LAN PHY: auto negotiation ENABLED\n") ;
   }
   return(0) ;
}


/******************************************************************************
*  sysPhyPcDisableAutoNeg()
*     disable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcDisableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, 1) == ERROR)
   {
      printk("ERROR - PC PHY: Cannot DISABLE auto negotiation\n") ;
   }
   else
   {
      printk("PC PHY: auto negotiation DISABLED\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyPcEnableAutoNeg()
*     enable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcEnableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, 0) == ERROR)
   {
      printk("ERROR - PC PHY: Cannot ENABLE auto negotiation\n") ;
   }
   else
   {
      printk("PC PHY: auto negotiation ENABLED\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyLanReset()
*     reset the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanReset(void)
{
   if (sw21553_PhyReset(INCA_IP_MDIO_LAN_PHY) == ERROR)
   {
      printk("LAN PHY: ERROR - while PHY RESET\n") ;
   }
   else
   {
      printk("LAN PHY: PHY RESET done \n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyPcReset()
*     reset the PC PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcReset(void)
{
   if (sw21553_PhyReset(INCA_IP_MDIO_PC_PHY) == ERROR)
   {
      printk("PC PHY: ERROR - while PHY RESET\n") ;
   }
   else
   {
      printk("PC PHY: PHY RESET done \n") ;
   }
   return(0) ;
}



/******************************************************************************
   Switch API: MAC table handling
******************************************************************************/

/* definitions of the MAC table entry */
#define INCA_IP_MAC_TABLE_ENTRY_STATIC       0
#define INCA_IP_MAC_TABLE_ENTRY_AGEDOUT      7

#define INCA_IP_MAC_TABLE_ENTRY_STATIC_STR   "static"
#define INCA_IP_MAC_TABLE_ENTRY_AGEDOUT_STR  "aged out"
#define INCA_IP_MAC_TABLE_ENTRY_DYNAMIC_STR  "dynamic"


/* FORMAT: MAC ENTRY Control Field
   BIT:  7 6 5 4   3 2 1 0
               C    Port
          AGE  R   V M L P
         0 0 0 0   0 0 0 0
*/
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_P       0
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_P_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_P )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_L       1
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_L_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_L )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_V       2
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_V_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_V )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_M       3
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_M_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_M )

#define INCA_IP_MAC_TABLE_ENTRY_CTRL_CR      4
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_CR_MASK (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_CR )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_POS     5
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_WIDTH   3
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_GET_AGE(ageValue) \
                            ( ( (ageValue) >> INCA_IP_MAC_TABLE_ENTRY_CTRL_POS ) &\
                              (  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_WIDTH) - 1   )   )



/******************************************************************************
*  sw21553_SwDisplayMacTableHeader()
*     display a header for show a MAC table
*
*/
LOCAL void sw21553_SwDisplayMacTableHeader(void)
{
   /*     
          " INCA-IP MAC table entries ||MAC Entry | PORTS |"
          "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
          "----------------------------------------------------------------"
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
           [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static  
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
   */
 printk("\n INCA-IP MAC table entries ||MAC Entry | PORTS |\n") ;
   printk("INDEX |     MAC Address    || Cntrl -> |M|V|L|P| C| age\n") ;
   printk("----------------------------------------------------------------\n") ;
   /*
          "----------------------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "----------------------------------------------------------------") ;
   */
}


/******************************************************************************
*  sw21553_SwDisplayMacTableTrailer()
*     display a trailer for show a MAC table
*
*/
LOCAL void sw21553_SwDisplayMacTableTrailer(int count)
{
   /*     
          "        INCA-IP MAC table entries"                   
          "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
          "----------------------------------------------------------------"
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
           [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static  
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
   */

   printk("----------------------------------------------------------------\n") ;
   printk("  ==>  MAC TABLE: %d entries found\n", count) ;
   printk("----------------------------------------------------------------\n") ;
   /*
          "----------------------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "----------------------------------------------------------------") ;
   */
}


/******************************************************************************
*  sw21553_SwDisplayMacEntry()
*     display an MAX entry 
*
*/
LOCAL void sw21553_SwDisplayMacEntry(int index, sw21553_MacTableEntry MacEntry)
{
   UINT32 tempAge ;

   tempAge = INCA_IP_MAC_TABLE_ENTRY_CTRL_GET_AGE((UCHAR)MacEntry.Cntrl) ;
   /*     
          "        INCA-IP MAC table entries"                   
          "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
          "----------------------------------------------------------------"
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
           [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static  
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
   */
   printk("[%3d] |  %02x:%02x:%02x:%02x:%02x:%02x"
                                    " ||  0x%02x "
                                              "-> |%c|%c|%c|%c| %c"
                                                            "| %2d %s\n", 
           index, 
           MacEntry.MacAddr[0], MacEntry.MacAddr[1], MacEntry.MacAddr[2],
           MacEntry.MacAddr[3], MacEntry.MacAddr[4], MacEntry.MacAddr[5],
           MacEntry.Cntrl,
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_M_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_V_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_L_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_P_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_CR_MASK)?'1':'0',
           tempAge, 
           (tempAge == INCA_IP_MAC_TABLE_ENTRY_STATIC) ? (INCA_IP_MAC_TABLE_ENTRY_STATIC_STR) : ( 
               (tempAge == INCA_IP_MAC_TABLE_ENTRY_AGEDOUT) ? (INCA_IP_MAC_TABLE_ENTRY_AGEDOUT_STR) : 
                                                              (INCA_IP_MAC_TABLE_ENTRY_DYNAMIC_STR) )
           ) ;
   /*
          "----------------------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "----------------------------------------------------------------") ;
   */
}

/******************************************************************************
*  sw21553_SwReadDisplMacEntry()
*     get the MAC entry from the MAC table and display
*
*/
LOCAL void sw21553_SwReadDisplMacEntry(int index)
{
   sw21553_MacTableEntry MacEntry ;

   if ( sw21553_SwMacReadTableEntry(index, &MacEntry) == 1 )
   {
      /* entry found --> display */
      printk("[%3d] |  %02x:%02x:%02x:%02x:%02x:%02x |   0x%02x\n", 
              index, 
              MacEntry.MacAddr[0], MacEntry.MacAddr[1], MacEntry.MacAddr[2],
              MacEntry.MacAddr[3], MacEntry.MacAddr[4], MacEntry.MacAddr[5],
              MacEntry.Cntrl) ;
   }

}

/******************************************************************************
*  sw21553_SwDisplayUnDest()
*     display the UnDest register settings
*
*/
LOCAL UINT32 sw21553_SwDisplayUnDest(UINT32 regValue)
{
   if (regValue == ERROR)
   {
      printk("Error - cannot show \"Unknown Destination Register\" \n") ;
      return(ERROR) ; 
   }

   /*     
          "  Unknown Destination Register: 0x00000000"
          "                  | Broad | Multi | Uni"
          "  Forward unknown | C|L|P | C|L|P | C|L|P"
          "------------------------------------------"
          "       packets to |  | |  |  | |  |       "
   */
   printk("\n Unknown Destination Register: 0x%08x\n", regValue) ;
   printk("                 || Broad | Multi | Uni \n") ;
   printk(" Forward unknown || C|L|P | C|L|P | C|L|P \n") ;
   printk(" ----------------||-------|-------|------ \n") ;
   printk("      packets to || %c|%c|%c | %c|%c|%c | %c|%c|%c \n\n",
           /* broadcast flags */
           (regValue&INCA_IP_Switch_UN_DEST_CB)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_LB)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_PB)?'+':'-',
           /* multicast flags */
           (regValue&INCA_IP_Switch_UN_DEST_CM)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_LM)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_PM)?'+':'-',
           /* unicast flags */
           (regValue&INCA_IP_Switch_UN_DEST_CU)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_LU)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_PU)?'+':'-'
         ) ;
   return(OK) ;
}

/******************************************************************************
*  sw21553_SwDisplaySpPt()
*     display the SpPt register settings
*
*/

LOCAL UINT32 sw21553_SwDisplaySpPt(int regValue)
{
   int tempState ;

   if (regValue == ERROR)
   {
      printk("Error - cannot show \"Spanning Tree Port Status Register\" \n") ;
      return(ERROR) ; 
   }

   printk("\nSpanning Tree Port Status Register: 0x%08x\n", regValue) ;

   tempState = INCA_IP_Switch_ST_PT_PPS_GET(regValue) ;
   printk("   PC-port state:   \"%s\" \n",
          (tempState == INCA_IP_Switch_SP_PT_STATE_DISABLED)?("DISABLED"):
             ( 
               (tempState == INCA_IP_Switch_SP_PT_STATE_LISTEN)?("LISTENING"):
                  ( 
                    (tempState == INCA_IP_Switch_SP_PT_STATE_LEARN)?("LEARNING"):("FORWARDING")
                  )         
             )
         ) ;

   tempState = INCA_IP_Switch_ST_PT_LPS_GET(regValue) ;
   printk("  LAN-port state:   \"%s\" \n",
          (tempState == INCA_IP_Switch_SP_PT_STATE_DISABLED)?("DISABLED"):
             ( 
               (tempState == INCA_IP_Switch_SP_PT_STATE_LISTEN)?("LISTENING"):
                  ( 
                    (tempState == INCA_IP_Switch_SP_PT_STATE_LEARN)?("LEARNING"):("FORWARDING")
                  )         
             )
         ) ;

   tempState = INCA_IP_Switch_ST_PT_CPS_GET(regValue) ;
   printk("  CPU-port state:   \"%s\" \n",
          (tempState == INCA_IP_Switch_SP_PT_STATE_DISABLED)?("DISABLED"):
             ( 
               (tempState == INCA_IP_Switch_SP_PT_STATE_LISTEN)?("LISTENING"):
                  ( 
                    (tempState == INCA_IP_Switch_SP_PT_STATE_LEARN)?("LEARNING"):("FORWARDING")
                  )         
             )
         ) ;

   return(OK) ;
}


/******************************************************************************
*  sysSwUnDestSetPorts()
*     set and change the UN_DEST register per port
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     portBehaviour: 3 bit field
*                 forwarding of unknown packets for broadcast/multicast/unicast packets
*                 0: do not forward a unknown packet 
*                 1: forward a unknown packet 
*
*     3 bit field:
*     ============
*        Bit 0: unicast packets
*        Bit 1: multicast packets
*        Bit 2: broadcast packets
*
*     Example: forwarding unknown packets to <portSelect> port
*        portBehaviour = 0x5 = 100b
*                              |||---> do not forward unknown unicast packets 
*                              ||----> do not forward unknown multicast packets 
*                              |-----> do     forward unknown broadcast packets 
*
*  Returns:
*     0: Success and display new settings
*     else ERROR (-1)
*
*/
int sysSwUnDestSetPorts(int portSelect, int portBehaviour)
{
   UINT32 regValue ;
   regValue = sw21553_SwUpdUnDestPorts(portSelect, portBehaviour) ;

   return(sw21553_SwDisplayUnDest(regValue) ) ;
}

/******************************************************************************
*  sysSwUnDestShow()
*     read and display the UN_DEST register
*
*  Returns:
*     0: success and display settings
*
*/
int sysSwUnDestShow(void)
{
   UINT32 regValue ;
   /* arg 3: read only settings */
   regValue = sw21553_SwIOUnDest(0,0,0) ;

   return(sw21553_SwDisplayUnDest(regValue) ) ;
}

/******************************************************************************
*  sysSwSpPtSetPorts()
*     update the port state within the Spanning Tree Port Status 
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     newPortState: possible states are
*                    INCA_IP_Switch_SP_PT_STATE_DISABLED  0
*                    INCA_IP_Switch_SP_PT_STATE_LISTEN    1
*                    INCA_IP_Switch_SP_PT_STATE_LEARN     2
*                    INCA_IP_Switch_SP_PT_STATE_FORWARD   3
*
*  Returns:
*     0: Success and display new settings
*     else ERROR (-1)
*/
int sysSwSpPtSetPorts(int portSelect, int newPortState)
{

   UINT32 regValue ;
   regValue = sw21553_SwUpdSpPtPortState(portSelect, newPortState) ;

   return(sw21553_SwDisplaySpPt(regValue) ) ;
}

/******************************************************************************
*  sysSwSpPtShow()
*     read and display the ST_PT register
*
*  Returns:
*     0: success and display settings
*
*/
int sysSwSpPtShow(void)
{
   UINT32 regValue ;
   /* arg 3: read only settings */
   regValue = sw21553_SwIOSpPt(0,0,0) ;

   return(sw21553_SwDisplaySpPt(regValue) ) ;
}

/******************************************************************************
*  sysSwPktLenSet()
*     set the PKT_LEN register (Packet Length Register)
*
*  Parameter:
*     newPktLen: contains the new maximal packet lenght
*     addTagLen: ADD 4 to MAX_PKT_LEN for Tagged Packets
*                0: MAC_PKT_LEN to be used for tagged and untagged packets
*                1: MAX_PKT_LEN + 4 is used for tagged packets.
*
*  Returns:
*     0: Success and display new settings
*     else ERROR (-1)
*/
int sysSwPktLenSet(UINT32 newPktLen, int addTagLen)
{

   UINT32 regValue ;
   regValue = sw21553_SwIOPacketLen(newPktLen, addTagLen, 1) ;

   if(regValue != ERROR)
   {
      printk("PKT_LEN register (0x%08x): max packet len = 0x%x (%d), TAG length %s\n",
              regValue, 
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
              (regValue&INCA_IP_Switch_PKT_LEN_ADD) ? "added" : "not added"
            ) ;
      return(OK) ;
   }
   else
   {
      printk("ERROR - cannot set new MAX_PKT_LEN = %d (add = %d), max len = 0x%x (%d)\n",
              newPktLen, addTagLen, 
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN,
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN ) ;

      return(ERROR) ;
   }
   return(ERROR) ;
}

/******************************************************************************
*  sysSwPktLenShow()
*     read and display the PKT_LEN register
*
*  Returns:
*     0: success and display settings
*
*/
int sysSwPktLenShow(void)
{
   UINT32 regValue ;
   regValue = sw21553_SwIOPacketLen(0, 0, 0) ;

   printk("PKT_LEN register (0x%08x): max packet len = 0x%x (%d), TAG length %s\n",
           regValue, 
           INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
           INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
           (regValue&INCA_IP_Switch_PKT_LEN_ADD) ? "added" : "not added"
         ) ;
   return(OK) ;
}



/******************************************************************************
*  sysSwMacShowTable()
*     Display the entries of the switch MAC table
*  Returns:
*     0 - no dummy
*
*/
int sysSwMacShowTable(void)
{
   UINT32 index, count ;
   sw21553_MacTableEntry MacEntry ;
   count = 0 ;

   sw21553_SwDisplayMacTableHeader() ;

   for (index = 0; index < INCA_IP_MAX_MAC_TABLE_ENTRIES; index++)
   {
      if (
           sw21553_SwMacReadTableEntry(index, &MacEntry) == 1
         )
      {
         count++ ;
         /* entry found --> display */
         sw21553_SwDisplayMacEntry(index, MacEntry) ;
      }
   }

   sw21553_SwDisplayMacTableTrailer(count) ;

   return(0) ;
}

/******************************************************************************
*  sysSwMacAddTableEntry()
*     Add an given MAC address to the switch MAC table
*     interprete and set the given parameters                              
*
*  Parameter:
*     pStrEtherAdr:  ptr to MAC address string with the form 00:11:22:aa:bb:cc
*     port:       defines the port to which the address is assigned
*                 possible values are:
*                 0: PC port
*                 1: LAN port
*                 2: CPU port VOICE
*                 3: CPU port MANAGEMENT
*     classification: calssify the frames
*                 0: packets will be classified as normal packets
*                 1: packets will be classified as critical packets
*     aging:      defines the aging behaviour of the entry
*                 0: no aging --> static entry into the MAC table
*                 1: normal aging --> entry will normal timed out
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwMacAddTableEntry(char *pStrEtherAdr, int port, int classification, int aging )
{
   sw21553_MacTableEntry tempMacTableEntry ;

   /* create a MAC table entry struct and init the data */
   if (pStrEtherAdr != NULL)
   {
      /* get and copy mac-addr */
      sw21553_SwMacConvertStr2Hex(pStrEtherAdr, tempMacTableEntry.MacAddr) ;
   }
   else
   {
      printk("ERROR - sysSwMacAddTableEntry(): no MAC address given\n") ;
      return(ERROR) ;
   }

   /* check and set assigned port */
   if ( sw21553_SwMacSetTableEntryCntrl(
                                       port, classification, aging, &tempMacTableEntry.Cntrl
                                     ) == ERROR )
   {
      /* parameter error */
      printk("ERROR - sysSwMacAddTableEntry(): invalid parameter\n") ;
      return(ERROR) ;
   }

   printk("sysSwMacAddTableEntry(): add MAC table entry: MAC = %02x:%02x:%02x:%02x:%02x:%02x, control = %2x ",
           tempMacTableEntry.MacAddr[0], tempMacTableEntry.MacAddr[1], 
           tempMacTableEntry.MacAddr[2], tempMacTableEntry.MacAddr[3], 
           tempMacTableEntry.MacAddr[4], tempMacTableEntry.MacAddr[5],
           tempMacTableEntry.Cntrl ) ;
   if (sw21553_SwMacAddEntry(tempMacTableEntry, 0) == OK) 
      printk("SUCCESSFUL\n") ;
   else
      printk("NOT SUCCESSFUL !!!\n") ;
   
   return(OK) ;
}



/******************************************************************************
*  sysSwMacClearEntryByMac()
*     Clear all MAC table entries with the given MAC address
*
*  Parameter:
*     pStrEtherAdr:  ptr to MAC address string with the form 00:11:22:aa:bb:cc
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*  REMARK:
*     Via modulation there are 4 possible MAC entry indexes for one MAC address
*     This functions checks all 4 possibilities for clearance.
*
*/
int sysSwMacClearEntryByMac(char *pStrEtherAdr)
{
   int count ;
   sw21553_MacTableEntry tempMacTableEntry ;

   /* create a MAC table entry struct and init the data */
   if (pStrEtherAdr != NULL)
   {
      /* get and copy mac-addr */
      sw21553_SwMacConvertStr2Hex(pStrEtherAdr, tempMacTableEntry.MacAddr) ;
      /* clear all entries with this MAC */
      tempMacTableEntry.Cntrl = 0 ;
   }
   else
   {
      printk("ERROR - sysSwMacClearEntryByMac(): no MAC address given\n") ;
      return(ERROR) ;
   }

   count = sw21553_SwMacClearEntryByMac(tempMacTableEntry) ;
   printk("sysSwMacClearEntryByMac(%02x:%02x:%02x:%02x:%02x:%02x): %d MAC table entries cleared \n ",
           tempMacTableEntry.MacAddr[0], tempMacTableEntry.MacAddr[1], 
           tempMacTableEntry.MacAddr[2], tempMacTableEntry.MacAddr[3], 
           tempMacTableEntry.MacAddr[4], tempMacTableEntry.MacAddr[5],
           count ) ;

   return(OK) ;
}



/******************************************************************************
*  sysSwMacClearEntryByIndex()
*     Clear a MAC table entry identified by the given index
*
*  Parameter:
*     index:  index of the MAC table entry for clearance
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*  REMARK:
*     The index must be a valid index for the MAC table (currently [0..255]
*
*/
int sysSwMacClearEntryByIndex(int index)
{
   /* create a MAC table entry struct and init the data */
   if (index < 0 || index >= INCA_IP_MAX_MAC_TABLE_ENTRIES)
   {
      printk("ERROR - sysSwMacClearEntryByIndex(): no valid MAC table index\n") ;
      return(ERROR) ;
   }

   if (sw21553_SwMacClearEntryByIndex(index) == OK)
   {
      printk("Success cleared entry: ") ;
      sw21553_SwReadDisplMacEntry(index) ;
   }
   else
   {
      printk("ERROR: sysSwMacClearEntryByIndex(index = %d): contains no vaid entry \n", index) ;
      return(ERROR) ;
   }

   return(OK) ;
}

/******************************************************************************
*  sysSwMacTableReset()
*     reset the complete MAC table
*
*  Returns:
*     0 - Success
*     -1 ERROR:  ERROR while reset write
*
*/
int sysSwMacTableReset(void)
{
   /* reset complete MAC table: 0 - (INCA_IP_MAX_MAC_TABLE_ENTRIES-1) */
   return( sw21553_SwMacResetMacTable() ) ;
}



/******************************************************************************
   Switch API: VLAN control
******************************************************************************/

/******************************************************************************
*  sw21553_SwVlanDisplayVLanCtrl()
*     display the VLAN_CTRL register
*
*/
LOCAL void sw21553_SwVlanDisplayVLanCtrl(UINT32 regValue)
{
   /* 
        "                              PCx  LAN  CPU"
        "      send only to Members:
        "accept no untagged packets:
        "               VLAN awware:

   */
 printk("\n              VLAN_CTRL:    PC   LAN  CPU \n") ;
   printk("   send only to Members:     %c    %c    %c \n",
           (regValue&INCA_IP_Switch_VLAN_CTRL_SP) ? '+' : '-',
           (regValue&INCA_IP_Switch_VLAN_CTRL_SL) ? '+' : '-',
           (regValue&INCA_IP_Switch_VLAN_CTRL_SC) ? '+' : '-'
         ) ;
   printk("accept untagged packets:     %c    %c    %c \n\n",
           (regValue&INCA_IP_Switch_VLAN_CTRL_TP) ? '-' : '+',
           (regValue&INCA_IP_Switch_VLAN_CTRL_TL) ? '-' : '+',
           (regValue&INCA_IP_Switch_VLAN_CTRL_TC) ? '-' : '+'
         ) ;
   printk("             VLAN aware:    %s \n\n",
           (regValue&INCA_IP_Switch_VLAN_CTRL_VA) ? "YES" : "NO") ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayPortVLanHeader()
*     display header for VLAN port configuration
*/
LOCAL void sw21553_SwVlanDisplayVLanTagHeader(void)
{
   /* 
        "Port      VLAN ID      PRIO"
        "PC  :
        "LAN :
        "CPU :  
   */
   printk("\nPort      VLAN ID       PRIO\n") ;
   return ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayVLanCtrl()
*     display the xx_VLAN port register
*
*/
LOCAL void sw21553_SwVlanDisplayPortVLan(int portSelect, UINT32 regValue)
{
   /* 
         "  Port      VLAN ID      PRIO "
         "  PC  :   0x0234 (xxxx)   4"
         "  LAN :
         "  CPU :  
   */
   printk("%s :   0x%04x (%4d)    %d\n", 
           (portSelect==INCA_IP_SWITCH_PORT_SELECT_PC) ? "PC " : 
               ( (portSelect==INCA_IP_SWITCH_PORT_SELECT_LAN)? "LAN" : 
                     ( (portSelect==INCA_IP_SWITCH_PORT_SELECT_CPU) ? "CPU" : "ERROR" ) 
               ) ,
           INCA_IP_Switch_PC_VLAN_VLAN_ID_GET(regValue),
           INCA_IP_Switch_PC_VLAN_VLAN_ID_GET(regValue),
           INCA_IP_Switch_PC_VLAN_PRI_GET(regValue)
         ) ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayVLanTableHeader()
*     display header for a VLAN table
*
*/
LOCAL void sw21553_SwVlanDisplayVLanTableHeader(void)
{
   /*     
          " INCA-IP VLAN table"
          "INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M|"
          "INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P|"
          "----------------------------------------------------"
           [  0] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  1] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  2] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
   */
 printk("\n INCA-IP VLAN table\n") ;
   printk("INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M| \n") ;
   printk("INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P| \n") ;
   printk("----------------------------------------------------\n") ;
   /*
          "---------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "---------------------------------------------------") ;
   */
   return ;
}



/******************************************************************************
*  sw21553_SwVlanDisplayVLanTableEntry()
*     display header for a VLAN table
*
*/
LOCAL void sw21553_SwVlanDisplayVLanTableEntry(int index, UINT32 vLanEntry)
{
   UINT32 vLanId, vLanCntrl ;

   vLanId    = INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(vLanEntry) ;
   vLanCntrl = INCA_IP_VLAN_TABLE_ENTRY_GET_CTRL_FLAGS(vLanEntry) ;

   /*     
          " INCA-IP VLAN table"
          "INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M| "
          "INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P| "
          "----------------------------------------------------"
           [  0] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | | 
           [  1] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | | 
           [  2] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |  
   */

   printk("[%3d] |  0x%03x (%4d)  ||  0x%02x  -> | %c|%c|%c|%c|%c|%c|%c|\n", 
           index, vLanId, (int)vLanId, vLanCntrl,
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK) ?'1':'0',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_EC_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_EL_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_ML_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP_MASK)?'+':' '
         ) ;

   /*
          "---------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "---------------------------------------------------") ;
   */
   return ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayVLanTableTrailer()
*     display trailer for a VLAN table
*
*/
LOCAL void sw21553_SwVlanDisplayVLanTableTrailer(int count)
{
   /*     
          " INCA-IP VLAN table"
          "INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M|"
          "INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P|"
          "----------------------------------------------------"
           [  0] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  1] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  2] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |

          "----------------------------------------------------") ;
          "  ==>  VLAN TABLE: %d entries found"
          "----------------------------------------------------") ;
   */
   printk("----------------------------------------------------\n") ;
   printk("  ==>  VLAN TABLE: %d entries found\n", count) ;
   printk("----------------------------------------------------\n") ;
   return ;
}


/******************************************************************************
*  sysSwVLanShowVLanCtrl()
*     display the VLAN control settings
*
*  Parameter:
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanShowVLanCtrl(void)
{
   /* arg 3 = 0: get the register value --> arg 1, arg 2 not used */
   sw21553_SwVlanDisplayVLanCtrl( sw21553_SwVLanIOVLanCntrl(0, 0, 0) ) ;

   return(OK) ;
}

/******************************************************************************
*  sysSwVLanSetVLanCtrlPort()
*     switch ON/OFF VLAN aware (register VLAN_CTRL)
*
*  Parameter:
*     vLanAware: 0: set VLAN unaware
*                1: set VLAN aware
*/
int sysSwVLanUpdVLanCtrlVAware(int vLanAware)
{

   sw21553_SwVlanDisplayVLanCtrl( sw21553_SwIoSetControl( eSwIoSetVlanCntrl_VA, 
                                                          (void *)vLanAware )  ) ;
   return(OK) ;
}


/******************************************************************************
*  sysSwVLanSetVLanCtrlPort()
*     set the VLAN settings for an given port
*
*  Parameter:
*     portSelect:            select a port 
*                               PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                               LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                               CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     sendOnlyToMember:      Symmetrical VLAN Settings for this port
*                                0: can send packets to VLANs of which it is not a member
*                                1: can only send packets to VLANs of which it is a member
*     notAcceptUntaggedPkt:  Tag Packets only on this Port
*                                0: Untagged packets will be accepted on this port
*                                1: Untagged packets will not be accepted on this port
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanSetVLanCtrlPort(int portSelect, int sendOnlyToMember, int notAcceptUntaggedPkt)
{
   UINT32 regValue ;
   sw21553_SwIoVLanCtrl tempVLanCtrl ;

   tempVLanCtrl.portSelect = portSelect ;
   tempVLanCtrl.sendOnlyToMember = sendOnlyToMember ;
   tempVLanCtrl.notAcceptUntaggedPkt = notAcceptUntaggedPkt ;

   if ( (regValue = sw21553_SwIoSetControl( eSwIoSetVlanCntrl, 
                                         (void *)(&tempVLanCtrl) ) ) == ERROR )
   {
      /* wrong parameter */
      printk("ERROR: sysSwVLanSetVLanCtrlPort(port = %d, ...): no valid port number (%d .. %d) \n", 
              portSelect, 
              INCA_IP_SWITCH_PORT_SELECT_PC,
              INCA_IP_SWITCH_PORT_SELECT_CPU) ;
      return(ERROR) ;
   }
   else
   {
      sw21553_SwVlanDisplayVLanCtrl(regValue) ;
      return(OK) ;
   }
}

/******************************************************************************
*  sysSwVLanShowPortConfig()
*     show the current port (PC, LAN, CPU) VLAN settings
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanShowPortConfig(void)
{
   UINT32 regValue ;

   sw21553_SwVlanDisplayVLanTagHeader() ;
   /* read and display PC-port */
   regValue = sw21553_SwVLanGetVLanConfig(INCA_IP_SWITCH_PORT_SELECT_PC) ;
   sw21553_SwVlanDisplayPortVLan(INCA_IP_SWITCH_PORT_SELECT_PC, regValue) ;
   /* read and display LAN-port */
   regValue = sw21553_SwVLanGetVLanConfig(INCA_IP_SWITCH_PORT_SELECT_LAN) ;
   sw21553_SwVlanDisplayPortVLan(INCA_IP_SWITCH_PORT_SELECT_LAN, regValue) ;
   /* read and display CPU-port */
   regValue = sw21553_SwVLanGetVLanConfig(INCA_IP_SWITCH_PORT_SELECT_CPU) ;
   sw21553_SwVlanDisplayPortVLan(INCA_IP_SWITCH_PORT_SELECT_CPU, regValue) ;

   return(OK) ;
}

/******************************************************************************
*  sysSwVLanSetPortConfig()
*     set the VLAN settings for an given port
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*
*     newPrio:    new priority value to set (range 0 - 7)
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanSetPortConfig(int portSelect, UINT32 newPrio, UINT32 newVLanId)
{
   UINT32 regValue ;
   sw21553_SwIoVLanCfg tempPortVLan ;

   tempPortVLan.portSelect = portSelect ;
   tempPortVLan.newVLanId  = newVLanId ;
   tempPortVLan.newPrio    = newPrio ;


   if ( (regValue = sw21553_SwIoSetControl( eSwIoSetPortVLanConfig, 
                                         (void *)(&tempPortVLan) ) ) == ERROR )
   {
      /* wrong parameter */
      printk("ERROR: sysSwVLanSetPortConfig(port = %d, ...): no valid port number (%d .. %d) \n", 
              portSelect, 
              INCA_IP_SWITCH_PORT_SELECT_PC,
              INCA_IP_SWITCH_PORT_SELECT_CPU) ;
      return(ERROR) ;
   }
   else
   {
      sw21553_SwVlanDisplayVLanTagHeader() ;
      sw21553_SwVlanDisplayPortVLan(portSelect, regValue) ;
      return(OK) ;
   }
}


/******************************************************************************
*  sysSwVLanTableReset()
*     reset the complete VLAN table
*
*  Returns:
*     0 - Success
*     -1 ERROR:  ERROR while reset write
*
*/
int sysSwVLanTableReset(void)
{
   /* reset complete VLAN table: 0 - (INCA_IP_MAX_VLAN_TABLE_ENTRIES-1) */
   return( sw21553_SwVLanResetVLanTable() ) ;
}


/******************************************************************************
*  sysSwVLanTableSetTag()
*     set a VLAN tag within the VLAN tag table
*
*  Parameter:
*     index:      index of the VLAN Table
*                 first step: this version is only for testing
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*     newMemberInfo: 3 bit field
*                 Membership information for PC/LAN/CPU Port
*                 0: Packets from PC/LAN/CPU port is not a member of VLAN
*                 1: Packets from PC/LAN/CPU port is member of this VLAN
*     newEgressTagInfo: 3 bit field
*                 Egress Tagging information for PC/LAN/CPU Port
*                 0: Packets to PC/LAN/CPU egress port is not tagged for this VLAN
*                 1: Packets to PC/LAN/CPU egress port is tagged for this VLAN
*
*     3 bit field:
*     ============
*        Bit 0: represents PC port
*        Bit 1: represents LAN port
*        Bit 2: represents CPU port
*
*     Example: newMemberInfo = 0x5 = 101b
*            Bit 0   Bit 1   Bit 2
*        --> MP = 1, ML = 0, MC = 1
*
*     Example: newEgressTagInfo = 0x6 = 110b
*            Bit 0   Bit 1   Bit 2
*        --> EP = 1, EL = 1, EC = 0 
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanTableSetTag( int index, 
                          UINT32 newVLanId, UINT32 newMemberInfo, UINT32 newEgressTagInfo)
{
   sw21553_VLanTableEntry vLanTableEntry ;

   /* check parameter */
   if (newMemberInfo > 7)
   {
      printk("ERROR: membership info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newMemberInfo) ;
      return(ERROR) ;
   }

   if (newEgressTagInfo > 7)
   {
      printk("ERROR: egress tag info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newEgressTagInfo) ;
      return(ERROR) ;
   }

   if ( (newVLanId & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK) )
   {
      printk("ERROR: VLAN ID (%x) out of range (max: %x) \n",
              newVLanId,
              INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK-1) ;
      return(ERROR) ;
   }

   /* create VLAN table struct */
   vLanTableEntry.vLanId = newVLanId ;
   vLanTableEntry.Cntrl  = (newMemberInfo & 0x7) | 
                           ( (newEgressTagInfo & 0x7) << 3 ) |
                           INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK ;

   if ( sw21553_SwVLanWriteTableEntry(index, vLanTableEntry) == OK )
   {
      /* add VLAN entry OK */
      printk("VLAN table entry added: index = %d, VLAN ID = %x (%d), Ctrl = %x \n",
              index, 
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(OK) ;
   }
   else
   {
      /* add VLAN entry OK */
      printk("ERROR: cannot add VLAN table entry: index = %d, VLAN ID = %x (%d), Ctrl = %x \n",
              index, 
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(OK) ;
   }
}





/******************************************************************************
*  sysSwVLanAddTableEntry()
*     add a VLAN tag within the VLAN tag table (index will be calculatet)
*
*  Parameter:
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*     newMemberInfo: 3 bit field
*                 Membership information for PC/LAN/CPU Port
*                 0: Packets from PC/LAN/CPU port is not a member of VLAN
*                 1: Packets from PC/LAN/CPU port is member of this VLAN
*     newEgressTagInfo: 3 bit field
*                 Egress Tagging information for PC/LAN/CPU Port
*                 0: Packets to PC/LAN/CPU egress port is not tagged for this VLAN
*                 1: Packets to PC/LAN/CPU egress port is tagged for this VLAN
*
*     3 bit field:
*     ============
*        Bit 0: represents PC port
*        Bit 1: represents LAN port
*        Bit 2: represents CPU port
*
*     Example: newMemberInfo = 0x5 = 101b
*            Bit 0   Bit 1   Bit 2
*        --> MP = 1, ML = 0, MC = 1
*
*     Example: newEgressTagInfo = 0x6 = 110b
*            Bit 0   Bit 1   Bit 2
*        --> EP = 1, EL = 1, EC = 0 
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*  REMARK:
*     Because the VLAN table entries must be in a increasing order a local 
*     copy of the current VLAN table will be used for sorting.
*     After insertion of a new entry into the local VLAN table the
*     INCA IP VLAN table will be updated.
*
*/
int sysSwVLanAddTableEntry(UINT32 newVLanId, UINT32 newMemberInfo, UINT32 newEgressTagInfo)
{
   int index ;
   sw21553_VLanTableEntry vLanTableEntry ;

   /* check parameter */
   if (newMemberInfo > 7)
   {
      printk("ERROR: membership info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newMemberInfo) ;
      return(ERROR) ;
   }

   if (newEgressTagInfo > 7)
   {
      printk("ERROR: egress tag info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newEgressTagInfo) ;
      return(ERROR) ;
   }

   if ( (newVLanId & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK) )
   {
      printk("ERROR: VLAN ID (%x) out of range (max: %x) \n",
              newVLanId,
              INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK-1) ;
      return(ERROR) ;
   }

   /* create VLAN table struct */
   vLanTableEntry.vLanId = newVLanId ;
   vLanTableEntry.Cntrl  = (newMemberInfo & 0x7) | 
                           ( (newEgressTagInfo & 0x7) << 3 ) |
                           INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK ;

   if ( (index=sw21553_SwVLanAddTableEntry(vLanTableEntry)) != ERROR )
   {
      /* add VLAN entry OK */
      printk("VLAN table entry added: index = %d, VLAN ID = %x (%d), Ctrl = %x \n",
              index, 
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(OK) ;
   }
   else
   {
      /* add VLAN entry OK */
      printk("ERROR: whilet add VLAN table entry: VLAN ID = %x (%d), Ctrl = %x \n",
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(ERROR) ;
   }
}



/******************************************************************************
*  sysSwVLanShowTable()
*     show the VLAN table
*
*  Parameter:
*
*/
int sysSwVLanShowTable(void)
{
   int count, index ;
   UINT32 vlanTable[INCA_IP_MAX_VLAN_TABLE_ENTRIES] ;

   count =  sw21553_SwVLanReadTable(vlanTable, 0) ;

   if (count != 0)
   {
      /* display VLAN table entries */
      sw21553_SwVlanDisplayVLanTableHeader() ;

      for (index = 0; index < count; index++)
      {
         sw21553_SwVlanDisplayVLanTableEntry(index, vlanTable[index]) ;
      }
   }

   sw21553_SwVlanDisplayVLanTableTrailer(count) ;

   return(0) ;
}


/******************************************************************************
*  sysSwVLanTableReadTag()
*     read and show a VLAN tag within the VLAN tag table
*
*  Parameter:
*     index:      index of the VLAN Table
*
*/
int sysSwVLanTableReadTag( int index )
{
   UINT32 vlanTableEntry ;

   vlanTableEntry = sw21553_SwVLanReadTableEntry(index) ;

   switch (vlanTableEntry) 
   {
      case 0: /* no valid entry found at this index */
         printk("NO entry found at VLAN table[%d]\n", index ) ;
         return(0) ;

      case ERROR: /* index not valid (out of range) */
         printk("ERROR: index not valid (range 0..%d) - cannot read VLAN table[%d]\n", 
                INCA_IP_MAX_VLAN_TABLE_ENTRIES-1 ,
                index ) ;
         return(ERROR) ;

      default :
         printk("VLAN table[%d] = %08x:  VLAN ID = %x (%d), Ctrl = %x \n",
                index,
                vlanTableEntry, 
                INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(vlanTableEntry),
                INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(vlanTableEntry),
                INCA_IP_VLAN_TABLE_ENTRY_GET_CTRL_FLAGS(vlanTableEntry) ) ;
         return(OK) ;

   }

   return(ERROR) ;
}


#ifdef INCLUDE_LED_MULTIPLEXER

/******************************************************************************
   Switch/END LED's ACCESS: functions
******************************************************************************/


/******************************************************************************
*  sysSwLedMuxAllLan()
*     enable all LED's for signal LAN port properties
*
*  Parameter:
*     txAct: activate also the activities of transmit
*        0: default, signal only receive activities
*        1: signal also the transmit acitvities
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedMuxAllLan(int txAct)
{
   /* set the config register */
   if (txAct)
   {
      sw21553_SwLedSetLedMuxConfig( (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;
   }
   else
   {
      sw21553_SwLedSetLedMuxConfig( ((INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) & ~(INCA_IP_TSF_LEDMUX_CONF_ETL1) ), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;
   }

   return(0) ;
}

/******************************************************************************
*  sysSwLedDeMuxAllLan()
*     deactivate multiplexing of all LED's for signal LAN port properties
*
*  Parameter:
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedDeMuxAllLan(void)
{
   /* reset LEDMUX_CONF register */
   sw21553_SwLedSetLedMuxConfig( 0, (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;

   return(0) ;
}


/******************************************************************************
*  sysSwLedMuxLan()
*     configure and set the LED's for signal the LAN port properties
*
*  Parameter:
*     status: display the status of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: normal
*                LED on:  ehternet port is disabled
*                LED flashing: ERROR condition
*     speed:  displays the speed of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: 10 Mbit/s, LED on: 100 Mbit/s
*     Duplex: displays the duplex mode of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: Half Duplex, LED on: Full Duplex
*     Activity: signals any receive activities on the ethernet port
*             0: do not signal activities
*             1: do signal activities 
*                LED on (non-flashing): Link without activity
*                LED on (flashing):     Port receiving (or transmitting, see txAct)
*     txAct:  signals also the transmit activities
*             0: do not signal transmit activities
*             1: do signal transmit activities 
*
*  Returns:
*     always 0: success
*
*  REMARK:
*
*/
int sysSwLedMuxLan(int status, int speed, int duplex, int Activity, int txAct)
{
   UINT32 newSettings = 0 ;

   /* create LEDMUX_CONF settings */
   newSettings |= (status == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_ESTA1) : 0 ;
   newSettings |= (speed == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_EACT1) : 0 ;
   newSettings |= (duplex == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_EDPX1) : 0 ;
   newSettings |= (Activity == 1) ? (INCA_IP_TSF_LEDMUX_CONF_ESPD1) : 0 ;
   newSettings |= (txAct == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_ETL1)  : 0 ;

   sw21553_SwLedSetLedMuxConfig( (newSettings), 
                                 (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;

   return(0) ;
}



#if defined(INCA_STANDARD)

/******************************************************************************
*  sysSwLedMuxAllPc()
*     enable all LED's for signal PC port properties
*
*  Parameter:
*     txAct: activate also the activities of transmit
*        0: default, signal only receive activities
*        1: signal also the transmit acitvities
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedMuxAllPc(int txAct)
{
   /* set the config register */
   if (txAct)
   {
      sw21553_SwLedSetLedMuxConfig( (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;
   }
   else
   {
      sw21553_SwLedSetLedMuxConfig( ((INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) & ~(INCA_IP_TSF_LEDMUX_CONF_ETL0) ), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;
   }

   return(0) ;
}

/******************************************************************************
*  sysSwLedDeMuxAllPc()
*     deactivate all LED's for signal PC port properties
*
*  Parameter:
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedDeMuxAllPc(void)
{
   /* reset LEDMUX_CONF register */
   sw21553_SwLedSetLedMuxConfig( 0, (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;

   return(0) ;
}

/******************************************************************************
*  sysSwLedMuxPc()
*     configure and set the LED's for signal the PC port properties
*
*  Parameter:
*     status: display the status of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: normal
*                LED on:  ehternet port is disabled
*                LED flashing: ERROR condition
*     speed:  displays the speed of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: 10 Mbit/s, LED on: 100 Mbit/s
*     Duplex: displays the duplex mode of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: Half Duplex, LED on: Full Duplex
*     Activity: signals any receive activities on the ethernet port
*             0: do not signal activities
*             1: do signal activities 
*                LED on (non-flashing): Link without activity
*                LED on (flashing):     Port receiving (or transmitting, see txAct)
*     txAct:  signals also the transmit activities
*             0: do not signal transmit activities
*             1: do signal transmit activities 
*
*  Returns:
*     always 0: success
*
*  REMARK:
*
*/
int sysSwLedMuxPc(int status, int speed, int duplex, int Activity, int txAct)
{
   UINT32 newSettings = 0 ;

   /* create LEDMUX_CONF settings */
   newSettings |= (status == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_ESTA0) : 0 ;
   newSettings |= (speed == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_EACT0) : 0 ;
   newSettings |= (duplex == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_EDPX0) : 0 ;
   newSettings |= (Activity == 1) ? (INCA_IP_TSF_LEDMUX_CONF_ESPD0) : 0 ;
   newSettings |= (txAct == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_ETL0)  : 0 ;

   sw21553_SwLedSetLedMuxConfig( (newSettings), 
                                 (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;

   return(0) ;
}

#endif      /* #if defined(INCA_STANDARD) */

#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */


/*
   for testing
   ===========
   calculate the appropriate hash value for the given 
   MAC address (string form 00:22:33:aa:bb:cc) and the 
   requested modulation

   Remark: 
      modulation = 3 --> calculate hash from original MAC address
*/
int sysSwMacCalcStrHashValue(int modulateNum, char *strMacAddr)
{
   UCHAR tempMacAddr[6] ;
   UCHAR hashValue ;
   
   sw21553_SwMacConvertStr2Hex(strMacAddr, tempMacAddr) ;
   printk("MAC Address: Str<%s> --> Hex %2x-%2x-%2x-%2x-%2x-%2x \n",
           strMacAddr,
           tempMacAddr[0], tempMacAddr[1], tempMacAddr[2],
           tempMacAddr[3], tempMacAddr[4], tempMacAddr[5] ) ;

   hashValue = sw21553_IncaIpSwitchCalcHash(modulateNum, tempMacAddr) ;

   printk("Modulation %d: MAC addr = %2x-%2x-%2x-%2x-%2x-%2x --> hash = 0x%02x (%3d) \n",
           modulateNum,
           tempMacAddr[0], tempMacAddr[1], tempMacAddr[2],
           tempMacAddr[3], tempMacAddr[4], tempMacAddr[5],
           hashValue, hashValue) ;
   return(OK) ;
}

/*
   for testing
   ===========
   calculate the appropriate hash value for the given 
   MAC address for all available modulations.

   Remark: 
      modulation = 3 --> calculate hash from original MAC address
*/
int sysSwMacCalcStrAllHashValue(char *strMacAddr)
{
   int modulateNum ;
   UCHAR tempMacAddr[6] ;
   UCHAR hashValue ;

   printk("\n-----------------------------------------\n") ;
   sw21553_SwMacConvertStr2Hex(strMacAddr, tempMacAddr) ;
   printk("MAC Address: Str<%s> --> Hex %2x-%2x-%2x-%2x-%2x-%2x \n",
           strMacAddr,
           tempMacAddr[0], tempMacAddr[1], tempMacAddr[2],
           tempMacAddr[3], tempMacAddr[4], tempMacAddr[5] ) ;
   for (modulateNum = 0; modulateNum < 4; modulateNum++)
   {
      printk("-----------------------------------------\n") ;
      hashValue = sw21553_IncaIpSwitchCalcHash(modulateNum, tempMacAddr) ;
      printk("Modulation %d: HASH value = 0x%02x (%3d) \n",
              modulateNum, hashValue, hashValue) ;
   }
   printk("-----------------------------------------\n\n") ;
   return(OK) ;
}

/* Linux specific enhancements */
void sw21553_PrintReg(UINT32 addr);
void sw21553_WrReg(UINT32 addr, UINT32 value);
static struct switch_api_command {
    const char *name;
    const void *function;
    const char *param;          /* i=int, l=long, s=string */
} commands[] = {
    { "sysPhyShowLan", sysPhyShowLan, "i" },
    { "sysPhyShowPc", sysPhyShowPc, "i" },
    { "sysPhySetLanAdvertise", sysPhySetLanAdvertise, "li" },
    { "sysPhySetPcAdvertise", sysPhySetPcAdvertise, "li" },
    { "sysPhySetLanFixed", sysPhySetLanFixed, "li" },
    { "sysPhySetPcFixed", sysPhySetPcFixed, "li" },
    { "sysPhyLanReAutoNeg", sysPhyLanReAutoNeg, "" },
    { "sysPhyPcReAutoNeg", sysPhyPcReAutoNeg, "" },
    { "sysPhyLanDisableAutoNeg", sysPhyLanDisableAutoNeg, "" },
    { "sysPhyLanEnableAutoNeg", sysPhyLanEnableAutoNeg, "" },
    { "sysPhyPcDisableAutoNeg", sysPhyPcDisableAutoNeg, "" },
    { "sysPhyPcEnableAutoNeg", sysPhyPcEnableAutoNeg, "" },
    { "sysPhyLanReset", sysPhyLanReset, "" },
    { "sysPhyPcReset", sysPhyPcReset, "" },
    { "sysSwUnDestShow", sysSwUnDestShow, "" },
    { "sysSwUnDestSetPorts", sysSwUnDestSetPorts, "ii" },
    { "sysSwSpPtSetPorts", sysSwSpPtSetPorts, "ii" },
    { "sysSwSpPtShow", sysSwSpPtShow, "" },
    { "sysSwPktLenSet", sysSwPktLenSet, "li" },
    { "sysSwPktLenShow", sysSwPktLenShow, "" },
    { "sysSwMacShowTable", sysSwMacShowTable, "" },
    { "sysSwMacAddTableEntry", sysSwMacAddTableEntry, "siii" },
    { "sysSwMacClearEntryByMac", sysSwMacClearEntryByMac, "s" },
    { "sysSwMacClearEntryByIndex", sysSwMacClearEntryByIndex, "i" },
    { "sysSwVLanShowVLanCtrl", sysSwVLanShowVLanCtrl, "" },
    { "sysSwVLanSetVLanCtrlPort", sysSwVLanSetVLanCtrlPort, "iii" },
    { "sysSwVLanUpdVLanCtrlVAware", sysSwVLanUpdVLanCtrlVAware, "i" },
    { "sysSwVLanShowPortConfig", sysSwVLanShowPortConfig, "" },
    { "sysSwVLanSetPortConfig", sysSwVLanSetPortConfig, "ill" },
    { "sysSwVLanTableReset", sysSwVLanTableReset, "" },
    { "sysSwVLanTableReadTag", sysSwVLanTableReadTag, "i" },
    { "sysSwVLanAddTableEntry", sysSwVLanAddTableEntry, "lll" },
    { "sysSwVLanShowTable", sysSwVLanShowTable, "" },
    { "sysSwVLanTableSetTag", sysSwVLanTableSetTag, "illl" },
#ifdef INCLUDE_LED_MULTIPLEXER
    { "sysSwLedMuxAllLan", sysSwLedMuxAllLan, "i" },
    { "sysSwLedDeMuxAllLan", sysSwLedDeMuxAllLan, "" },
    { "sysSwLedMuxLan", sysSwLedMuxLan, "iiiii" },
#if defined(INCA_STANDARD)
    { "sysSwLedMuxAllPc", sysSwLedMuxAllPc, "i" },
    { "sysSwLedDeMuxAllPc", sysSwLedDeMuxAllPc, "" },
    { "sysSwLedMuxPc", sysSwLedMuxPc, "iiiii" },
#endif      /* #if defined(INCA_STANDARD) */
#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */
    { "sw21553_SwMacResetMacTable", sw21553_SwMacResetMacTable, "" },
    { "sw21553_SwUpdSpPtPortState", sw21553_SwUpdSpPtPortState, "ii" },
    { "sw21553_PrintReg", sw21553_PrintReg, "l" },
    { "sw21553_WrReg", sw21553_WrReg, "ll" }
};

static const char *get_string(const char **buffer, int *len)
{
    const char *s1 = *buffer;
    const char *s2;
    int quoted = 0;

    while (*s1 == ' ') { s1++; }

    if ((*s1 == '\0') || (*s1 == '\n')) /* end of string */
    {
        *len = 0;
        return NULL;
    }

    if (*s1 == '"')
    {
        quoted = 1;
        s1++;
    }

    s2 = s1;
    while ((*s2) && (*s2 != '\n'))
    {
        if (((*s2 == ' ') && (!quoted)) || 
            ((*s2 == '"') && ( quoted)) ||
            ((*s2 == ',') && (!quoted)))
        {
            break;
        }
        s2++;
    }

    *len = (s2 - s1);
    
    if (quoted && (*s2 == '"')) s2++;
    if (*s2 == ',') s2++;       /* remove comma between arguments */
    *buffer = s2;               /* save start of next token */
    
/*      { */
/*          int i; */
/*          printk("get_string: \""); */
/*          for (i = 0; i < *len; i++) printk("%c", s1[i]); */
/*          printk("\"\n"); */
/*      } */

    return s1;
}

static long atoi(const char *buffer, int len)
{
    long val = 0;
    int hex = 0;

    if ((len > 2) && (buffer[0]=='0') && (buffer[1]=='x'))
    {
        hex = 1;
        len -= 2;
        buffer += 2;
    }

    while (*buffer && len)
    {
        if ((*buffer >= '0') && (*buffer <= '9'))
        {
            val = val * ((hex) ? 16 : 10) + (*buffer - '0');
        }
        else if ((hex && (*buffer >= 'A') && (*buffer <= 'F')) ||
                 (hex && (*buffer >= 'a') && (*buffer <= 'f')))
        {
            val = val * 16 + ((*buffer & ~0x20) - 'A' + 10);
        }
        else
        {
            printk("atoi: invalid character: '%c'\n", *buffer);
            return 0;
        }
        len--;
        buffer++;
    }

    return val;
}

static int inca_switch_read_procmem(char *buf, char **start, off_t offset, 
                                        int count, int *eof, void *data)
{
    int len = 0, c;
    
    len += sprintf(buf+len,"Available commands:\n");

    for (c = 0; c < (sizeof(commands)/sizeof(commands[0])); c++)
    {
        len += sprintf(buf+len, "\t%s\t%s\n", 
                       commands[c].name, commands[c].param);
    }    
    *eof = 1;	// No more data available
    
    return len;
}

static int inca_switch_write_procmem(struct file *file, const char *buffer,
					unsigned long count, void *data)
{
    const char *buf = buffer;
    const char *cmd;
    const char *parg_string;
    const char *arg;
    int     slen, c;
    long    cmd_arg[10]={}; /* max. 10 command arguments (integer/long/string) */
    char    cmd_arg_strings[100]; /* max 100 characters as string parameters */
    int     cmd_arg_str_offset = 0;
    int     argc = 0;
    int     ret = 0;
    
    cmd = get_string(&buf, &slen);
    if ((cmd == NULL) || (slen == 0))
    {
        printk("No/invalid command specified (%s)\n", buffer);
        return -EINVAL;
    }
    
    for (c = 0; c < (sizeof(commands)/sizeof(commands[0])); c++)
    {
        if (strncmp(cmd, commands[c].name, slen) == 0)
        {
            /* yeah, we found our command */
/*              printk("Command found: name=\"%s\", fct=%p, param=\"%s\"\n", */
/*                     commands[c].name, commands[c].function, commands[c].param); */
            break;
        }
    }
    
    if (c >= (sizeof(commands)/sizeof(commands[0])))
    {
        int i;
        printk("Command not found (");
        for (i = 0; i < slen; i++) 
            printk("%c", cmd[i]);
        printk(")\n");
        return -EINVAL;
    }
    
    parg_string = commands[c].param;
    while ((*parg_string) && (argc < (sizeof(cmd_arg)/sizeof(cmd_arg[0]))))
    {
        arg = get_string(&buf, &slen);
        if ((arg == NULL) || (slen == 0))
        {
            printk("Too few parameters for command \"%s\", required are \"%s\"\n",
                   commands[c].name, commands[c].param);
            printk("arg=%p, slen=%d\n", arg, slen);
            return -EINVAL;
        }
        
        switch (*parg_string)
        {
            case 'i':
            case 'l':
                cmd_arg[argc++] = atoi(arg, slen);
                break;
            case 's':
                memcpy(&cmd_arg_strings[cmd_arg_str_offset], arg, slen);
                cmd_arg_strings[cmd_arg_str_offset + slen] = '\0';
                cmd_arg[argc++] = (long)&cmd_arg_strings[cmd_arg_str_offset];
                cmd_arg_str_offset += slen+1;
                break;
            default:
                printk("Invalid format string in module, please fix and re-compile\n");
                return -EINVAL;
        }
        
        parg_string++;
    }

    arg = get_string(&buf, &slen);
    if ((arg != NULL) && (slen != 0))
    {
        printk("Too many parameters for command \"%s\", required are \"%s\"\n",
               commands[c].name, commands[c].param);
        return -EINVAL;
    }
    
    /* This instructions casts the void pointer commands.function to a function that
     returns int and has 10 long parameters. Though the function actually called has
     a different argument list it works if the parameter are set up accordingly
     (e.g. int/long or string pointer) */
    ret = ((int (*)(long,long,long,long,long,long,long,long,long,long))(commands[c].function))
        (cmd_arg[0], cmd_arg[1], cmd_arg[2], cmd_arg[3], 
         cmd_arg[4], cmd_arg[5], cmd_arg[6], cmd_arg[7], 
         cmd_arg[8], cmd_arg[9]);
    printk("return value is: %d (0x%08x)\n", ret, ret);
    
    return count;
	
}

/**
 *	inca_switch_read_procmem - Create proc file output
 *	@buf:      Buffer to write the string to
 *	@start:    not used (Linux internal)
 *	@offset:   not used (Linux internal)
 *	@count:    not used (Linux internal)
 *	@eof:      Set to 1 when all data is stored in buffer
 *	@data:     not used (Linux internal)
 *
 *	This function creates the output for the switch proc file.
 *
 *  	Return Value:	
 *	@len = Lenght of data in buffer
 */
int inca_switch_lan_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
  long	len = 0;
  UINT32  properties;
  int phyInfo = 0;

  if ((properties = sw21553_PhyGetProperties(INCA_IP_MDIO_LAN_PHY, phyInfo)) != ERROR) {
    char c =((properties & sw21553_PHY_CTRL_LINK_STATE) ? '1' : '0');

    len = sprintf(buf,"%c\n", c);
  }
  return len;
}

static int inca_switch_open(struct inode *inode, struct file *filep)
{
  return OK;
}

static int inca_switch_release(struct inode *inode, struct file *filep)
{
  return OK;
}

static int inca_switch_set_auto_negotiation(volatile struct inca_switch_ioctl_properties *parm, int enable)
{
  UINT32 properties = 0;
              
  if (parm->phy == 0) {
    properties = sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, !enable);
  } else if (parm->phy == 1) {
    properties = sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, !enable);
  }

#ifdef DEBUG
  printk("KERNEL: PhyDisEnAutoNeg Result: %d!\n", properties);
#endif

  if (properties == ERROR) {
    printk("ERROR - while request PhyDisEnAutoNeg PHY %d Enable %d informations.\n", parm->phy, enable);
    return -1;
  }
                
  return 0;
}

static int inca_switch_ioctl(struct inode *inode, struct file *filep, 
                             unsigned int cmd, unsigned long arg) 
{
	int ret = 0;
// 	volatile struct inca_switch_ioctl_parm parm;

// 	if (_IOC_TYPE(cmd) != INCA_SWITCH_IOC_MAGIC) return -EINVAL;
	
// 	if (_IOC_DIR(cmd) & _IOC_WRITE) {
// 		if (!access_ok(VERIFY_READ, arg, sizeof(struct inca_switch_ioctl_parm))) 
// 			return -EFAULT;
// 		ret=copy_from_user((void*)&parm,(void*)arg,sizeof(struct inca_switch_ioctl_parm));
// 	}
// 	if (_IOC_DIR(cmd) & _IOC_READ) {
// 		if (!access_ok(VERIFY_WRITE, arg, sizeof(struct inca_switch_ioctl_parm))) 
// 			return -EFAULT;
// 	}

	switch (cmd) {
          case INCA_SWITCH_IOC_LAN_LINK:
          {
            UINT32 properties;
            int phyInfo = 0;

            if ((properties = sw21553_PhyGetProperties(INCA_IP_MDIO_LAN_PHY, phyInfo)) != ERROR) {
              /* display and interpret informations */              
              ret=((properties & sw21553_PHY_CTRL_LINK_STATE) ? 1 : 0);
#ifdef DEBUG
              printk("Status LAN Port PHY (%d)\n", ret) ;              
#endif
//            copy_to_user((void*)arg,(void*)&parm,sizeof(struct inca_switch_ioctl_parm));
            }
            else {
              printk("ERROR - while request LAN PHY informations.\n") ;
            }
          }
          break;
          
          case INCA_SWITCH_IOC_GET_PROPERTIES:
          {
            volatile struct inca_switch_ioctl_properties parm;
            UINT32 properties = 0L;

            ret = copy_from_user((void*)&parm,(void*)arg,sizeof(struct inca_switch_ioctl_properties));

#ifdef DEBUG
            printk("KERNEL: Read Link %d!\n", parm.phy);
#endif
            if (parm.phy == 0) {
              properties = sw21553_PhyGetProperties(INCA_IP_MDIO_LAN_PHY, sw21553_PHY_PROPS_CURRENT);
            } else if (parm.phy == 1) {
              properties = sw21553_PhyGetProperties(INCA_IP_MDIO_PC_PHY, sw21553_PHY_PROPS_CURRENT);
            }
              
            if (properties != ERROR) {

// parm value description:
// phy:       0 = LAN port, 1 = PC port
// speed:    10 = 10Mbit, 100 = 100Mbit
// half_full: 1 = half duplex, 2 = full duplex

              parm.link             = (properties & sw21553_PHY_CTRL_LINK_STATE) ? 1 : 0;
              parm.speed            = (properties & sw21553_PHY_CTRL_SPEED_10_IND) ? 10 : 0;
              parm.speed            = parm.speed + ((properties & sw21553_PHY_CTRL_SPEED_100_IND) ? 100 : 0);
              parm.half_full        = (properties & sw21553_PHY_CTRL_SPEED_DUPLEX_IND) ? 2 : 1;
              parm.auto_negotiation = (properties & sw21553_PHY_CTRL_AUTO_NEG_COMPL) ? 1 : 0;
              parm.valid            = (properties & sw21553_PHY_CTRL_DATA_CORRUPTED) ? 0 : 1;
              parm.properties       = properties;
              
              ret = parm.link;
#ifdef DEBUG
              printk("Status PHY %d (%d)\n", parm.phy, ret);
#endif
              copy_to_user((void*)arg,(void*)&parm,sizeof(struct inca_switch_ioctl_properties));
            }
            else {
              printk("ERROR - while request PhyGetProperties PHY %d informations.\n", parm.phy);
              ret = -1;
            }
          }
          break;
          
          case INCA_SWITCH_IOC_SET_PROPERTIES:
          {
            volatile struct inca_switch_ioctl_properties parm;
            UINT32 properties = 0L;
            UINT32 speed = 0L; // half duplex

// parm value description:
// phy:       0 = LAN port, 1 = PC port
// speed:    10 = 10Mbit, 100 = 100Mbit, 0 = auto negotiation
// half_full: 1 = half duplex, 2 = full duplex, 0 = auto negotiation

            ret = copy_from_user((void*)&parm,(void*)arg,sizeof(struct inca_switch_ioctl_properties));
#ifdef DEBUG
            printk("KERNEL: Set Properties of port %d (ret = %d)!\n", parm.phy, ret);
#endif
            if (parm.phy < 0 || parm.phy > 1) {
              printk("KERNEL: No port set, return!\n");
              break;
            }
            
            if (parm.auto_negotiation == 0){
#ifdef DEBUG
              printk("KERNEL: Auto Negotiation off!\n");
#endif            
              if (inca_switch_set_auto_negotiation(&parm, 0) < 0) {
                break;
              }

              if (parm.speed == 10) {
#ifdef DEBUG
                printk("KERNEL: 10 Mbit!\n");
#endif
                speed = sw21553_PHY_SPEED_10_BIT; // 10Mbit
              }
              else if (parm.speed == 100) {
#ifdef DEBUG
                printk("KERNEL: 100 Mbit!\n");
#endif
                speed = sw21553_PHY_SPEED_100_BIT; // 100Mbit
              } else {
                printk("KERNEL: No speed set, return!\n");
                break;
              }
                
              if (parm.half_full == 2) {
#ifdef DEBUG
                printk("KERNEL: Full Duplex!\n");
#endif
                speed = speed | sw21553_PHY_SPEED_DUPLEX_BIT; // full duplex
              } else if (parm.half_full == 1) {
#ifdef DEBUG
                printk("KERNEL: Half Duplex!\n");
#endif
              } else {
                printk("KERNEL: No half_full set, return!\n");
                break;
              }
                
              if (parm.phy == 0) {
                properties = sw21553_PhySetFixSpeed(INCA_IP_MDIO_LAN_PHY, speed);
              } else if (parm.phy == 1) {
                properties = sw21553_PhySetFixSpeed(INCA_IP_MDIO_PC_PHY, speed);
              }

#ifdef DEBUG
              printk("KERNEL: PhySetFixSpeed Result: %d!\n", properties);
#endif
              if (properties != ERROR) {
                ret = 0;
              }
              else {
                ret = -1;
                printk("ERROR - while request PhySetFixSpeed PHY %d informations.\n", parm.phy);
                break;
              } 

            } else if (parm.auto_negotiation == 1){
#ifdef DEBUG
              printk("KERNEL: Auto Negotiation on!\n");
#endif              
              if (inca_switch_set_auto_negotiation(&parm, 1) < 0) {
                break;
              }

              if (parm.phy == 0) {
                properties = sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_LAN_PHY);
              } else if (parm.phy == 1) {
                properties = sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_PC_PHY);
              }
  
#ifdef DEBUG
              printk("KERNEL: PhyRestartAutoNeg Result: %d!\n", properties);
#endif
              if (properties == ERROR) {
                printk("ERROR - while request PhyRestartAutoNeg PHY %d informations.\n", parm.phy);
                ret = -3;
                break;
              }

            } else {
              ret = -2;
              printk("KERNEL: No Auto Negotiation info set, return!\n");
              break;
            }
            
            copy_to_user((void*)arg,(void*)&parm,sizeof(struct inca_switch_ioctl_properties));
          }            
          break;
          default:
            ret = -EINVAL;
	}

	return ret;
}

static const char *dev_name="driver/switch_api";
static const char *dev2_name="driver/inca_switch_lan";

static struct file_operations switch_fops = {
 	open    : inca_switch_open,
 	release : inca_switch_release,
	ioctl   : inca_switch_ioctl
};

int inca_switch_api_init(void)
{
    struct proc_dir_entry *inca_switch_proc;
    struct proc_dir_entry *inca_switch_lan_proc;
    int err = 0;

    inca_switch_proc = create_proc_entry(dev_name, 0, 0);
    if (!inca_switch_proc) {
        printk(KERN_ERR "could not create /proc/%s!\n", dev_name);
        return -ENOENT;
    }
    inca_switch_proc->read_proc  = inca_switch_read_procmem;
    inca_switch_proc->write_proc = inca_switch_write_procmem;
#ifdef DEBUG
    printk(KERN_NOTICE "/proc/%s created!\n", dev_name);
#endif

    sw21553_IncaIpInitHashGenPoly();

    /* register switch device */
    err = register_chrdev(0,"inca-switch-lan", &switch_fops);
    if (err > 0) switch_major = err;
    else {
      printk("inca-port: Error! Could not register switch_lan device. #%d\n",err);
      return err;
    }

    SET_MODULE_OWNER(&switch_fops);
	
    printk(KERN_NOTICE "Switch device registered. major: %d.\n", switch_major);
	
    inca_switch_lan_proc = create_proc_read_entry(dev2_name, 0, NULL, inca_switch_lan_read_procmem, NULL);
    if (!inca_switch_lan_proc) {
      printk(KERN_ERR "could not create /proc/%s!\n", dev2_name);
      return -ENOENT;
    }
    inca_switch_proc->read_proc  = inca_switch_read_procmem;
//  inca_switch_proc->write_proc = inca_switch_write_procmem;

    printk(KERN_NOTICE "/proc/%s created!\n", dev2_name);

    return 0;
}

void inca_switch_api_exit(void)
{
    remove_proc_entry(dev_name, 0);
#ifdef DEBUG
    printk(KERN_NOTICE "/proc/%s removed!\n", dev_name);
#endif
}

MODULE_LICENSE("Infineon");

module_init(inca_switch_api_init);
module_exit(inca_switch_api_exit);

EXPORT_SYMBOL(sysPhyShowLan);
EXPORT_SYMBOL(sysPhyShowPc);
EXPORT_SYMBOL(sysPhySetLanAdvertise);
EXPORT_SYMBOL(sysPhySetPcAdvertise);
EXPORT_SYMBOL(sysPhySetLanFixed);
EXPORT_SYMBOL(sysPhySetPcFixed);
EXPORT_SYMBOL(sysPhyLanReAutoNeg);
EXPORT_SYMBOL(sysPhyPcReAutoNeg);
EXPORT_SYMBOL(sysPhyLanDisableAutoNeg);
EXPORT_SYMBOL(sysPhyLanEnableAutoNeg);
EXPORT_SYMBOL(sysPhyPcDisableAutoNeg);
EXPORT_SYMBOL(sysPhyPcEnableAutoNeg);
EXPORT_SYMBOL(sysPhyLanReset);
EXPORT_SYMBOL(sysPhyPcReset);
EXPORT_SYMBOL(sysSwUnDestShow);
EXPORT_SYMBOL(sysSwUnDestSetPorts);
EXPORT_SYMBOL(sysSwSpPtSetPorts);
EXPORT_SYMBOL(sysSwSpPtShow);
EXPORT_SYMBOL(sysSwPktLenSet);
EXPORT_SYMBOL(sysSwPktLenShow);
EXPORT_SYMBOL(sysSwMacShowTable);
EXPORT_SYMBOL(sysSwMacAddTableEntry);
EXPORT_SYMBOL(sysSwMacClearEntryByMac);
EXPORT_SYMBOL(sysSwMacClearEntryByIndex);
EXPORT_SYMBOL(sysSwVLanShowVLanCtrl);
EXPORT_SYMBOL(sysSwVLanSetVLanCtrlPort);
EXPORT_SYMBOL(sysSwVLanUpdVLanCtrlVAware);
EXPORT_SYMBOL(sysSwVLanShowPortConfig);
EXPORT_SYMBOL(sysSwVLanSetPortConfig);
EXPORT_SYMBOL(sysSwVLanTableReset);
EXPORT_SYMBOL(sysSwVLanTableReadTag);
EXPORT_SYMBOL(sysSwVLanAddTableEntry);
EXPORT_SYMBOL(sysSwVLanShowTable);
EXPORT_SYMBOL(sysSwVLanTableSetTag);
#ifdef INCLUDE_LED_MULTIPLEXER
EXPORT_SYMBOL(sysSwLedMuxAllLan);
EXPORT_SYMBOL(sysSwLedDeMuxAllLan);
EXPORT_SYMBOL(sysSwLedMuxLan);
#if defined(INCA_STANDARD)
EXPORT_SYMBOL(sysSwLedMuxAllPc);
EXPORT_SYMBOL(sysSwLedDeMuxAllPc);
EXPORT_SYMBOL(sysSwLedMuxPc);
#endif      /* #if defined(INCA_STANDARD) */
#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */
