/******************************************************************************
       Copyright (c) 2003, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : inca_switch_control.c
   Date        : 14.07.2003
   Description : 

   $Log: inca_switch_control.c,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.3  2004/02/24 12:38:58  de6wanme
   Initial Auto-MDIX support added

   Revision 1.1  2003/07/16 14:31:13  rjanse_a
   Initial revision.


   
******************************************************************************/

#define EXPORT_SYMTAB

#include <linux/module.h>
#include <linux/init.h>
#include <asm/incaip/inca-ip.h>

#include "vxWorks_type.h"
#include "inca_switch_settings.h"
#include "inca_mdio_settings.h"
#include "inca_switch_access.h"

/******************************************************************************
   Common definitions
******************************************************************************/

#define SW_WRITE_REG(reg, value)   \
         *((volatile u32*)reg) = (u32)value;\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         *((volatile u32*)reg) = (u32)value;

#define SW_READ_REG(reg, value)    \
         value = (u32)*((volatile u32*)reg);\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         asm("nop;nop;nop;nop;nop");\
         value = (u32)*((volatile u32*)reg);

#define INCA_IP_DEBUG
#undef  INCA_IP_DEBUG

/* DEBUG MACROS */
#ifdef SW_DEBUG_ENABLE
   /*#define SW_DEBUG_RX_PRINTOUT*/

   /* debug print out masks */
   #define SW_DEBUG_OFF          0x00000000
   #define SW_DEBUG_INIT         0x00000001
   #define SW_DEBUG_RX           0x00000002
   #define SW_DEBUG_TX           0x00000004
   #define SW_DEBUG_POLL_RX      0x00000008
   #define SW_DEBUG_POLL_TX      0x00000010
   #define SW_DEBUG_POLL         0x00000018 /*(POLL_TX + POLL_RX)*/
   #define SW_DEBUG_IOCTL        0x00000020
   #define SW_DEBUG_INT          0x00000040
   #define SW_DEBUG_QUEUE        0x00000080
   #define SW_DEBUG_EVENT        0x00000100
   #define SW_DEBUG_PHY          0x00000200
   #define SW_DEBUG_FULL         0xFFFFFFFF

/*   #define SW_DEBUG_MASK       (SW_DEBUG_FULL & ~SW_DEBUG_QUEUE & ~SW_DEBUG_INT)*/
/*   #define SW_DEBUG_MASK         (SW_DEBUG_INIT | SW_DEBUG_RX | SW_DEBUG_TX)*/
/*   #define SW_DEBUG_MASK         (SW_DEBUG_PHY | SW_DEBUG_INT) */
/*   #define SW_DEBUG_MASK         (SW_DEBUG_PHY | SW_DEBUG_INT) */
/*   #define SW_DEBUG_MASK         (SW_DEBUG_FULL) */
/*   #define SW_DEBUG_MASK         (SW_DEBUG_INIT) */
   #define SW_DEBUG_MASK         (SW_DEBUG_FULL)

   #define SW_DEBUG(type,X)         if((UINT32)type & SW_DEBUG_MASK) printf X
   #define SW_LOG(type,S)           if((UINT32)type & SW_DEBUG_MASK) logMsg(S,0,0,0,0,0,0)
   #define SW_LOG1(type,S,P1)       if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,0,0,0,0,0)
   #define SW_LOG2(type,S,P1,P2)    if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,0,0,0,0)
   #define SW_LOG3(type,S,P1,P2,P3) if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,P3,0,0,0)
   #define SW_LOG4(type,S,P1,P2,P3,P4)    if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,0,0)
   #define SW_LOG5(type,S,P1,P2,P3,P4,P5) if((UINT32)type & SW_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,P5,0)

   #undef LOCAL
   #define LOCAL

#else
   #define SW_DEBUG(type,X)          
   #define SW_LOG(type,S)            
   #define SW_LOG1(type,S,P1)        
   #define SW_LOG2(type,S,P1,P2)     
   #define SW_LOG3(type,S,P1,P2,P3)
   #define SW_LOG4(type,S,P1,P2,P3,P4)
   #define SW_LOG5(type,S,P1,P2,P3,P4,P5)

#endif /* SW_DEBUG_ENABLE */

/******************************************************************************
*  sw21553_InitChip_PowerONSwitch()
*     do additional basic initialization of Ethernet and PHY 
*     Power ON the switch module via the PMU (Power Management Unit)
*/
LOCAL void sw21553_InitChip_PowerONSwitch(void)
{
   /* 
      switch on the PMU bit for the switch module and the PC port 
      EN5: ethernet switch global enable
      EN6: ethernet port 2 (PC) global enabel
   */
#if !defined(CONFIG_INCAIP_COMPACT)
    /* 'Standard' INCA-IP are supported */
    *INCA_IP_PMU_PM_GEN |= INCA_IP_PMU_PM_GEN_EN5 | INCA_IP_PMU_PM_GEN_EN6;
#else
    /* 'Compact' INCA-IP are supported */
    *INCA_IP_PMU_PM_GEN |= INCA_IP_PMU_PM_GEN_EN5;
#endif

    /* To workaround a problem with collision counter (see Errata sheet) */
    SW_WRITE_REG (INCA_IP_Switch_PC_TX_CTL, 0x00000001);
    SW_WRITE_REG (INCA_IP_Switch_LAN_TX_CTL, 0x00000001);
    SW_DEBUG (SW_DEBUG_INIT, ("sw21553_InitChip_PowerONSwitch: PMU switch ON switch module\n"));
   return ;
}

/******************************************************************************
*  sw21553_InitChip_DisableAllPorts()
*
*     do additional basic initialization of Ethernet and PHY 
*     IncaIpDisableAllPorts:  disable all port, 
*     --> do at the beginning to avoid traffic while init
*/
LOCAL void sw21553_InitChip_DisableAllPorts(void)
{

   /* enable all ports of the INCA IP ethernet switch   
      ST_PT:
         bit[5:4]: CPS (CPU port status)   0x0 (disabled)
         bit[3:2]: LPS (LAN port status)   0x0 (disabled)
         bit[1:0]: PPS (PC port status)    0x0 (disabled)
   */
   sw21553_SwIOSpPt(0x00, 0xFFFFFFFF, 1) ;
   SW_DEBUG (SW_DEBUG_INIT, ("sw21553_InitChip_DisableAllPorts: all ports disabled\n"));
   return ;
}

/******************************************************************************
*  sw21553_InitChip_SwitchBasicInit()
*     do additional basic initialization of Ethernet and PHY 
*     --> enable CRC generation
*     --> disable "ADD TAG" for virtual LAN
*     --> setup the own MAC address to the PMAC 
*
*  Parameter:
*     pDrvCtrl: ptr to the device control sturct (contains MAC address)
*
*/
LOCAL void sw21553_InitChip_SwitchBasicInit(void)
{

    /* drop all non critical packets when corresponding WM-2 is reached */
    sw21553_SwNonCritPacketDrop( TRUE, TRUE, TRUE, TRUE );			
	
    /* set watermarks */
    sw21553_SwCoSSetGlobalWatermarks( 0x06, 0x03, 0x00, 0x0c ); /* default */	
    sw21553_SwCoSSetRxWatermarks( INCA_IP_SWITCH_PORT_SELECT_PC,  
                                  0x18, 0x1E, 0x20, 0x14 );	/* 1/3 of buffer */
    sw21553_SwCoSSetRxWatermarks( INCA_IP_SWITCH_PORT_SELECT_LAN, 
	                              0x18, 0x1E, 0x20, 0x14 );	/* 1/3 of buffer */									 
    sw21553_SwCoSSetRxWatermarks( INCA_IP_SWITCH_PORT_SELECT_CPU, 
                                  0x18, 0x1E, 0x20, 0x14 );	/* 1/3 of buffer */

   /* init switch modul control sturct */
   sw21553_SwVLanCoSInitControl() ;

   /* reset switch MAC and VLAN table */
   sw21553_SwMacResetMacTable() ;
   sw21553_SwVLanResetVLanTable() ;

   SW_DEBUG (SW_DEBUG_INIT, ("sw21553_InitChip1: basic init of switch module\n"));
   return ;
}

/******************************************************************************
*  sw21553_InitChip_SetupPhyCfg()
*     do additional basic initialization of Ethernet and PHY 
*     Setup the PHY configuration for default use
*
*/
LOCAL void sw21553_InitChip_SetupPhyCfg()
{
   UINT32 tempRegValue ;

   /* init MDIO configuration: 
         MDS (Poll speed):       0x01 (4ms)
         PHY_LAN_ADDR:           0x06
         PHY_PC_ADDR:            0x05
         UEP (Use External PHY): 0x00 (Internal PHY is used)
         PS (Port Select):       0x00 (PT/UMM for LAN)
         PT (PHY Test):          0x00 (no test mode)
         UMM (Use MDIO Mode):    0x01 (state machine is disabled)
   */
   tempRegValue = 0 ;
   tempRegValue = (  (INCA_IP_Switch_MDIO_CFG_MDS_DEFAULT) |
                     (INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_DEFAULT) |
                     (INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_DEFAULT) |
                     (INCA_IP_Switch_MDIO_CFG_UEP_DEFAULT) |
                     (INCA_IP_Switch_MDIO_CFG_PS_DEFAULT) |
                     (INCA_IP_Switch_MDIO_CFG_UMM_DEFAULT)
                  ) ;
   SW_DEBUG (SW_DEBUG_INIT, ("sw21553_InitChip_SetupPhyCfg: PHY setup - MDIO_CFG=%x\n", tempRegValue));

   sw21553_SwSetMdioConfig(tempRegValue, 0xFFFFFFFF) ;

   return ;
}

MODULE_LICENSE("Infineon");

int inca_switch_control_init(void)
{
#ifdef SW_DEBUG_ENABLE
    printk("inca_switch_control_init\n");
#endif
    /* enable switch module */
    sw21553_InitChip_PowerONSwitch();

    /* stop traffic until initialization is done */
    sw21553_InitChip_DisableAllPorts();

    /* Reset MAC/VLan tables, enable CRC generation, disable tag insertion */
    sw21553_InitChip_SwitchBasicInit();

    /* set MDIO state machine to a known state (reset) */
    sw21553_SwResetMDIO();
    sw21553_InitChip_SetupPhyCfg();

    sw21553_PhyReset(INCA_IP_SWITCH_PORT_SELECT_LAN); /* LAN Phy reset */
    sw21553_PhyReset(INCA_IP_SWITCH_PORT_SELECT_PC); /* PC Phy reset */

    /* Now it should be safe to enable LAN and PC port */
    /* CPU ports remains disabled until the ethernet driver
       is ready to receive data */
    sw21553_SwUpdSpPtPortState(INCA_IP_SWITCH_PORT_SELECT_LAN, 
                               INCA_IP_Switch_SP_PT_STATE_FORWARD);
    sw21553_SwUpdSpPtPortState(INCA_IP_SWITCH_PORT_SELECT_PC, 
                               INCA_IP_Switch_SP_PT_STATE_FORWARD);
    return 0;
}

void __exit inca_switch_control_exit(void)
{
}

#ifdef MODULE
module_init(inca_switch_control_init);
module_exit(inca_switch_control_exit);
#endif

EXPORT_SYMBOL(inca_switch_control_init);
