/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ******************************************************************************
   Module      : INCAIPSwitchPhy.h
   Date        : 26.09.2003
   Description :
   
   $Log: inca_switch_phy.h,v $
   Revision 1.1.1.1  2004-06-03 10:25:35  fischesv
   initial from infineon sdk 2.1

   Revision 1.1  2004/01/12 09:16:21  de6wmigr
   Definitions and Phy driver function prototypes taken from VxWorks BSP vers. 1.7.

   Revision 1.7  2003/11/26 13:03:51  aschmidt
   - type definition of PHY callback function
   - moved PHY function prototypes from IncaSwitchAccess.h to here

   Revision 1.6  2003/11/07 15:43:01  aschmidt
   * corrected value of sw21553_PHY_SPEED_IND_100_FULL
   * added fields to the PhyStruct structure

   Revision 1.5  2003/10/21 14:45:31  aschmidt
   changed xxx_INTERRUPTS_ENABLE/DISABLE macros to be interrupt safe

   Revision 1.4  2003/10/16 10:55:58  aschmidt
   extended "swPhyStruct" by "IntDetect" flag

   Revision 1.3  2003/10/13 19:07:23  neumann
   * Added link status definition

   Revision 1.2  2003/09/26 17:24:34  mangele
   export new PHY routines

   Revision 1.1  2003/09/26 14:26:29  aschmidt
   This new file contains the PHY and AutoMDIX definitions.
   It mainly consists of definitions moved from "IncaSwitchEnd.h" and
   "IncaSwitchAccess.h"


   
******************************************************************************/
#ifndef _INCA_SWITCH_PHY_H
#define _INCA_SWITCH_PHY_H
/*
#include <vxWorks.h>
#include <end.h>
#include "incaip.h"
#include "RegIncaIp.h"
*/
/*
#include "inca_switch_access.h"
*/
/* switch ON/OFF debugging, only for testing */
#define PHY_DEBUG_ENABLE
#undef  PHY_DEBUG_ENABLE

/* -------------------------------------------------------------------------- */
/* Default setting                                                            */
/* -------------------------------------------------------------------------- */
/* tbd */

/* -------------------------------------------------------------------------- */
/* Basic macro definitions                                                    */
/* -------------------------------------------------------------------------- */
/* tbd */


/*****************************************************************************
   Default and startup configurations for the INCA IP Phy module
*****************************************************************************/


/***********************************************************************
    MDIO PHY registers 
*/


/*----- PHY register 0: Basic Control Register -----*/
/* PHYReg0 - reserved bits: [0..6] */
#define  INCA_IP_PHY_REG0_BCR_RESERVE_MASK   0x0000007F 

/* PHYReg0 - Bit 15: RESET */
/* 0: normal operation, 1: software reset */
#define  INCA_IP_PHY_REG0_BCR_RESET          15 
#define  INCA_IP_PHY_REG0_BCR_RESET_MASK     (1 << INCA_IP_PHY_REG0_BCR_RESET)

/* PHYReg0 - Bit 13: Speed Select */
/* 0: 10 Mbit/s, 1: 100 Mbit/s */
#define  INCA_IP_PHY_REG0_BCR_SS             13
#define  INCA_IP_PHY_REG0_BCR_SS_MASK        (1 << INCA_IP_PHY_REG0_BCR_SS)

/* PHYReg0 - Bit 12: Auto Negotiation Enable */
#define  INCA_IP_PHY_REG0_BCR_ANE            12
#define  INCA_IP_PHY_REG0_BCR_ANE_MASK       (1 << INCA_IP_PHY_REG0_BCR_ANE)

/* PHYReg0 - Bit 9: Restart Negotiation Enable */
#define  INCA_IP_PHY_REG0_BCR_RAN            9
#define  INCA_IP_PHY_REG0_BCR_RAN_MASK       (1 << INCA_IP_PHY_REG0_BCR_RAN)

/* PHYReg0 - Bit 8: Duplex Mode */
/* 0: Half duplex, 1: Full duplex */
#define  INCA_IP_PHY_REG0_BCR_DM             8
#define  INCA_IP_PHY_REG0_BCR_DM_MASK        (1 << INCA_IP_PHY_REG0_BCR_DM)



/*----- PHY register 1: Basic Status Register  -----*/
/* PHYReg1 - reserved bits: [6..10] */
#define  INCA_IP_PHY_REG1_BSR_RESERVE_MASK   0x000003C0

/* PHYReg 1 - Bit 0: Extended Capabilities */
/* 0: does not support ext. capabilities, 1: support */
#define INCA_IP_PHY_REG1_BSR_EXC          0
#define INCA_IP_PHY_REG1_BSR_EXC_MASK     (1 << INCA_IP_PHY_REG1_BSR_EXC)

/* PHYReg 1 - Bit 1: Jabber Detect */
/* 0: No jabber condition detected, 1: jabber condition detected */
#define INCA_IP_PHY_REG1_BSR_JD           1
#define INCA_IP_PHY_REG1_BSR_JD_MASK      (1 << INCA_IP_PHY_REG1_BSR_JD)

/* PHYReg 1 - Bit 2: Link Status */
/* 0: link down, 1: link up */
#define INCA_IP_PHY_REG1_BSR_LS           2
#define INCA_IP_PHY_REG1_BSR_LS_MASK      (1 << INCA_IP_PHY_REG1_BSR_LS)

/* PHYReg 1 - Bit 3: Auto-Negotiate Ability */
/* 0: Unable to perform auto-negotiate function, 1: Able to perform auto-negotiate function */
#define INCA_IP_PHY_REG1_BSR_ANA          3
#define INCA_IP_PHY_REG1_BSR_ANA_MASK     (1 << INCA_IP_PHY_REG1_BSR_ANA)

/* PHYReg 1 - Bit 4: Remote Fault */
/* 0: No remote fault, 1: Remote fault condition detected */
#define INCA_IP_PHY_REG1_BSR_RF           4
#define INCA_IP_PHY_REG1_BSR_RF_MASK      (1 << INCA_IP_PHY_REG1_BSR_RF)

/* PHYReg 1 - Bit 5: Auto-Negotiate Complete */
/* 0: Auto-negotiate process not completed, 1: Auto-negotiate process completed */
#define INCA_IP_PHY_REG1_BSR_ANC          5
#define INCA_IP_PHY_REG1_BSR_ANC_MASK     (1 << INCA_IP_PHY_REG1_BSR_ANC)


/*----- PHY register 2: Register 2, PHY Identifier 1 -----*/
/* PHYReg0 - reserved bits:  */
#define  INCA_IP_PHY_REG2_ID1_RESERVE_MASK   0x00000000 

/*----- PHY register 3: Basic Control Register -----*/
/* PHYReg0 - reserved bits:  */
#define  INCA_IP_PHY_REG0_ID2_RESERVE_MASK   0x00000000 


/*----- PHY register 4: Auto Negotiation Advertisement Register (PHY_ANAR)  -----*/
/* PHYReg4 - reserved bits: [11..12] */
#define INCA_IP_PHY_REG4_ANAR_RESERVE_MASK   0x00001800

/* PHYReg 4 - Bit 10: Pause Operation */
/* 0: Pause Operation is not supported by MAC , 1: Pause Operation is supported by MAC */
#define INCA_IP_PHY_REG4_ANAR_PO       10
#define INCA_IP_PHY_REG4_ANAR_PO_MASK  (1 << INCA_IP_PHY_REG4_ANAR_PO)

/* PHYReg 4 - Bit 13: Remote Fault */
/* 0: No remote fault, 1: Remote fault detected */
#define INCA_IP_PHY_REG4_ANAR_RF       13
#define INCA_IP_PHY_REG4_ANAR_RF_MASK  (1 << INCA_IP_PHY_REG4_ANAR_RF)

/* PHYReg 4 - Bit 15: Next Page */
/* 0: No next page ability, 1: Next page capable */
#define INCA_IP_PHY_REG4_ANAR_NP       15
#define INCA_IP_PHY_REG4_ANAR_NP_MASK  (1 << INCA_IP_PHY_REG4_ANAR_NP)


/*----- PHY register 5: Auto Negotiation Link Partner Ability Register (PHY_ANLPAR)  -----*/
/* PHYReg5 - reserved bits: [11..12] */
#define INCA_IP_PHY_REG5_ANLPAR_RESERVE_MASK   0x00001800

/* PHYReg 5 - Bit 10: Pause Operation */
/* 0: Pause Operation is not supported by remote MAC , 1: Pause Operation is supported */
#define INCA_IP_PHY_REG5_ANLPAR_PO        10
#define INCA_IP_PHY_REG5_ANLPAR_PO_MASK   (1 << INCA_IP_PHY_REG5_ANLPAR_PO)

/* PHYReg 5 - Bit 13: Remote Fault */
/* 0: No remote fault, 1: Remote fault detected */
#define INCA_IP_PHY_REG5_ANLPAR_RF        13
#define INCA_IP_PHY_REG5_ANLPAR_RF_MASK   (1 << INCA_IP_PHY_REG5_ANLPAR_RF)

/* PHYReg 5 - Bit 14: Acknowledge */
/* 0: Link code word not yet received, 1: Link code word received from partner */
#define INCA_IP_PHY_REG5_ANLPAR_ACK       14
#define INCA_IP_PHY_REG5_ANLPAR_ACK_MASK  (1 << INCA_IP_PHY_REG5_ANLPAR_ACK)

/* PHYReg 5 - Bit 15: Next Page */
/* 0: No next page ability, 1: Next page capable */
#define INCA_IP_PHY_REG5_ANLPAR_NP        15
#define INCA_IP_PHY_REG5_ANLPAR_NP_MASK   (1 << INCA_IP_PHY_REG5_ANLPAR_NP)


/*----- PHY register 6: Auto Negotiation Expansion Register (PHY_ANER) -----*/
/* PHYReg6 - reserved bits: [5..15] */
#define INCA_IP_PHY_REG6_ANER_RESERVE_MASK   0x0000FFE0

/* PHYReg 6 - Bit 0: Link Partner Auto-Negotiation Able */
/* 0: Link partner does not have auto-negotiation ability, 
   1: Link partner has auto-negotiation ability */
#define INCA_IP_PHY_REG6_ANER_LPANA       0
#define INCA_IP_PHY_REG6_ANER_LPANA_MASK  (1 << INCA_IP_PHY_REG6_ANER_LPANA)

/* PHYReg 6 - Bit 1: Page Received */
/* 0: New page not yet received, 1: New page received */
#define INCA_IP_PHY_REG6_ANER_PR          1
#define INCA_IP_PHY_REG6_ANER_PR_MASK     (1 << INCA_IP_PHY_REG6_ANER_PR)

/* PHYReg 6 - Bit 2: Next Page Able */
/* 0: Local device does not have next page ability, 
   1: Local device has next page ability */
#define INCA_IP_PHY_REG6_ANER_NPA         2
#define INCA_IP_PHY_REG6_ANER_NPA_MASK    (1 << INCA_IP_PHY_REG6_ANER_NPA)

/* PHYReg 6 - Bit 3: Link Partner Next Page Able */
/* 0: Link partner does not have next page ability, 
   1: Link partner has next page ability */
#define INCA_IP_PHY_REG6_ANER_LPNPA       3
#define INCA_IP_PHY_REG6_ANER_LPNPA_MASK  (1 << INCA_IP_PHY_REG6_ANER_LPNPA)

/* PHYReg 6 - Bit 4: Parallel Detection Fault */
/* 0: No fault detected by parallel detection logic, 
   1: Fault detected by parallel detection logic */
#define INCA_IP_PHY_REG6_ANER_PDF         4
#define INCA_IP_PHY_REG6_ANER_PDF_MASK    (1 << INCA_IP_PHY_REG6_ANER_PDF)


/*----- PHY register 16: Silicon Revision Register (PHY_SRR) -----*/
/* PHYReg16 - reserved bits: [0..5], [10..15] */
#define INCA_IP_PHY_REG16_SRR_RESERVE_MASK   0x0000FC3F

/*----- PHY register 17: Mode Control / Status register (PHY_MCSR) -----*/
/* PHYReg17 - reserved bits: [2..12], [14..15] */
#define INCA_IP_PHY_REG17_MCSR_RESERVE_MASK   0x0000DFFC

/* PHYReg 17 - Bit 1: ENERGYON signal state */
/* 0: no valid energy from the line within 256ms, 1: signal detected */
#define INCA_IP_PHY_REG17_MCSR_ENRG_BIT	   1
#define INCA_IP_PHY_REG17_MCSR_ENRG_MASK     (1 << INCA_IP_PHY_REG17_MCSR_ENRG_BIT)

/* PHYReg 17 - Bit 1: Energy Detect Power-Down Mode */
/* 0: Disabled, 1: Enabled */
#define INCA_IP_PHY_REG17_MCSR_EDPD_BIT	   13
#define INCA_IP_PHY_REG17_MCSR_EDPD_MASK     (1 << INCA_IP_PHY_REG17_MCSR_EDPD_BIT)

/*----- PHY register 18: Special Modes Register (PHY_SMR) -----*/
/* PHYReg18 - reserved bits: [8..15] */
#define INCA_IP_PHY_REG18_SMR_RESERVE_MASK   0x000000FF

/*----- PHY register 29: Interrupt Source Register (PHY_ISR) -----*/
/*----- PHY register 30: Interrupt Mask Register (PHY_IMR)   -----*/
/* PHYReg29/30 - reserved bits: [0] [8..15] */
#define INCA_IP_PHY_REG_INT_RESERVE_MASK   0x0000FF01

/* PHYReg 30: interrupt mask bit field [1..7] */
#define  INCA_IP_MDIO_IMR_FIELD_MASK     0x000000FE


/* PHYReg 29/30 - Bit 1: Auto-Negotiation Page Received */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ANPREC_BIT       1
#define INCA_IP_PHY_REG_INT_ANPREC_MASK      (1 << INCA_IP_PHY_REG_INT_ANPREC_BIT)

/* PHYReg 29/30 - Bit 2: Parallel Detection Fault */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_PDFAULT_BIT      2
#define INCA_IP_PHY_REG_INT_PDFAULT_MASK     (1 << INCA_IP_PHY_REG_INT_PDFAULT_BIT)

/* PHYReg 29/30 - Bit 3: Auto-Negotiation LP Acknowledge */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ANLPACK_BIT      3
#define INCA_IP_PHY_REG_INT_ANLPACK_MASK     (1 << INCA_IP_PHY_REG_INT_ANLPACK_BIT)

/* PHYReg 29/30 - Bit 4: Link Down (link status negated) */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_LDOWN_BIT        4
#define INCA_IP_PHY_REG_INT_LDOWN_MASK       (1 << INCA_IP_PHY_REG_INT_LDOWN_BIT)

/* PHYReg 29/30 - Bit 5: Remote Fault Detected */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_RFDETECT_BIT     5
#define INCA_IP_PHY_REG_INT_RFDETECT_MASK    (1 << INCA_IP_PHY_REG_INT_RFDETECT_BIT)

/* PHYReg 29/30 - Bit 6: Auto-Negotiation Complete */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ANCOMPL_BIT      6
#define INCA_IP_PHY_REG_INT_ANCOMPL_MASK     (1 << INCA_IP_PHY_REG_INT_ANCOMPL_BIT)

/* PHYReg 29/30 - Bit 7: ENERGYON Generated */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ENRGON_BIT       7
#define INCA_IP_PHY_REG_INT_ENRGON_MASK      (1 << INCA_IP_PHY_REG_INT_ENRGON_BIT)

/*----- PHY register 31: PHY Special Control/Status Register (PHY_SCSR) -----*/
/* PHYReg31 - reserved bits: [0..1], [5..11], [13..15] */
#define INCA_IP_PHY_REG31_SCSR_RESERVE_MASK   0x0000EFE3

/* PHYReg31 - Bit [2..4]: Speed Indication */
/* 001B: 10Base-T half-duplex
   101B: 10Base-T full-duplex
   010B: 100Base-TX half-duplex
   110B: 100Base-TX full-duplex */
#define INCA_IP_PHY_REG31_SCSR_SPEED_GET(value)  ( ((value) >> 2) & ((1 << 3) - 1) )
#define INCA_IP_PHY_REG31_SCSR_SPEED_SET(value)  ( ( ((1 << 3) - 1) &(value) ) << 2 )

/* PHYReg31 - Bit 12: Auto-Negotiation Done indication (Autodone) */
/* 0: Auto-negotiation is not done or disabled 
   1: IAuto-negotiation is done */
#define INCA_IP_PHY_REG31_SCSR_AUTODONE         12
#define INCA_IP_PHY_REG31_SCSR_AUTODONE_MASK    (1<<INCA_IP_PHY_REG31_SCSR_AUTODONE)

/*
    END: MDIO PHY registers 
*************************************************************************/

/*********************************************************************
      port link status control word
      =============================

         Control part     |    Property part
      --------------------|--------------------
      3322 2222 2222 1111   1111 11
      1098 7654 3210 9876   5432 1098 7654 3210
      |    |___ |||| ||||         |||    |    |
      |    | |  |||| ||||         |||    |-------> line properties (speed/duplex)
      |    | |  |||| ||||         |||
      |    | |  |||| ||||         |||--> Pause 
      |    | |  |||| ||||         ||---> Next Page
      |    | |  |||| ||||         |----> Auto Nego.
      |    | |  |||| ||||
      |    | |  |||| ||||--> extended Capabilities
      |    | |  |||| |||---> Auto Neg possible (Reg 1), maybe Auto-Neg. disabled
      |    | |  ||||
      |    | |  ||||--> Link Status
      |    | |  |||---> Remote Fault
      |    | |  ||----> Auto Neg. Complete
      |    | |  |----->
      |    | |
      |    | |--> current Speed
      |    | 
      |    |----> Auto Neg Done
      |
      |----------> data may corrupted
*/

#define sw21553_PHY_CTRL_LINE_PROPS_GET(value)  ( (value) & ((1 << 5) - 1)  )
#define sw21553_PHY_CTRL_LINE_PROPS_SET(value)  ( ((1 << 5) - 1) &(value) )
#define sw21553_PHY_CTRL_PAUSE                  (1<<8)
#define sw21553_PHY_CTRL_NEXT_PAGE              (1<<9)
#define sw21553_PHY_CTRL_AUTO_NEG_SUPPORT       (1<<10)

#define sw21553_PHY_CTRL_EXT_CAP                (1<<16)
#define sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE      (1<<17)

#define sw21553_PHY_CTRL_LINK_STATE             (1<<20)
#define sw21553_PHY_CTRL_REMOTE_FAULT           (1<<21)
#define sw21553_PHY_CTRL_AUTO_NEG_COMPL         (1<<22)

/* see PHY register 31: speed indicaton */
#define sw21553_PHY_CTRL_CURR_SPEED_GET(value)  ( ((value) >> 24) & ((1 << 3) - 1) )
#define sw21553_PHY_CTRL_CURR_SPEED_SET(value)  ( ( ((1 << 3) - 1) &(value) ) << 24 )

/* 001B 10Base-T half-duplex */
#define sw21553_PHY_SPEED_IND_10_HALF     1
/* 101B 10Base-T full-duplex */
#define sw21553_PHY_SPEED_IND_10_FULL     5
/* 010B 100Base-TX half-duplex */
#define sw21553_PHY_SPEED_IND_100_HALF    2
/* 110B 100Base-TX full-duplex */
#define sw21553_PHY_SPEED_IND_100_FULL    6

/* interprete speed bits */
#define sw21553_PHY_SPEED_10_BIT        (1)
#define sw21553_PHY_SPEED_100_BIT       (1<<1)
#define sw21553_PHY_SPEED_DUPLEX_BIT    (1<<2)


/* interprete current speed indication */
#define sw21553_PHY_CTRL_SPEED_10_IND        (1<<24)
#define sw21553_PHY_CTRL_SPEED_100_IND       (1<<25)
#define sw21553_PHY_CTRL_SPEED_DUPLEX_IND    (1<<26)

#define sw21553_PHY_CTRL_AUTO_NEG_DONE       (1<<27)

#define sw21553_PHY_CTRL_DATA_CORRUPTED         (1<<31)


/* current used properties from "Basic Status Register" (PHY_BSR) */
#define sw21553_PHY_PROPS_CURRENT     0
#define sw21553_PHY_PROPS_CURRENT_STR "STATUS"
/* take care about: link state | Remote Fault */
#define sw21553_PHY_PROPS_CURRENT_VALID_MASK   ( sw21553_PHY_CTRL_LINK_STATE | sw21553_PHY_CTRL_REMOTE_FAULT )
/* data are valid if the state mask matches */
#define sw21553_PHY_PROPS_CURRENT_VALID_STATE  (sw21553_PHY_CTRL_LINK_STATE)


/* advertised properties from "Auto-Negotiation Advertisement Register" (PHY_ANAR) */
#define sw21553_PHY_PROPS_ADVERTISE   1
#define sw21553_PHY_PROPS_ADVERTISE_STR "ADVERTISEMENT"

/* only this settings are writeable within the advertise register */
#define sw21553_PHY_PROPS_ADVERTISE_SET_MASK   ( sw21553_PHY_CTRL_PAUSE | \
                                                 sw21553_ETHERNET_LINE_100_FULL | \
												 sw21553_ETHERNET_LINE_100_HALF | \
												 sw21553_ETHERNET_LINE_10_FULL | \
												 sw21553_ETHERNET_LINE_10_HALF )

/* partner link properties from "Auto-Negotiation Link Partner Ability Register" (PHY_ANLPAR) */
#define sw21553_PHY_PROPS_PARTNER     2
#define sw21553_PHY_PROPS_PARTNER_STR "LINK-PARTNER"

/* data are valid only if: 
      partner supports Auto Neg | 
      Auto Neg. must be complete    */
#define sw21553_PHY_PROPS_PARTNER_VALID_MASK   (sw21553_PHY_CTRL_AUTO_NEG_SUPPORT | \
                                                sw21553_PHY_CTRL_AUTO_NEG_COMPL )
/* data are valid if the state mask matches */
#define sw21553_PHY_PROPS_PARTNER_VALID_STATE  (sw21553_PHY_CTRL_AUTO_NEG_SUPPORT | \
                                                sw21553_PHY_CTRL_AUTO_NEG_COMPL )
												

												 
/*
      END: port link status control word
******************************************************************************/



/* define the PHY port number within the INCA IP */
#define INCA_IP_PHY0_LAN      0
#define INCA_IP_PHY1_PC       1

/* PHY0 (LAN) interrupt line: IM1_IRLx */
#define INCA_IP_IM_IRL_LAN    INT_NUM_IM1_IRL2 
/* PHY1 (PC) interrupt line: IM1_IRLx */
#define INCA_IP_IM_IRL_PC     INT_NUM_IM1_IRL1 

#define INCA_IRQ_LINE_PHY0LAN 0x00000004  /* PHY0 (LAN) Interrupt Request */
#define INCA_IRQ_LINE_PHY1PC  0x00000002  /* PHY1 (PC) Interrupt Request */

#define SW_INTERRUPTS_ENABLE(X) 	sysIncaHWRegSetBit(INCA_IP_ICU_IM1_IER, X);   
#define SW_INTERRUPTS_DISABLE(X) 	sysIncaHWRegResetBit(INCA_IP_ICU_IM1_IER, X);
#define SW_INTERRUPTS_CLEAR(X) 	sysIncaHWRegWrite(INCA_IP_ICU_IM1_ISR, X);   
#define SW_INTERRUPTS_GEN(X) 	sysIncaHWRegWrite(INCA_IP_ICU_IM1_IRSR, X);


/**************************************************/
/*  PHY  S t a t e  Definitions                   */
/**************************************************/

#define  AWAIT_ENERGY               0
#define  AWAIT_LINK                 1
#define  LINK_UP					2
#define  PHY_STATE_MAX              (LINK_UP+1)

/**************************************************/
/*  PHY  E v e n t  Definitions                   */
/**************************************************/

#define  PHY_INIT                   0
#define  ENERGY_DETECT              1
#define  LINK_ACTIVE                2
#define  LINK_INACTIVE              3
#define  LINK_TRIES_TOUT            4
#define  LINK_TRY_FAIL              5
#define  REMOTE_FAULT				6
#define  PARDET_FAULT               7
#define  PHY_EVENT_MAX              (PARDET_FAULT+1)


#define LINK_RETRY_TICKS			(2500 /*ms*/ * sysClkRateGet()/1000)
/* call back function type, called when registered and an event occured */
typedef void (*sw21553_PhyCallbackFunction)(int event);

/* control struct for handle the Ethernet PHY port (PC/LAN) */
typedef struct
{
   UINT32 			 Port;
   UINT32            PhyAddr;		/* Phy Address on MDIO bus */
   SEM_ID            semPhy;
   BOOL              EventDetect;
   int               LinkRetries;
   int               LinkRetriesReload;
   int               RetryTicks;
   BOOL              Polling;
   BOOL				 AutoNeg;
   UINT32			 FixSpeed;	/* only valid if AutoNeg = FALSE */
   UINT32			 State;		/* for Finate State Machine */
   sw21553_PhyCallbackFunction fct;
} swPhyStruct;


typedef unsigned int WDOG_ID;

/******************************************************************************
   PHY ACCESS: functions
******************************************************************************/


UINT32 sw21553_PhyReadReg(int portSelect, UINT32 phyRegIndex);
UINT32 sw21553_PhyWriteReg(int portSelect, UINT32 phyRegIndex, UINT32 newRegValue);

/* returns the selected PHY properties
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      phyInfo: specify properties - 0: current , 1: advertised 2: partner 
*/
UINT32 sw21553_PhyGetProperties(int portSelect, int phyInfo) ;






/* registers callback function for link-up/down state indications
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      fct: function to be called on state change 
*/
sw21553_PhyCallbackFunction sw21553_PhySetCallback(int portSelect, 
                                                   sw21553_PhyCallbackFunction fct);

/* sets the link retry parameters
      portSelect    : selects the MDIO PHY (LAN = 0 / PC = 1)
      retryCount    : number of Link Retries
      retryDelay_ms : delay between link retries
*/
UINT32 sw21553_PhySetLinkRetry(int portSelect, int retryCount, int retryDelay_ms);



/* set the advertise properties
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      newSettings: bit[0..4] - 10MBit/s HD | 10MBit/s FD | 100MBit/s HD | 100MBit/s FD 
                   bit 4-7 - no used
                   bit 8: pause operation
*/
UINT32 sw21553_PhySetAdvertiseProperties(int portSelect, UINT32 newSettings) ;

/* set the fixed properties (without auto neg)
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      newSpeed:   001B:  10MBit/s HD  
                  101B:  10MBit/s FD 
                  010B: 100MBit/s HD  
                  110B: 100MBit/s FD 
*/
UINT32 sw21553_PhySetFixSpeed(int portSelect, UINT32 newSpeed) ;

/* disable / enable auto negotiation 
      portSelect: selects PHY (LAN = 0 / PC = 1) 
      disableAutoNeg: 0 -> enable, 1 -> disable
*/
UINT32 sw21553_PhyDisEnAutoNeg(int portSelect, int disableAutoNeg) ;

/* restart the auto negotiation for the selected PHY
      portSelect: selects PHY (LAN = 0 / PC = 1)
*/
int sw21553_PhyRestartAutoNeg(int portSelect) ;

/* reset the selected PHY
      portSelect: selects PHY (LAN = 0 / PC = 1)
*/
int sw21553_PhyReset(int portSelect) ;

int sw21553_PhyInit( void ) ;
int sw21553_PhyStart( void ) ;
#endif /* _INCASWITCHPHY_H */
