

#ifndef __INCA_SWITCH_SETTINGS_H__
#define __INCA_SWITCH_SETTINGS_H__

/*****************************************************************************
   Default and startup configurations for the INCA IP Switch and Phy module
*****************************************************************************/

/* init UN_DEST unknown destination register with a default configuration 
   setup the UN_DEST: 0x0000 01db
   (given form Yong Yean Kee, THANKS)
*/

#define INCA_IP_Switch_UN_DEST_RESERVED_MASK    0xFFFFFE00

/* BIT 0: PC unicast destination */
#define INCA_IP_Switch_UN_DEST_PU_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_PU_FWD        INCA_IP_Switch_UN_DEST_PU
#define INCA_IP_Switch_UN_DEST_PU_DEFAULT    INCA_IP_Switch_UN_DEST_PU_FWD
/* BIT 1: LAN unicast destination */
#define INCA_IP_Switch_UN_DEST_LU_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_LU_FWD        INCA_IP_Switch_UN_DEST_LU
#define INCA_IP_Switch_UN_DEST_LU_DEFAULT    INCA_IP_Switch_UN_DEST_LU_FWD
/* BIT 2: CPU unicast destination */
#define INCA_IP_Switch_UN_DEST_CU_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_CU_FWD        INCA_IP_Switch_UN_DEST_CU

#if (INCA_IP_SWITCH_LEARN_MAC_ADDR == 0)
   #define INCA_IP_Switch_UN_DEST_CU_DEFAULT    INCA_IP_Switch_UN_DEST_CU_NOT_FWD
#else
   /* REMEMBER: all unknown unicast packets will be forwarded to the CPU port too */
   #define INCA_IP_Switch_UN_DEST_CU_DEFAULT    INCA_IP_Switch_UN_DEST_CU_FWD
#endif

/* BIT 3: PC multicast destination */
#define INCA_IP_Switch_UN_DEST_PM_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_PM_FWD        INCA_IP_Switch_UN_DEST_PM
#define INCA_IP_Switch_UN_DEST_PM_DEFAULT    INCA_IP_Switch_UN_DEST_PM_FWD
/* BIT 4: LAN multicast destination */
#define INCA_IP_Switch_UN_DEST_LM_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_LM_FWD        INCA_IP_Switch_UN_DEST_LM
#define INCA_IP_Switch_UN_DEST_LM_DEFAULT    INCA_IP_Switch_UN_DEST_LM_FWD
/* BIT 5: CPU multicast destination */
#define INCA_IP_Switch_UN_DEST_CM_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_CM_FWD        INCA_IP_Switch_UN_DEST_CM
#define INCA_IP_Switch_UN_DEST_CM_DEFAULT    INCA_IP_Switch_UN_DEST_CM_NOT_FWD

/* BIT 6: PC braodcast destination */
#define INCA_IP_Switch_UN_DEST_PB_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_PB_FWD        INCA_IP_Switch_UN_DEST_PB
#define INCA_IP_Switch_UN_DEST_PB_DEFAULT    INCA_IP_Switch_UN_DEST_PB_FWD
/* BIT 7: LAN braodcast destination */
#define INCA_IP_Switch_UN_DEST_LB_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_LB_FWD        INCA_IP_Switch_UN_DEST_LB
#define INCA_IP_Switch_UN_DEST_LB_DEFAULT    INCA_IP_Switch_UN_DEST_LB_FWD
/* BIT 8: CPU braodcast destination */
#define INCA_IP_Switch_UN_DEST_CB_NOT_FWD    0
#define INCA_IP_Switch_UN_DEST_CB_FWD        INCA_IP_Switch_UN_DEST_CB
#define INCA_IP_Switch_UN_DEST_CB_DEFAULT    INCA_IP_Switch_UN_DEST_CB_FWD

/* port behaviour states: 3 Bit field
      000b
      |||---> unicast packets
      ||----> multicast packets
      |-----> broadcast packets
*/
#define INCA_IP_Switch_UN_DEST_FORWARD_NO    0
#define INCA_IP_Switch_UN_DEST_FORWARD_U     1
#define INCA_IP_Switch_UN_DEST_FORWARD_M     2
#define INCA_IP_Switch_UN_DEST_FORWARD_UM    3
#define INCA_IP_Switch_UN_DEST_FORWARD_B     4
#define INCA_IP_Switch_UN_DEST_FORWARD_UB    5
#define INCA_IP_Switch_UN_DEST_FORWARD_MB    6
#define INCA_IP_Switch_UN_DEST_FORWARD_ALL   7

#define INCA_IP_Switch_UN_DEST_PC_PORT_MASK  INCA_IP_Switch_UN_DEST_PU | \
                                             INCA_IP_Switch_UN_DEST_PM | \
                                             INCA_IP_Switch_UN_DEST_PB
#define INCA_IP_Switch_UN_DEST_LAN_PORT_MASK INCA_IP_Switch_UN_DEST_LU | \
                                             INCA_IP_Switch_UN_DEST_LM | \
                                             INCA_IP_Switch_UN_DEST_LB
#define INCA_IP_Switch_UN_DEST_CPU_PORT_MASK INCA_IP_Switch_UN_DEST_CU | \
                                             INCA_IP_Switch_UN_DEST_CM | \
                                             INCA_IP_Switch_UN_DEST_CB

/* END default settings: UN_DEST - unknown destination register */

/* 
   VLAN_CTRL register default configuration 

*/
/* reserved bits [7..31] */
#define INCA_IP_Switch_VLAN_CTRL_RESERVED_MASK    0xFFFFFF80
#define INCA_IP_Switch_VLAN_CTRL_RESET_VALUE      0x00000000
/* END default settings: VLAN_CTRL - VLAN Control Register */

/* 
   PC_VLAN register default configuration 
   LAN_VLAN register default configuration
   CPU_VLAN register default configuration
       
   ATTENTION:
      the prio-field is bit[12..14] 
      but within a VLAN tag (TCI) there is the CFI bit between prio field and VLAN ID
*/
/* reserved bits [15..31] */
#define INCA_IP_Switch_XXX_VLAN_RESERVED_MASK    0xFFFF8000

#define INCA_IP_Switch_PC_VLAN_RESET_VALUE      0x00000000
#define INCA_IP_Switch_LAN_VLAN_RESET_VALUE      0x00000000
#define INCA_IP_Switch_CPU_VLAN_RESET_VALUE      0x00000000
/* END default settings: PC_VLAN/LAN_VLAN/CPU_VLAN - xxx VLAN Configuration Register */


/* 
   PRIO_CoS register default configuration 

*/
/* reserved bits [7..31] */
#define INCA_IP_Switch_PRIO_COS_RESERVED_MASK    0xFFFFFF00
#define INCA_IP_Switch_PRIO_COS_RESET_VALUE      0x000000F0
/* END default settings: PRIO_CoS - Priority CoS Mapping Register */

/* 
   SP_PT register default configuration 

*/
/* BIT[16:31] Reserved  */
#define INCA_IP_Switch_SP_PT_RESERVED_MASK       0xFFFFFFC0
/* BIT[0:1]: Port Status for PC Port */
#define INCA_IP_Switch_SP_PT_PPS_MASK        0x00000003
/* BIT[2:3]: Port Status for LAN Port */
#define INCA_IP_Switch_SP_PT_LPS_MASK        0x0000000C
/* BIT[4:5]: Port Status for CPU Port */
#define INCA_IP_Switch_SP_PT_CPS_MASK        0x00000030

/* possible port states */
#define INCA_IP_Switch_SP_PT_STATE_DISABLED  0
#define INCA_IP_Switch_SP_PT_STATE_LISTEN    1
#define INCA_IP_Switch_SP_PT_STATE_LEARN     2
#define INCA_IP_Switch_SP_PT_STATE_FORWARD   3

/* END default settings: SP_PT - spanning tree port status */


/* 
   init ARL control register with a default configuration 

*/
/* BIT[16:31] Reserved  */
#define INCA_IP_Switch_ARL_CTL_RESERVED_MASK       0xFFFF0000

/* BIT[0] ENL: Enable Learning from LAN Port */
#define INCA_IP_Switch_ARL_CTL_ENL_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_ENL_ENABLED     INCA_IP_Switch_ARL_CTL_ENL
#define INCA_IP_Switch_ARL_CTL_ENL_DEFAULT     INCA_IP_Switch_ARL_CTL_ENL_ENABLED

/* BIT[1] MAF: MA Freeze */
#define INCA_IP_Switch_ARL_CTL_MAF_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_MAF_ENABLED     INCA_IP_Switch_ARL_CTL_MAF
#define INCA_IP_Switch_ARL_CTL_MAF_DEFAULT     INCA_IP_Switch_ARL_CTL_MAF_DISABLED

/* BIT[2:4] AGE_TICK_SEL: Aging Tick Select     */
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_MASK         0x0000001C
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_IMMMEDIATE   0x0
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_20_SEC       0x1
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_5_MIN        0x2
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_15_MIN       0x3
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_1_HR         0x4
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_4_HRS        0x5
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_1_DAY        0x6
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_NO_AGING     0x7
#define INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_DEFAULT      INCA_IP_Switch_ARL_CTL_AGE_TICK_SEL_20_SEC

/* BIT[5] ATS: Aging Tick Software */
#define INCA_IP_Switch_ARL_CTL_ATS_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_ATS_ENABLED     INCA_IP_Switch_ARL_CTL_ATS
#define INCA_IP_Switch_ARL_CTL_ATS_DEFAULT     INCA_IP_Switch_ARL_CTL_ATS_DISABLED

/* BIT[6] SRC: Send Reserved Packets to CPU */
#define INCA_IP_Switch_ARL_CTL_SRC_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_SRC_ENABLED     INCA_IP_Switch_ARL_CTL_SRC
#define INCA_IP_Switch_ARL_CTL_SRC_DEFAULT     INCA_IP_Switch_ARL_CTL_SRC_DISABLED

/* BIT[7] MRO: MA Replace Oldest Entry */
#define INCA_IP_Switch_ARL_CTL_MRO_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_MRO_ENABLED     INCA_IP_Switch_ARL_CTL_MRO
#define INCA_IP_Switch_ARL_CTL_MRO_DEFAULT     INCA_IP_Switch_ARL_CTL_MRO_DISABLED

/* BIT[8]     PS : Priority Selection */
#define INCA_IP_Switch_ARL_CTL_PS_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_PS_ENABLED     INCA_IP_Switch_ARL_CTL_PS
#define INCA_IP_Switch_ARL_CTL_PS_DEFAULT     INCA_IP_Switch_ARL_CTL_PS_DISABLED

/* BIT[9]     CG : CRC GEN for CPU Port */
#define INCA_IP_Switch_ARL_CTL_CG_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_CG_ENABLED     INCA_IP_Switch_ARL_CTL_CG
#define INCA_IP_Switch_ARL_CTL_CG_DEFAULT     INCA_IP_Switch_ARL_CTL_CG_DISABLED

/* BIT[10]    CP : CoS for PC Port */
#define INCA_IP_Switch_ARL_CTL_CP_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_CP_ENABLED     INCA_IP_Switch_ARL_CTL_CP
#define INCA_IP_Switch_ARL_CTL_CP_DEFAULT     INCA_IP_Switch_ARL_CTL_CP_DISABLED

/* BIT[11]    CL : CoS for LAN Port */
#define INCA_IP_Switch_ARL_CTL_CL_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_CL_ENABLED     INCA_IP_Switch_ARL_CTL_CL
#define INCA_IP_Switch_ARL_CTL_CL_DEFAULT     INCA_IP_Switch_ARL_CTL_CL_DISABLED

/* BIT[12]    CC : CoS for CPU Port */
#define INCA_IP_Switch_ARL_CTL_CC_DISABLED    0
#define INCA_IP_Switch_ARL_CTL_CC_ENABLED     INCA_IP_Switch_ARL_CTL_CC
#define INCA_IP_Switch_ARL_CTL_CC_DEFAULT     INCA_IP_Switch_ARL_CTL_CC_DISABLED

/* BIT[13]    CHCP: Classify all High Priority packets from PC port as Critica. */
/* BIT[14]    CHCL: Classify all High Priority packets from LAN port as Critical. */
/* BIT[15]    CHCC: Classify all High Priority packets from CPU port as Critica. */

/* END default settings: ARL_CTL - ARL control register */

/* init CPU Port Control Register with a default configuration 
   most settings refer to the learning behaviour of the CPU port
   (given form Yong Yean Kee, THANKS)
   BIT[0:0]    EDL: enable DA lookup         0x1 --> enabled
   BIT[1:1]    EML: enable MA learn          0x1 --> enabled
   BIT[2:2]    MAC: MA critical (if EML=1)   0x1     
   BIT[3:5]    MA_PORTS, if EML=1 upd V,L,P  0x4 --> update V-bit (mac table entry)
   BIT[6:6]    MAM: MA management (if EML=1) 0x0 --> M-bit = 0 (mac table entry)
   BIT[7:9]    MA_STATE: (if EML=1)          0x0 --> MA_ST bit = 0 (mac table entry)
   BIT[10:10]  DAC: DA CoS (used if EDL=0)   0x0 --> packet forward with low priority
   BIT[11:13]  DA_PORTS: DA destination      0x0 --> packet is dropped
   BIT[14:31]  reserved
   
   setup the CPU_PCTL: 0x0000 0027
*/

/* BIT[16:31] Reserved  */
#define INCA_IP_Switch_CPU_PCTL_RESERVED_MASK       0xFFFFC000

/* BIT[0]: enable DA look up */
#define INCA_IP_Switch_CPU_PCTL_EDL_DISABLED    0
#define INCA_IP_Switch_CPU_PCTL_EDL_ENABLED     INCA_IP_Switch_CPU_PCTL_EDL
#define INCA_IP_Switch_CPU_PCTL_EDL_DEFAULT     INCA_IP_Switch_CPU_PCTL_EDL_ENABLED

/* BIT[1]: enable MA learning */
#define INCA_IP_Switch_CPU_PCTL_EML_DISABLED    0
#define INCA_IP_Switch_CPU_PCTL_EML_ENABLED     INCA_IP_Switch_CPU_PCTL_EML
#if (INCA_IP_SWITCH_LEARN_MAC_ADDR == 0)
   #define INCA_IP_Switch_CPU_PCTL_EML_DEFAULT     INCA_IP_Switch_CPU_PCTL_EML_DISABLED
#else
   #define INCA_IP_Switch_CPU_PCTL_EML_DEFAULT     INCA_IP_Switch_CPU_PCTL_EML_ENABLED
#endif

/* BIT[2]: if EML = 1: set CR bit within the MAC table entry */
#define INCA_IP_Switch_CPU_PCTL_MAC_NORMAL      0
#define INCA_IP_Switch_CPU_PCTL_MAC_CRITICAL    INCA_IP_Switch_CPU_PCTL_MAC
#define INCA_IP_Switch_CPU_PCTL_MAC_DEFAULT     INCA_IP_Switch_CPU_PCTL_MAC_CRITICAL

/* BIT[3:5]: MAC table entry: (V: voice port | L: LAN port | P: PC port 
   0: port is not a destination
   1: port is a destination      */
#define INCA_IP_Switch_CPU_PCTL_MA_PORTS_V_BIT     0x1
#define INCA_IP_Switch_CPU_PCTL_MA_PORTS_L_BIT     0x0
#define INCA_IP_Switch_CPU_PCTL_MA_PORTS_P_BIT     0x0
#define INCA_IP_Switch_CPU_PCTL_MA_PORTS_DEF_VAL   ((INCA_IP_Switch_CPU_PCTL_MA_PORTS_V_BIT << 2) | \
                                                    (INCA_IP_Switch_CPU_PCTL_MA_PORTS_L_BIT << 1) | \
                                                    (INCA_IP_Switch_CPU_PCTL_MA_PORTS_P_BIT) )
#define INCA_IP_Switch_CPU_PCTL_MA_PORTS_DEFAULT   INCA_IP_Switch_CPU_PCTL_MA_Ports_SET(INCA_IP_Switch_CPU_PCTL_MA_PORTS_DEF_VAL) 

/* BIT[6]: if EML = 1: set M bit within the MAC table entry */
#define INCA_IP_Switch_CPU_PCTL_MAM_IS_NO_DEST  0
#define INCA_IP_Switch_CPU_PCTL_MAM_IS_DEST     INCA_IP_Switch_CPU_PCTL_MAM
#define INCA_IP_Switch_CPU_PCTL_MAM_DEFAULT     INCA_IP_Switch_CPU_PCTL_MAM_IS_NO_DEST

/* BIT[7:9]: MAC state with in the MAC table entry 
   000:  static entry, cannot be aged
   001:  learned entry
   010 .. 110: valid entries, progressive aging stages
   111:  invalid entry, aged out
*/
#define INCA_IP_Switch_CPU_PCTL_MA_STATE_STATIC    0x00
#define INCA_IP_Switch_CPU_PCTL_MA_STATE_LEARNED_1 0x01
#define INCA_IP_Switch_CPU_PCTL_MA_STATE_AGED_OUT  0x07
#define INCA_IP_Switch_CPU_PCTL_MA_STATE_DEFAULT   INCA_IP_Switch_CPU_PCTL_MA_STATE_STATIC

/* BIT[10]: if EDL = 0: forward packet with high/low priority */
#define INCA_IP_Switch_CPU_PCTL_DAC_LOW         INCA_IP_API_CPU_PCTL_DAC_LOW
#define INCA_IP_Switch_CPU_PCTL_DAC_HIGH        INCA_IP_Switch_CPU_PCTL_DAC
#define INCA_IP_Switch_CPU_PCTL_DAC_DEFAULT     INCA_IP_Switch_CPU_PCTL_DAC_LOW

/* BIT[11:13]: DA destination
   000:  Packet is dropped
   001:  Packet is forward to the PC port
   010:  Packet is forward to the LAN port
   011:  Packet is forward to both LAN and PC port
*/
#define INCA_IP_Switch_CPU_PCTL_DA_PORTS_MASK      0x00003800
#define INCA_IP_Switch_CPU_PCTL_DAC_MASK           INCA_IP_Switch_CPU_PCTL_DAC
#define INCA_IP_Switch_CPU_PCTL_DA_PORTS_DEF_VAL   INCA_IP_API_CPU_PCTL_DA_PORTS_DROP
#define INCA_IP_Switch_CPU_PCTL_DA_PORTS_DEFAULT   INCA_IP_Switch_CPU_PCTL_DA_PORTS_SET(INCA_IP_Switch_CPU_PCTL_DA_PORTS_DEF_VAL)
/* END default settings: CPU_PCTL - CPU Port Control Register */

/* 
   PMAC_HD_CTL register a default configuration 
*/
/* BIT[22:31] Reserved  */
#define INCA_IP_Switch_PMAC_HD_CTL_RESERVED_MASK   0xFFC00000
#define INCA_IP_Switch_PMAC_HD_CTL_RESET_VALUE     0x00000000
/* END default settings: PMAC_HD_CTL - PM Header Control Register */

/* 
   DSCP_COS1/DSCP_COS2 register default configuration 

*/
/* no reserved bits */
#define INCA_IP_Switch_DSCP_COS1_RESERVED_MASK    0x00000000
#define INCA_IP_Switch_DSCP_COS1_RESET_VALUE      0x00000000

/* no reserved bits */
#define INCA_IP_Switch_DSCP_COS2_RESERVED_MASK    0x00000000
#define INCA_IP_Switch_DSCP_COS2_RESET_VALUE      0x00000000

/* END default settings: DSCP_COS1/DSCP_COS2 - DSCP CoS Mapping Register 1/2 */

/* 
   PC_WFQ_CTL  - PC  WFQ Control Register
   LAN_WFQ_CTL - LAN WFQ Control Register
   CPU_WFQ_CTL - CPU WFQ Control Register
*/

/* reset values form switch module */
#define INCA_IP_Switch_PC_WFQ_CTL_RESET_VALUE      0x00000203
#define INCA_IP_Switch_LAN_WFQ_CTL_RESET_VALUE     0x00000203
#define INCA_IP_Switch_CPU_WFQ_CTL_RESET_VALUE     0x00000203

/* BIT[10:31] Reserved  */
#define INCA_IP_Switch_XXX_WFQ_CTL_RESERVED_MASK   0xFFFFFC00
/* BIT[0..1]: Select Scheduling scheme */
#define INCA_IP_Switch_XXX_WFQ_CTL_SCH_SEL_MASK    0x00000003
/* BIT[2..4]: WFQ weight for Queue 0 */
#define INCA_IP_Switch_XXX_WFQ_CTL_WT0_MASK        0x0000001C
/* BIT[5..7]: WFQ weight for Queue 1 */
#define INCA_IP_Switch_XXX_WFQ_CTL_WT1_MASK        0x000000E0
/* BIT[8]: Priority for Queue 0 */
#define INCA_IP_Switch_XXX_WFQ_CTL_P0_MASK         0x00000100
/* BIT[9]: Priority for Queue 1 */
#define INCA_IP_Switch_XXX_WFQ_CTL_P1_MASK         0x00000200

/* possible scheduling types */
#define INCA_IP_Switch_XXX_WFQ_CTL_STRICT_SCHED 0x0
#define INCA_IP_Switch_XXX_WFQ_CTL_WFQ_SCHED    0x3

/* END settings: PP_WFQ_CTL/LAN_WFQ_CTL/CPU_WFQ_CTL - xxx WFQ Control Register */


/* 
   PC_WM  - PM  PC  RX Watermark Register
   LAN_WM - PM  LAN RX Watermark Register
   CPU_WM - PM  CPU RX Watermark Register
   GLB_WM - PM  Global RX Watermark Register
*/

#define INCA_IP_Switch_PC_RX_WM_RESET_VALUE     0x282E3024
#define INCA_IP_Switch_LAN_RX_WM_RESET_VALUE    0x282E3024
#define INCA_IP_Switch_CPU_RX_WM_RESET_VALUE    0x282E3024
#define INCA_IP_Switch_GLB_RX_WM_RESET_VALUE    0x0603000C

/* no bits reserved  */
#define INCA_IP_Switch_XXX_RX_WM_RESERVED_MASK   0x00000000
/* BIT[0..7]: RX_WM1 */
#define IINCA_IP_Switch_XXX_RX_WM_WM1_MASK    0x000000FF
/* BIT[8..15]: RX_WM2 */
#define IINCA_IP_Switch_XXX_RX_WM_WM2_MASK    0x0000FF00
/* BIT[16..23]: RX_WM3 */
#define IINCA_IP_Switch_XXX_RX_WM_WM3_MASK    0x00FF0000
/* BIT[24..31]: RX_WM1 */
#define IINCA_IP_Switch_XXX_RX_WM_WM4_MASK    0xFF000000
/* END settings: PC_WM/LAN_WM/CPU_WM - xxx RX Watermark Register */


/* 
   PKT_LEN register a default configuration 
*/
/* BIT[12:31] Reserved  */
#define INCA_IP_Switch_PKT_LEN_RESERVED_MASK       0xFFFFF000
#define INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_MASK    0x000007FF
#define INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN         0x6A4


/* 
   EPHY register a default configuration 
*/
/* BIT[16:31] Reserved  */
#define INCA_IP_Switch_EPHY_RESERVED_MASK       0xFFFFFF00

/* BIT[0] Auto Negotiated Pause Status for PC PHY */
#define INCA_IP_Switch_EPHY_PP_PAUSE_NOT_SUP    0
#define INCA_IP_Switch_EPHY_PP_PAUSE_SUP        INCA_IP_Switch_EPHY_PP
#define INCA_IP_Switch_EPHY_PP_DEFAULT          INCA_IP_Switch_EPHY_PP_PAUSE_SUP

/* BIT[1] Auto Negotiated Pause Status for LAN PHY */
#define INCA_IP_Switch_EPHY_PL_PAUSE_NOT_SUP    0
#define INCA_IP_Switch_EPHY_PL_PAUSE_SUP        INCA_IP_Switch_EPHY_PL
#define INCA_IP_Switch_EPHY_PL_DEFAULT          INCA_IP_Switch_EPHY_PL_PAUSE_SUP

/* BIT[2] Duplex Status for PC PHY */
#define INCA_IP_Switch_EPHY_DP_HALF          0
#define INCA_IP_Switch_EPHY_DP_FULL          INCA_IP_Switch_EPHY_DP
#define INCA_IP_Switch_EPHY_DP_DEFAULT       INCA_IP_Switch_EPHY_DP_FULL

/* BIT[3] Duplex Status for LAN PHY */
#define INCA_IP_Switch_EPHY_DL_HALF          0
#define INCA_IP_Switch_EPHY_DL_FULL          INCA_IP_Switch_EPHY_DL
#define INCA_IP_Switch_EPHY_DL_DEFAULT       INCA_IP_Switch_EPHY_DL_FULL

/* BIT[4] Link Status for PC PHY */
#define INCA_IP_Switch_EPHY_LP_LINK_DOWN     0
#define INCA_IP_Switch_EPHY_LP_LINK_UP       INCA_IP_Switch_EPHY_LP
#define INCA_IP_Switch_EPHY_LP_DEFAULT       INCA_IP_Switch_EPHY_LP_LINK_DOWN

/* BIT[5] Link Status for LAN PHY */
#define INCA_IP_Switch_EPHY_LL_LINK_DOWN     0
#define INCA_IP_Switch_EPHY_LL_LINK_UP       INCA_IP_Switch_EPHY_LL
#define INCA_IP_Switch_EPHY_LL_DEFAULT       INCA_IP_Switch_EPHY_LL_LINK_DOWN

/* BIT[6] Auto Negotiated Speed Settings for PC PHY */
#define INCA_IP_Switch_EPHY_SP_10MBIT        0
#define INCA_IP_Switch_EPHY_SP_100MBIT       INCA_IP_Switch_EPHY_SP
#define INCA_IP_Switch_EPHY_SP_DEFAULT       INCA_IP_Switch_EPHY_SP_100MBIT

/* BIT[7] Auto Negotiated Speed Settings for LAN PHY */
#define INCA_IP_Switch_EPHY_SL_10MBIT        0
#define INCA_IP_Switch_EPHY_SL_100MBIT       INCA_IP_Switch_EPHY_SL
#define INCA_IP_Switch_EPHY_SL_DEFAULT       INCA_IP_Switch_EPHY_SL_100MBIT

/* END default settings: EPHY - Ethernet PHY Register */
#endif /* __INCA_SWITCH_SETTINGS_H__ */
