
#include <linux/string.h>
#include <linux/sched.h>

#if defined(CIPHER_BITS_128)
# define BS 16
#else
# define BS 8
#endif


static int CIPHER_NAME(_ofb_encrypt)(struct cipher_context *cx, 
				const u8 *in_blk, u8 *out_blk, int size, 
				int atomic, const u8 *_iv)
{
	const u32 *iv = (const u32 *) _iv;
	int SS = cx->ci->segmentsize/8;

#ifdef HW_ENCRYPT
	u8 tmp[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
# if defined(CIPHER_BITS_64) /* DES / 3DES Cipher */
	*INCA_IP_DES_3DES_DES_CONTROLR = (*INCA_IP_DES_3DES_DES_CONTROLR &
			~INCA_IP_DES_3DES_DES_CONTROLR_E_D) | /* encrypt */
			INCA_IP_DES_3DES_DES_CONTROLR_O(2) | /* OFB Mode */
			INCA_IP_DES_3DES_DES_CONTROLR_F(
				SS ==  8 ? 1 :
				SS ==  4 ? 2 :
				SS ==  2 ? 3 :
				SS ==  1 ? 4 : 1); /* feedback size */

	*INCA_IP_DES_3DES_DES_IVHR = iv[0];
	*INCA_IP_DES_3DES_DES_IVLR = iv[1];
# elif defined(CIPHER_BITS_128) /* AES Cipher */
	*INCA_IP_AES_AES_CONTROLR = (*INCA_IP_AES_AES_CONTROLR &
			~INCA_IP_AES_AES_CONTROLR_E_D) | /* encrypt */
			INCA_IP_AES_AES_CONTROLR_O(2) | /* OFB Mode */
			INCA_IP_AES_AES_CONTROLR_F(
				SS == 16 ? 0 :
				SS ==  8 ? 1 :
				SS ==  4 ? 2 :
				SS ==  2 ? 3 :
				SS ==  1 ? 4 : 0); /* feedback size */

	*INCA_IP_AES_AES_IV3R = iv[0];
	*INCA_IP_AES_AES_IV2R = iv[1];
	*INCA_IP_AES_AES_IV1R = iv[2];
	*INCA_IP_AES_AES_IV0R = iv[3];
# else
#  error unsupported cipher blocksize
# endif	
	
	for (size -= SS; size >= 0; size -= SS) {
		CIPHER_NAME(_encrypt)(cx, in_blk, out_blk, SS, atomic);
		in_blk += SS; out_blk += SS;

# if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0)
		if (!atomic && current->need_resched)
			schedule ();
# else
		if (!atomic)
			cond_resched ();
# endif
	} /* for */
	if (size != -SS) {
		memcpy(tmp, in_blk, size+SS);
		CIPHER_NAME(_encrypt)(cx, tmp, out_blk, SS, atomic);
	} /* if */
#else /* Software implementation */
# warning no software implementation for encryption in OFB mode
#endif

	return 0;
}

static int CIPHER_NAME(_ofb_decrypt)(struct cipher_context *cx, 
				const u8 *in_blk, u8 *out_blk, int size, 
				int atomic, const u8 *_iv)
{
	const u32 *iv = (const u32 *) _iv;
	int SS = cx->ci->segmentsize/8;
#ifdef HW_DECRYPT
	u8 tmp[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
# if defined(CIPHER_BITS_64) /* DES / 3DES Cipher */
	*INCA_IP_DES_3DES_DES_CONTROLR |=
	/*--- Workaround --------------------------------------------------*/
			(SS == 8 ? 0 : INCA_IP_DES_3DES_DES_CONTROLR_E_D) |
//			INCA_IP_DES_3DES_DES_CONTROLR_E_D | /* decrypt */
	/*-----------------------------------------------------------------*/
			INCA_IP_DES_3DES_DES_CONTROLR_O(2) | /* OFB Mode */
			INCA_IP_AES_AES_CONTROLR_F(
				SS ==  8 ? 1 :
				SS ==  4 ? 2 :
				SS ==  2 ? 3 :
				SS ==  1 ? 4 : 1); /* feedback size */


	*INCA_IP_DES_3DES_DES_IVHR = iv[0];
	*INCA_IP_DES_3DES_DES_IVLR = iv[1];
# elif defined(CIPHER_BITS_128) /* AES Cipher */
	*INCA_IP_AES_AES_CONTROLR |=
			INCA_IP_AES_AES_CONTROLR_E_D | /* decrypt */
			INCA_IP_AES_AES_CONTROLR_O(2) | /* OFB Mode */
			INCA_IP_AES_AES_CONTROLR_F(
				SS == 16 ? 0 :
				SS ==  8 ? 1 :
				SS ==  4 ? 2 :
				SS ==  2 ? 3 :
				SS ==  1 ? 4 : 0); /* feedback size */
			
	/* Processing of decryption key already started in aes_set_key() ! */
	while ((*INCA_IP_AES_AES_CONTROLR & INCA_IP_AES_AES_CONTROLR_KV) == 0)
		printk(KERN_NOTICE "processing decryption key!\n");
			
	*INCA_IP_AES_AES_IV3R = iv[0];
	*INCA_IP_AES_AES_IV2R = iv[1];
	*INCA_IP_AES_AES_IV1R = iv[2];
	*INCA_IP_AES_AES_IV0R = iv[3];
# else
#  error unsupported cipher blocksize
# endif	

	for (size -= SS; size >= 0; size -= SS) {
		CIPHER_NAME(_decrypt)(cx, in_blk, out_blk, SS, atomic);
		in_blk += SS; out_blk += SS;

# if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0)
		if (!atomic && current->need_resched)
			schedule ();
# else
		if (!atomic)
			cond_resched ();
# endif
	}

	if (size != -SS) { 
		CIPHER_NAME(_decrypt)(cx, in_blk, tmp, SS, atomic);
		memcpy(out_blk, tmp, size+SS);
	}
#else /* Software implementation */
# warning no software implementation for decryption in OFB mode
#endif
	return 0;
}

#undef BS
