/* define this to get DMA on SSC2 */
#undef INCAIP_SSC_USEDMA
#undef SSC_MULTISLAVE
/*
 *  linux/drivers/char/incaip_ssc.c
 *  Copyright (c) 2003 Gary Jennejohn <gj@denx.de>
 *
 *  This driver is partly based on the VxWorks drivers Copyright (c)
 *  2002, Infineon Technologies.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 */

/*
 * Include section 
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/fcntl.h>
#include <linux/ptrace.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/spinlock.h>
#include <linux/slab.h>
#include <linux/poll.h>

#include <asm/system.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/bitops.h>

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/version.h>

/* INCAIP specific includes */
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>
#include <asm/incaip/incaip_ssc.h>
#ifdef INCAIP_SSC_USEDMA
#include <asm/incaip/dma.h>
#endif

/*
 * Deal with CONFIG_MODVERSIONS
 */
#if CONFIG_MODVERSIONS==1
# include <linux/modversions.h>
#endif

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Gary Jennejohn <gj@denx.de>");
MODULE_DESCRIPTION("SSC driver");
MODULE_SUPPORTED_DEVICE("ssc");
MODULE_PARM(maj, "i");
MODULE_PARM_DESC(maj, "Major device number");

#define INCA_MUX

/* allow the user to set the major device number */
static int maj = 0;

#define INCA_MUX

/* a macro to turn an interrupt number into the correct bit for the SSC */
#define BIT_NO(irq) (1 << (irq - 64))

/* the maximum possible number of ports */
#define NR_PORTS 2
/* this is the actual physical number of ports - some INCAs don't */
/* have SSC2. */
static int nr_ports;

/*
 * This is the per-channel data structure containing pointers, flags
 * and variables for the port. This driver supports a maximum of NR_PORTS.
 * isp is allocated in incaip_ssc_init() based on the chip version.
 */
static struct incaip_ssc_port *isp;

#ifdef INCAIP_SSC_USEDMA
#define INCAIP_SSC_DMACHAN 2
/* DMA is only used for transmissions */
#define INCAIP_SSC_TX_DESC 16
#define INCAIP_SSC_DMA_PRIO 0
#endif

/* prototypes for fops */
static ssize_t incaip_ssc_read(struct file *, char *, size_t, loff_t *);
static ssize_t incaip_ssc_write(struct file *, const char *, size_t, loff_t *);
static unsigned int incaip_ssc_poll(struct file *, struct poll_table_struct *);
int incaip_ssc_ioctl(struct inode *, struct file *, unsigned int, unsigned long);
int incaip_ssc_open(struct inode *, struct file *);
int incaip_ssc_close(struct inode *, struct file *);

/* kernel interface for read and write */
ssize_t incaip_ssc_kernel_read(int, char *, size_t);
ssize_t incaip_ssc_kernel_write(int, const char *, size_t);

/* other forward declarations */
#ifdef INCAIP_SSC_USEDMA
static unsigned long incaip_ssc_dma_txdone(void *);
#endif
static void incaip_ssc_rx_int(int, void *, struct pt_regs *);
static void incaip_ssc_tx_int(int, void *, struct pt_regs *);
static void incaip_ssc_er_int(int, void *, struct pt_regs *);
static void tx_int_FIFO(struct incaip_ssc_port *);
static void tx_int_noFIFO(struct incaip_ssc_port *);
static int incaip_ssc1_read_proc(char *, char **, off_t, int, int *, void *);
static int incaip_ssc2_read_proc(char *, char **, off_t, int, int *, void *);

/* declarations of external functions to make gcc happy */
extern unsigned int incaip_get_fpi_hz(void);
extern unsigned int incaip_get_cpu_ver(void);
extern long incaDMA_InitTxChannel (unsigned long, unsigned long, unsigned long, 
                              void *, unsigned long (*)(void*));
#ifdef INCA_MUX
/* requested by the customer, but I don't know what they do */
extern unsigned int inca_mux_ssc1_pins(void);
extern unsigned int inca_mux_ssc2_pins(void);
#endif

/* Fops-struct */
static struct file_operations incaip_ssc_fops = {
        owner: THIS_MODULE,
	read: incaip_ssc_read,	/* read */
	write: incaip_ssc_write, /* write */
        poll: incaip_ssc_poll,  /* poll */
        ioctl: incaip_ssc_ioctl, /* ioctl */
        open: incaip_ssc_open,  /* open */
        release: incaip_ssc_close, /* release */
};

#ifdef USE_BH
/*
 * This routine is used by the interrupt handler to schedule
 * processing in the software interrupt portion of the driver
 * (also known as the "bottom half").  This can be called any
 * number of times for any channel without harm.
 */
static inline void
incaip_ssc_sched_event(struct incaip_ssc_port *info, int event)
{
    info->event |= 1 << event; /* remember what kind of event and who */
    queue_task(&info->tqueue, &tq_cyclades); /* it belongs to */
    mark_bh(CYCLADES_BH);                       /* then trigger event */
} /* incaip_ssc_sched_event */


/*
 * This routine is used to handle the "bottom half" processing for the
 * serial driver, known also the "software interrupt" processing.
 * This processing is done at the kernel interrupt level, after the
 * cy#/_interrupt() has returned, BUT WITH INTERRUPTS TURNED ON.  This
 * is where time-consuming activities which can not be done in the
 * interrupt driver proper are done; the interrupt driver schedules
 * them using incaip_ssc_sched_event(), and they get done here.
 *
 * This is done through one level of indirection--the task queue.
 * When a hardware interrupt service routine wants service by the
 * driver's bottom half, it enqueues the appropriate tq_struct (one
 * per port) to the tq_cyclades work queue and sets a request flag
 * via mark_bh for processing that queue.  When the time is right,
 * do_incaip_ssc_bh is called (because of the mark_bh) and it requests
 * that the work queue be processed.
 *
 * Although this may seem unwieldy, it gives the system a way to
 * pass an argument (in this case the pointer to the incaip_ssc_port
 * structure) to the bottom half of the driver.  Previous kernels
 * had to poll every port to see if that port needed servicing.
 */
static void
do_incaip_ssc_bh(void)
{
    run_task_queue(&tq_cyclades);
} /* do_incaip_ssc_bh */

static void
do_softint(void *private_)
{
  struct incaip_ssc_port *info = (struct incaip_ssc_port *) private_;

    if (test_and_clear_bit(Cy_EVENT_HANGUP, &info->event)) {
        wake_up_interruptible(&info->open_wait);
        info->flags &= ~(ASYNC_NORMAL_ACTIVE|
                             ASYNC_CALLOUT_ACTIVE);
    }
    if (test_and_clear_bit(Cy_EVENT_OPEN_WAKEUP, &info->event)) {
        wake_up_interruptible(&info->open_wait);
    }
    if (test_and_clear_bit(Cy_EVENT_DELTA_WAKEUP, &info->event)) {
	wake_up_interruptible(&info->delta_msr_wait);
    }
    if (test_and_clear_bit(Cy_EVENT_WRITE_WAKEUP, &info->event)) {
        wake_up_interruptible(&tty->write_wait);
    }
#ifdef Z_WAKE
    if (test_and_clear_bit(Cy_EVENT_SHUTDOWN_WAKEUP, &info->event)) {
        wake_up_interruptible(&info->shutdown_wait);
    }
#endif
} /* do_softint */
#endif /* USE_BH */

static void
rx_int_FIFO(struct incaip_ssc_port *info)
{
	int bytes_in_fifo, nbytes, i;
	unsigned short uval;

	/* number of bytes waiting in the RX FIFO */
	bytes_in_fifo = info->regs->ssc_fstat & INCAIP_SSC_RXFIFO_MASK;
	/* free space in the RX buffer */
	nbytes = info->rxrp - info->rxwp - 1;
	if (nbytes < 0)
		nbytes += RX_BUFSIZE;
	if (bytes_in_fifo > nbytes )
	{
		bytes_in_fifo = nbytes;
		/* not enough space in the RX buffer, disable the rxirq */
		info->bRxIrqEnabled = 0;
	}
	/* all accesses 16 bits wide */
	for (i = 0; i < bytes_in_fifo; i++)
	{
		uval = info->regs->ssc_rbuf;
		*info->rxwp = uval;

		if (++info->rxwp > &info->rxbuf[RX_BUFSIZE - 1])
			info->rxwp = info->rxbuf;
	}
	info->stats.rxBytes += bytes_in_fifo * info->elem_size;

	if (!SSC_IS_MASTER(info) && info->opts.duplexMode == SSC_FULL_DUPLEX)
	{
		tx_int_FIFO(info);
	}
	/* wakeup any processes waiting in read() */
	wake_up_interruptible(&info->rwait);
	/* and in poll() */
	wake_up_interruptible(&info->pwait);
	if (info->bRxIrqEnabled == 0)
		disable_irq(info->rxirq);
}

static void
rx_int_noFIFO(struct incaip_ssc_port *info)
{
	int free_space;
	unsigned short uval;

	/* avoid a panic */
	if (info->rxbuf == NULL)
	{
		uval = info->regs->ssc_rbuf;
		return;
	}

	/* free space in the RX buffer */
	free_space = info->rxrp - info->rxwp - 1;
	if (free_space < 0)
		free_space += RX_BUFSIZE;
	if (free_space > 0)
	{
		info->runIdxRxIrq++;
		uval = info->regs->ssc_rbuf;
		*info->rxwp = uval;

		if (++info->rxwp > &info->rxbuf[RX_BUFSIZE - 1])
			info->rxwp = info->rxbuf;
		/* update the RX count */
		info->stats.rxBytes += info->elem_size;
	}
	else
		/* flag to turn off the interrupt */
		info->bRxIrqEnabled = 0;

	/* turn off the interrupt */
	if (info->bRxIrqEnabled == 0)
		disable_irq(info->rxirq);
	/* wakeup any processes waiting in read() */
	wake_up_interruptible(&info->rwait);
	/* and in poll() */
	wake_up_interruptible(&info->pwait);

	/* since we don't have a FIFO we have to synchronise TX with RX */
	if(SSC_IS_MASTER(info))
	{
		if(info->runIdxTxIrq != info->runIdxRxIrq)
			tx_int_noFIFO(info);
	}
#ifdef SSC_MULTISLAVE
	else
	{
		if (info->bDisTransmit)
			*INCA_IP_Ports_P2_ALTSEL |= info->mrst;
	}
#endif
}

static void
tx_int_FIFO(struct incaip_ssc_port *info)
{
	int fifo_space, fill, i;
	unsigned short val;

	fifo_space = INCAIP_SSC_TXFIFO_FULL - ((info->regs->ssc_fstat & INCAIP_SSC_TXFIFO_MASK) >> INCAIP_SSC_TXFIFO_SHIFT);

	fill = info->txwp - info->txrp;
	if (fill < 0)
		fill += TX_BUFSIZE;

	if (fill == 0)
	{
#ifdef SSC_MULTISLAVE
		if (!SSC_IS_MASTER(info) && fifo_space != 0)
		{
			fifo_space /= info->elem_size;
			for (i = 0; i < fifo_space; i++)
				info->regs->ssc_tbuf = 0xffff;
		}
#endif
		return;
	}

	if (fifo_space == 0)
		return;

	if (fill > fifo_space)
		fill = fifo_space;

#ifdef SSC_MULTISLAVE
	if (!SSC_IS_MASTER(info))
	{
		/* add dummy symbols if required */
		fifo_space -= fill;
		if (fifo_space)
		{
			fill += fifo_space;
			while(fifo_space--)
			{
				*info->txwp = 0xffff;
				if (++info->txwp > &info->txbuf[TX_BUFSIZE - 1])
					info->txwp = info->txbuf;
			}
		}
	}
#endif

	for (i = 0; i < fill; i++)
	{
		val = *info->txrp;
		info->regs->ssc_tbuf = val;

		if (++info->txrp > &info->txbuf[TX_BUFSIZE - 1])
			info->txrp = info->txbuf;
	}

	info->stats.txBytes += fill * info->elem_size;
	/* wake up any process waiting in poll() */
	wake_up_interruptible(&info->pwait);
}

static void
tx_int_noFIFO(struct incaip_ssc_port *info)
{
	unsigned short val;

	/* if there are data.. */
	if (info->txrp != info->txwp)
	{
		if (SSC_IS_MASTER(info))
			/* don't transmit if out of sync */
			if (info->runIdxTxIrq == info->runIdxRxIrq)
				return;
		/* update the sync counter */
		info->runIdxTxIrq++;

		/* transmit the next element */
		val = *info->txrp;
		info->regs->ssc_tbuf = val;

		if (++info->txrp > &info->txbuf[TX_BUFSIZE - 1])
			info->txrp = info->txbuf;
		/* update the TX count */
		info->stats.txBytes += info->elem_size;
		/* wake up any process waiting in poll() */
		wake_up_interruptible(&info->pwait);
	}
	else
	{
		if (!SSC_IS_MASTER(info))
		{
			info->bDisTransmit = 1;
#ifdef SCC_MULTISLAVE
			disable_irq(info->txirq);
#endif
		}
	}
}

static void
incaip_ssc_rx_int(int irq, void *dev_id, struct pt_regs *regs)
{
	struct incaip_ssc_port *info = (struct incaip_ssc_port *)dev_id;

	if (SSC_USE_FIFO(info))
		rx_int_FIFO(info);
	else
		rx_int_noFIFO(info);
}

static void
incaip_ssc_tx_int(int irq, void *dev_id, struct pt_regs *regs)
{
	struct incaip_ssc_port *info = (struct incaip_ssc_port *)dev_id;

	if (SSC_USE_FIFO(info))
		tx_int_FIFO(info);
	else
		tx_int_noFIFO(info);
}

static void
incaip_ssc_er_int(int irq, void *dev_id, struct pt_regs *regs)
{
	struct incaip_ssc_port *info = (struct incaip_ssc_port *)dev_id;

	/* avoid getting an RX interrupt */
	*INCA_IP_ICU_IM2_ISR = BIT_NO(info->rxirq);

	if (info->regs->ssc_con & INCA_IP_SSC1_SCC_CON_OPR_RE)
	{
		info->stats.rxErr++;
		info->regs->ssc_whbcon |= INCA_IP_SSC1_SSC_WHBCON_CLRRE;
	}

	if (info->regs->ssc_con & INCA_IP_SSC1_SCC_CON_OPR_PE)
	{
		info->stats.phErr++;
		info->regs->ssc_whbcon |= INCA_IP_SSC1_SSC_WHBCON_CLRPE;
	}

	/* supposedly this bit is never set by the master, but I see it! */
	if (info->regs->ssc_con & INCA_IP_SSC1_SCC_CON_OPR_BE)
	{
		info->stats.brErr++;
		info->regs->ssc_whbcon |= INCA_IP_SSC1_SSC_WHBCON_CLRBE;
	}

	if (!SSC_IS_MASTER(info))
	{
		if (info->regs->ssc_con & INCA_IP_SSC1_SCC_CON_OPR_TE)
		{
			info->stats.txErr++;
			info->regs->ssc_whbcon |= INCA_IP_SSC1_SSC_WHBCON_CLRTE;
		}
	}

}

#ifdef INCAIP_SSC_USEDMA
/* this is the callback for DMA transmissions */
static unsigned long
incaip_ssc_dma_txdone(void *dev)
{
	struct incaip_ssc_port *info = (struct incaip_ssc_port *)dev;

	/* we only get one interrupt after completion of the transfer */
	info->dma_freecnt = INCAIP_SSC_TX_DESC;
	return 0;
}
#endif

/*
 * This routine is called whenever a port is opened.  It enforces
 * exclusive opening of a port and enables interrupts, etc.
 */
int
incaip_ssc_open(struct inode *inode, struct file * filp)
{
	struct incaip_ssc_port  *info;
	int line;
	int from_kernel = 0;

	if ((inode == (struct inode *)0) || (inode == (struct inode *)1))
	{
		from_kernel = 1;
		line = (int)inode;
	}
	else
	{
		line = MINOR(filp->f_dentry->d_inode->i_rdev);
		filp->f_op = &incaip_ssc_fops;
	}

	/* don't open more minor devices than we can support */
	if (line < 0 || line >= nr_ports)
		return -ENXIO;


	info = &isp[line];

	/* exclusive open */
	if (info->port_is_open != 0)
		return -EBUSY;
	info->port_is_open++;

	/* logically flush the software FIFOs */
	info->rxrp = info->rxwp;
	info->txrp = info->txwp;
	info->bRxIrqEnabled = 1;
	info->runIdxTxIrq = 0;
	info->runIdxRxIrq = 1;
	if (line == 0)
	{
		/* clear all interrupts */
		*INCA_IP_ICU_IM2_ISR = 0x380;
		/* enable all interrupts */
		*INCA_IP_ICU_IM2_IER |= 0x380;
		if (!SSC_IS_MASTER(info) && !SSC_USE_FIFO(info))
		{
#ifdef SSC_MULTISLAVE
			disable_irq(info->txirq);
			*INCA_IP_Ports_P2_ALTSEL |= info->mrst;
#endif
			info->bDisTransmit = 1;
		}
	}
	else
	{
#ifdef INCAIP_SSC_USEDMA
		/* clear all interrupts */
		*INCA_IP_ICU_IM2_ISR = 0x1c00;
		/* disable all interrupts */
		*INCA_IP_ICU_IM2_IER &= ~0x1c00;
#else
		/* clear all interrupts */
		*INCA_IP_ICU_IM2_ISR = 0x1c00;
		/* enable all interrupts */
		*INCA_IP_ICU_IM2_IER |= 0x1c00;
#endif
	}
	/* make sure that the RB is empty */
	line = info->regs->ssc_rbuf & 0xffff;

	/* clear all error bits */
	info->regs->ssc_whbcon |= INCA_IP_SSC1_SSC_WHBCON_CLRRE |
		INCA_IP_SSC1_SSC_WHBCON_CLRPE | INCA_IP_SSC1_SSC_WHBCON_CLRBE |
		INCA_IP_SSC1_SSC_WHBCON_CLRTE;
#ifdef SSC_MULTISLAVE
	if (!SSC_IS_MASTER(info))
	{
		/* put some daa into the slave */
		if (SSC_USE_FIFO(info))
			tx_int_FIFO(info);
		else
			info->regs->ssc_tbuf = 0xffff;
	}
#endif

	MOD_INC_USE_COUNT;

	return 0;
} /* incaip_ssc_open */
EXPORT_SYMBOL(incaip_ssc_open);

/*
 * This routine is called when a particular device is closed.
 */
int
incaip_ssc_close(struct inode *inode, struct file *filp)
{
	struct incaip_ssc_port *info;
        int idx;
	int from_kernel = 0;

	if ((inode == (struct inode *)0) || (inode == (struct inode *)1))
	{
		from_kernel = 1;
		idx = (int)inode;
	}
	else
		idx = MINOR(filp->f_dentry->d_inode->i_rdev);

	if (idx < 0 || idx >= nr_ports)
		return -ENXIO;

	info = &isp[idx];
	info->port_is_open = 0;

	/* flush the HW FIFOs */
	if(SSC_USE_FIFO(info))
	{
		info->regs->ssc_rxfcon |= INCA_IP_SSC1_SSC_RXFCON_RXFLU;
		info->regs->ssc_txfcon |= INCA_IP_SSC1_SSC_TXFCON_TXFLU;
	}

	MOD_DEC_USE_COUNT;
	return 0;
} /* incaip_ssc_close */
EXPORT_SYMBOL(incaip_ssc_close);

/* helper routine to handle reads from the kernel or user-space */
static ssize_t
incaip_ssc_read_helper(struct incaip_ssc_port *info, char *buf, size_t len,
		int from_kernel)
{
        ssize_t N, nbytes;
	int idx;
	unsigned long flags;
	unsigned char *Cdst, *Cscr;
	unsigned short *Sdst, *Sscr;

	local_irq_save(flags);
	N = info->rxwp - info->rxrp;
	local_irq_restore(flags);
	if (N < 0)
		N += RX_BUFSIZE;
	/* NB that rxbuf is an array of ushorts! */
	/* doing byte-wide transfers */
	if (info->elem_size == 1)
	{
		if (len < N)
			nbytes = len;
		else
			nbytes = N;

		if (!from_kernel)
		{
			Cdst = (unsigned char *)kmalloc(nbytes, GFP_KERNEL);
			if (Cdst == NULL)
			{
				N = -ENOMEM;
				goto out;
			}
		}
		else
			Cdst = (unsigned char *)buf;
		Cscr = Cdst;
		/* rxbuf contains shorts so copy them to chars */
		for (idx = 0; idx < nbytes; idx++)
		{
			*Cdst++ = *info->rxrp++;
			if (info->rxrp > &info->rxbuf[RX_BUFSIZE - 1])
				info->rxrp = info->rxbuf;
		}
		if (!from_kernel)
		{
			if (copy_to_user (buf, Cscr, nbytes))
			{
				N = -EFAULT;
				kfree(Cscr);
				goto out;
			}
			kfree(Cscr);
		}
		N = nbytes;
	}
	else
	{
		if (len < (N << 1))
		{
			nbytes = len;
			N  = len >> 1;
		}
		else
			nbytes = N << 1;
		if (!from_kernel)
		{
			Sdst = (unsigned short *)kmalloc(sizeof(unsigned short)
				* nbytes, GFP_KERNEL);
			if (Sdst == NULL)
			{
				N = -ENOMEM;
				goto out;
			}
		}
		else
			Sdst = (unsigned short *)buf;
		Sscr = Sdst;
		/* copy shorts instead of bytes */
		for (idx = 0; idx < N; idx++)
		{
			*Sdst++ = *info->rxrp++;
			if (info->rxrp > &info->rxbuf[RX_BUFSIZE - 1])
				info->rxrp = info->rxbuf;
		}
		if (!from_kernel)
		{
			if (copy_to_user (buf, Sscr, nbytes))
			{
				N = -EFAULT;
				kfree(Sscr);
				goto out;
			}
			kfree(Sscr);
		}
		N = nbytes;
	}

	/* if the RX irq was disabled then reenable it now */
	if (info->bRxIrqEnabled == 0)
	{
		info->bRxIrqEnabled = 1;
		enable_irq(info->rxirq);
	}

  out:
	return N;
}

/* helper routine to handle writes to the kernel or user-space */
static ssize_t
incaip_ssc_write_helper(struct incaip_ssc_port *info, const char *buf,
		size_t len, int from_kernel)
{
        int idx;
	size_t N, spc;
	unsigned long flags;
	size_t nbytes;
	unsigned char *Csrc, *tmp = NULL;
	unsigned short *Ssrc;

	/* in half-duplex mode the slave can only receive */
	if (!SSC_IS_MASTER(info) && info->opts.duplexMode == SSC_HALF_DUPLEX)
		return -EIO;

	idx = info - &isp[0];

#ifdef INCAIP_SSC_USEDMA
	/* only SSC1 needs this in the DMA case */
	if (idx == 0)
	{
#endif
	N = len;
	/* XXX - may not need this */
	local_irq_save(flags);
	/* free space in the TX SW-FIFO */
	spc = info->txrp - info->txwp - 1;
	if (spc < 0)
		spc += TX_BUFSIZE;
	local_irq_restore(flags);
	if (N > spc)
		N = spc;
	if (!from_kernel)
	{
		Csrc = (unsigned char *)kmalloc(N, GFP_KERNEL);
		if (Csrc == NULL)
			return -ENOMEM;
		tmp = Csrc;

		/* copy data in from user */
		if (copy_from_user(Csrc, buf, N))
		{
			kfree(tmp);
			return -EFAULT;
		}
	}
	else
		Csrc = (unsigned char *)buf;

	/* XXX - may not need this */
	local_irq_save(flags);
	/* copy data into txbuf and honor elem_size */
	nbytes = N;
	if (info->elem_size == 1)
	{
		for (idx = 0; idx < nbytes; idx++)
		{
			*info->txwp = *Csrc++;
			/* check for a wrap */
			if (++info->txwp > &info->txbuf[TX_BUFSIZE - 1])
				info->txwp = info->txbuf;
		}
	}
	else
	{
		Ssrc = (unsigned short *)Csrc;
		for (idx = 0; idx < nbytes/2; idx++)
		{
			*info->txwp = *Ssrc++;
			/* check for a wrap */
			if (++info->txwp > &info->txbuf[TX_BUFSIZE - 1])
				info->txwp = info->txbuf;
		}
	}
	if (!from_kernel)
		kfree(tmp);

	if (info->bDisTransmit)
	{
#ifdef SSC_MULTISLAVE
		/* clear TX interrupt */
		*INCA_IP_ICU_IM2_ISR = BIT_NO(info->txirq);
		/* connect the pin to SSC again */
		*INCA_IP_Ports_P2_ALTSEL &= ~info->mrst;
		/* enable TX intr */
		enable_irq(info->txirq);
#endif
		info->bDisTransmit = 0;
	}
	/* start the transmission */
	if (SSC_USE_FIFO(info))
		tx_int_FIFO(info);
	else
		tx_int_noFIFO(info);
	local_irq_restore(flags);
#ifdef INCAIP_SSC_USEDMA
	}
	/* transmit using DMA */
	else
	{
		/* XXX block interrupts here? */
		/* DMA can only send a multiple of sizeof(long) */
		if (len & 3)
			return -EINVAL;
		/* DMA buffers are 1520 bytes in size - see dma.c */
		/* only transmit as much data as we have free DMA */
		/* descriptors to handle */
		local_irq_save(flags);
		if (len > 1520 * info->dma_freecnt)
			N = 1520 * info->dma_freecnt;
		else
			N = len;
		nbytes = N;
		if (!from_kernel)
		{
			Csrc = (unsigned char *)kmalloc(nbytes, GFP_KERNEL);
			if (Csrc == NULL)
			{
				local_irq_restore(flags);
				return -ENOMEM;
			}
			tmp = Csrc;
			/* copy data in from user */
			if (copy_from_user(Csrc, buf, nbytes))
			{
				local_irq_restore(flags);
				kfree(tmp);
				return -EFAULT;
			}
		}
		else
			Csrc = (unsigned char *)buf;
		while (nbytes > 0)
		{
			if (nbytes > 1520)
			{
				if (incaDMA_Transmit(INCAIP_SSC_DMACHAN,
					Csrc, 1520, 0) == (unsigned long)-1)
				{
					if (!from_kernel)
						kfree(tmp);
					return -EIO;
				}
				nbytes -= 1520;
				Csrc += 1520;
				--info->dma_freecnt;
			}
			else
			{
				if (incaDMA_Transmit(INCAIP_SSC_DMACHAN,
					Csrc, nbytes, 1) == (unsigned long)-1)
				{
					if (!from_kernel)
						kfree(tmp);
					return -EIO;
				}
				nbytes = 0;
				--info->dma_freecnt;
			}
		}
		local_irq_restore(flags);
		if (!from_kernel)
			kfree(tmp);
		info->stats.txBytes += N;
	}
#endif
        return N;
}

/*
 * kernel interfaces for read and write.
 * The caller must set port to
 *	0 for SSC1
 *	1 for SSC2
 */
ssize_t
incaip_ssc_kernel_read(int port, char *buf, size_t len)
{
	struct incaip_ssc_port *info;
	unsigned long flags;

	if (port < 0 || port >= nr_ports)
		return -ENXIO;

        if (len == 0)
                return 0;

#ifdef INCAIP_SSC_USEDMA
	/* SSC2 can only transmit using DMA - no receive  */
	if (port == 1)
		return -EIO;
#endif

	info = &isp[port];

	local_irq_save(flags);
	if (info->rxrp == info->rxwp)
	{
		local_irq_restore(flags);
		return 0;
	}
	return(incaip_ssc_read_helper(info, buf, len, 1));
}
EXPORT_SYMBOL(incaip_ssc_kernel_read);

ssize_t
incaip_ssc_kernel_write(int port, const char *buf, size_t len)
{
	struct incaip_ssc_port *info;

	if (port < 0 || port >= nr_ports)
		return -ENXIO;

        if (len == 0)
                return 0;

	info = &isp[port];

	return(incaip_ssc_write_helper(info, buf, len, 1));
}
EXPORT_SYMBOL(incaip_ssc_kernel_write);


/* user interfaces to read and write */
static ssize_t
incaip_ssc_read(struct file *filp, char *buf, size_t len, loff_t *off)
{
        ssize_t N;
        DECLARE_WAITQUEUE(wait, current);
        int idx;
	struct incaip_ssc_port *info;
	unsigned long flags;

        if (len == 0)
                return (0);

        idx = MINOR(filp->f_dentry->d_inode->i_rdev);

#ifdef INCAIP_SSC_USEDMA
	/* SSC2 can only transmit using DMA - no receive  */
	if (idx == 1)
		return -EIO;
#endif

	info = &isp[idx];

	/* remember to do a wakeup!! */
        __add_wait_queue(&info->rwait, &wait);
        set_current_state(TASK_INTERRUPTIBLE);

        do {
		local_irq_save(flags);
                if (info->rxrp != info->rxwp)
                        break;
		local_irq_restore(flags);

                if (filp->f_flags & O_NONBLOCK)
                {
                        N = -EAGAIN;
                        goto out;
                }
                if (signal_pending(current)) {
                        N = -ERESTARTSYS;
                        goto out;
                }
                schedule();
        } while (1);

	N = incaip_ssc_read_helper(info, buf, len, 0);

  out:
        current->state = TASK_RUNNING;
        __remove_wait_queue(&info->rwait, &wait);

        return (N);
}

/*
 * As many bytes as we have free space for are copied from the user
 * into txbuf and the actual byte count is returned. The transmission is
 * always kicked off by calling the appropriate TX routine.
 *
 * In the DMA case as many bytes as we have free DMA descriptors to
 * handle are transmitted and the actual byte count is returned.
 */
static ssize_t
incaip_ssc_write(struct file *filp, const char *buf, size_t len, loff_t *off)
{
        int idx;
	struct incaip_ssc_port *info;

        if (len == 0)
                return (0);

        idx = MINOR(filp->f_dentry->d_inode->i_rdev);
	info = &isp[idx];

	return(incaip_ssc_write_helper(info, buf, len, 0));

} /* incaip_ssc_write */


/*
 * ------------------------------------------------------------
 * incaip_ssc_ioctl() and friends
 * ------------------------------------------------------------
 */

/*
 * This routine intializes the SSC and Port2 bits appropriately depending
 * on slave/master and full-/half-duplex mode.
 * It also flushes all buffers and initializes some state information
 * to reasonable values.
 */
static int
incaip_ssc_sethwopts(struct incaip_ssc_port *info, int portnr)
{
	unsigned long flags, bits;
	struct incaip_ssc_hwopts *opts = &info->opts;

	bits = 0;

	/* sanity checks */
	if ((opts->dataWidth < SSC_MIN_DATA_WIDTH) ||
	    (opts->dataWidth > SSC_MAX_DATA_WIDTH) ||
	    /* SSC2 is always master... */
	    ((portnr == 1 && !SSC_IS_MASTER(info)) ||
	    /* does not have a FIFO... */
	    (portnr == 1 && SSC_USE_FIFO(info))
#ifdef INCAIP_SSC_USEDMA
	    /* DMA always uses 16 bit data */
	   || (portnr == 1 && opts->dataWidth != SSC_MAX_DATA_WIDTH)
#endif	    
	    ))
	{
		printk("%s: sanity check failed\n", __FUNCTION__);
	    	return -1;
	}
	if (SSC_IS_MASTER(info))
		bits |=  INCA_IP_SSC1_SCC_CON_PRG_MS;
	bits |= opts->dataWidth - 1;
	info->elem_size = (opts->dataWidth > 8)? 2 : 1;
	if (opts->autoReset)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_AREN;
	if (opts->brErrDetect)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_BEN;
	if (opts->phErrDetect)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_PEN;
	if (opts->rxErrDetect)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_REN;
	if (opts->txErrDetect)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_TEN;
	if (opts->loopBack)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_LB;
	if (opts->headingControl)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_HB;
	if (opts->clockPhase)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_PH;
	if (opts->clockPolarity)
		bits |= INCA_IP_SSC1_SCC_CON_PRG_PO;
	local_irq_save(flags);
	info->regs->ssc_con &= ~INCA_IP_SSC1_SCC_CON_PRG_EN;
	info->regs->ssc_con = bits;
	/* set up the port pins */
	if (portnr == 0)
	{
		if (SSC_IS_MASTER(info))
		{
			/* for master mode: set P2_DIR.MTSR1 and */
			/* P2_DIR.SCLK1 bits */
			/*                  clear  P2_DIR.MRST1 bit */
			*INCA_IP_Ports_P2_DIR &= ~(0x00000080);
			*INCA_IP_Ports_P2_DIR |= (0x00000140);
			if (!SSC_USE_FIFO(info))
			{
#ifdef SSC_MULTISLAVE
				enable_irq(info->txirq);
				*INCA_IP_Ports_P2_ALTSEL &= ~info->mrst;
#endif
				info->bDisTransmit = 0;
			}
		}
		else
		{
			/* for slave mode: set P2_DIR.MRST1 */
			/* clear  P2_DIR.MTSR1 bit */
			/* and P2_DIR.SCLK1 bits */
			*INCA_IP_Ports_P2_DIR &= ~(0x00000140);
			*INCA_IP_Ports_P2_DIR |= (0x00000080);
#ifdef SSC_MULTISLAVE
			*INCA_IP_Ports_P2_OUT |= (1 << 7);
#endif
			if (!SSC_USE_FIFO(info))
			{
#ifdef SSC_MULTISLAVE
				disable_irq(info->txirq);
				*INCA_IP_Ports_P2_ALTSEL |= info->mrst;
#endif
				info->bDisTransmit = 1;
			}
		}
	}
	else
	{
		/* SSC2 is always the master */
		*INCA_IP_Ports_P2_DIR &= ~(0x00000600);
		*INCA_IP_Ports_P2_DIR |= (0x00000400);
	}

	if (opts->duplexMode == SSC_HALF_DUPLEX)
	{
		if (SSC_IS_MASTER(info))
		{
			if (portnr == 0)
			{
				/* turn on TxIrq */
				enable_irq(info->txirq);
			}
		}
		else
		{
			/* For slave mode we just disable TX interrupt */
			/* So slave will only receive data */
			disable_irq(info->txirq);
			/* Workaround for half-duplex mode */
			/* Setup direction for MRST to input */
			*INCA_IP_Ports_P2_DIR &= ~0x80;
		}
	}
	else /* full duplex */
	{
#ifdef INCAIP_SSC_USEDMA
		/* if using DMA this is only needed for SSC1 */
		if (portnr == 0)
#endif
		{
			/* turn on TxIrq */
			enable_irq(info->txirq);
		}
		if (!SSC_IS_MASTER(info))
		{
			/* Reset MRST direction to output */
			*INCA_IP_Ports_P2_DIR |= 0x80;
		}
	}
	/* re-enable the SSC */
	info->regs->ssc_con |= INCA_IP_SSC1_SCC_CON_PRG_EN;
	if (SSC_USE_FIFO(info))
	{
		info->regs->ssc_txfcon |= INCA_IP_SSC1_SSC_TXFCON_TXFLU;
		info->regs->ssc_rxfcon |= INCA_IP_SSC1_SSC_RXFCON_RXFLU;
	}
	/* make sure that the RB is empty */
	bits = (unsigned short)(info->regs->ssc_rbuf & 0xffff);
	local_irq_restore(flags);
	/* logically flush the software FIFOs */
	info->rxrp = info->rxwp;
	info->txrp = info->txwp;
	/* initialize some values */
	info->bRxIrqEnabled = 1;
	info->runIdxTxIrq = 0;
	info->runIdxRxIrq = 1;
#ifdef SSC_MULTISLAVE
	if (info->port_is_open && !SSC_IS_MASTER(info))
	{
		/* put some data into the slave */
		if (SSC_USE_FIFO(info))
			tx_int_FIFO(info);
		else
			info->regs->ssc_tbuf = 0xffff;
	}
#endif
	return 0;
}

static int
incaip_ssc_set_baud(struct incaip_ssc_port *info, unsigned int baud)
{
	unsigned int incaip_ssc_clock;
	unsigned int br;
	unsigned long flags;

	incaip_ssc_clock = incaip_get_fpi_hz();

	local_irq_save(flags);
	/* have to disable the SSC to set the baudrate */
	info->regs->ssc_con &= ~INCA_IP_SSC1_SCC_CON_PRG_EN;
	local_irq_restore(flags);

	br = ((incaip_ssc_clock >> 1)/baud) - 1;
	if (br > 0xffff || ((br == 0) && ((info->regs->ssc_con &
				INCA_IP_SSC1_SCC_CON_PRG_MS) == 0)))
	{
		printk("%s: illegal baudrate %u\n", __FUNCTION__, baud);
		return -1;
	}
	local_irq_save(flags);
	info->regs->ssc_br = br;
	info->regs->ssc_con |= INCA_IP_SSC1_SCC_CON_PRG_EN;
	local_irq_restore(flags);
	return 0;
}

static int
incaip_ssc_hwinit(struct incaip_ssc_port *info)
{
	unsigned long flags;
	int portnr;

	if (info == &isp[0])
		portnr = 0;
	else
		portnr = 1;

	if (incaip_ssc_sethwopts(info, portnr) < 0)
	{
		printk("%s: setting the hardware options failed\n",
							__FUNCTION__);
		return -1;
	}

	if (SSC_USE_FIFO(info))
	{
		local_irq_save(flags);
		/* TX FIFO */
		info->regs->ssc_txfcon &= ~INCAIP_SSC_TXFIFO_MASK;
		info->regs->ssc_txfcon |= (INCAIP_SSC_TXFIFO_FL << 8);
		info->regs->ssc_txfcon |= INCA_IP_SSC1_SSC_TXFCON_TXFEN;
		/* RX FIFO */
		info->regs->ssc_rxfcon &= ~INCAIP_SSC_RXFIFO_MASK;
		info->regs->ssc_rxfcon |= (INCAIP_SSC_RXFIFO_FL << 8);
		info->regs->ssc_rxfcon |= INCA_IP_SSC1_SSC_RXFCON_RXFEN;
		local_irq_restore(flags);
	}
	else
	{
		local_irq_save(flags);
		/* TX FIFO */
		info->regs->ssc_txfcon = 0;
		/* RX FIFO */
		info->regs->ssc_rxfcon = 0;
		local_irq_restore(flags);
	}
	if (incaip_ssc_set_baud(info, info->baud) < 0)
	{
		printk("%s: setting the baud rate failed\n", __FUNCTION__);
		return -1;
	}
	return 0;
}

/*
 * This routine allows the driver to implement device-
 * specific ioctl's.  If the ioctl number passed in cmd is
 * not recognized by the driver, it should return ENOIOCTLCMD.
 */
int
incaip_ssc_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
							unsigned long data)
{
	struct incaip_ssc_port *info;
	int line, ret_val = 0;
	unsigned long flags;
	int from_kernel = 0;

	if ((inode == (struct inode *)0) || (inode == (struct inode *)1))
	{
		from_kernel = 1;
		line = (int)inode;
	}
	else
		line = MINOR(filp->f_dentry->d_inode->i_rdev);

	/* don't open more minor devices than we can support */
	if (line < 0 || line >= nr_ports)
		return -ENXIO;

	info = &isp[line];

	switch (cmd) {
               /* data must be a pointer to an int */
		case FIONREAD:
			/* reuse line */
			line = info->rxwp - info->rxrp;
			if (line < 0)
				line += RX_BUFSIZE;
			/* allow for whether data are 8 bits or 16 bits */
			line <<= (info->elem_size - 1);
			if (from_kernel)
				memcpy((void *)data, (void *)&line,
						sizeof(line));
			else
				if (copy_to_user((void *)data, (void *)&line,
						sizeof(line)))
					ret_val = -EFAULT;
			break;
               /* data must be a pointer to a struct incaip_ssc_statistics */
                case SSC_STATS_READ:
			if (from_kernel)
				memcpy((void *)data, (void *)&info->stats,
					sizeof(struct incaip_ssc_statistics));
			else
				if (copy_to_user((void *)data,
					(void *)&info->stats,
					sizeof(struct incaip_ssc_statistics)))
					ret_val = -EFAULT;
                        break;
                /* just resets the statistics counters */
                case SSC_STATS_RESET:
                        memset((void *)&info->stats, 0, sizeof(struct incaip_ssc_statistics));
                        break;
		case SSC_SET_BAUD:
			/* if the buffers are not empty then the port is */
			/* busy and we shouldn't change things on-the-fly! */
			if ((info->txrp != info->txwp) ||
				(info->rxrp != info->rxwp))
			{
				ret_val = -EINVAL;
				break;
			}
			/* misuse flags */
			if (from_kernel)
				flags = *((unsigned long *)data);
			else
				if (copy_from_user((void *)&flags,
					(void *)data, sizeof(flags)))
				{
					ret_val = -EFAULT;
					break;
				}
			if (flags == 0)
			{
				ret_val = -EINVAL;
				break;
			}
			if (incaip_ssc_set_baud(info, flags) < 0)
			{
				ret_val = -EINVAL;
				break;
			}
			info->baud = flags;
			break;
		case SSC_GET_BAUD:
			if (from_kernel)
				*((unsigned int *)data) = info->baud;
			else
				if (copy_to_user((void *)data,
						(void *)&info->baud,
						sizeof(unsigned long)))
					ret_val = -EFAULT;
			break;
		case SSC_GET_DMA:
			/* reuse line */
			line = 0;
#ifdef INCAIP_SSC_USEDMA
			if (info == &isp[1])
				line = 1;
#endif
			if (from_kernel)
				*((int *)data) = line;
			else
				if (copy_to_user((void *)data, (void *)&line,
						sizeof(int)))
					ret_val = -EFAULT;
			break;
               /* data must be a pointer to a struct incaip_ssc_hwopts */
		case SSC_SET_HWOPTS:
			/* if the buffers are not empty then the port is */
			/* busy and we shouldn't change things on-the-fly! */
			if ((info->txrp != info->txwp) ||
				(info->rxrp != info->rxwp))
			{
				ret_val = -EINVAL;
				break;
			}
			if (from_kernel)
				memcpy((void *)&info->opts, (void *)data,
					sizeof(struct incaip_ssc_hwopts));
			else
				if (copy_from_user((void *)&info->opts,
					(void *)data,
					sizeof(struct incaip_ssc_hwopts)))
				{
					ret_val = -EFAULT;
					break;
				}
			if (incaip_ssc_hwinit(info) < 0)
			{
				ret_val = -EIO;
			}
			break;
               /* data must be a pointer to a struct incaip_ssc_hwopts */
		case SSC_GET_HWOPTS:
			if (from_kernel)
				memcpy((void *)data, (void *)&info->opts,
					sizeof(struct incaip_ssc_hwopts));
			else
				if (copy_to_user((void *)data,
					(void *)&info->opts,
					sizeof(struct incaip_ssc_hwopts)))
					ret_val = -EFAULT;
			break;
               /* data must be a pointer to an int */
		case SSC_FIONWRITE:
			/* reuse line */
			line = 0;
#ifdef INCAIP_SSC_USEDMA
			/*
			 * if DMA is used then the data are immediately
			 * transmitted for SSC2, so we can only do this
			 * for SCC1 in that case.
			 */
			if (info == &isp[0])
#endif
			{
				/* number of bytes in the TX SW-FIFO */
				line = info->txwp - info->txrp;
				if (line < 0)
					line += TX_BUFSIZE;
			}
			/* bytes in the FIFO or TB */
			if (SSC_USE_FIFO(info))
				line += ((info->regs->ssc_fstat &
					INCAIP_SSC_TXFIFO_MASK) >>
					INCAIP_SSC_TXFIFO_SHIFT);
			else
				if (SSC_IS_MASTER(info) == 0 &&
						info->bDisTransmit == 0)
					line += info->elem_size;

			if (from_kernel)
				*((int *)data) = line;
			else
				if (copy_to_user((void *)data, (void *)&line,
						sizeof(line)))
					ret_val = -EFAULT;
			break;
		case SSC_FIFOFLUSH:
			info->rxrp = info->rxwp;
			if (SSC_USE_FIFO(info))
				info->regs->ssc_rxfcon |= INCA_IP_SSC1_SSC_RXFCON_RXFLU;
			/* FALLTHROUGH */
		case SSC_FIFOWFLUSH:
			info->txrp = info->txwp;
			if (SSC_USE_FIFO(info))
			{
				info->regs->ssc_txfcon |= INCA_IP_SSC1_SSC_TXFCON_TXFLU;
#ifdef SSC_MULTISLAVE
				/* only for slave and full-duplex:
					fill the FIFO */
				if (!SSC_IS_MASTER(info) &&
					info->opts.duplexMode ==
							SSC_FULL_DUPLEX)
					tx_int_FIFO(info);
#endif
			}
			else
			{
#ifdef SSC_MULTISLAVE
				disable_irq(info->txirq);
				*INCA_IP_Ports_P2_ALTSEL |= info->mrst;
#endif
				info->bDisTransmit = 1;
			}
			break;
		case SSC_FIFORFLUSH:
			info->rxrp = info->rxwp;
			if (SSC_USE_FIFO(info))
				info->regs->ssc_rxfcon |= INCA_IP_SSC1_SSC_RXFCON_RXFLU;
			break;
		default:
			ret_val = -ENOIOCTLCMD;
	}

	return ret_val;
} /* incaip_ssc_ioctl */
EXPORT_SYMBOL(incaip_ssc_ioctl);

/* the poll routine */
static unsigned int
incaip_ssc_poll(struct file *filp, struct poll_table_struct *pts)
{
        int unit = MINOR(filp->f_dentry->d_inode->i_rdev);
	struct incaip_ssc_port *info;
        unsigned int mask = 0; 
	int spc;

	info = &isp[unit];

        /* add event to the wait queues */
        /* DO NOT FORGET TO DO A WAKEUP ON THESE !!!! */
        poll_wait(filp, &info->pwait, pts);

	/* are there bytes in the RX SW-FIFO? */
        if (info->rxrp != info->rxwp)
                mask |= POLLIN | POLLRDNORM;

	/* free space in the TX SW-FIFO */
	spc = info->txrp - info->txwp - 1;
	if (spc < 0)
		spc += TX_BUFSIZE;
#ifdef INCAIP_SSC_USEDMA
	/* writing always works, except in the DMA case when all descriptors */
	/* are used up */
	if (unit == 1 && info->dma_freecnt == 0)
		spc = 0;
#endif
	if (spc > 0)
		mask |= POLLOUT | POLLWRNORM;

        return (mask);
}

static int
incaip_ssc1_read_proc(char *page, char **start, off_t offset, int count, int *eof, void *data)
{
	int off = 0;
	unsigned long flags;

	/* don't want any interrupts here */
	save_flags(flags);
	cli();


	/* print statistics */
	off += sprintf(page+off, "Statistics for SSC1\n");
	off += sprintf(page+off, "RX errors %d\n", isp[0].stats.rxErr);
	off += sprintf(page+off, "Phase errors %d\n", isp[0].stats.phErr);
	off += sprintf(page+off, "Baudrate errors %d\n", isp[0].stats.brErr);
	off += sprintf(page+off, "TX errors %d\n", isp[0].stats.txErr);
	off += sprintf(page+off, "TX overflows %d\n", isp[0].stats.txOvErr);
	off += sprintf(page+off, "RX overflows %d\n", isp[0].stats.rxOvErr);
	off += sprintf(page+off, "RX underruns %d\n", isp[0].stats.rxUnErr);
	off += sprintf(page+off, "RX Bytes %d\n", isp[0].stats.rxBytes);
	off += sprintf(page+off, "TX Bytes %d\n", isp[0].stats.txBytes);
#undef GJDEBUG
#ifdef GJDEBUG
	off += sprintf(page+off, "runIdxTxIrq %d\n", isp[0].runIdxTxIrq);
	off += sprintf(page+off, "runIdxRxIrq %d\n", isp[0].runIdxRxIrq);
	off += sprintf(page+off, "bDisTransmit %d\n", isp[0].bDisTransmit);
	off += sprintf(page+off, "bRxIrqEnabled %d\n", isp[0].bRxIrqEnabled);
	off += sprintf(page+off, "IM2_IER %#x\n", *INCA_IP_ICU_IM2_IER & 0x380);
	off += sprintf(page+off, "CON_OPR %#x\n", isp[0].regs->ssc_con);
	isp[0].regs->ssc_con &= ~INCA_IP_SSC1_SCC_CON_PRG_EN;
	off += sprintf(page+off, "CON_PRG %#x\n", isp[0].regs->ssc_con);
	off += sprintf(page+off, "BR %#x\n", isp[0].regs->ssc_br);
	off += sprintf(page+off, "CLC %#x\n", isp[0].regs->ssc_clc);
	off += sprintf(page+off, "TXFCON %#x\n", isp[0].regs->ssc_txfcon);
	off += sprintf(page+off, "RXFCON %#x\n", isp[0].regs->ssc_rxfcon);
	isp[0].regs->ssc_con |= INCA_IP_SSC1_SCC_CON_PRG_EN;
	off += sprintf(page+off, "P2_DIR %#x\n", *INCA_IP_Ports_P2_DIR);
	off += sprintf(page+off, "P2_ALTSEL %#x\n", *INCA_IP_Ports_P2_ALTSEL);
#endif

	restore_flags (flags); /* XXXXX */
	*eof = 1;
	return (off);
}

static int
incaip_ssc2_read_proc(char *page, char **start, off_t offset, int count, int *eof, void *data)
{
	int off = 0;
	unsigned long flags;

	/* don't want any interrupts here */
	save_flags(flags);
	cli();


	/* print statistics */
	off += sprintf(page+off, "Statistics for SSC2\n");
	off += sprintf(page+off, "RX errors %d\n", isp[1].stats.rxErr);
	off += sprintf(page+off, "Phase errors %d\n", isp[1].stats.phErr);
	off += sprintf(page+off, "Baudrate errors %d\n", isp[1].stats.brErr);
	off += sprintf(page+off, "TX errors %d\n", isp[1].stats.txErr);
	off += sprintf(page+off, "TX overflows %d\n", isp[1].stats.txOvErr);
	off += sprintf(page+off, "RX overflows %d\n", isp[1].stats.rxOvErr);
	off += sprintf(page+off, "RX underruns %d\n", isp[1].stats.rxUnErr);
	off += sprintf(page+off, "RX Bytes %d\n", isp[1].stats.rxBytes);
	off += sprintf(page+off, "TX Bytes %d\n", isp[1].stats.txBytes);
#undef GJDEBUG
#ifdef GJDEBUG
#ifdef INCAIP_SSC_USEDMA
	off += sprintf(page+off, "DMAfree %d\n", isp[1].dma_freecnt);
#endif
	off += sprintf(page+off, "runIdxTxIrq %d\n", isp[1].runIdxTxIrq);
	off += sprintf(page+off, "runIdxRxIrq %d\n", isp[1].runIdxRxIrq);
	off += sprintf(page+off, "bDisTransmit %d\n", isp[1].bDisTransmit);
	off += sprintf(page+off, "bRxIrqEnabled %d\n", isp[1].bRxIrqEnabled);
	off += sprintf(page+off, "IM2_IER %#x\n", *INCA_IP_ICU_IM2_IER & 0x1c00);
	off += sprintf(page+off, "CON_OPR %#x\n", isp[1].regs->ssc_con);
	isp[1].regs->ssc_con &= ~INCA_IP_SSC1_SCC_CON_PRG_EN;
	off += sprintf(page+off, "CON_PRG %#x\n", isp[1].regs->ssc_con);
	off += sprintf(page+off, "BR %#x\n", isp[1].regs->ssc_br);
	off += sprintf(page+off, "CLC %#x\n", isp[1].regs->ssc_clc);
	isp[1].regs->ssc_con |= INCA_IP_SSC1_SCC_CON_PRG_EN;
	off += sprintf(page+off, "P2_DIR %#x\n", *INCA_IP_Ports_P2_DIR);
	off += sprintf(page+off, "P2_ALTSEL %#x\n", *INCA_IP_Ports_P2_ALTSEL);
#endif

	restore_flags (flags); /* XXXXX */
	*eof = 1;
	return (off);
}

/*
 * This routine prints out the appropriate serial driver version number
 */
#define INCAIP_SSC_VERS "0.4"
static inline void
show_version(void)
{
    printk("INCA-IP SSC driver %s: ", INCAIP_SSC_VERS);
    printk("built %s %s\n", __DATE__, __TIME__);
} /* show_version */

/*
 * Due to the fact that a port can be dynamically switched between slave
 * and master mode using an IOCTL the hardware is not initialized here,
 * but in incaip_ssc_hwinit() as a result of an IOCTL.
 */
int __init
incaip_ssc_init(void)
{
	struct incaip_ssc_port  *info;
	int i, nbytes;
	unsigned int ver;
	unsigned long flags;

	/* figure out what kind of INCAIP we have */
	/* and set the number of SSCs accordingly */
	ver = incaip_get_cpu_ver();
	if ((INCA_IP_WDT_CHIPID_PART_NUMBER_GET(ver) & 0xc1) == 0xc1)
		nr_ports = 1;
	else
		nr_ports = 2;

#ifdef INCA_MUX
	/* I don't know what these routines really do, but the */
	/* customer requested this */
	if (inca_mux_ssc1_pins() == 0)
		return 1;
	if (nr_ports == 2 && inca_mux_ssc2_pins() == 0)
		/* graceful degradation - we can use at least SSC1 */
		nr_ports = 1;
#endif

	nbytes = nr_ports * sizeof(struct incaip_ssc_port);
	isp = (struct incaip_ssc_port *)kmalloc(nbytes, GFP_KERNEL);
	if (isp == NULL)
	{
		printk("%s: no memory for isp\n", __FUNCTION__);
		return 1;
	}
	memset(isp, 0, nbytes);

	/* allocate RX/TX buffers for each port */
	for (i = 0; i < nr_ports; i++)
	{
#ifdef INCAIP_SSC_USEDMA
		/* SSC2 does not need any buffers if using DMA */
		if (i == 1)
			break;
#endif
		info = &isp[i];
		info->rxbuf = (unsigned short *)kmalloc(sizeof(unsigned short)
			* RX_BUFSIZE, GFP_KERNEL);
		if (info->rxbuf == NULL)
		{
			printk("%s: no memory for rxbuf[%d]\n", __FUNCTION__, i);
			goto errout;
		}
		memset(info->rxbuf, 0, RX_BUFSIZE);
		info->rxrp = info->rxwp = info->rxbuf;
		info->txbuf = (unsigned short *)kmalloc(sizeof(unsigned short)
			* TX_BUFSIZE, GFP_KERNEL);
		if (info->txbuf == NULL)
		{
			printk("%s: no memory for txbuf[%d]\n", __FUNCTION__, i);
			goto errout;
		}
		memset(info->txbuf, 0, TX_BUFSIZE);
		info->txrp = info->txwp = info->txbuf;
	}

	show_version();

	/* register the device */
	i = maj;
	if ((i = register_chrdev(maj, "ssc", &incaip_ssc_fops)) < 0)
	{
		printk("Unable to register major %d for the SSC\n", maj);
		if (maj == 0)
			goto errout;
		else
		{
			maj = 0;
			if ((i = register_chrdev(maj, "ssc", &incaip_ssc_fops)) < 0)
			{
				printk("Unable to register major %d for the SSC\n", maj);
				goto errout;
			}
		}
	}
	maj = i;
	printk("registered major %d for SSC\n", i);

	/* set default values in incaip_ssc_port */
	for (i = 0; i < nr_ports; i++)
	{
		info = &isp[i];
		/* default values for the HwOpts */
		info->opts.brErrDetect = SSC_DEFAULT_BR_ERR_DETECT;
		info->opts.phErrDetect = SSC_DEFAULT_PH_ERR_DETECT;
		info->opts.rxErrDetect = SSC_DEFAULT_RX_ERR_DETECT;
		info->opts.txErrDetect = SSC_DEFAULT_TX_ERR_DETECT;
		info->opts.autoReset = SSC_DEFAULT_AUTO_RESET;
		info->opts.loopBack = SSC_DEFAULT_LOOP_BACK;
		info->opts.clockPolarity = SSC_DEFAULT_CLOCK_POLARITY;
		info->opts.clockPhase = SSC_DEFAULT_CLOCK_PHASE;
		info->opts.headingControl = SSC_DEFAULT_HEADING_CONTROL;
		info->opts.dataWidth = SSC_DEFAULT_DATA_WIDTH;
		info->opts.useFIFO = SSC_DEFAULT_USE_FIFO;
		info->opts.duplexMode = SSC_DEFAULT_DUPLEX_MODE;
		info->elem_size = 2;
		info->baud = SSC_DEFAULT_BAUDRATE;
		/* values specific to SSC1 */
		if (i == 0)
		{
			info->regs = (struct incaip_ssc_regs *)INCA_IP_SSC1;
			/* CLC.DISS = 0 and CLC.RMC = 1 */
			info->regs->ssc_clc &= ~INCA_IP_SSC1_SSC_CLC_DISS;
			info->regs->ssc_clc &= ~0x0000ff00;
			info->regs->ssc_clc |= (0x01 << 8);
			/* enable SSC1 */
			*INCA_IP_PMU_PM_GEN |= INCA_IP_PMU_PM_GEN_EN11;
			/* set pin 6-8 to default function: P2_ALTSEL */
			*INCA_IP_Ports_P2_ALTSEL &= ~0x1c0;
			/* NB default mode is SLAVE */
			info->txirq = INT_NUM_IM2_IRL7;
			info->rxirq = INT_NUM_IM2_IRL8;
			info->erirq = INT_NUM_IM2_IRL9;
			info->mrst = (1 << 7);
		}
		/* values specific to SSC2 */
		if (i == 1)
		{
			info->regs = (struct incaip_ssc_regs *)INCA_IP_SSC2;
			/* CLC.DISS = 0 and CLC.RMC = 1 */
			info->regs->ssc_clc &= ~INCA_IP_SSC1_SSC_CLC_DISS;
			info->regs->ssc_clc &= ~0x0000ff00;
			info->regs->ssc_clc |= (0x01 << 8);
			/* enable SSC2 */
			*INCA_IP_PMU_PM_GEN |= INCA_IP_PMU_PM_GEN_EN12;
			/* set pin 9-10 to default function: P2_ALTSEL */
			*INCA_IP_Ports_P2_ALTSEL &= ~0x600;
			info->txirq = INT_NUM_IM2_IRL10;
			info->rxirq = INT_NUM_IM2_IRL11;
			info->erirq = INT_NUM_IM2_IRL12;
			info->mrst = (1 << 9);
			/* SSC2 is _always_ master and has no FIFOs! */
			info->opts.masterSelect = SSC_MASTER_MODE;
			info->opts.useFIFO = 0;
#ifdef INCAIP_SSC_USEDMA
			info->dma_active = 1;
			info->dma_freecnt = INCAIP_SSC_TX_DESC;
			/* clear all interrupts */
			*INCA_IP_ICU_IM2_ISR = 0x1c00;
			/* disable all interrupts */
			*INCA_IP_ICU_IM2_IER &= ~0x1c00;
			/* initialize the DMA channel */
			/* this only works because only SSC2 uses DMA */
			if (incaDMA_InitTxChannel (INCAIP_SSC_DMACHAN,
				INCAIP_SSC_TX_DESC, INCAIP_SSC_DMA_PRIO, 
					info, incaip_ssc_dma_txdone) < 0)
			{
				printk("%s: %s\n",
					__FUNCTION__,
					"DMA initialization of SSC2 failed");
				goto errout;
			}
#endif
		}
		init_waitqueue_head(&info->rwait);
		init_waitqueue_head(&info->pwait);

		/* try to get the interrupts - reuses nbytes */
		local_irq_save(flags);
		nbytes = request_irq(info->txirq, incaip_ssc_tx_int, 0, "ssc_tx", info);
		if (nbytes)
		{
			printk("%s: unable to get irq %d\n", __FUNCTION__,
					info->txirq);
			local_irq_restore(flags);
			goto errout;
		}
		nbytes = request_irq(info->rxirq, incaip_ssc_rx_int, 0, "ssc_rx", info);
		if (nbytes)
		{
			printk("%s: unable to get irq %d\n", __FUNCTION__,
					info->rxirq);
			local_irq_restore(flags);
			goto irqerr;
		}
		nbytes = request_irq(info->erirq, incaip_ssc_er_int, 0, "ssc_er", info);
		if (nbytes)
		{
			printk("%s: unable to get irq %d\n", __FUNCTION__,
					info->erirq);
			local_irq_restore(flags);
			goto irqerr;
		}
		local_irq_restore(flags);
	}

	/* init the SSCs with default values */
	for (i = 0; i < nr_ports; i++)
	{
		info = &isp[i];
		if (incaip_ssc_hwinit(info) < 0)
		{
			printk("%s: hardware init failed for port %d\n",
				__FUNCTION__, i);
			goto irqerr;
		}
	}

	/* register /proc read handler */
	/* for SSC1, which is always present */
	create_proc_read_entry("driver/ssc1", 0, NULL, incaip_ssc1_read_proc, NULL);
	/* for SSC2, which may not be present */
	if (nr_ports > 1)
		create_proc_read_entry("driver/ssc2", 0, NULL, incaip_ssc2_read_proc, NULL);
	return 0;

irqerr:
	free_irq(isp[0].txirq, &isp[0]);
	free_irq(isp[0].rxirq, &isp[0]);
	free_irq(isp[0].erirq, &isp[0]);
	free_irq(isp[1].txirq, &isp[1]);
	free_irq(isp[1].rxirq, &isp[1]);
	free_irq(isp[1].erirq, &isp[1]);

errout:
	/* free up any allocated memory in the error case */
	for (i = 0; i < nr_ports; i++)
	{
#ifdef INCAIP_SSC_USEDMA
		/* SSC2 does not need any buffers if using DMA */
		if (i == 1)
			break;
#endif
		info = &isp[i];
		if (info->rxbuf != NULL)
			kfree(info->rxbuf);
		if (info->txbuf != NULL)
			kfree(info->txbuf);
	}
	kfree(isp);
	return 1;
} /* incaip_ssc_init */

#ifdef MODULE
void
incaip_ssc_cleanup_module(void)
{
	int i;

	/* free up any allocated memory */
	for (i = 0; i < nr_ports; i++)
	{
		/* disable the SSC */
		isp[i].regs->ssc_con &= ~INCA_IP_SSC1_SCC_CON_PRG_EN;
		/* free the interrupts */
		free_irq(isp[i].txirq, &isp[i]);
		free_irq(isp[i].rxirq, &isp[i]);
		free_irq(isp[i].erirq, &isp[i]);
#ifdef INCAIP_SSC_USEDMA
		/* SSC2 does not need any buffers if using DMA */
		if (i == 1)
			break;
#endif
		if (isp[i].rxbuf != NULL)
			kfree(isp[i].rxbuf);
		if (isp[i].txbuf != NULL)
			kfree(isp[i].txbuf);
	}
	kfree(isp);

#ifdef INCAIP_SSC_USEDMA
        /* Close DMA channel.
         */
        incaDMA_CloseTxChannel(INCAIP_SSC_DMACHAN);
#endif

	/* unregister the device */
	if (unregister_chrdev(maj, "ssc"))
	{
		printk("Unable to unregister major %d for the SSC\n", maj);
	}
	/* delete /proc read handler */
	remove_proc_entry("driver/ssc1", NULL);
	remove_proc_entry("driver/ssc2", NULL);
} /* incaip_ssc_cleanup_module */

module_exit(incaip_ssc_cleanup_module);
#endif /* MODULE */

/* Module entry-points */
module_init(incaip_ssc_init);

#ifndef MODULE
static int __init
incaip_ssc_set_maj(char *str)
{
	maj = simple_strtol(str, NULL, 0);
	return 1;
}
__setup("maj=", incaip_ssc_set_maj);
#endif /* !MODULE */
