#ifndef _DRV_INCA_IOM2_API_H_
#define _DRV_INCA_IOM2_API_H_
/****************************************************************************
       Copyright (c) 2003, Infineon Technologies.  All rights reserved.

                               No Warranty                                     
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties   
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The  
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.                               
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or      
   redistribute the program as permitted above, be liable to you for        
   damages, including any general, special, incidental or consequential     
   damages arising out of the use or inability to use the program           
   (including but not limited to loss of data or data being rendered        
   inaccurate or losses sustained by you or third parties or a failure of   
   the program to operate with any other programs), even if such holder or  
   other party has been advised of the possibility of such damages. 
 ****************************************************************************   
   Module      : $RCSfile: incaip_iom2_api.h,v $
   Date        : $ $
   Description : This file contains the API of the INCA-IP IOM2 device driver.

   $Log: incaip_iom2_api.h,v $
   Revision 1.1.1.1  2004-06-03 10:26:01  fischesv
   initial from infineon sdk 2.1

   Revision 1.2  2003/08/15 13:50:42  de6wthne
   * moved define for debug output to incaip_iom2.h
   * some cosmetic

   Revision 1.1  2003/08/14 11:30:23  de6wthne
   IOM-2 kernel module delivered by EPSCOM - initial version (after integration)

*******************************************************************************/

/******************************************************************************/
/*                        IOM-2 module configuration                          */
/******************************************************************************/

#define INCA_IOM2_MAJOR                0 /*  0 - dynamic major number
                                             other - predefined major number */

#define INCA_IOM2_DEV_NAME             "iom2"

/* IOM_CR: clock mode */
#define INCA_IOM2_BITCLOCK             0 /*  0 - single bit clock
                                             1 - double bit clock */
/* MODE1: Master/Slave mode select */
#define INCA_IOM2_MASTER               1 /*  0 - slave mode
                                             1 - master mode */
/* MODE1: Cofiguration Select */
#define INCAIOM2_CONFIGURATION         0

/* CIX0: TIC Bus Address */
#define INCAIOM2_TIC_BUS_ADDR          7

/* SDS_CR: Control for Serial Data Strobes */
#define INCAIOM2_TSS_ENABLE            0 /* Enable/Disable TSS + 0 */
#define INCAIOM2_TSS_1_ENABLE          0 /* Enable/Disable TSS + 1 */
#define INCAIOM2_TSS_3_ENABLE          0 /* Enable/Disable TSS + 3 */
#define INCAIOM2_TIMESLOT_SEL          0 /* Timeslot selection: 0 - 12 */

/* SDS_CONF: Configuration for Serial Data Strobes */
#define INCAIOM2_ENABLE_IOM2_CLK       0 /* 0 - Serial Data Strobe is generated 
                                            in programmed TS */
                                         /* 1 - IOM bit clock is generated 
                                            in programmed TS */

/******************************************************************************/
/*                         Additional IOCTL commands                          */
/******************************************************************************/
#define FIO_IOM2_MAGIC           '2'
#define FIO_IOM2_BASE            0

/* ---------------------------*/
/*    CDA's commands          */
/* ---------------------------*/
   /* CDA HW Options are: */
   typedef struct
   {
      u_int   DPS      :1;      /* Data Port Selection */
      u_int   TSS      :4;      /* Time Slot Selection */
   }inca_cda_opts_t;
   /* 1. DPS: Data Port Selection */
   #define CDA_DD_OUT_DU_IN      0x0
   #define CDA_DU_OUT_DD_IN      0x1
   /* 2. TSS: Time Slot Selection 0 - 11 */

#define FIO_IOM2_CDA_GET_HW_OPT     _IOR(FIO_IOM2_MAGIC,FIO_IOM2_BASE  ,inca_cda_opts_t)
#define FIO_IOM2_CDA_SET_HW_OPT     _IOW(FIO_IOM2_MAGIC,FIO_IOM2_BASE+1,inca_cda_opts_t)

   /* IOM2 CDA modes */
   typedef enum
   {
      CDA_LOOPING,
      CDA_SHIFTING_A2B,
      CDA_SHIFTING_B2A,
      CDA_SWITCHING,
      CDA_MONITORING,
      CDA_MONITORING_TIC,
      CDA_WRITING,
      CDA_DISABLED
   }inca_cda_mode_e;

#define FIO_IOM2_CDA_GET_MODE       _IOR(FIO_IOM2_MAGIC,FIO_IOM2_BASE+2,inca_cda_mode_e)
#define FIO_IOM2_CDA_SET_MODE       _IOW(FIO_IOM2_MAGIC,FIO_IOM2_BASE+3,inca_cda_mode_e)
#define FIO_IOM2_CDA_CLEAR_COUNTERS _IO(FIO_IOM2_MAGIC,FIO_IOM2_BASE+4)

/* ---------------------------*/
/*    CI Handler's commands   */
/* ---------------------------*/
   /* CI Handler HW Options are: */
   typedef struct
   {
      u_int   CICW     :1;      /* CI Channel 1 Width */
      u_int   DPS      :1;      /* Data Port Selection */
   }inca_ci_opts_t;
   /* 1. CICW: Channel 1 Width */
   #define CI1_CHANNEL_WIDTH_4      0x0
   #define CI1_CHANNEL_WIDTH_6      0x1
   /* 2. DPS: Data port selection */
   #define CI_DD_OUT_DU_IN          0x0
   #define CI_DU_OUT_DD_IN          0x1

#define FIO_IOM2_CI_GET_HW_OPT      _IOR(FIO_IOM2_MAGIC,FIO_IOM2_BASE+10,inca_ci_opts_t)
#define FIO_IOM2_CI_SET_HW_OPT      _IOW(FIO_IOM2_MAGIC,FIO_IOM2_BASE+11,inca_ci_opts_t)
#define FIO_IOM2_CI_CLEAR_COUNTERS  _IO(FIO_IOM2_MAGIC,FIO_IOM2_BASE+12)

/* ---------------------------*/
/*    D Channel commands      */
/* ---------------------------*/
   typedef struct
   {
      u_int   MDS      :3;      /* Message Transfer Mode */
      u_int   DIM0     :1;      /* Digital Interface Mode 0 */
      u_int   DIM1     :1;      /* Digital Interface Mode 1 */
      u_int   TxFIFO   :1;      /* Tx FIFO size */
      u_int   RxFIFO   :2;      /* Rx FIFO size */
      u_int   SRA      :1;      /* Store receive address */
      u_int   TxCRC    :1;      /* Transmit CRC */
      u_int   RxCRC    :1;      /* Store received CRC */
      u_int   ITF      :1;      /* Interframe time fill */
      u_int   DPS      :1;      /* Data port selection */
   }inca_d_opts_t;
   /* D Channel HW Options are: */
   /* 1. MDS: Message Transfer Mode */
   #define D_MODE_NON_AUTO_1           0x2
   #define D_MODE_NON_AUTO_2           0x3
   #define D_MODE_TRANSPARENT_EXT      0x4
   #define D_MODE_TRANSPARENT_0        0x6
   #define D_MODE_TRANSPARENT_1        0x7
   #define D_MODE_TRANSPARENT_2        0x5
   /* 2. DIM0: Collision enabled */
   #define D_COLLISION_DETECT_ENABLE   0x1
   #define D_COLLISION_DETECT_DISABLE  0x0
   /* 3. DIM1: TIC bus enabled */
   #define D_TIC_BUS_ENABLE            0x0
   #define D_TIC_BUS_DISABLE           0x1
   /* 4. TxFIFO: Tx FIFO Block Size */
   #define D_TX_FIFO_SIZE_8            0x1 
   #define D_TX_FIFO_SIZE_16           0x0 
   /* 5. RxFIFO: Rx FIFO Block Size */
   #define D_RX_FIFO_SIZE_32           0x0 
   #define D_RX_FIFO_SIZE_16           0x1 
   #define D_RX_FIFO_SIZE_8            0x2 
   #define D_RX_FIFO_SIZE_4            0x3 
   /* 6. SRA: Store Receive Address */
   #define D_STORE_RX_ADDR_ENABLE      0x1
   #define D_STORE_RX_ADDR_DISABLE     0x0
   /* 7. TxCRC: CRC is transmitted */
   #define D_TX_CRC_DISABLE            0x1
   #define D_TX_CRC_ENABLE             0x0
   /* 8. RxCRC: CRC is stored in FIFO */
   #define D_RX_CRC_ENABLE             0x1
   #define D_RX_CRC_DISABLE            0x0
   /* 9. ITF: Interframe Time Fill */
   #define D_ITF_IDLE                  0x0
   #define D_ITF_FLAGS                 0x1
   /* 10. DPS: Data Port Selection */
   #define D_DD_OUT_DU_IN              0x0
   #define D_DU_OUT_DD_IN              0x1

#define FIO_IOM2_D_GET_HW_OPT       _IOR(FIO_IOM2_MAGIC,FIO_IOM2_BASE+20,inca_d_opts_t)
#define FIO_IOM2_D_SET_HW_OPT       _IOW(FIO_IOM2_MAGIC,FIO_IOM2_BASE+21,inca_d_opts_t)
#define FIO_IOM2_D_TEST_MODE        _IO(FIO_IOM2_MAGIC,FIO_IOM2_BASE+21)

   typedef struct
   {
      u_char    SAPI1;      /* Service Access Point Identifier 1 */
      u_char    SAPI2;      /* Service Access Point Identifier 2 */
      u_char    TEI1;       /* Terminal Endpoint Identifier 1 */
      u_char    TEI2;       /* Terminal Endpoint Identifier 2 */
   }inca_sapi_tei_t;

#define FIO_IOM2_D_GET_SAPI_TEI     _IOR(FIO_IOM2_MAGIC,FIO_IOM2_BASE+22,inca_sapi_tei_t)
#define FIO_IOM2_D_SET_SAPI_TEI     _IOW(FIO_IOM2_MAGIC,FIO_IOM2_BASE+23,inca_sapi_tei_t)
#define FIO_IOM2_D_GET_NUM_BYTES    _IOR(FIO_IOM2_MAGIC,FIO_IOM2_BASE+24,int)
#define FIO_IOM2_D_CLEAR_COUNTERS   _IO(FIO_IOM2_MAGIC,FIO_IOM2_BASE+25)

/* ---------------------------*/
/*    IOM-2 test commands     */
/* ---------------------------*/
#define FIO_IOM2_PRINT_REGS         _IO(FIO_IOM2_MAGIC,FIO_IOM2_BASE+100)
#define FIO_IOM2_DEBUG_LEVEL        _IOW(FIO_IOM2_MAGIC,FIO_IOM2_BASE+101,int)

   /* Debug level masks */
   #define _NO_DEBUG_MASK     0x00000000
   #define _ERROR_MASK        0x00000001
   #define _INFO_MASK         0x00000002
   #define _TRACE_MASK        0x00000004
   #define _DATA_MASK         0x00000008
   #define _IRQ_ERROR_MASK    0x00000010
   #define _IRQ_TRACE_MASK    0x00000020
   #define _FUNC_NAME_MASK    0x10000000  /*add the function name to every debug msg*/
   #define _FULL_DEBUG_MASK   0xFFFFFFFF

/******************************************************************************/
/*                         IOM 2 Default configuration                        */
/******************************************************************************/
/* ---------------------------------------*/
/*    CDA's default configuration         */
/* ---------------------------------------*/
#define CDA10_DPS_DEFAULT     CDA_DD_OUT_DU_IN
#define CDA10_TSS_DEFAULT     0
#define CDA10_MODE_DEFAULT    CDA_DISABLED

#define CDA11_DPS_DEFAULT     CDA_DD_OUT_DU_IN
#define CDA11_TSS_DEFAULT     1
#define CDA11_MODE_DEFAULT    CDA_DISABLED

#define CDA20_DPS_DEFAULT     CDA_DU_OUT_DD_IN
#define CDA20_TSS_DEFAULT     0
#define CDA20_MODE_DEFAULT    CDA_DISABLED

#define CDA21_DPS_DEFAULT     CDA_DU_OUT_DD_IN
#define CDA21_TSS_DEFAULT     1
#define CDA21_MODE_DEFAULT    CDA_DISABLED

/* ---------------------------------------*/
/*    CI Handler's default configuration  */
/* ---------------------------------------*/
#define CI0_DPS_DEFAULT       CI_DU_OUT_DD_IN
#define CI1_DPS_DEFAULT       CI_DU_OUT_DD_IN
#define CI1_CICW_DEFAULT      CI1_CHANNEL_WIDTH_6

/* ---------------------------------------*/
/*    D Channel's default configuration   */
/* ---------------------------------------*/
#define D_MDS_DEFAULT         D_MODE_TRANSPARENT_0
#define D_DIM0_DEFAULT        D_COLLISION_DETECT_DISABLE
#define D_DIM1_DEFAULT        D_TIC_BUS_ENABLE
#define D_TX_FIFO_DEFAULT     D_TX_FIFO_SIZE_16
#define D_RX_FIFO_DEFAULT     D_RX_FIFO_SIZE_32
#define D_SRA_DEFAULT         D_STORE_RX_ADDR_DISABLE
#define D_TX_CRC_DEFAULT      D_TX_CRC_ENABLE
#define D_RX_CRC_DEFAULT      D_RX_CRC_DISABLE
#define D_ITF_DEFAULT         D_ITF_IDLE
#define D_DPS_DEFAULT         D_DU_OUT_DD_IN

#define D_SAPI1_DEFAULT       0xFC
#define D_SAPI2_DEFAULT       0xFC
#define D_TEI1_DEFAULT        0xFF
#define D_TEI2_DEFAULT        0xFF


/******************************************************************************/
/*                         DSP <---> IOM 2 interface                          */
/******************************************************************************/
/*---------------------------------------------------------------------------
   Communication between DSP and IOM2

   The communication between DSP and IOM2 is done in a direct way. To setup 
   this communication the CO_TSDPxy (codec time slot & data port) selection 
   register have to be to used.

   The following function provides an API to setup this communication. The 
   setup has to be done once a time and the communication is done under HW 
   control. 
   So the IOM2 driver is not directly involved.

   Register naming convention: communication DSP - IOM2 
   ===========================
   INCA_IOM_DSP_CHANNEL1_LOW  = 0 :  CO_TSDP10   DSP Channel 1   LOW  Byte
   INCA_IOM_DSP_CHANNEL1_HIGH = 1 :  CO_TSDP11   DSP Channel 1   HIGH Byte
   INCA_IOM_DSP_CHANNEL2_LOW  = 2 :  CO_TSDP20   DSP Channel 2   LOW  Byte
   INCA_IOM_DSP_CHANNEL2_HIGH = 3 :  CO_TSDP21   DSP Channel 2   HIGH Byte

---------------------------------------------------------------------------*/
/* ---------------------------*/
/*    CO's commands           */
/* ---------------------------*/
/* 
   for setup the communication between DSP and IOM2 
   we have to set the corresponding CO_TSDPxy register
   and you have to enable the corresponding data channel
   This is done via the "inca_iom2_co_set_TSDPxy" API function
*/
typedef enum
{
   DATA_CHANNEL_DISABLE,
   DATA_CHANNEL_ENABLE,
   DATA_CHANNEL_IGNORE
}inca_data_channel_state_e;

typedef struct
{
   u_int   DPS      :1;      /* Data Port Selection */
   u_int   TSS      :4;      /* Time Slot Selection */
}inca_co_opts_t;


/* CO HW Options are: */
/* 1. DPS: Data Port Selection */
#define CO_DD_OUT_DU_IN      0x0
#define CO_DU_OUT_DD_IN      0x1
/*
   communication DSP - IOM2 
   CO_TSDP10      DSP Channel 1   LOW  Byte
   CO_TSDP11      DSP Channel 1   HIGH Byte
   CO_TSDP20      DSP Channel 2   LOW  Byte
   CO_TSDP21      DSP Channel 2   HIGH Byte
*/
#define INCA_IOM_DSP_CHANNEL1_LOW   0
#define INCA_IOM_DSP_CHANNEL1_HIGH  1
#define INCA_IOM_DSP_CHANNEL2_LOW   2
#define INCA_IOM_DSP_CHANNEL2_HIGH  3

/* ---------------------------------------*/
/*    CO_TSDPxy default configuration     */
/* ---------------------------------------*/
/* output on B1-DU */
#define CO_TSDP10_DPS_DEFAULT     CO_DU_OUT_DD_IN
#define CO_TSDP10_TSS_DEFAULT     0
/* output on B2-DU */
#define CO_TSDP11_DPS_DEFAULT     CO_DU_OUT_DD_IN
#define CO_TSDP11_TSS_DEFAULT     1
/* output on B2-DU */
#define CO_TSDP20_DPS_DEFAULT     CO_DU_OUT_DD_IN
#define CO_TSDP20_TSS_DEFAULT     1
/* output on IC2-DU */
#define CO_TSDP21_DPS_DEFAULT     CO_DU_OUT_DD_IN
#define CO_TSDP21_TSS_DEFAULT     5


/* ---------------------------------------*/
/*    CO API functions                    */
/* ---------------------------------------*/
void inca_iom2_co_regs_default(void);
/*
   Remark:
   The CO_TSDP register correspond to the following data channels
      CO_TSDP10 --> BCHA_CR.ENBC1
      CO_TSDP11 --> BCHA_CR.ENBC2
      CO_TSDP20 --> BCHB_CR.ENBC1
      CO_TSDP21 --> BCHB_CR.ENBC2
*/
int inca_iom2_co_set_TSDPxy( int reg_num, 
                      inca_co_opts_t *co_opts,
                      inca_data_channel_state_e state);
int inca_iom2_co_get_TSDPxy( int reg_num, 
                      inca_co_opts_t *co_opts);

#endif /* _DRV_INCA_IOM2_API_H_ */
