/******************************************************************************
       Copyright (c) 2002, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for
   the program, to the extent permitted by applicable law.  Except when
   otherwise stated in writing the copyright holders and/or other parties
   provide the program "as is" without warranty of any kind, either
   expressed or implied, including, but not limited to, the implied
   warranties of merchantability and fitness for a particular purpose. The
   entire risk as to the quality and performance of the program is with
   you.  should the program prove defective, you assume the cost of all
   necessary servicing, repair or correction.

   In no event unless required by applicable law or agreed to in writing
   will any copyright holder, or any other party who may modify and/or
   redistribute the program as permitted above, be liable to you for
   damages, including any general, special, incidental or consequential
   damages arising out of the use or inability to use the program
   (including but not limited to loss of data or data being rendered
   inaccurate or losses sustained by you or third parties or a failure of
   the program to operate with any other programs), even if such holder or
   other party has been advised of the possibility of such damages.
 ******************************************************************************
   Module      : ledmatrix.h
   Date        : 2003-06-03
   Description :
      Driver Module for the LED hardware of INCA-IP
******************************************************************************/
#ifndef _inca_led_H
#define _inca_led_H

#ifdef __KERNEL__
/* Function Prototypes */
extern u32	inca_led_off		(u32 indexLED);
extern u32	inca_led_on		(u32 indexLED);
extern u8	inca_led_eth_ctrl	(int eth_port, u8 eth_leds);
#endif

/* IOCTLs */
#define LED_ON		_IOW('T', 1, unsigned int)
#define LED_OFF		_IOW('T', 2, unsigned int)
#define LED_ETH_PC	_IOW('T', 3, unsigned char)
#define LED_ETH_LAN	_IOW('T', 4, unsigned char)

/* LED Matrix acc. to User's Manual; LED(x,y) corresponds to Lxy

   pins  |   LED0     LED1     LED2*    LED3*
  -------+-------------------------------------	   
   LED4  | LED(0,4) LED(1,4) LED(2,4) LED(3,4)
   LED5  | LED(0,5) LED(1,5) LED(2,5) LED(3,5)
   ...	 |	...	...	...	...
         |
   LED8  | LED(0,8) LED(1,8) LED(2,8) LED(3,8)
   LED9* | LED(0,9) LED(1,9) LED(2,9) LED(3,9)                    
   
   *: Pins LED2, LED3 and LED9 may be unavailable!                */
#define LED(x,y) 	(1 << ((y-4)*4 + x)) 	/* x=0..3, y=4..9 */

/* LEDs optionally controlled autonomously by Ethernet HW */
#define LED_ETHPORT_PC	0	
#define LED_ETHPORT_LAN	1

#define LED_ETH_SPD	(1 << 0)	/* LAN: LED(0,4) ; PC: LED(0,5) */
#define LED_ETH_ACT	(1 << 1)	/* LAN: LED(1,4) ; PC: LED(1,5) */
#define LED_ETH_DPX	(1 << 2)	/* LAN: LED(2,4) ; PC: LED(2,5) */
#define LED_ETH_STA	(1 << 3)	/* LAN: LED(3,4) ; PC: LED(3,5) */
#define LED_ETH_TL	(1 << 4)


#endif /* _inca_led_H */
