/*
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * port.h
 *
 * Global Inca-IP port driver header file
 *
 * Andre Messerschmidt 
 * Infineon Technologies AG (C) 2003
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#ifndef PORT_H
#define PORT_H

#define GS_DISP_DATA_SIZE 1024
struct inca_port_ioctl_parm {
	int port;
	int pin;
	int value;
#ifdef DISP_GS_240x128
        unsigned char disp_bytes[GS_DISP_DATA_SIZE];
#endif
};

#define INCA_PORT_IOC_MAGIC	0xbf
#define INCA_PORT_IOCOD		_IOW(INCA_PORT_IOC_MAGIC,0,struct inca_port_ioctl_parm)
#define INCA_PORT_IOCPUDSEL	_IOW(INCA_PORT_IOC_MAGIC,1,struct inca_port_ioctl_parm)
#define INCA_PORT_IOCPUDEN	_IOW(INCA_PORT_IOC_MAGIC,2,struct inca_port_ioctl_parm)
#define INCA_PORT_IOCSTOFF	_IOW(INCA_PORT_IOC_MAGIC,3,struct inca_port_ioctl_parm)
#define INCA_PORT_IOCDIR		_IOW(INCA_PORT_IOC_MAGIC,4,struct inca_port_ioctl_parm)
#define INCA_PORT_IOCOUTPUT	_IOW(INCA_PORT_IOC_MAGIC,5,struct inca_port_ioctl_parm)
#define INCA_PORT_IOCINPUT	_IOWR(INCA_PORT_IOC_MAGIC,6,struct inca_port_ioctl_parm)
#define INCA_PORT_DISPCMD	_IOWR(INCA_PORT_IOC_MAGIC,7,struct inca_port_ioctl_parm)
#define INCA_PORT_DISPDATA	_IOWR(INCA_PORT_IOC_MAGIC,8,struct inca_port_ioctl_parm)
#define INCA_PORT_GS_DISPCMD    _IOWR(INCA_PORT_IOC_MAGIC,9,struct inca_port_ioctl_parm)
#define INCA_PORT_GS_DISPDATA   _IOWR(INCA_PORT_IOC_MAGIC,10,struct inca_port_ioctl_parm)
#define INCA_PORT_GS_MDISPDATA  _IOWR(INCA_PORT_IOC_MAGIC,11,struct inca_port_ioctl_parm)
#define INCA_PORT_GS_MDISPDATA_2B3P  _IOWR(INCA_PORT_IOC_MAGIC,12,struct inca_port_ioctl_parm)
#define INCA_PORT_GS_DISPBYTES  _IOWR(INCA_PORT_IOC_MAGIC,13,struct inca_port_ioctl_parm)


int inca_port_reserve_pin(int port, int pin, int module_id);
int inca_port_free_pin(int port, int pin, int module_id);
int inca_port_set_open_drain(int port, int pin, int module_id);
int inca_port_clear_open_drain(int port, int pin, int module_id);
int inca_port_set_pudsel(int port, int pin, int module_id);
int inca_port_clear_pudsel(int port, int pin, int module_id);
int inca_port_set_puden(int port, int pin, int module_id);
int inca_port_clear_puden(int port, int pin, int module_id);
int inca_port_set_stoff(int port, int pin, int module_id);
int inca_port_clear_stoff(int port, int pin, int module_id);
int inca_port_set_dir_out(int port, int pin, int module_id);
int inca_port_set_dir_in(int port, int pin, int module_id);
int inca_port_set_output(int port, int pin, int module_id);
int inca_port_clear_output(int port, int pin, int module_id);
int inca_port_get_input(int port, int pin, int module_id);


#endif /* PORT_H */


